{$IFNDEF FPC}
  {$N+}
  {$E+}
{$ENDIF}
{
        Program Name : APM
        Package Name : Subroutine Library - Layer 2
        Purpose      : Application Performance Monitor
        Institution  : Conroy & Conroy Co.
        Date Written : 31-January-2005
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2005-2015 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          The classes in this unit are used to provide a performance monitor for
        an application.
}

unit APM ;

interface

uses { Subroutine Library - Layer 1... }
     _APM, { TSApplication_Performance_Monitor }
     Collect ; { TString_Collection }

{$I EDEFINES.INC}

type TSStandard_Application_Performance_Monitor = object( TSApplication_Performance_Monitor )
                                                      private
                                                          _Mult : double ;
                                                          Transactions : TString_Collection ;

                                                      public { Constructors and destructors... }
                                                          constructor Init ;
                                                          destructor Done ;
                                                             virtual ;

                                                      public { API... }
                                                          function Begin_Transaction : integer ;
                                                              virtual ;
                                                          procedure End_Transaction( TN, Max : integer ;
                                                              const Title : string ) ;
                                                              virtual ;
                                                          procedure Set_Multiplier( Value : double ) ;
                                                              virtual ;
                                                          function Get_Multiplier : double ;
                                                              virtual ;
                                                          function Enabled : boolean ;
                                                              virtual ;
                                                  end ;
     PSStandard_Application_Performance_Monitor = ^TSStandard_Application_Performance_Monitor ;

{$IFDEF DYNAMIC_OBJECTS}
type TStandard_Application_Performance_Monitor = class( TApplication_Performance_Monitor )
                                                     private
                                                         _SAPM : TSStandard_Application_Performance_Monitor ;

                                                     public { Constructors and destructors... }
                                                         constructor Create ;
                                                         destructor Destroy ;
                                                            override ;

                                                     public { API... }
                                                         function Begin_Transaction : integer ;
                                                             override ;
                                                         procedure End_Transaction( TN, Max : integer ;
                                                             const Title : string ) ;
                                                             override ;
                                                         procedure Set_Multiplier( Value : double ) ;
                                                             override ;
                                                         function Get_Multiplier : double ;
                                                             override ;
                                                 end ;

type TStandard_COM_Application_Performance_Monitor = class( TCOM_Application_Performance_Monitor )
                                                         private
                                                             _DAPM : TStandard_Application_Performance_Monitor ;

                                                         public { Constructors and destructors... }
                                                             constructor Create ;
                                                             destructor Destroy ;
                                                                override ;

                                                         public { API... }
                                                             function Begin_Transaction : integer ;
                                                                 override ;
                                                             procedure End_Transaction( TN, Max : integer ;
                                                                 Title : PChar ) ;
                                                                 override ;
                                                             procedure Set_Multiplier( Value : double ) ;
                                                                 override ;
                                                             function Get_Multiplier : double ;
                                                                 override ;
                                                     end ;
{$ENDIF}

function Get_Tick_Count : cardinal ;


implementation

uses
{$IFDEF FPC_OR_DELPHI}
     { Borland... }
     Windows, { GetTickCount }
{$ENDIF}

     { Subroutine Library... }
     _Applic, { Application_Manager }
     L10n, { Substitute2 }
     Num1s ; { Num1 }

{ API... }

function Get_Tick_Count : LongWord ;

{$IFNDEF FPC_OR_DELPHI}
var Year, Month, Day, Hour, Minute, Second, Hundreths, DOW : word ;
    Temp : longword ;
{$ENDIF}

begin
{$IFDEF FPC_OR_DELPHI}
    Get_Tick_Count := GetTickCount ; { Number of ms since Windows started }
{$ELSE}
    GetTime( Hour, Minute, Second, Hundreths ) ;
    GetDate( Year, Month, Day, DOW ) ;
    Temp := Hundreths * 10 ;
    Temp := Temp + Second * 1000 ;
    Temp := Temp + Minute * 60000 ;
    Temp := Temp + Hour * 3600000 ;
    Temp := Temp + Day * 86400000 ;
    Get_Tick_Count := Temp ; { Number of ms since start of month }
{$ENDIF}
end ;



{ TSStandard_Application_Performance_Monitor methods... }

{ Constructors and destructors... }

constructor TSStandard_Application_Performance_Monitor.Init ;

begin
    inherited Init ;

    Transactions.Init( 0, 4 ) ;
    _Mult := 0 ;
end ;


destructor TSStandard_Application_Performance_Monitor.Done ;

begin
    Transactions.Done ;

    inherited Done ;
end ;


{ API... }

function TSStandard_Application_Performance_Monitor.Begin_Transaction : integer ;

begin
    if( not Enabled ) then
    begin
        Begin_Transaction := 0 ;
        exit ;
    end ;
    Transactions.Insert( Num1( Get_Tick_Count ) ) ;
    Begin_Transaction := Transactions.Count - 1 ;
end ;


procedure TSStandard_Application_Performance_Monitor.End_Transaction( TN, Max : integer ;
    const Title : string ) ;

var E : integer ;
    Temp : double ;
    Timing : longword ;

begin
    if( not Enabled ) then
    begin
        exit ;
    end ;
    if( ( TN >= 0 ) and ( TN < Transactions.Count ) ) then
    begin
{$R-}
        val( Transactions.At( TN ), Timing, E ) ;
        Timing := Get_Tick_Count - Timing ;
{$R+}
        if( TN = Transactions.Count - 1 ) then
        begin
            Transactions.AtDelete( TN ) ;
        end ;
        Temp := Max ;
        if( _Mult > 0 ) then
        begin
            Temp := Temp * _Mult ;
        end ;
        if( ( Timing > trunc( Temp ) ) and ( Application_Manager <> nil ) ) then
        begin
            Application_Manager^.Popup_Message( Substitute2( Text_Error_Execution_Exceeds_ms, Title, Num1( Timing ) ) ) ;
        end ;
    end ;
end ;


procedure TSStandard_Application_Performance_Monitor.Set_Multiplier( Value : double ) ;

begin
    _Mult := Value ;
end ;


function TSStandard_Application_Performance_Monitor.Get_Multiplier : double ;

begin
    Get_Multiplier := _Mult ;
end ;


function TSStandard_Application_Performance_Monitor.Enabled : boolean ;

begin
    Enabled := _Mult > 0 ;
end ;



{$IFDEF DYNAMIC_OBJECTS}
{ TStandard_COM_Application_Performance_Monitor methods... }

{ Constructors and destructors... }

constructor TStandard_COM_Application_Performance_Monitor.Create ;

begin
    inherited Create ;

    _DAPM := TStandard_Application_Performance_Monitor.Create ;
end ;


destructor TStandard_COM_Application_Performance_Monitor.Destroy ;

begin
    _DAPM.Free ;

    inherited Destroy ;
end ;


{ API... }

function TStandard_COM_Application_Performance_Monitor.Begin_Transaction : integer ; stdcall ;

begin
    Result := _DAPM.Begin_Transaction ;
end ;


procedure TStandard_COM_Application_Performance_Monitor.End_Transaction( TN, Max : integer ;
    Title : PChar ) ; stdcall ;

begin
    _DAPM.End_Transaction( TN, Max, Title ) ;
end ;


procedure TStandard_COM_Application_Performance_Monitor.Set_Multiplier( Value : double ) ; stdcall ;

begin
    _DAPM.Set_Multiplier( Value ) ;
end ;


function TStandard_COM_Application_Performance_Monitor.Get_Multiplier : double ; stdcall ;

begin
    Get_Multiplier := _DAPM.Get_Multiplier ;
end ;



{ TStandard_Application_Performance_Monitor methods... }

{ Constructors and destructors... }

constructor TStandard_Application_Performance_Monitor.Create ;

begin
    inherited Create ;

    _SAPM.Init ;
end ;


destructor TStandard_Application_Performance_Monitor.Destroy ;

begin
    _SAPM.Done ;

    inherited Destroy ;
end ;


{ API... }

function TStandard_Application_Performance_Monitor.Begin_Transaction : integer ;

begin
    Result := _SAPM.Begin_Transaction ;
end ;


procedure TStandard_Application_Performance_Monitor.End_Transaction( TN, Max : integer ;
    const Title : string ) ;

begin
    _SAPM.End_Transaction( TN, Max, Title ) ;
end ;


procedure TStandard_Application_Performance_Monitor.Set_Multiplier( Value : double ) ;

begin
    _SAPM.Set_Multiplier( Value ) ;
end ;


function TStandard_Application_Performance_Monitor.Get_Multiplier : double ;

begin
    Get_Multiplier := _SAPM.Get_Multiplier ;
end ;
{$ENDIF}



end.
