{$IFNDEF FPC}
  {$N+}
{$ENDIF}
{$A-}
unit Allocatr64 ;
{
        Program Name : Allocatr64
        Package Name : Subroutine Library - Layer 3
        Purpose      : 64-bit Allocator class
        Institution  : Conroy & Conroy Co.
        Date Written : 1998
        Written By   : Alan Conroy
        Version      : 1.0D

        Copyright (C) 1998-2016 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

         15-Nov-2002    EAC         Added debug interface.
         16-Nov-2002    EAC         Allow viewing of AT in debug.
          2-Dec-2002    EAC         Handle strings in objects.
         14-May-2003    EAC         Fix problem with uninitialized
                                    Lowest_Free_Space.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

	    This unit contains the TCOM_Allocator64 class, which is used to manage
        storage.  It also contains TAT64, a TAllocator which manages storage via
        an allocation table.
}

interface

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

{$I EDEFINES.INC}

uses { Subroutine Library - Layer 0... }
     CommBase, { TBase_Object }
{$IFNDEF MOBILE}
     _DebugIn, { PSDebug_Interface }
{$ENDIF}
     TypeDefs, { pByte_Array }

     { Subroutine Library - Layer 1... }
     _Stores ; { TStore_Address }

{$IFDEF DYNAMIC_OBJECTS}
  {$IFDEF INT64}
type TCOM_Allocator64 = class( TBase_Delphi_Object )
                           public { API... }
                               function Get_Offset : TStore_Address64 ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function Get_Resolution : cardinal ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function Get_Size : cardinal ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function Get_Store : TCOM_Store64 ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;

                               procedure Set_Offset( Value : TStore_Address64 ) ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Set_Resolution( Value : cardinal ) ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Set_Size( Value : cardinal ) ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Set_Store( Value : TCOM_Store64 ) ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;

                           public { API... }
                               procedure Flush ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function Allocate( Size : TStore_Address64 ) : TStore_Address64 ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function Allocate_At( Pos, Size : TStore_Address64 ) : boolean ;
                                   virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Deallocate( PTR : TStore_Address64 ;
                                   Size : TStore_Address64 ) ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Load_Table ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function MaxSpace : TStore_Size64 ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function SpaceAvail : TStore_Size64 ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               function Get_Data : pointer ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Set_Data( Value : pointer ;
                                   Size : cardinal ) ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;
                               procedure Find_Lowest_Space ; virtual ;
{$IFDEF Use_STDCALL}
                                   stdcall ;
{$ENDIF}
                                   abstract ;

                               property Offset : TStore_Address64
                                   read Get_Offset
                                   write Set_Offset ;
                               property Resolution : cardinal
                                   read Get_Resolution
                                   write Set_Resolution ;
                               property Size : cardinal
                                   read Get_Size
                                   write Set_Size ;
                               property Store : TCOM_Store64
                                   read Get_Store
                                   write Set_Store ;
                       end ; { TCOM_Allocator64 }

     TAT64 = class( TCOM_Allocator64 )
               public { Constructors and destructors... }
                   constructor Create( _Resolution : cardinal ) ;
                   destructor Destroy ; override ;

               private { Instance data... }
                   _AT : pByte_Array ; // Allocation table image
                   Byte_Mapping : cardinal ; // How much storage is mapped by one byte (8*Resolution)
                   _Offset : TStore_Address64 ; { AT offset in store }
                   _Size : cardinal ; { AT size, in bytes }
                   _Store : TCOM_Store64 ; { Store where table is saved }
                   Lowest_Free_Space : TStore_Address64 ; { Address of lowest unallocated space }
                   _Modified : boolean ; { True if unsaved modifications }
                   _Resolution : cardinal ; { How many bytes each bit represents }

               protected { Internal utility routines... }
                   function AT : pByte_Array ;

               public { API... }
                   function Get_Modified : boolean ; virtual ;
{$IFDEF Use_STDCALL}
                       stdcall ;
{$ENDIF}
                   function Get_Offset : TStore_Address64 ; override ;
                   function Get_Resolution : cardinal ; override ;
                   function Get_Size : cardinal ; override ;
                   function Get_Store : TCOM_Store64 ; override ;

                   procedure Set_Modified( Value : boolean ) ; virtual ;
{$IFDEF Use_STDCALL}
                       stdcall ;
{$ENDIF}
                   procedure Set_Offset( Value : TStore_Address64 ) ; override ;
                   procedure Set_Resolution( Value : cardinal ) ; override ;
                   procedure Set_Size( Value : cardinal ) ; override ;
                   procedure Set_Store( Value : TCOM_Store64 ) ; override ;

               public { API... }
                   procedure Flush ; override ; { Write AT changes to store }
                   function Allocate( Size : TStore_Address64 ) : TStore_Address64 ;
                       override ;
                   function Allocate_At( Pos, Size : TStore_Address64 ) : boolean ;
                       override ;
                   procedure Deallocate( PTR : TStore_Address64 ;
                       Size : TStore_Size64 ) ; override ;
                   function Reallocate( PTR : TStore_Address ;
                       Old, New : TStore_Address ) : TStore_Address ;
{$IFNDEF MOBILE}
                   function Debugger : TDebug_Interface ; override ;
{$ENDIF}
                   procedure Load_Table ; override ;
                   function MaxSpace : TStore_Size64 ; override ;
                   function Space_At( PTR : TStore_Address64 ) : TStore_Size64 ;
		               function SpaceAvail : TStore_Size64 ; override ;
                   function Get_Data : pointer ; override ;
                   procedure Set_Data( Value : pointer ; Size : cardinal ) ;
                       override ;
                   procedure Find_Lowest_Space ; override ;

                   property Modified : boolean
                       read Get_Modified
                       write Set_Modified ;
                   property Resolution : cardinal
                       read Get_Resolution
                       write Set_Resolution ;
           end ; { TAT64 }
  {$ENDIF}
{$ENDIF}

{$IFDEF CC_Regression_Test}
procedure Test_Unit ;
{$ENDIF}

implementation

uses { Subroutine Library - Layer 0... }
     L10n, { Text_* }
     _Types, { PAnsichar }
     _UE, { TUnified_Exception }

     { Subroutine Library - Layer 1... }
     CommonUt, { Boolean_To_String }
     Compatib, { ReallocMem }
{$IFNDEF MOBILE}
     DebugInt, { TText_Debugger }
{$ENDIF}
     Num1s, { Num1 }

     { Subroutine Library - Layer 3... }
     GIO, { Log_Error }
     _CCStandard, { Bit_Values }
     UStrings { Boolean_To_String }
{$IFDEF DYNAMIC_OBJECTS}
  {$IFDEF CC_Regression_Test}
     ,Forms
  {$ENDIF}
     ,sysutils
     ,Heaps // TCOM_Heap64
     ,UE // TStandard_Error_Interface
{$ENDIF}
     ;

{ TByte_Array_Debugger... }

{$IFDEF CC_INTERNALS}
type PByte_Array_Debugger = ^TByte_Array_Debugger ;
     TByte_Array_Debugger = object( TSText_Debugger )
                                private
                                    _Instance : PByte_Array ;
                                    _Size : integer ;
                                    S : string[ 255 ] ;

                                public
                                    constructor Init( I : PByte_Array ;
					Size : integer ) ;

                                    function Count : longint ; virtual ;

                                    function Child( Ordinal : longint ) : PSDebug_Interface ;
                                        virtual ;
                            end ;

constructor TByte_Array_Debugger.Init( I : PByte_Array ; Size : integer ) ;

begin
    inherited Init ;

    _Instance := I ;
    _Size := Size ;
end ;


function TByte_Array_Debugger.Count : longint ;

begin
    Count := _Size ;
end ;


function TByte_Array_Debugger.Child( Ordinal : longint ) : PSDebug_Interface ;

var R : PSText_Debugger ;

begin
    new( R, Init ) ;
    S := Num1( Ordinal ) + ' : ' + Num1( _Instance^[ Ordinal ] ) ;
    R^.Set_Title( @S[ 1 ] ) ;
    Child := PSDebug_Interface( R ) ;
end ;
{$ENDIF}



{$IFDEF DYNAMIC_OBJECTS}
  {$IFDEF INT64}
const AT_Error_Load_Failure = 1 ;


function ERT( C : longint ) : string ;

begin
    case C of
        AT_Error_Load_Failure : Result := 'Load failure' ;
        else Result := 'Unknown error' ;
    end ;
end ;


type tAllocator_Exception = class( TStandard_Error_Interface )
                                                        private
                                                            _Code : longint ;
                                                            _Text : Ansistring ;
                                                            _Previous : tUnified_Exception ;

                                                        public
                                                            function Get_Facility : longint ; override ;
                                                            function Get_Facility_Version : longint ;
                                                                override ;
                                                            function Version : longint ; override ;
                                                            function Severity : longint ; override ;
                                                            function Error_Text( var Size, Typ : longint ) : PAnsiChar ;
                                                                override ;
                                                            function Get_Error : longint ; override ;
                                                            function Get_Previous : tUnified_Exception ;
                                                                override ;
                                                            procedure Terminate ; override ;
                                                    end ;


function tAllocator_Exception.Get_Facility : longint ;

begin
    Result := 137 ;
end ;


function tAllocator_Exception.Get_Facility_Version : longint ;

begin
    Result := 10 ;
end ;


function tAllocator_Exception.Version : longint ;

begin
    Result := 10 ;
end ;


function tAllocator_Exception.Severity : longint ;

begin
    Result := UE_Error ;
end ;


function tAllocator_Exception.Error_Text( var Size, Typ : longint ) : PAnsiChar ;

begin
    _Text := ERT( _Code ) ;
    Size := length( _Text ) ;
    Typ := 0 ;
    Result := PAnsiChar( _Text ) ;
end ;


function tAllocator_Exception.Get_Error : longint ;

begin
    Result := _Code ;
end ;


function tAllocator_Exception.Get_Previous : tUnified_Exception ;

begin
    Result := _Previous ;
end ;


procedure tAllocator_Exception.Terminate ;

begin
    Free ;
end ;



function Create_Exception( C : longint ; E : TUnified_Exception ) : tAllocator_Exception ;

begin
    Result := tAllocator_Exception.Create ;
    Result._Code := C ;
    Result._Previous := E ;
end ;



{$IFDEF CC_INTERNALS}
type PAT_Debugger64 = ^TAT_Debugger64 ;
     TAT_Debugger64 = object( TSText_Debugger )
                          private
                              _Instance : TAT64 ;
                              S : string[ 255 ] ;

                          public
                              constructor Init( I : TAT64 ) ;

                              function Count : longint ; virtual ;

                              function Child( Ordinal : longint ) : PSDebug_Interface ;
                                  virtual ;
                      end ;

constructor TAT_Debugger64.Init( I : TAT64 ) ;

begin
    inherited Init ;

    _Instance := I ;
end ;


function TAT_Debugger64.Count : longint ;

begin
    Count := 5 ;
end ;


function TAT_Debugger64.Child( Ordinal : longint ) : PSDebug_Interface ;

var R : PSText_Debugger ;
    R1 : PByte_Array_Debugger ;

begin
    new( R, Init ) ;
    case Ordinal of
        0 : begin
                R^.Free ;
                R1 := new( PByte_Array_Debugger, Init( _Instance.AT, _Instance.Byte_Mapping ) ) ;
                R := PSText_Debugger( R1 ) ;
                S := 'AT' ;
            end ;
        1 : S := 'Modified = ' + Boolean_To_String( _Instance.Modified ) ;
        2 : S := 'Byte_Mapping = ' + Num1( _Instance.Byte_Mapping ) ;
        3 : S := 'Size = ' + Num1( _Instance.Get_Size ) ;
        4 : S := 'Lowest_Free_Space = ' + Num1( _Instance.Lowest_Free_Space ) ;
    end ;
    R^.Set_Title( @S[ 1 ] ) ;
    Child := PSDebug_Interface( R ) ;
end ; { TAT_Debugger.Child }
{$ENDIF}



{ TAT64 methods... }

{ Constructors and destructors... }

constructor TAT64.Create( _Resolution : cardinal ) ;

begin
    inherited Create ;

    Resolution := _Resolution ;
    Byte_Mapping := 8 * Resolution ; { How much one AT byte maps }
end ;


destructor TAT64.Destroy ;

begin
    Flush ; { Update store }
    Set_Size( 0 ) ; { Clear cache }

    inherited Destroy ;
end ;


{ Internal utility routines... }

function TAT64.AT : pByte_Array ;

begin
    if( _AT = nil ) then
    begin
        _AT := Allocmem( _Size ) ;
        fillchar( _AT^, _Size, 0 ) ;
        _AT^[ 0 ] := 1 ; // Lowest cluster is always allocated
        Lowest_Free_Space := Resolution ;
    end ;
    Result := _AT ;
end ;


{ API... }

function TAT64.Get_Modified : boolean ;

begin
    Get_Modified := _Modified ;
end ;


function TAT64.Get_Offset : TStore_Address64 ;

begin
    Get_Offset := _Offset ;
end ;


function TAT64.Get_Resolution : cardinal ;

begin
    Get_Resolution := _Resolution ;
end ;


function TAT64.Get_Size : cardinal ;

begin
    Get_Size := _Size ;
end ;


function TAT64.Get_Store : TCOM_Store64 ;

begin
    Get_Store := _Store ;
end ;


procedure TAT64.Set_Modified( Value : boolean ) ;

begin
    _Modified := Value ;
end ;


procedure TAT64.Set_Offset( Value : TStore_Address64 ) ;

begin
    if( _Offset <> Value ) then
    begin
        _Offset := Value ;
    end ;
end ;


procedure TAT64.Set_Resolution( Value : cardinal ) ;

begin
    if( Value <> Resolution ) then
    begin
        _Resolution := Value ;
        Byte_Mapping := 8 * _Resolution ; { How much one AT byte maps }
    end ;
end ;


procedure TAT64.Set_Size( Value : cardinal ) ;

begin
    if( _Size <> Value ) then
    begin
        if( _AT <> nil ) then
        begin
            reallocmem( pointer( _AT ), _Size, Value ) ;
            if( Value > Size ) then
            begin
                fillchar( _AT[ _Size ], Value - _Size, 0 ) ;
            end ;
        end ;
        _Size := Value ;
    end ;
end ; { TAT64.Set_Size }


procedure TAT64.Set_Store( Value : TCOM_Store64 ) ;

var C : int64 ;

begin
    if( Value <> nil ) then
    begin
        Value.Attach ;
        if( Resolution < Value.Min_Storage ) then { else untested }
        begin
            Resolution := Value.Min_Storage ;
        end ;
        if( Resolution <> 0 ) then
        begin
            C := Value.Max_Storage div Resolution ; // Total clusters on store
        end else
        begin
            C := 0 ;
        end ;
        _Size := C div 8 ; // Size of AT
        Byte_Mapping := 8 * Resolution ; { How much one AT byte maps }
    end ;
    if( _Store <> nil ) then
    begin
        _Store.Detach ;
    end ;
    _Store := Value ;
end ;


procedure TAT64.Flush ; { Write AT changes to store }

var UEC : TUnified_Exception ;

begin
    if( ( _Offset <> 0 ) and ( _Store <> nil ) and Modified ) then
    begin
        _Store.Write_Data( AT^, _Offset, _Size, UEC ) ;
        Set_Last_Error( UEC ) ;
        Modified := False ;
    end ;
end ; { TAT64.Flush }


function TAT64.Allocate( Size : TStore_Address64 ) : TStore_Address64 ;
{ Allocate Size bytes and return pointer.  Return 0 if error. }

var AT_Byte, AT_Bit : integer ; { Current byte and bit within AT }
    Counter : TStore_Size64 ;
    Lowest_Free_Byte : integer ;
    Offset : TStore_Address64 ;
    Offset_Byte, Offset_Bit : cardinal ;

    procedure Set_Start ;

    begin
        Offset := AT_Byte ; { Allocation table byte offset }
        Offset := Offset * Byte_Mapping + AT_Bit * Resolution ; { Remember start }
        Offset_Byte := AT_Byte ;
        Offset_Bit := AT_Bit ;
    end ;

begin
    Allocate := 0 ; { Return zero (0) if error }
    if( Size < 1 ) then { Cannot allocate nothing, or less than nothing }
    begin
        exit ;
    end ;
    Size := ( Size + Resolution - 1 ) and ( not ( Resolution - 1 ) ) ;
    { Allocate minimum multiple bytes at a time }
    Lowest_Free_Byte := Lowest_Free_Space div Byte_Mapping ;
    AT_Byte := Lowest_Free_Byte ;
    if( AT_Byte + ( ( Size div Resolution + 7 ) div 8 ) - 1 >= _Size ) then { More space than we have }
    begin
        exit ;
    end ;

    { Find a place where we can allocate the requested space... }
    Offset := 0 ; { So we know if allocation failed }
    { Start at the beginning }

    AT_Bit := 0 ;
    Counter := 0 ; { Number of bytes available in current segment }
    while( Counter < Size ) do { Find necessary free space }
    begin
        if( AT^[ AT_Byte ] = 255 ) then { No room in entire byte }
        begin
            if( Lowest_Free_Byte = AT_Byte ) then { If lowest free byte is this byte, which is full }
            begin
                inc( Lowest_Free_Byte ) ;
                Lowest_Free_Space := Lowest_Free_Space + Byte_Mapping ;
            end ;
            Counter := 0 ;
            AT_Bit := 7 ; { Skip rest of bits in this byte }
        end else
        begin
            if(
                ( AT_Bit = 0 )
                and
                ( Size - Counter >= Byte_Mapping )
                and
                ( AT^[ AT_Byte ] = 0 )
              ) then
            begin { Quick check for entire byte }
                if( Counter = 0 ) then
                begin
                    Set_Start ;
                end ;
                Counter := Counter + Byte_Mapping ;
                AT_Bit := 7 ;
                if( Counter >= Size ) then
                begin
                    break ;
                end ;
            end else
            begin { Check bit by bit }
                if( ( AT^[ AT_Byte ] and Bit_Values[ AT_Bit ] ) = 0 ) then
                begin
                    if( Counter = 0 ) then { New/first free block }
                    begin
                        Set_Start ;
                    end ;
                    Counter := Counter + Resolution ; { Count up free bytes }
                    if( Counter >= Size ) then
                    begin
                        break ;
                    end ;
                end else { Ran out of free blocks before we found sufficient space }
                begin
                    Counter := 0 ; { Start over }
                end ;
            end ;
        end ;
        inc( AT_Bit ) ; { Check next bit of current byte }
        if( AT_Bit > 7 ) then { Ran out of bits on this byte }
        begin
            AT_Bit := 0 ; { Start at first bit of next byte }
            inc( AT_Byte ) ; { Move to next byte }
            if( AT_Byte >= _Size ) then { AT was not big enough }
            begin
                exit ;
            end ; { if( AT_Byte > Segment_Size ) }
        end ; { if( AT_Bit > 7 ) }
    end ; { while( Counter > _Size ) }

    { Now mark space as allocated in AT... }
    AT_Byte := Offset_Byte ;
    AT_Bit := Offset_Bit ;
    while( Counter > 0 ) do
    begin
        Modified := True ;
        if( ( AT_Bit = 0 ) and ( Counter >= Byte_Mapping ) ) then
        begin { Quick check for entire byte }
            AT^[ AT_Byte ] := 255 ;
            Counter := Counter - Byte_Mapping ;
            AT_Bit := 7 ;
            if( Counter <= 0 ) then
            begin
                break ;
            end ;
        end else
        begin
            AT^[ AT_Byte ] := AT^[ AT_Byte ] or Bit_Values[ AT_Bit ] ; { Mark as used }
            Counter := Counter - Resolution ;
        end ;
        inc( AT_Bit ) ; { Check next bit of current byte }
        if( AT_Bit > 7 ) then { Ran out of bits on this byte } if( Counter > 0 ) then
        { ...and not yet finished }
        begin
            AT_Bit := 0 ; { Start at first bit... }
            inc( AT_Byte ) ; {...of next byte}
        end ; { if( AT_Bit > 7 ) }
    end ; { while Counter > 0 }
    Modified := True ;
    Allocate := Offset ; { Return offset of allocation }

    { Update lowest free space }
    if( Offset <> 0 ) then
    begin
        if( Offset = Lowest_Free_Space ) then
        begin
            Lowest_Free_Space := Lowest_Free_Space + Size ;
        end ;
    end ; { if( Offset <> 0 ) }

    if( _Offset <> 0 ) then
    begin
        if( ( Offset < _Offset + _Size ) and ( Offset + Size > _Offset ) ) then
        begin
            { This indicates that the allocation was over the area of the store
              occupied by the allocation table itself - this is a serious error
              and indicates either AT corruption or some errant prior
              Deallocate.  The only time this is valid is during the initial
              setup of the allocation table, where we are setting the bits for
              the allocation table so that this situation doesn't happen. }
            Offset := 0 ;
        end ;
    end ;
end ; { TAT64.Allocate }


function TAT64.Allocate_At( Pos, Size : TStore_Address64 ) : boolean ;

var AT_Byte, AT_Bit : longint ; { Current byte and bit within AT }
    Counter : TStore_Size64 ;

begin
    Allocate_At := False ; { Return false if error }
    if( ( Size < 1 ) or ( Pos < 1 ) ) then
    begin
        exit ; { Cannot allocate nothing, or at an invalid position }
    end ;
    Size := ( Size + Resolution - 1 ) and ( not ( Resolution - 1 ) ) ;
    { Allocate minimum multiple bytes at a time }

    AT_Byte := Pos div Byte_Mapping ;
    if( AT_Byte >= _Size ) then // Not enough room
    begin
        exit ;
    end ;
    AT_Bit := ( Pos - ( AT_Byte * Byte_Mapping ) ) div Resolution ;
    Counter := 0 ; { Number of bytes available in current segment }
    while( Counter < Size ) do { Check for necessary free space }
    begin
        if(
            ( AT_Bit = 0 )
            and
            ( Size - Counter >= Byte_Mapping )
            and
            ( AT^[ AT_Byte ] = 0 )
          ) then
        begin { Quick check for entire byte }
            Counter := Counter + Byte_Mapping ;
            AT_Bit := 7 ;
            if( Counter >= Size ) then
            begin
                break ;
            end ;
        end else
        begin { Check bit by bit }
            if( ( AT^[ AT_Byte ] and Bit_Values[ AT_Bit ] ) = 0 ) then
            begin
                Counter := Counter + Resolution ; { Count up free bytes }
                if( Counter >= Size ) then
                begin
                    break ;
                end ;
            end else
            begin
                exit ; { Insufficient space at location }
            end ;
        end ;
        inc( AT_Bit ) ; { Check next bit of current byte }
        if( AT_Bit > 7 ) then { Ran out of bits on this byte }
        begin
            AT_Bit := 0 ; { Start at first bit of next byte }
            inc( AT_Byte ) ; { Move to next byte }
        end ; { if( AT_Bit > 7 ) }
    end ; { while( Counter > _Size ) }

    { Now mark space as allocated in AT... }
    AT_Byte := Pos div Byte_Mapping ;
    AT_Bit := ( Pos - ( AT_Byte * Byte_Mapping ) ) div Resolution ;
    while( Counter > 0 ) do
    begin
        Modified := True ;
        if( ( AT_Bit = 0 ) and ( Counter >= Byte_Mapping ) ) then
        begin { Quick check for entire byte }
            AT^[ AT_Byte ] := 255 ;
            Counter := Counter - Byte_Mapping ;
            AT_Bit := 7 ;
            if( Counter <= 0 ) then
            begin
                break ;
            end ;
        end else
        begin
            AT^[ AT_Byte ] := AT^[ AT_Byte ] or Bit_Values[ AT_Bit ] ; { Mark as used }
            Counter := Counter - Resolution ;
        end ;
        inc( AT_Bit ) ; { Check next bit of current byte }
        if( AT_Bit > 7 ) then { Ran out of bits on this byte } if( Counter > 0 ) then
        { ...and not yet finished }
        begin
            AT_Bit := 0 ; { Start at first bit... }
            inc( AT_Byte ) ; {...of next byte}
        end ; { if( AT_Bit > 7 ) }
    end ; { while Counter > 0 }
    Modified := True ;
    Allocate_At := True ; { Success }
    if( Pos = Lowest_Free_Space ) then
    begin
        Lowest_Free_Space := Lowest_Free_Space + Size ;
    end ;
end ; { TAT64.Allocate_At }


procedure TAT64.Deallocate( PTR : TStore_Address64 ; Size : TStore_Address64 ) ;

var AT_Byte, AT_Bit : longint ;

begin
    if( ( Size < 1 ) or ( PTR < Resolution ) ) then
    begin
        exit ;
    end ;
    Size := ( Size + Resolution - 1 ) and ( not ( Resolution - 1 ) ) ;
    { Deallocate minimum multiple bytes at a time }

    if( Lowest_Free_Space > PTR ) then
    begin
        Lowest_Free_Space := PTR ;
    end ;

    AT_Byte := PTR div Byte_Mapping ;
    if( AT_Byte > _Size ) then
    begin
        exit ;
    end ;
    PTR := PTR - AT_Byte * Byte_Mapping ;
    AT_Bit := PTR div Resolution ;
    while( Size > 0 ) do
    begin
        if( ( AT_Bit = 0 ) and ( Size >= Byte_Mapping ) ) then
        begin
            AT_Bit := 7 ;
            AT^[ AT_Byte ] := 0 ;
            Size := Size - Byte_Mapping ;
        end else
        begin
            AT^[ AT_Byte ] := AT^[ AT_Byte ] and ( not Bit_Values[ AT_Bit ] ) ;
            Size := Size - Resolution ;
        end ;

        if( Size > 0 ) then
        begin
            inc( AT_Bit ) ;
            if( AT_Bit > 7 ) then
            begin
                AT_Bit := 0 ; { Move to first bit... }
                inc( AT_Byte ) ; { ...of next byte }
                if( AT_Byte = _Size ) then
                begin
                    Modified := True ;
                    Size := 0 ; { Done if at end of AT }
                end ;
            end ; { if AT_Bit > 7 }
        end ; { if _Size > 0 }
    end ; { while _Size > 0 }
    Modified := True ;
end ; { TAT64.Deallocate }


function TAT64.Reallocate( PTR : TStore_Address ; Old, New : TStore_Address ) : TStore_Address ;

var _P : TStore_Address ;

begin
    Old := ( Old + Resolution - 1 ) and ( not ( Resolution - 1 ) ) ;
    New := ( New + Resolution - 1 ) and ( not ( Resolution - 1 ) ) ;
    if( Old = New ) then // No change
    begin
        Reallocate := PTR ;
        exit ;
    end ;
    if( New < Old ) then // Shrinking allocation
    begin
        Deallocate( PTR + New, Old - New ) ;
        Reallocate := PTR ;
        exit ;
    end ;

    // Expanding allocation...
    if( Allocate_At( PTR + Old, New - Old ) ) then // Success at expanding in place
    begin
        Reallocate := PTR ;
        exit ;
    end ;

    // Cannot reallocate in place - "move" allocation...
    _P := Allocate( New ) ;
    if( _P = 0 ) then // No room for reallocation
    begin
        Reallocate := 0 ;
        exit ;
    end ;
    Deallocate( PTR, Old ) ; // Deallocate old allocation
    Reallocate := _P ; // Return new allocation
end ; { TAT64.Reallocate }


{$IFNDEF MOBILE}
function TAT64.Debugger : TDebug_Interface ;

begin
    Result := nil ;
end ;
{$ENDIF}


procedure TAT64.Load_Table ; { Load from store }

var Dummy : TRecord_Size ;
    P : TStore_Address ;
    U_E : TUnified_Exception ;

begin
    if( ( _Store <> nil ) and ( _Size > 0 ) ) then
    begin
        P := _Offset ;
        Dummy := _Store.Read_Data( AT^, P, _Size, U_E ) ;
        if( Dummy <> _Size ) then
        begin
            Set_Last_Error( Create_Exception( AT_Error_Load_Failure, _Store.Last_Error ) ) ;
            exit ;
        end ;
        Modified := False ;
    end ;
end ;


function TAT64.MaxSpace : TStore_Size64 ; { Return largest contiguous space }

var AT_Byte, AT_Bit : integer ; { Current byte and bit within AT }
    Counter, Max : TStore_Size64 ;

begin
    Result := 0 ;
    AT_Byte := Lowest_Free_Space div Byte_Mapping ;
    if( AT_Byte >= _Size ) then
    begin
        exit ;
    end ;
    AT_Bit := 0 ;
    Counter := 0 ; { Number of bytes available in current segment }
    Max := 0 ; { Maximum space found }
    while( AT_Bit <= 7 ) do { Find free space }
    begin
        if( ( AT_Bit = 0 ) and ( AT^[ AT_Byte ] = 0 ) ) then
        begin
            Counter := Counter + Byte_Mapping ;
            AT_Bit := 7 ;
        end else
        if( ( AT^[ AT_Byte ] and Bit_Values[ AT_Bit ] ) = 0 ) then
        begin
            Counter := Counter + Resolution ; { Count up free bytes }
        end else
        begin { Ran out of free blocks }
            if( Counter > Max ) then
            begin
                Max := Counter ;
            end ;
            Counter := 0 ; { Start over }
        end ;
        inc( AT_Bit ) ; { Check next bit of current byte }
        if( AT_Bit > 7 ) then { Ran out of bits on this byte }
        begin
            AT_Bit := 0 ; { Start at first bit of next byte }
            inc( AT_Byte ) ; { Move to next byte }
            if( AT_Byte >= _Size ) then { AT was not big enough }
            begin
                AT_Bit := 8 ; { Terminate loop }
                if( Counter > Max ) then
                begin
                    Max := Counter ;
                end ;
            end ; { if AT_Byte >= AT_Segment_Size }
        end ; { if AT_Bit > 7 }
    end ; { while AT_Bit <= 7 }
    MaxSpace := Max ;
end ; { TAT64.MaxSpace }


function TAT64.SpaceAvail : TStore_Size64 ;
{ Return available space within table }

var AT_Byte, AT_Bit : integer ; { Current byte and bit within AT }
    Counter : longint ;

begin
    Result := 0 ;
    AT_Byte := Lowest_Free_Space div Byte_Mapping ;
    if( AT_Byte >= _Size ) then
    begin
        exit ;
    end ;
    AT_Bit := 0 ;
    Counter := 0 ; { Number of bytes available in current segment }
    while( AT_Bit <= 7 ) do { Find free space }
    begin
        if( ( AT_Bit = 0 ) and ( AT^[ AT_Byte ] = 0 ) ) then // Whole byte is free
        begin
            Counter := Counter + Byte_Mapping ;
            AT_Bit := 8 ;
        end else
        begin
            if( ( AT^[ AT_Byte ] and Bit_Values[ AT_Bit ] ) = 0 ) then
            begin
                Counter := Counter + Resolution ; { Count up free bytes }
            end ;
            inc( AT_Bit ) ; { Check next bit of current byte }
        end ;
        if( AT_Bit > 7 ) then { Ran out of bits on this byte }
        begin
            AT_Bit := 0 ; { Start at first bit of next byte }
            inc( AT_Byte ) ; { Move to next byte }
            if( AT_Byte >= _Size ) then { AT was not big enough }
            begin
                AT_Bit := 8 ; { Terminate loop }
            end ;
        end ;
    end ;
    SpaceAvail := Counter ;
end ; { TAT64.SpaceAvail }


function TAT64.Space_At( PTR : TStore_Address64 ) : TStore_Size64 ;

var AT_Byte, AT_Bit : integer ; { Current byte and bit within AT }
    Counter : longint ;

begin
    { Setup... }
    PTR := ( PTR + Resolution - 1 ) and ( not ( Resolution - 1 ) ) ;
    AT_Byte := PTR div Byte_Mapping ;
    AT_Bit := ( PTR - ( AT_Byte * Byte_Mapping ) ) div Resolution ;
    Result := 0 ;

    if( AT_Byte >= _Size ) then
    begin
        exit ;
    end ;
    Counter := 0 ; { Number of bytes available in current segment }
    while( AT_Bit <= 7 ) do { Find free space }
    begin
        if( ( AT_Bit = 0 ) and ( AT^[ AT_Byte ] = 0 ) ) then // Whole byte is free
        begin
            Counter := Counter + Byte_Mapping ;
            AT_Bit := 8 ;
        end else
        begin
            if( ( AT^[ AT_Byte ] and Bit_Values[ AT_Bit ] ) <> 0 ) then
            begin
                exit ;
            end ;
            Counter := Counter + Resolution ; { Count up free bytes }
            inc( AT_Bit ) ; { Check next bit of current byte }
        end ;
        if( AT_Bit > 7 ) then { Ran out of bits on this byte }
        begin
            AT_Bit := 0 ; { Start at first bit of next byte }
            inc( AT_Byte ) ; { Move to next byte }
            if( AT_Byte >= _Size ) then { AT was not big enough }
            begin
                AT_Bit := 8 ; { Terminate loop }
            end ;
        end ;
    end ; { while( AT_Bit <= 7 ) }
    Space_At := Counter ;
end ; { TAT64.Space_At }


function TAT64.Get_Data : pointer ;

begin
    Result := AT ;
end ;


procedure TAT64.Set_Data( Value : pointer ; Size : cardinal ) ;

var P : pointer ;

begin
    if( _Size < Size ) then
    begin
        P := AT ;
        reallocmem( P, Size, _Size ) ;
        _AT := P ;
        _Size := Size ;
    end ;
    move( Value^, AT^, Size ) ;
    Find_Lowest_Space ;
end ;


procedure TAT64.Find_Lowest_Space ;

var B, Loop, Loop1 : integer ;

begin
    Lowest_Free_Space := 0 ;
    AT^[ 0 ] := AT^[ 0 ] or 1 ; // Make sure lowest chunk is always allocated
    for Loop := 0 to _Size do
    begin
        if( AT^[ Loop ] = 255 ) then
        begin
            Lowest_Free_Space := Lowest_Free_Space + Byte_Mapping ;
        end else
        begin
            B := AT^[ Loop ] ;
            for Loop1 := 0 to 7 do
            begin
                if( ( B and 1 ) = 0 ) then
                begin
                    exit ;
                end ;
                Lowest_Free_Space := Lowest_Free_Space + Resolution ;
                B := B shr 1 ;
            end ;
        end ;
    end ;
end ;
  {$ENDIF}
{$ENDIF}


{$IFDEF CC_Regression_Test}
procedure Error ;

begin
    halt ;
end ;


var F : text ;
    OpCount : int64 = 0 ;

procedure Log( const S : string ) ;

begin
    inc( OpCount ) ;
    if( ( OpCount and 4095 ) = 0 ) then
    begin
        Application.MainForm.Caption := inttostr( OpCount ) ;
    end ;
    if( ( OpCount and 4194303 ) = 0 ) then
    begin
        rewrite( F ) ;
    end else
    begin
        append( F ) ;
    end ;
    writeln( F, S ) ;
    closefile( F ) ;
end ;


procedure Test_Unit ;

  {$IFDEF DYNAMIC_OBJECTS}
    {$IFDEF INT64}
var Avail, Dummy, Dummy1, S : integer ;
    Test : TAT64 ;
    Clusters : array[ 0..79 ] of byte ;
    {$ENDIF}
  {$ENDIF}

begin
  {$IFDEF DYNAMIC_OBJECTS}
    {$IFDEF INT64}
    Test := TAT64.Create( 1 ) ;
    if( Test.Get_Modified ) then
    begin
        Error ;
    end ;
    if( Test.Get_Offset <> 0 ) then
    begin
        Error ;
    end ;
    Test.Set_Offset( 1 ) ;
    if( Test.Get_Offset <> 1 ) then
    begin
        Error ;
    end ;
    Test.Set_Offset( 1 ) ;
    if( Test.Get_Offset <> 1 ) then
    begin
        Error ;
    end ;
    Test.Set_Offset( 0 ) ;
    if( Test.Get_Offset <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Get_Resolution <> 1 ) then
    begin
        Error ;
    end ;
    if( Test.Get_Store <> nil ) then
    begin
        Error ;
    end ;
    if( Test.Get_Size <> 0 ) then
    begin
        Error ;
    end ;
    Test.Set_Resolution( 1 ) ;
    if( Test.Get_Resolution <> 1 ) then
    begin
        Error ;
    end ;
    Test.Set_Size( 10 ) ;
    if( Test.Get_Size <> 10 ) then
    begin
        Error ;
    end ;
    Test.Set_Size( 10 ) ;
    if( Test.Get_Size <> 10 ) then
    begin
        Error ;
    end ;
    if( Test.MaxSpace <> 79 ) then
    begin
        Error ;
    end ;
    if( Test.SpaceAvail <> 79 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 4 ) <> 1 ) then
    begin
        Error ;
    end ;
    if( not Test.Get_Modified ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( -1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 4 ) <> 5 ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 1, 4 ) ;
    if( Test.MaxSpace <> 71 ) then
    begin
        Error ;
    end ;
    if( Test.SpaceAvail <> 75 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 1024 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate_At( 5, 2 ) ) then
    begin
        Error ;
    end ;
    if( not Test.Allocate_At( 70, 2 ) ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 16 ) = 0 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 18 ) = 0 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 60 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 1 ) <> 1 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate_At( 1, -1 ) ) then
    begin
        Error ;
    end ;
    if( Test.Allocate_At( -1, 1 ) ) then
    begin
        Error ;
    end ;
    if( Test.Allocate_At( 1024, 1 ) ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 0, 0 ) ;
    if( Test.Debugger <> nil ) then
    begin
        Error ;
    end ;
    Test.Load_Table ;
    Test.Set_Resolution( 2 ) ;

    Test.Set_Store( TCOM_Heap64.Create ) ;
    Test.Set_Modified( False ) ; // Don't write to store
    Test.Set_Store( nil ) ;
    Test.Free ;

    Test := TAT64.Create( 1 ) ;
    Test.Set_Size( 3 ) ;
    if( not Test.Allocate_At( 23, 1 ) ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 22 ) <> 1 ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 1, 23 ) ;
    if( Test.Allocate( 23 ) <> 1 ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 1, 23 ) ;
    if( Test.Allocate( 1 ) <> 1 ) then
    begin
        Error ;
    end ;
    if( Test.Allocate( 22 ) <> 2 ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 1, 23 ) ;
    if( not Test.Allocate_At( 1, 16 ) ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 10, 2 ) ;
    if( Test.Allocate_At( 10, 10 ) ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 8, 2 ) ;
    Test.Deallocate( 12, 4 ) ;
    if( not Test.Allocate_At( 8, 8 ) ) then
    begin
        Error ;
    end ;
    Test.Deallocate( 1, 100 ) ;
    Test.Allocate_At( 12, 10 ) ;
    if( Test.MaxSpace <> 11 ) then
    begin
        Error ;
    end ;
    Test.Free ;

    // Stress test...
    assignfile( F, 'k:\test.txt' ) ;
    rewrite( F ) ;
    closefile( F ) ;
    fillchar( Clusters, sizeof( Clusters ), 0 ) ;
    Clusters[ 0 ] := 255 ;
    Avail := 79 ;
    Test := TAT64.Create( 1 ) ;
    Test.Set_Size( 10 ) ;
    if( Test.MaxSpace <> 79 ) then
    begin
        Error ;
    end ;
    while( True ) do
    begin
        case random( 5 ) of
            0 : // Allocate
                begin
                    if( Avail = 0 ) then
                    begin
                        Log( 'A ' ) ;
                        if( Test.Allocate( 1 ) <> 0 ) then
                        begin
                            Error ;
                        end ;
                    end else
                    begin
                        S := 0 ;
                        Dummy1 := random( 78 ) + 1 ;
                        for Dummy := Dummy1 to 79 do
                        begin
                            if( Clusters[ Dummy ] = 0 ) then
                            begin
                                for Dummy1 := Dummy to 79 do
                                begin
                                    if( Clusters[ Dummy1 ] = 0 ) then
                                    begin
                                        S := S + 1 ;
                                    end else
                                    begin
                                        break ;
                                    end ;
                                end ;
                                break ;
                            end ;
                        end ;
                        if( S = 0 ) then
                        begin
                            continue ;
                        end ;
                        Log( 'A ' + inttostr( S ) ) ;
                        Dummy := Test.Allocate( S ) ;
                        if( Dummy = 0 ) then
                        begin
                            Test.Allocate( S ) ;
                            Error ;
                        end ;
                        fillchar( Clusters[ Dummy ], S, 255 ) ;
                        Avail := Avail - S ;
                        if( Avail <> Test.SpaceAvail ) then
                        begin
                            Error ;
                        end ;
                    end ;
                end ;
            1 : // Allocate_At
                begin
                    if( Avail = 0 ) then
                    begin
                        continue ;
                    end ;
                    Dummy := random( 78 ) + 1 ;
                    while( ( Clusters[ Dummy ] <> 0 ) and ( Dummy < 80 ) ) do
                    begin
                        inc( Dummy ) ;
                    end ;
                    if( Dummy = 80 ) then
                    begin
                        Dummy := 1 ;
                        while( ( Clusters[ Dummy ] <> 0 ) and ( Dummy < 80 ) ) do
                        begin
                            inc( Dummy ) ;
                        end ;
                    end ;
                    S := 0 ;
                    for Dummy1 := Dummy to 79 do
                    begin
                        if( Clusters[ Dummy1 ] = 0 ) then
                        begin
                            S := S + 1 ;
                        end else
                        begin
                            break ;
                        end ;
                    end ;
                    Log( 'AA' + inttostr( Dummy ) + ' ' + inttostr( S ) ) ;
                    if( not Test.Allocate_At( Dummy, S ) ) then
                    begin
                        Test.Allocate_At( Dummy, S ) ;
                        Error ;
                    end ;
                    fillchar( Clusters[ Dummy ], S, 255 ) ;
                    Avail := Avail - S ;
                    if( Avail <> Test.SpaceAvail ) then
                    begin
                        Test.SpaceAvail ;
                        Error ;
                    end ;
                end ;
            2 : // Deallocate
                begin
                    if( Avail <> Test.SpaceAvail ) then
                    begin
                        Error ;
                    end ;
                    Dummy := random( 78 ) + 1 ;
                    S := random( 78 ) + 1 ;
                    if( S + Dummy >= 80 ) then
                    begin
                        S := 80 - Dummy ;
                    end ;
                    Log( 'D' + inttostr( Dummy ) + ' ' + inttostr( S ) ) ;
                    Test.Deallocate( Dummy, S ) ;
                    for Dummy1 := Dummy to Dummy + S - 1 do
                    begin
                        if( Clusters[ Dummy1 ] <> 0 ) then
                        begin
                            Clusters[ Dummy1 ] := 0 ;
                            Avail := Avail + 1 ;
                        end ;
                    end ;
                    if( Avail <> Test.SpaceAvail ) then
                    begin
                        Test.SpaceAvail ;
                        Error ;
                    end ;
                end ;
            3 : // MaxSpace
                begin
                    Log( 'M' ) ;
                    if( Test.MaxSpace > Avail ) then
                    begin
                        Test.MaxSpace ;
                        Error ;
                    end ;
                end ;
            4 : // SpaceAvail
                begin
                    Log( 'S' ) ;
		            if( Test.SpaceAvail <> Avail ) then
                    begin
                        Error ;
                    end ;
                end ;
        end ;
    end ;
    Test.Free ;
    {$ENDIF}
  {$ENDIF}
end ; { Test_Unit }
{$ENDIF}


end.
