{$IFNDEF FPC}
  {$N+}
{$ENDIF}
{
        Program Name : BaseUt
        Package Name : Subroutine Library - Layer 1
        Purpose      : Common Utility functions and classes
        Institution  : Conroy & Conroy Co.
        Date Written : 28-Nov-88
        Written By   : Alan Conroy
        Version      : 2.1H

        Copyright (c) 1988-2017 by Alan Conroy.  Released to the Public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *********************************************************

             DATE      BY             REASON

          16-Jul-2017  EAC            Extracted from CommonUt.

        *********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

          This unit contains base common utility function and classes.

          LEVEL 2 CERTIFICATION: 6-Sep-2008 by EAC (Untested: TXInteger_List.Delete)
}

{$I eDefines.inc}

unit BaseUt ;

interface

uses
{$IFDEF FPC_OR_DELPHI}
     { Borland... }
     Classes,
{$ENDIF}

     { Layer 0... }
     _DebugIn ; { PSDebug_Interface }

{$IFDEF FPC_OR_DELPHI}
type TInteger_List = class( TList )
                         public { Constructors and destructors... }
                             constructor Create ;
                             destructor Destroy ; override ;

  {$IFDEF CC_Debug}
                         private { Instance data... }
                             Serial_Number : integer ;
  {$ENDIF}

                         protected { Property handlers }
                             function Get_Index( Index : integer ) : integer ;
                                 virtual ;
                             procedure Set_Index( Index : integer ; Value : integer ) ;
                                 virtual ;

                         protected { Internal utility methods }
                             procedure Check_Capacity ;
  {$IFDEF INLINE_FUNCTIONS}
                                 inline ;
  {$ENDIF}

                         public { API... }
                             function Add( Value : integer ) : integer ;
                                 virtual ;
                             procedure Assign( Source : TInteger_List ) ;
                                 virtual ;
                             function Debugger : TCOM_Debug_Interface ;
                             procedure Deserialize( const Image : string ) ;
                             function Remove( Value : integer ) : integer ;
                                 virtual ;
                             procedure Increment( Index : integer ) ;
                             function IndexOf( Value : integer ) : integer ;
                                 virtual ;
                             procedure Insert( Index, Value : integer ) ;
                                 virtual ;
                             function Serialize : string ;
                             procedure Sort( Mask : integer ) ; virtual ;

                             property Items[ Index : integer ] : integer
                                 read Get_Index
                                 write Set_Index ; default ;
  {$IFDEF CC_Regression_Test}
                             class procedure Test_Class ;
  {$ENDIF}
                     end ;


     { TXInteger_List is the same as TInteger_List but maintains sorted state,
       and supports objects associated with the list. }
type TXInteger_List = class( TInteger_List )
                         public { Constructors and destructors... }
                             constructor Create ;
                             destructor Destroy ; override ;

                         private { Instance data... }
                             _Sorted : boolean ;
                             Sort_Mask : integer ;

                         protected { Property handlers }
                             procedure Set_Sorted( Value : boolean ) ;

                         public { API... }
                             Objects : TList ;

                             function Add( Value : integer ) : integer ;
                                 override ;
                             function Add_Object( Value : integer ; O : pointer ) : integer ;
                                 virtual ;
                             procedure Assign( Source : TXInteger_List ) ;
  {$IFDEF REINTRODUCE}
                                 reintroduce ;
  {$ELSE}
                                 virtual ;
  {$ENDIF}
                             procedure Delete( Index : integer ) ;
                             function Remove( Value : integer ) : integer ;
                                 override ;
                             function IndexOf( Value : integer ) : integer ;
                                 override ;
                             procedure Insert( Index, Value : integer ) ;
                                 override ;
                             procedure Insert_Object( Index, Value : integer ;
                                 O : pointer ) ; virtual ;
                             procedure Sort( Mask : integer ) ; override ;

                             property Sorted : boolean
                                 read _Sorted
                                 write Set_Sorted ;
  {$IFDEF CC_Regression_Test}
                             class procedure Test_Class ;
  {$ENDIF}
                     end ;
{$ENDIF}


implementation

{$IFDEF FPC_OR_DELPHI}
uses { Borland... }
     _Applic, { Application_Manager }
     SysUtils, { inttostr }

     { Layer 1... }
     DebugInt ; { TText_Debugger }
{$ENDIF}

{$IFDEF FPC_OR_DELPHI}
{ TInteger_List_Debugger... }

type TInteger_List_Debugger = class( TText_Debugger )
                                  public { Instance data }
                                      List : TInteger_List ;

                                  public { API... }
                                      function Count : longint ;
                                          override ;

                                      function Child( Ordinal : longint ) : PDebug_Interface ;
                                          override ;
                              end ;

function TInteger_List_Debugger.Count : longint ;

begin
    Count := List.Count ;
end ;


function TInteger_List_Debugger.Child( Ordinal : longint ) : PDebug_Interface ;

var Res : PText_Debugger ;

begin
    Res := TText_Debugger.Create ;
    Res.Set_Title( PChar( inttostr( Ordinal ) + ': ' + inttostr( List[ Ordinal ] ) ) ) ;
    Child := Res ;
end ;



{ TXInteger_List methods... }

{ Constructors and destructors... }

constructor TXInteger_List.Create ;

begin
    inherited Create ;

    Objects := TList.Create ;
end ;


destructor TXInteger_List.Destroy ;

begin
    Objects.Free ;
    Objects := nil ;

    inherited Destroy ;
end ;


{ Property handlers... }

procedure TXInteger_List.Set_Sorted( Value : boolean ) ;

begin
    if( Value <> _Sorted ) then
    begin
        if( Value ) then
        begin
            Sort( -1 ) ;
        end ;
        _Sorted := Value ;
    end ;
end ;


{ API... }

function TXInteger_List.Add( Value : integer ) : integer ;

var Loop : integer ;

begin
    Check_Capacity ;
    if( Sorted ) then
    begin
        for Loop := 0 to Count - 1 do
        begin
            if( ( Items[ Loop ] and Sort_Mask ) >= ( Value and Sort_Mask ) ) then
            begin
                inherited Insert( Loop, Value ) ;
                Objects.Insert( Loop, nil ) ;
                Result := Loop ;
                exit ;
            end ;
        end ;
    end ;
    Result := inherited Add( Value ) ;
    Objects.Add( nil ) ;
end ;


function TXInteger_List.Add_Object( Value : integer ; O : pointer ) : integer ;

begin
    Result := Add( Value ) ;
    Objects[ Result ] := O ;
end ;


procedure TXInteger_List.Assign( Source : TXInteger_List ) ;

{$IFNDEF DELPHI5UP}
var Dummy : integer ;
{$ENDIF}

begin
    inherited Assign( Source ) ;
    _Sorted := Source.Sorted ;
{$IFDEF DELPHI5UP}
    Objects.assign( Source.Objects ) ;
{$ELSE}
    Objects.Clear ;
    for Dummy := 0 to Source.Objects.Count - 1 do
    begin
        Objects.Add( Source.Objects[ Dummy ] ) ;
    end ;
{$ENDIF}
end ;


procedure TXInteger_List.Delete( Index : integer ) ;

begin
    Objects.Delete( Index ) ;
    inherited Delete( Index ) ;
end ;


function TXInteger_List.Remove( Value : integer ) : integer ;

begin
    Result := inherited Remove( Value ) ;
    if( Result <> -1 ) then
    begin
        Objects.Delete( Result ) ;
    end ;
end ;


function TXInteger_List.IndexOf( Value : integer ) : integer ;

var Loop, Low, High : integer ;

begin
    if( Sorted and ( Count > 600 ) and ( Sort_Mask = -1 ) ) then // 600 was chosen for a 166 Mhz system - revisit for adapting to machine speed
    begin
        // Bi-search...
        Low := 0 ;
        High := Count - 1 ;
        while( Low < High ) do
        begin
            Loop := ( ( High - Low ) div 2 ) + Low ;
            if( Items[ Loop ] < Value ) then
            begin
                if( Loop = Low ) then
                begin
                    Low := Low + 1 ;
                end else
                begin
                    Low := Loop ;
                end ;
            end else
            begin
                High := Loop ;
            end ;
        end ; { while( Low < High ) }

        Result := Low ;
        if( Items[ Result ] <> Value ) then
        begin
            Result := -1 ; // Not found
        end ;
    end else
    begin
        Result := inherited IndexOf( Value ) ; // Linear search
    end ;
end ; // TXInteger_List.IndexOf


procedure TXInteger_List.Insert( Index, Value : integer ) ;

begin
    if( Sorted ) then
    begin
        Add( Value ) ;
    end else
    begin
        Check_Capacity ;
        inherited Insert( Index, Value ) ;
        Objects.Insert( Index, nil ) ;
    end ;
end ;


procedure TXInteger_List.Insert_Object( Index, Value : integer ; O : pointer ) ;

begin
    if( Sorted ) then
    begin
        Add_Object( Value, O ) ;
    end else
    begin
        Check_Capacity ;
        inherited Insert( Index, Value ) ;
        Objects.Insert( Index, O ) ;
    end ;
end ;


procedure TXInteger_List.Sort( Mask : integer ) ;

var Loop, Loop1, I, I1, M, M1 : integer ;
    O, O1 : TObject ;

begin
    Sort_Mask := Mask ;
    for Loop := 0 to Count - 1 do
    begin
        I := Items[ Loop ] ;
        M := I and Mask ;
        for Loop1 := Loop to Count - 1 do
        begin
            I1 := Items[ Loop1 ] ;
            M1 := I1 and Mask ;
            if( M > M1 ) then
            begin
                O := Objects[ Loop ] ;
                O1 := Objects[ Loop1 ] ;
                Objects[ Loop ] := O1 ;
                Objects[ Loop1 ] := O ;
                Items[ Loop ] := I1 ;
                Items[ Loop1 ] := I ;
                I := I1 ;
                M := M1 ;
            end ;
        end ;
    end ;
    _Sorted := True ;
end ;


  {$IFDEF CC_Regression_Test}
class procedure TXInteger_List.Test_Class ;

var {Intermediate : integer ;}
    Loop : integer ;
    S : string ;
    Test, Test1 : TXInteger_List ;

begin
    { Set up memory manager }
{    Start_Memory_Test ;}

    { Create it... }
    Test := TXInteger_List.Create ;
    Test1 := TXInteger_List.Create ;
{    Intermediate := Allocation_Count ;}

    if( Test.Serialize <> '0' ) then
    begin
        raise Exception.Create( 'TXInteger_List.Serialize of empty list failed' ) ;
    end ;

    { Test Add and IndexOf... }
    for Loop := 0 to 10 do
    begin
        Test.Add( Loop ) ;
        if( Test[ Loop ] <> Loop ) then
        begin
            raise Exception.Create( 'TXInteger_List.Add (or default array) failed' ) ;
        end ;
        if( Test.IndexOf( Loop ) <> Loop ) then
        begin
            raise Exception.Create( 'TXInteger_List.IndexOf failed' ) ;
        end ;
    end ;
    if( Test.IndexOf( 11 ) <> -1 ) then
    begin
        raise Exception.Create( 'TXInteger_List.IndexOf of non-existant value failed' ) ;
    end ;
    S := Test.Serialize ;
    if( S <> '11|0|1|2|3|4|5|6|7|8|9|10' ) then
    begin
        raise Exception.Create( 'TXInteger_List.Serialize failed' ) ;
    end ;

    { Test Insert... }
    Test.Insert( 0, 11 ) ;
    if( ( Test[ 0 ] <> 11 ) or ( Test.Count <> 12 ) ) then
    begin
        raise Exception.Create( 'TXInteger_List.Insert failed' ) ;
    end ;
    for Loop := 1 to 11 do
    begin
        if( Test[ Loop ] <> Loop - 1 ) then
        begin
            raise Exception.Create( 'TXInteger_List.Insert failed' ) ;
        end ;
    end ;
    Test.Insert( 12, -1 ) ;
    if( Test[ 12 ] <> -1 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Insert failed' ) ;
    end ;

    { Test Assign... }
    Test1.Assign( Test ) ;
    if( Test1.Count <> Test.Count ) then
    begin
        raise Exception.Create( 'TXInteger_List.Assign failed' ) ;
    end ;
    for Loop := 0 to Test.Count - 1 do
    begin
        if( Test[ Loop ] <> Test1[ Loop ] ) then
        begin
            raise Exception.Create( 'TXInteger_List.Assign failed' ) ;
        end ;
    end ;

    Test.Clear ;
    Test1.Clear ;
    Test.Deserialize( '0' ) ;
    if( Test.Count > 0 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Deserialize failed' ) ;
    end ;
    Test.Deserialize( S ) ;
    for Loop := 0 to 10 do
    begin
        if( Test[ Loop ] <> Loop ) then
        begin
            raise Exception.Create( 'TXInteger_List.Deserialize failed' ) ;
        end ;
    end ;
    Test.Clear ;
    for Loop := 0 to 1025 do
    begin
        Test.Add( Loop ) ;
    end ;
    if( Test.Count <> 1026 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Add failed' ) ;
    end ;
    if( Test.Capacity <> 2048 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Add failed' ) ;
    end ;
    if( Test[ 0 ] <> 0 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Add failed' ) ;
    end ;
    Test.Increment( 0 ) ;
    if( Test[ 0 ] <> 1 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Increment failed' ) ;
    end ;
    Test.Remove( 1 ) ;
    if( Test[ 0 ] <> 1 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Remove failed' ) ;
    end ;
    if( Test.Count <> 1025 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Remove failed' ) ;
    end ;
    Test.Remove( 0 ) ;
    if( Test[ 0 ] <> 1 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Remove failed' ) ;
    end ;
    if( Test.Count <> 1025 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Remove failed' ) ;
    end ;
    Test.Clear ;
    Test.Sort( -1 ) ;
    if( Test.Count <> 0 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Sort failed' ) ;
    end ;
    Test.Add( 0 ) ;
    Test.Add( 2 ) ;
    Test.Add( 1 ) ;
    Test.Add( 3 ) ;
    Test.Sorted := True ;
    Test.Sorted := True ;
    Test.Sorted := False ;
    if( Test.Sorted ) then
    begin
        raise Exception.Create( 'TXInteger_List.Sorted failed' ) ;
    end ;
    if( Test.Count <> 4 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Sort failed' ) ;
    end ;
    for Loop := 0 to Test.Count - 1 do
    begin
        if( Test[ Loop ] <> Loop ) then
        begin
            raise Exception.Create( 'TXInteger_List.Sort failed' ) ;
        end ;
    end ;
    Test.Sort( -1 ) ;
    if( not Test.Sorted ) then
    begin
        raise Exception.Create( 'TXInteger_List.Sort failed' ) ;
    end ;
    Test.Add( 10 ) ;
    if( Test[ Test.Count - 1 ] <> 10 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Add after sort failed' ) ;
    end ;
    Test.Insert( 0, 9 ) ;
    if( Test[ Test.Count - 1 ] <> 10 ) then
    begin
        raise Exception.Create( 'TXInteger_List.Insert after sort failed' ) ;
    end ;
    Test.Sorted := False ;
    Test.Sorted := True ;
    Test.Clear ;
    for Loop := 0 to 1023 do
    begin
        Test.Add( Loop ) ;
    end ;
    for Loop := 0 to Test.Count - 2 do
    begin
        if( Test.IndexOf( Loop ) <> Loop ) then
        begin
            raise Exception.Create( 'TXInteger_List.indexof after sort failed' ) ;
        end ;
    end ;
    if( Test.IndexOf( 1024 ) <> -1 ) then
    begin
        raise Exception.Create( 'TXInteger_List.indexof after sort failed' ) ;
    end ;
    if( Test.Objects[ 0 ] <> nil ) then
    begin
        raise Exception.Create( 'TXInteger_List.Objects failed' ) ;
    end ;
    Test.Objects[ 0 ] := pointer( 1 ) ;
    if( Test.Objects[ 0 ] <> pointer( 1 ) ) then
    begin
        raise Exception.Create( 'TXInteger_List.Objects failed' ) ;
    end ;
    Test.Insert_Object( 0, 1025, pointer( $11 ) ) ;
    if( Test.Objects[ Test.Count - 1 ] <> pointer( $11 ) ) then
    begin
        raise Exception.Create( 'TXInteger_List.Add_Object failed' ) ;
    end ;
    Test.Sorted := False ;
    Test.Insert_Object( 0, 1026, pointer( $12 ) ) ;
    if( Test.Objects[ 0 ] <> pointer( $12 ) ) then
    begin
        raise Exception.Create( 'TXInteger_List.Add_Object failed' ) ;
    end ;
    Test.Clear ;

    { Check memory usage... }
{
    if( Allocation_Count <> Intermediate ) then
    begin
        raise Exception.Create( 'Memory loss' ) ;
    end ;
}

    Test.Free ;
    Test1.Free ;

    { Check memory usage... }
{
    if( Allocation_Count <> 0 ) then
    begin
        raise Exception.Create( 'Memory loss' ) ;
    end ;
    End_Memory_Test ;
}
end ;
  {$ENDIF}


const Integer_List_Facility = 88 ;

{ TInteger_List methods... }

{ Constructors and destructors... }

constructor TInteger_List.Create ;

begin
    inherited Create ;

{$IFDEF CC_Debug}
    if( Application_Manager <> nil ) then {UNTESTED: = nil}
    begin
        Serial_Number := Application_Manager^.Get_Serial_Number( 0 ) ;
        Application_Manager^.Object_Manager^.Add_Object( self, Integer_List_Facility, pointer( Serial_Number ) ) ;
    end ;
{$ENDIF}
end ;


destructor TInteger_List.Destroy ;

begin
{$IFDEF CC_Debug}
    if( Application_Manager <> nil ) then {UNTESTED: = nil}
    begin
        Application_Manager^.Object_Manager^.Remove_Object( self, Integer_List_Facility ) ;
    end ;
{$ENDIF}

    inherited Destroy ;
end ;


{ Property handlers... }

function TInteger_List.Get_Index( Index : integer ) : integer ;

begin
{$IFDEF Development}
    Result := integer( inherited Items[ Index ] ) ;
{$ELSE}
    Result := integer( List[ Index ] ) ;
{$ENDIF}
end ;


procedure TInteger_List.Set_Index( Index : integer ; Value : integer ) ;

begin
{$IFDEF Development}
    inherited Items[ Index ] := pointer( Value ) ;
{$ELSE}
    List[ Index ] := pointer( Value ) ;
{$ENDIF}
end ;


{ Internal utility methods... }

procedure TInteger_List.Check_Capacity ;

begin
    if( Count = Capacity ) then { Need to expand }
    begin
        if( Count < 1024 ) then
        begin
            Capacity := Capacity * 2 ;
        end else
        begin
            Capacity := Capacity + 1024 ;
        end ;
    end ;
end ;


{ API... }

function TInteger_List.Add( Value : integer ) : integer ;

begin
    Check_Capacity ;
    inherited Add( pointer( Value ) ) ;
    Result := Count - 1 ;
end ;


procedure TInteger_List.Assign( Source : TInteger_List ) ;

begin
    if( Source = nil ) then
    begin
        Clear ;
        exit ;
    end ;
    Count := Source.Count ;
    System.move( Source.List[ 0 ], List[ 0 ], Count * sizeof( integer ) ) ;
end ;


function TInteger_List.Debugger : TCOM_Debug_Interface ; {UNTESTED}

var Res : TInteger_List_Debugger ;

begin
    Res := TInteger_List_Debugger.Create ;
    Res.List := self ;
    Debugger := Res ;
end ;


procedure TInteger_List.Deserialize( const Image : string ) ;

var Dummy : integer ;
    Temp : string ;

begin
    Clear ;
    if( Image = '0' ) then
    begin
        exit ;
    end ;
    Dummy := pos( '|', Image ) ;
    Capacity := strtoint( copy( Image, 1, Dummy - 1 ) ) ;
    Temp := copy( Image, Dummy + 1, length( Image ) ) ;
    while( length( Temp ) > 0 ) do
    begin
        Dummy := pos( '|', Temp ) ;
        if( Dummy = 0 ) then
        begin
            Dummy := length( Temp ) + 1 ;
        end ;
        Add( strtoint( copy( Temp, 1, Dummy - 1 ) ) ) ;
        Temp := copy( Temp, Dummy + 1, length( Temp ) ) ;
    end ;
end ;


function TInteger_List.Remove( Value : integer ) : integer ;

begin
    Result := IndexOf( Value ) ;
    if( Result <> -1 ) then
    begin
        Delete( Result ) ;
    end ;
end ;


procedure TInteger_List.Increment( Index : integer ) ;

begin
    Items[ Index ] := Items[ Index ] + 1 ;
end ;


function TInteger_List.IndexOf( Value : integer ) : integer ;

begin
    Result := inherited IndexOf( pointer( Value ) ) ;
end ;


procedure TInteger_List.Insert( Index, Value : integer ) ;

begin
    Check_Capacity ;
    inherited Insert( Index, pointer( Value ) ) ;
end ;


function TInteger_List.Serialize : string ;

var Loop : integer ;

begin
    Result := inttostr( Count ) ;
    for Loop := 0 to Count - 1 do
    begin
        Result := Result + '|' + inttostr( Items[ Loop ] ) ;
    end ;
end ;


procedure TInteger_List.Sort( Mask : integer ) ;

var Loop, Loop1, I, I1, M, M1 : integer ;

begin
    for Loop := 0 to Count - 1 do
    begin
        I := Items[ Loop ] ;
        M := I and Mask ;
        for Loop1 := Loop to Count - 1 do
        begin
            I1 := Items[ Loop1 ] ;
            M1 := I1 and Mask ;
            if( M > M1 ) then
            begin
                Items[ Loop ] := I1 ;
                Items[ Loop1 ] := I ;
                I := I1 ;
                M := M1 ;
            end ;
        end ;
    end ;
end ;


  {$IFDEF CC_Regression_Test}
class procedure TInteger_List.Test_Class ;

var {Intermediate : integer ;}
    Loop : integer ;
    S : string ;
    Test, Test1 : TInteger_List ;

begin
    { Set up memory manager }
{    Start_Memory_Test ;}

    { Create it... }
    Test := TInteger_List.Create ;
    Test1 := TInteger_List.Create ;
{    Intermediate := Allocation_Count ;}

    if( Test.Serialize <> '0' ) then
    begin
        raise Exception.Create( 'TInteger_List.Serialize of empty list failed' ) ;
    end ;

    { Test Add and IndexOf... }
    for Loop := 0 to 10 do
    begin
        Test.Add( Loop ) ;
        if( Test[ Loop ] <> Loop ) then
        begin
            raise Exception.Create( 'TInteger_List.Add (or default array) failed' ) ;
        end ;
        if( Test.IndexOf( Loop ) <> Loop ) then
        begin
            raise Exception.Create( 'TInteger_List.IndexOf failed' ) ;
        end ;
    end ;
    if( Test.IndexOf( 11 ) <> -1 ) then
    begin
        raise Exception.Create( 'TInteger_List.IndexOf of non-existant value failed' ) ;
    end ;
    S := Test.Serialize ;
    if( S <> '11|0|1|2|3|4|5|6|7|8|9|10' ) then
    begin
        raise Exception.Create( 'TInteger_List.Serialize failed' ) ;
    end ;

    { Test Insert... }
    Test.Insert( 0, 11 ) ;
    if( ( Test[ 0 ] <> 11 ) or ( Test.Count <> 12 ) ) then
    begin
        raise Exception.Create( 'TInteger_List.Insert failed' ) ;
    end ;
    for Loop := 1 to 11 do
    begin
        if( Test[ Loop ] <> Loop - 1 ) then
        begin
            raise Exception.Create( 'TInteger_List.Insert failed' ) ;
        end ;
    end ;
    Test.Insert( 12, -1 ) ;
    if( Test[ 12 ] <> -1 ) then
    begin
        raise Exception.Create( 'TInteger_List.Insert failed' ) ;
    end ;

    { Test Assign... }
    Test1.Assign( Test ) ;
    if( Test1.Count <> Test.Count ) then
    begin
        raise Exception.Create( 'TInteger_List.Assign failed' ) ;
    end ;
    for Loop := 0 to Test.Count - 1 do
    begin
        if( Test[ Loop ] <> Test1[ Loop ] ) then
        begin
            raise Exception.Create( 'TInteger_List.Assign failed' ) ;
        end ;
    end ;

    Test.Clear ;
    Test1.Clear ;
    Test.Deserialize( '0' ) ;
    if( Test.Count > 0 ) then
    begin
        raise Exception.Create( 'TInteger_List.Deserialize failed' ) ;
    end ;
    Test.Deserialize( S ) ;
    for Loop := 0 to 10 do
    begin
        if( Test[ Loop ] <> Loop ) then
        begin
            raise Exception.Create( 'TInteger_List.Deserialize failed' ) ;
        end ;
    end ;
    Test.Clear ;
    for Loop := 0 to 1025 do
    begin
        Test.Add( Loop ) ;
    end ;
    if( Test.Count <> 1026 ) then
    begin
        raise Exception.Create( 'TInteger_List.Add failed' ) ;
    end ;
    if( Test.Capacity <> 2048 ) then
    begin
        raise Exception.Create( 'TInteger_List.Add failed' ) ;
    end ;
    if( Test[ 0 ] <> 0 ) then
    begin
        raise Exception.Create( 'TInteger_List.Add failed' ) ;
    end ;
    Test.Increment( 0 ) ;
    if( Test[ 0 ] <> 1 ) then
    begin
        raise Exception.Create( 'TInteger_List.Increment failed' ) ;
    end ;
    Test.Remove( 1 ) ;
    if( Test[ 0 ] <> 1 ) then
    begin
        raise Exception.Create( 'TInteger_List.Remove failed' ) ;
    end ;
    if( Test.Count <> 1025 ) then
    begin
        raise Exception.Create( 'TInteger_List.Remove failed' ) ;
    end ;
    Test.Remove( 0 ) ;
    if( Test[ 0 ] <> 1 ) then
    begin
        raise Exception.Create( 'TInteger_List.Remove failed' ) ;
    end ;
    if( Test.Count <> 1025 ) then
    begin
        raise Exception.Create( 'TInteger_List.Remove failed' ) ;
    end ;
    Test.Clear ;
    Test.Sort( -1 ) ;
    if( Test.Count <> 0 ) then
    begin
        raise Exception.Create( 'TInteger_List.Sort failed' ) ;
    end ;
    Test.Add( 0 ) ;
    Test.Add( 2 ) ;
    Test.Add( 1 ) ;
    Test.Add( 3 ) ;
    Test.Sort( -1 ) ;
    if( Test.Count <> 4 ) then
    begin
        raise Exception.Create( 'TInteger_List.Sort failed' ) ;
    end ;
    for Loop := 0 to Test.Count - 1 do
    begin
        if( Test[ Loop ] <> Loop ) then
        begin
            raise Exception.Create( 'TInteger_List.Sort failed' ) ;
        end ;
    end ;
    Test.Clear ;

    { Check memory usage... }
{
    if( Allocation_Count <> Intermediate ) then
    begin
        raise Exception.Create( 'Memory loss' ) ;
    end ;
}

    Test.Free ;
    Test1.Free ;

    { Check memory usage... }
{
    if( Allocation_Count <> 0 ) then
    begin
        raise Exception.Create( 'Memory loss' ) ;
    end ;
    End_Memory_Test ;
}
end ;
  {$ENDIF}

{$ENDIF}


end.
