{
        Program Name : TCommon_COM_Interface
        Package Name : Subroutine Library - Layer 0
        Purpose      : Abstract virtual common COM interface class
        Institution  : Conroy & Conroy Co.
        Date Written : 21-March-2002
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2002-2015 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TCommon_COM_Interface is a base class definition for COM objects used
        in the Application Foundation.
}

unit COMInter ;

{$I EDEFINES.INC}

interface

{$IFDEF DYNAMIC_OBJECTS}
  {$IFDEF STDCALL_SUPPORTED}

uses { Subroutine Library - layer 0... }
{$IFDEF CC_Track_Resources}
     ResMon,
{$ENDIF}
     _COMInter, { TBase_COM_Interface }
{$IFNDEF MOBILE}
     _DebugIn, { TDebug_Interface }
{$ENDIF}
     _UE ; { TUnified_Exception }

type TCommon_COM_Interface = class( TBase_COM_Interface )
                                 public { API... }
{$IFDEF CC_Track_Resources}
                                     constructor Create ;
{$ENDIF}
                                     destructor Destroy ; override ;

                                 protected { Instance data... }
                                     Reference_Count : longint ;
                                     _Last_Error : TUnified_Exception ;

                                 public { API... }
                                     { Initialize the object.  Returns result of
                                       initialization attempt. }
                                     function Initialize : TUnified_Exception ;
                                         override ; stdcall ;

                                     { Terminate the use of the object.  Returns
                                       result of termination.  This generally
                                       should not be used - instead, use the
                                       Detach method. }
                                     function Terminate : TUnified_Exception ;
                                         override ; stdcall ;

                                     { Increments the reference count for the
                                       object. }
                                     procedure Attach ; override ; stdcall ;

                                     { Decrements the reference count for the
                                       object.  When count reaches 0, the object
                                       is destroyed. }
                                     function Detach : boolean ;
                                         override ; stdcall ;

                                     { Version of the COM interface for this
                                       object. }
                                     function Interface_Version : longint ;
                                         override ; stdcall ;

                                     { Facility ID for the facility represented
                                       by this object.  Returns -1 if no
                                       facility assigned. }
                                     function Facility_ID : longint ;
                                         override ; stdcall ;

                                     { Facility version.  Result only has
                                       meaning if Facility_ID doesn't return
                                       -1. }
                                     function Facility_Version : longint ;
                                         override ; stdcall ;

{$IFNDEF MOBILE}
                                     { Returns a debugging interface for the
                                       object.  Returns nil if not supported. }
                                     function Debugger : TDebug_Interface ;
                                         override ; stdcall ;
{$ENDIF}

                                     { Returns last error. }
                                     function Last_Error : TUnified_Exception ;
                                         override ; stdcall ;

                                     { Returns a pointer to an object with
                                       extended common COM methods for object.
                                       Always returns nil for now. }
                                     function Extension : pointer ; override ;
                                         stdcall ;

                                 public // Internal utility routines...
                                     { Note: Don't make any of these virtual, as
                                       that could mess up VMT layout.  This class
                                       is simply to provide implementation of
                                       the abstract base class. }
                                     function Set_Last_Error( E : TUnified_Exception ) : TUnified_Exception ;
                             end ; { TCommon_COM_Interface }
     PCommon_COM_Interface = class( TCommon_COM_Interface ) ;

type TBase_COM_Interface2 = class
                               public { API... }
                                   { Initialize the object.  Returns result of
                                     initialization attempt. }
                                   function Initialize : TUnified_Exception ;
                                       virtual ;
                                       stdcall ; abstract ;

                                   { Terminate the use of the object.  Returns
                                     result of termination.  This generally
                                     should not be used - instead, use the
                                     Detach method. }
                                   function Terminate : TUnified_Exception ;
                                       virtual ;
                                       stdcall ; abstract ;

                                   { Increments the reference count for the
                                     object. }
                                   procedure Attach ; virtual ;
                                       stdcall ; abstract ;

                                   { Decrements the reference count for the
                                     object.  When count reaches 0, the object
                                     is destroyed. }
                                   procedure Detach ; virtual ;
                                       stdcall ; abstract ;

                                   { Version of the COM interface for this
                                     object. }
                                   function Interface_Version : longint ;
                                       virtual ;
                                       stdcall ; abstract ;

                                   { Facility ID for the facility represented
                                     by this object.  Returns -1 if no
                                     facility assigned. }
                                   function Facility_ID : longint ; virtual ;
                                       stdcall ; abstract ;

                                   { Facility version.  Result only has
                                     meaning if Facility_ID doesn't return
                                     -1. }
                                   function Facility_Version : longint ;
                                       virtual ;
                                       stdcall ; abstract ;

{$IFNDEF MOBILE}
                                   { Returns a debugging interface for the
                                     object.  Returns nil if not supported. }
                                   function Debugger : TDebug_Interface ;
                                       virtual ;
                                       stdcall ; abstract ;
{$ENDIF}

                                   { Returns a pointer to an object with
                                     extended common COM methods for object.
                                     Always returns nil for now. }
                                   function Extension : pointer ; virtual ;
                                       stdcall ; abstract ;
                           end ; { TBase_COM_Interface2 }

     TCommon_COM_Interface2 = class( TBase_COM_Interface2 )
                                 private { Instance data... }
                                     Reference_Count : longint ;

                                 public { API... }
                                     { Initialize the object.  Returns result of
                                       initialization attempt. }
                                     function Initialize : TUnified_Exception ;
                                         override ;
                                         stdcall ;

                                     { Terminate the use of the object.  Returns
                                       result of termination.  This generally
                                       should not be used - instead, use the
                                       Detach method. }
                                     function Terminate : TUnified_Exception ;
                                         override ;
                                         stdcall ;

                                     { Increments the reference count for the
                                       object. }
                                     procedure Attach ; override ;
                                         stdcall ;

                                     { Decrements the reference count for the
                                       object.  When count reaches 0, the object
                                       is destroyed. }
                                     procedure Detach ; override ;
                                         stdcall ;

                                     { Version of the COM interface for this
                                       object. }
                                     function Interface_Version : longint ;
                                         override ;
                                         stdcall ;

                                     { Facility ID for the facility represented
                                       by this object.  Returns -1 if no
                                       facility assigned. }
                                     function Facility_ID : longint ; override ;
                                         stdcall ;

                                     { Facility version.  Result only has
                                       meaning if Facility_ID doesn't return
                                       -1. }
                                     function Facility_Version : longint ;
                                         override ;
                                         stdcall ;

{$IFNDEF MOBILE}
                                     { Returns a debugging interface for the
                                       object.  Returns nil if not supported. }
                                     function Debugger : TDebug_Interface ;
                                         override ;
                                         stdcall ;
{$ENDIF}

                                     { Returns a pointer to an object with
                                       extended common COM methods for object.
                                       Always returns nil for now. }
                                     function Extension : pointer ; override ;
                                         stdcall ;
                             end ; { TCommon_COM_Interface }
     PCommon_COM_Interface2 = class( TCommon_COM_Interface2 ) ;
  {$ENDIF}
{$ENDIF}

implementation

{$IFDEF DYNAMIC_OBJECTS}
  {$IFDEF STDCALL_SUPPORTED}
{ TCommon_COM_Interface methods... }

{ Constructors and destructors... }
    {$IFDEF CC_Track_Resources}
constructor TCommon_COM_Interface.Create ;

begin
    inherited Create ;

    Add_Resource( RT_Object, integer( self ) ) ;
end ;
    {$ENDIF}


destructor TCommon_COM_Interface.Destroy ;

begin
    Set_Last_Error( nil ) ;
    {$IFDEF CC_Track_Resources}
    Remove_Resource( RT_Object, integer( self ) ) ;
    {$ENDIF}

    inherited Destroy ;
end ;


{ API... }

function TCommon_COM_Interface.Initialize : TUnified_Exception ; stdcall ;

begin
    Result := nil ;
    Reference_Count := 0 ;
end ;


function TCommon_COM_Interface.Terminate : TUnified_Exception ; stdcall ;

begin
    Set_Last_Error( nil ) ;
    Result := nil ;
    Free ;
end ;


procedure TCommon_COM_Interface.Attach ; stdcall ;

begin
    inc( Reference_Count ) ;
end ;


function TCommon_COM_Interface.Detach : boolean ; stdcall ;

begin
    dec( Reference_Count ) ;
    Result := Reference_Count < 1 ;
    if( Result ) then
    begin
        Terminate ;
    end ;
end ;


function TCommon_COM_Interface.Interface_Version : longint ; stdcall ;

begin
    Interface_Version := 0 ;
end ;


function TCommon_COM_Interface.Facility_ID : longint ; stdcall ;

begin
    Facility_ID := -1 ;
end ;


function TCommon_COM_Interface.Facility_Version : longint ; stdcall ;

begin
    Facility_Version := 0 ;
end ;


    {$IFNDEF MOBILE}
function TCommon_COM_Interface.Debugger : TDebug_Interface ; stdcall ;

begin
    Debugger := nil ;
end ;
    {$ENDIF}


function TCommon_COM_Interface.Last_Error : TUnified_Exception ; stdcall ;

begin
    Last_Error := _Last_Error ;
end ;


function TCommon_COM_Interface.Extension : pointer ; stdcall ;

begin
    Extension := nil ;
end ;


// Internal utility routines...

function TCommon_COM_Interface.Set_Last_Error( E : TUnified_Exception ) : TUnified_Exception ;

begin
    if( E <> nil ) then
    begin
        E.Attach ;
    end ;
    if( _Last_Error <> nil ) then
    begin
        _Last_Error.Detach ;
    end ;
    _Last_Error := E ;
    Result := E ;
end ;



{ TCommon_COM_Interface2 methods... }

{ API... }

function TCommon_COM_Interface2.Initialize : TUnified_Exception ; stdcall ;

begin
    Result := nil ;
    Reference_Count := 0 ;
end ;


function TCommon_COM_Interface2.Terminate : TUnified_Exception ; stdcall ;

begin
    Result := nil ;
    Free ;
end ;


procedure TCommon_COM_Interface2.Attach ; stdcall ;

begin
    inc( Reference_Count ) ;
end ;


procedure TCommon_COM_Interface2.Detach ; stdcall ;

begin
    dec( Reference_Count ) ;
    if( Reference_Count < 1 ) then
    begin
        Terminate ;
    end ;
end ;


function TCommon_COM_Interface2.Interface_Version : longint ; stdcall ;

begin
    Interface_Version := 0 ;
end ;


function TCommon_COM_Interface2.Facility_ID : longint ; stdcall ;

begin
    Facility_ID := -1 ;
end ;


function TCommon_COM_Interface2.Facility_Version : longint ; stdcall ;

begin
    Facility_Version := 0 ;
end ;


    {$IFNDEF MOBILE}
function TCommon_COM_Interface2.Debugger : TDebug_Interface ; stdcall ;

begin
    Debugger := nil ;
end ;
    {$ENDIF}


function TCommon_COM_Interface2.Extension : pointer ; stdcall ;

begin
    Extension := nil ;
end ;
  {$ENDIF}
{$ENDIF}



end.
