{
        Program Name : HTML
        Package Name : Subroutine Library - Layer 4
        Purpose      : HTML support
        Institution  : Conroy & Conroy Co.
        Date Written : 3-April-2003
        Written By   : Alan Conroy
        Version      : 1.0K

        Copyright (C) 2003-2015 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY        REASON

          6-Jan-2004    EAC       Added Max_X to Process_HTML.
         25-Jun-2004              Fix problem with translated HTML text output.
         12-Jan-2004    EAC       Added XML support.
          2-Feb-2005    EAC       Added HTML_To_Text.
          7-Mar-2005    EAC       Added strikethrough support.
          7-Mar-2006    EAC       Fixed infinite loop.
         16-Mar-2006    NLR       Fixed infinite loop.
         10-Dec-2006    EAC       Fixed FONT tag parsing.
         25-Jan-2008    EAC       Added RGB support to FONT COLOR.
         31-Dec-2008    EAC       Added ITALIC support.
          8-Jul-2009    EAC       Added HTML_Text_Width and HTML_Text_Height.
         21-Jul-2015    EAC       Added URL_Encode.
         21-Oct-2015              Fixed problem with FONT FACE prsisting after
                                  </FONT>.
         29-Jun-2019    EAC       Added Search_HTML.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          HTML support routines.
}

unit HTML ;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

{$I EDEFINES.INC}

uses { Borland... }
{$IFDEF FPC}
     LCLIntf, LCLType,
{$ENDIF}
{$IFDEF FPC_OR_DELPHI}
     Classes, {  TStringList }
  {$IFDEF NOVCL}
     FMX.Graphics,
     System.UITypes, // TAlphaColor
  {$ELSE}
    {$IFDEF DELPHI10UP}
     VCL.Graphics, { TFont }
    {$ELSE}
     Graphics, { TFont }
    {$ENDIF}
  {$ENDIF}
{$ENDIF}

     { Subroutine Library - Layer 0... }
     CommBase, { TBase_Delphi_Object }
{$IFDEF FPC_OR_DELPHI}
     ComInter, { TCommon_COM_Interface }
{$ENDIF}
     _Types, // Ansistring
     _UE, // TUnified_Exception
{$IFDEF MSWINDOWS}
     WinAPI, // HDC
{$ENDIF}

     { Subroutine Library - Layer 1... }
     CVT, { Hex }

     { Subroutine Library - Layer 3... }
     Parse, { TString_Parser }
     UStrings ; { TASCII_String }

function HTML_ERT( X : integer ) : Ansistring ; { Return text of HTML error code }
function From_HTML( X : Ansistring ; Options : longint ) : Ansistring ;
{ Convert HTML to ASCII text.
  Options is processing flags: and 1 = keep unknown entity references in string
                               and 2 = Convert <br> to CRLF
                               and 4 = Convert <p to CRLF }

function To_HTML( const X : Ansistring ) : Ansistring ; { Convert ASCII text to HTML }
function To_HTML_Leave_Tags( const X : Ansistring ) : Ansistring ;
function HTML_To_Text( X : Ansistring ) : Ansistring ; { Convert HTML to plain text }

function URL_Decode( const S : Ansistring ) : Ansistring ;
function URL_Encode( const S : Ansistring ) : Ansistring ;

function Decode_Entities( X : Ansistring ; Options : integer ) : Ansistring ;

{$IFDEF MSWINDOWS}
  {$IFDEF DYNAMIC_OBJECTS}
type THTML = class( TBase_Delphi_Object )
                 public // API...
                     function Process_Text( DC : HDC ; Font : TFont ;
                         const S : Ansistring ; Draw : boolean ;
                         X, Y, Max_X : integer ; var Max_Height : integer ;
                         Single_Line, Center : boolean ) : integer ; virtual ;
    {$IFDEF NOVCL}
                     procedure Write_Text( Handle : HDC ; Font : HFONT ;
                         Color : TAlphaColor ; X, Y, Max_X : integer ;
                         Text : Ansistring ) ; virtual ;
    {$ELSE}
                     procedure Write_Text( Handle : HDC ; Font : HFONT ;
                         Color : TColor ; X, Y, Max_X : integer ;
                         Text : Ansistring ) ; virtual ;
    {$ENDIF}
             end ; // THTML
  {$ENDIF}

// Outputs HTML to DC.  Returns last pixel column written.
function Process_HTML_Text( DC : HDC ; Font : TFont ; const S : Ansistring ;
     Draw : boolean ; X, Y, Max_X : integer ; var Max_Height : integer ;
     Single_Line, Center : boolean ) : integer ;

function HTML_Text_Width( DC : HDC ; Font : TFont ;
    const S : Ansistring ) : integer ;
function HTML_Text_Height( DC : HDC ; Font : TFont ;
    const S : Ansistring ) : integer ;

function Pixels_To_Size( X : integer ) : integer ;
function Size_To_Pixels( X : integer ) : integer ;
{$ENDIF}

function Extract_Tag_ID( const S : Ansistring ) : Ansistring ;
function Remove_Tags( const S, Tag : string ) : string ;
procedure Search_HTML( Sub, Source : string ; var Starting, Ending : integer ;
    Options : integer ; Start : integer = -1 ) ;

type TSHTML_Parser = object( TSParser )
                        public { Constructors and destructors... }
                            constructor Init ;
                            destructor Done ; virtual ;

                        private { Data... }
                            _Section_End : Ansistring ;

{$IFDEF FPC_OR_DELPHI}
                        protected { Internal utility routines... }
{$ENDIF}
                            function Get_Token : UStrings.PString ; virtual ;
                            function _Get_Section( S : Ansistring ;
                                XML : boolean ) : UStrings.PString ;

                        public { API... }
                            Preserve_Whitespace : boolean ;
                            Source : TASCII_String ;

                            function EOF : boolean ;
                            function Get_Section( const S : Ansistring ) : UStrings.PString ;
                                virtual ;
                            function Read_Next_Line : boolean ; virtual ;
                            function Section_End : Ansistring ;
                    end ;
     PSHTML_Parser = ^TSHTML_Parser ;

type TSXML_Parser = object( TSHTML_Parser )
                        public { API... }
                            function Get_Section( const S : Ansistring ) : UStrings.PString ;
                                virtual ;
                            function Get_Section_String( const S : Ansistring ) : Ansistring ;
                            function Get_Token_String : Ansistring ;
                    end ;
     PSXML_Parser = ^TSXML_Parser ;

{$IFDEF DYNAMIC_OBJECTS}
type THTML_Parser = class( TParser )
                        public { Constructors and destructors... }
                            constructor Create ;

                        protected { Internal utility routines... }
                            function Get_Source : Ansistring ;
                            procedure Set_Source( const S : Ansistring ) ;
                            function Get_Preserve_Whitespace : boolean ;
                            procedure Set_Preserve_Whitespace( Value : boolean ) ;

                            function _Get_Section( const S : Ansistring ;
                                XML : boolean ) : Ansistring ; virtual ;

                        public { API... }
                            function EOF : boolean ;
                            function Get_Section( const S : Ansistring ) : Ansistring ;
                                virtual ;
                            function Section_End : Ansistring ;

                        public { Properties... }
                            property Preserve_Whitespace : boolean
                                read Get_Preserve_Whitespace
                                write Set_Preserve_Whitespace ;

                            property Source : Ansistring
                                read Get_Source
                                write Set_Source ;
                    end ; { THTML_Parser }

     THTML_Tag = class
                     private
                         Names, Values : TStringList ;

                     public
                         Name : Ansistring ;

                         constructor Create( S : Ansistring ) ;
                         destructor Destroy ; override ;

                         function Attribute_Name( Index : integer ) : Ansistring ;
                         function Attribute_Value( Index : integer ) : Ansistring ;
                         function Count : integer ;
                 end ;

type TXML_Parser = class( TCommon_COM_Interface )
                        public { Constructors and destructors... }
                            constructor Create ;
                            destructor Destroy ; override ;

                        private { Instance data }
                            _Parser : THTML_Parser ;
                            Temp : Ansistring ;

                        public { API... }
                            function Is_Class( N : PAnsiChar ) : boolean ;
                                override ; stdcall ;

                            function Last_Error : TUnified_Exception ;
                                override ; stdcall ;

                            function Get_Current_Line : longint ;
                                virtual ; stdcall ;
                            function Get_Non_Terminals : PAnsiChar ;
                                virtual ; stdcall ;
                            procedure Set_Non_Terminals( S : PAnsiChar ) ;
                                virtual ; stdcall ;
                            function Get_Preserve_Whitespace : boolean ;
                                virtual ; stdcall ;
                            procedure Set_Preserve_Whitespace( Value : boolean ) ;
                                virtual ; stdcall ;
                            function Get_Source : PAnsiChar ; virtual ; stdcall ;
                            procedure Set_Source( S : PAnsiChar ) ;
                                virtual ; stdcall ;
                            function Get_Token_Line : integer ;
                                virtual ; stdcall ;
                            procedure Set_Token_Line( V : integer ) ;
                                virtual ; stdcall ;

                            procedure Add_Special_Token( S : PAnsiChar ) ;
                                virtual ; stdcall ;
                            function EOF : boolean ; virtual ; stdcall ;
                            function Get_Section( S : PAnsiChar ) : PAnsiChar ;
                                virtual ; stdcall ;
                            function Get_Token : PAnsiChar ; virtual ; stdcall ;
                            function Previous_Token_Line : Integer ;
                                virtual ; stdcall ;
                            procedure Push_Token_Line ; virtual ; stdcall ;
                            function Read_Next_Line : boolean ;
                                virtual ; stdcall ;
                            function Grab_Line : PAnsiChar ; virtual ; stdcall ;
                            procedure Put_Token( S : PAnsiChar ) ;
                                virtual ; stdcall ;
                            function Token : PAnsiChar ; virtual ; stdcall ;
                            function Token_EOL : boolean ; virtual ; stdcall ;

                        public { Properties... }
                            property Source : PAnsiChar
                                read Get_Source
                                write Set_Source ;
                    end ; { TXML_Parser }

     TXML_Tag = class( TCommon_COM_Interface )
                     public { Constructors and destructors }
                         constructor Create( const S : Ansistring ) ;
                         destructor Destroy ; override ;

                     private { Instance data... }
                         _Tag : THTML_Tag ;
                         Temp : Ansistring ;

                     public { API... }
                         function Attribute_Name( Index : integer ) : PAnsiChar ;
                             virtual ; stdcall ;
                         function Attribute_Value( Index : integer ) : PAnsiChar ;
                             virtual ; stdcall ;
                         function Count : integer ; virtual ; stdcall ;
                         function Get_Name : PAnsiChar ; virtual ; stdcall ;
                         procedure Set_Name( P : PAnsiChar ) ; virtual ; stdcall ;
                 end ;
{$ENDIF}

{$IFDEF CC_Regression_Test}
procedure Unit_Test ;
{$ENDIF}

const Font_Scaling = 3 ;
const Font_Base_Size = 8 ;


implementation

uses { Borland... }
{$IFDEF FPC_OR_DELPHI}
     SysUtils, { MemAlloc }
{$ENDIF}
     strutils, // Posex

     { Subroutine Library - Layer 0... }
     _ASCII, { CR }
     Compatib,
     L10n, { Text_ }

     { Subroutine Library - Layer 1... }
     CommonUt, { Edit }
     Instrs, { Instr }
     Num1s ; { Num1 }


function URL_Decode( const S : Ansistring ) : Ansistring ;

var Dummy : integer ;

begin
    Result := '' ;
    Dummy := 1 ;
    while( Dummy <= length( S ) ) do
    begin
        if( S[ Dummy ] = '%' ) then
        begin
            Result := Result + AnsiChar( From_Hex( copy( S, Dummy + 1, 2 ) ) ) ;
            Dummy := Dummy + 3 ;
        end else
        begin
            Result := Result + S[ Dummy ] ;
            inc( Dummy ) ;
        end ;
    end ;
end ;


function URL_Encode( const S : Ansistring ) : Ansistring ;

var C : Ansichar ;
    I : Integer ;

begin
    Result := '' ;
    for I := 1 to length( S ) do
    begin
        C := S[ I ] ;
        case C of
            'A'..'Z' : Result := Result + C ;
            'a'..'z' : Result := Result + C ;
            '0'..'9' : Result := Result + C ;
            '-' : Result := Result + C ;
            '_' : Result := Result + C ;
            '.' : Result := Result + C ;
            '~' : Result := Result + C ;
            else Result := Result + '%' + Hex( ord( C ) ) ;
        end ;
    end ;
end ;


function HTML_ERT( X : integer ) : Ansistring ;

begin
    case X of
        204 : HTML_ERT := Text_Error_HTML_No_Content ;
        301 : HTML_ERT := Text_Error_HTML_Moved_Permanently ;
        302 : HTML_ERT := Text_Error_HTML_Moved_Temporarily ;
        400 : HTML_ERT := Text_Error_HTML_Bad_Request ;
        401 : HTML_ERT := Text_Error_HTML_Unauthorized ;
        402 : HTML_ERT := Text_Error_HTML_Payment_Required ;
        403 : HTML_ERT := Text_Error_HTML_Forbidden ;
        404 : HTML_ERT := Text_Error_HTML_Not_Found ;
        405 : HTML_ERT := Text_Error_HTML_Method_Not_Allowed ;
        407 : HTML_ERT := Text_Error_HTML_Proxy_Authentication_Required ;
        408 : HTML_ERT := Text_Error_HTML_Request_Timeout ;
        409 : HTML_ERT := Text_Error_HTML_Conflict ;
        410 : HTML_ERT := Text_Error_HTML_Gone ;
        502 : HTML_ERT := Text_Error_HTML_Bad_Gateway ;
        503 : HTML_ERT := Text_Error_HTML_Service_Unavailable ;
        504 : HTML_ERT := Text_Error_HTML_Gateway_Timeout ;
        else HTML_ERT := Substitute1( Text_Unknown_Error_Code, Num1( X ) ) ;
    end ;
end ;


// Return a number (0-255) from a string, returning 0 if any errors.
function NFS( S : Ansistring ) : integer ;

var Dummy, Loop : integer ;

begin
    S := Edit( S, -1 ) ;
    if( ( length( S ) = 0 ) ) then
    begin
        NFS := 0 ;
        exit ;
    end ;
    val( S, Loop, Dummy ) ;
    if( Dummy <> 0 ) then // Invalid value
    begin
        NFS := 0 ;
        exit ;
    end ;
    NFS := Loop and 255 ;
end ;


{$IFDEF VER140}
type tagSize = TSize ;
{$ENDIF}

type TFont_Info = class
                      public { API... }
                          Color : TColor ;
                          Face : Ansistring ;
                          Size : integer ;
                  end ;

function HTML_To_Text( X : Ansistring ) : Ansistring ; { Convert HTML to plain text }

var Dummy, Dummy1 : integer ;

begin
    Dummy := pos( '<', X ) ;
    while( Dummy > 0 ) do
    begin
        Dummy1 := instr( Dummy + 1, X + '>', '>' ) ;
        X := copy( X, 1, Dummy - 1 ) + copy( X, Dummy1 + 1, length( X ) ) ;
        Dummy := pos( '<', X ) ;
    end ;
    Result := From_HTML( X, 0 ) ;
end ; { HTML_To_Text }


{$IFDEF MSWINDOWS}
function Max_Position( DC : HDC ; Temp : Ansistring ; Max_X : integer ) : integer ;

var Dummy : integer ;
    Size : tSize ;

begin
    if( Max_X <= 0 ) then
    begin
        Result := 0 ;
        exit ;
    end ;
    Dummy := length( Temp ) ;
    Temp := Edit( Temp, 128 ) ; { Remove trailing spaces }
    if( ( length( Temp ) = 0 ) and ( Dummy > 0 ) ) then // Must have been all space(s)
    begin
        Result := Dummy ;
        exit ;
    end ;
    GetTextExtentPoint( DC, Temp, length( Temp ), Size ) ;
    while( Size.cx > Max_X ) do
    begin
        while( ( length( Temp ) > 0 ) and ( Temp[ length( Temp ) ] <> ' ' ) ) do
        begin
            setlength( Temp, length( Temp ) - 1 ) ;
        end ;
        Temp := Edit( Temp, 128 ) ; { Remove trailing spaces }
        GetTextExtentPoint( DC, PAnsichar( Temp ), length( Temp ), Size ) ;
        if( Temp = '' ) then
        begin
            break ;
        end ;
    end ;
    Result := length( Temp ) ;
end ;
{$ENDIF}


procedure Decode_Color( Work : Ansistring ; var Color : TColor ) ;

begin
    if( Work = 'BLACK' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Black ;
{$ELSE}
        Color := clBlack ;
{$ENDIF}
    end else
    if( Work = 'WHITE' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.White ;
{$ELSE}
        Color := clWhite ;
{$ENDIF}
    end else
    if( Work = 'GRAY' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Gray ;
{$ELSE}
        Color := clGray ;
{$ENDIF}
    end else
    if( Work = 'GREY' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Gray ;
{$ELSE}
        Color := clGray ;
{$ENDIF}
    end else
    if( Work = 'SILVER' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Silver ;
{$ELSE}
        Color := clSilver ;
{$ENDIF}
    end else
    if( Work = 'FUCHSIA' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Fuchsia ;
{$ELSE}
        Color := clFuchsia ;
{$ENDIF}
    end else
    if( Work = 'RED' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Red ;
{$ELSE}
        Color := clRed ;
{$ENDIF}
    end else
    if( Work = 'PURPLE' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Purple ;
{$ELSE}
        Color := clPurple ;
{$ENDIF}
    end else
    if( Work = 'MAROON' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Maroon ;
{$ELSE}
        Color := clMaroon ;
{$ENDIF}
    end else
    if( Work = 'YELLOW' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Yellow ;
{$ELSE}
        Color := clYellow ;
{$ENDIF}
    end else
    if( Work = 'LIME' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Lime ;
{$ELSE}
        Color := clLime ;
{$ENDIF}
    end else
    if( Work = 'OLIVE' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Olive ;
{$ELSE}
        Color := clOlive ;
{$ENDIF}
    end else
    if( Work = 'AQUA' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Aqua ;
{$ELSE}
        Color := clAqua ;
{$ENDIF}
    end else
    if( Work = 'BLUE' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Blue ;
{$ELSE}
        Color := clBlue ;
{$ENDIF}
    end else
    if( Work = 'TEAL' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Teal ;
{$ELSE}
        Color := clTeal ;
{$ENDIF}
    end else
    if( Work = 'NAVY' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Navy ;
{$ELSE}
        Color := clNavy ;
{$ENDIF}
    end else
    if( Work = 'GREEN' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Green ;
{$ELSE}
        Color := clGreen ;
{$ENDIF}
    end else
    if( Work = 'GRAYTEXT' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Gray ;
{$ELSE}
        Color := clGrayText ;
{$ENDIF}
    end else
    if( Work = 'GREYTEXT' ) then
    begin
{$IFDEF NOVCL}
        Color := TAlphaColorRec.Gray ;
{$ELSE}
        Color := clGrayText ;
{$ENDIF}
    end ;
end ; // Decode_Color


{$IFDEF MSWINDOWS}
  {$IFDEF DYNAMIC_OBJECTS}
    {$IFDEF NOVCL}
procedure THTML.Write_Text( Handle : HDC ; Font : HFONT ;
                         Color : TAlphaColor ; X, Y, Max_X : integer ;
                         Text : Ansistring ) ;
    {$ELSE}
procedure THTML.Write_Text( Handle : HDC ; Font : HFONT ;
                         Color : TColor ; X, Y, Max_X : integer ;
                         Text : Ansistring ) ;
    {$ENDIF}

    procedure Write_Segment( X, Y : integer ; const Text : Ansistring ) ;

    var Rect : TRect ;

    begin
        if( length( Text ) > 0 ) then
        begin
            SetBKMode( Handle, TRANSPARENT ) ; { Something resets this, so change it back }
//            WinAPI.TextOut( Handle, X, Y, PAnsichar( Text ), length( Text ) ) ;
            Rect.Top := 0 ;
            Rect.Bottom := 32767 ;
            Rect.Left := X ;
            Rect.Right := X + Max_X ;
            if( Rect.Right > 32767 ) then
            begin
                Rect.Right := 32767 ;
            end ;
            ExtTextOut( Handle, X, Y, ETO_CLIPPED, @Rect, PAnsichar( Text ), length( Text ), nil ) ;
        end ;
    end ; { Write_Text.Write_Segment }

var Dummy : integer ;
    S : Ansistring ;
    Saved : Thandle ;
    Size : tSize ;

begin
    { Setup... }
    Saved := SelectObject( Handle, Font ) ;
    try
    {$IFDEF NOVCL}
        SetTextColor( Handle, Color ) ;
    {$ELSE}
        SetTextColor( Handle, ColorToRGB( Color ) ) ;
    {$ENDIF}

        { Handle up to tab... }
        Dummy := pos( #9, Text + #9 ) ;
        S := copy( Text, 1, Dummy - 1 ) ;
        Write_Segment( X, Y, S ) ;

        { Write text following tab... }
        if( Dummy < length( Text ) ) then
        begin
            S := copy( Text, Dummy + 1, length( Text ) ) ;
            GetTextExtentPoint32( Handle, PAnsichar( S ), length( S ), Size ) ;
            X := Max_X - Size.cx ;
            Write_Segment( X, Y, S ) ;
        end ;
    finally
        { Cleanup }
        SelectObject( Handle, Saved ) ;
    end ;
end ; { THTML.Write_Text }


function THTML.Process_Text( DC : HDC ; Font : TFont ; const S : Ansistring ;
     Draw : boolean ; X, Y, Max_X : integer ;
     var Max_Height : integer ; Single_Line, Center : boolean ) : integer ;
    { Process HTML text in the context of the passed DC.
      Parameters:
          DC = Device Context
          Font = Default font
          S = Text to process
          Draw = True to write out to DC.  False to calculate heights and widths
          X = Starting X position of text.
          Y = Starting Y position of text.
          Max_X = Maximum width before wrapping or truncation
          Max_Height = Passed value ignored.  On return, this is the maximum height,
                       in pixels, for all lines.
          List = If Draw is false, the heights of each line are added to this list;
                 otherwise each line gets its height from the corresponding entry in
                 the list.
          List_W = If Draw is false, the widths of each line are added to this list;
                   otherwise each line gets its width from the corresponding entry
                   in the list.
          Single_Line = True to limit to a single line (multiple lines otherwise)
          Center = True to center line(s)
    }


    function _Process_Text( DC : HDC ; Font : TFont ; S : Ansistring ;
         Draw : boolean ; X, Y, Max_X : integer ;
         var Max_Height : integer ; List, List_W : TInteger_List ;
         Single_Line, Center : boolean ) : integer ;

    var Big_Count : integer ;
        Bold_Count : integer ;
    {$IFDEF NOVCL}
        Color : TAlphaColor ; // Current font color
    {$ELSE}
        Color : TColor ; // Current font color
    {$ENDIF}
        Default_Font : TFont_Info ; // Initial font information
        Italic_Count : integer ;
        This_Font : HFONT ;
        Fonts : TList ;
        LogFont : TLogFont ;
        P : TFont_Info ;
        Size : tSize ;
        Strikeout_Count : integer ;
        Superscript_Count : integer ;
        Tag : Ansistring ;
        Temp : Ansistring ;
        TM : TTextMetric ;
        Underline_Count : integer ;
        Work : Ansistring ;

        procedure Create_New_Font ;

        var Temp : HFONT ;

        begin
            Temp := This_Font ;
            This_Font := CreateFontIndirect( LogFont ) ;
            selectobject( DC, This_Font ) ;
            if( Temp <> 0 ) then
            begin
                deleteobject( Temp ) ;
            end ;
            GetTextMetrics( DC, TM ) ;
        end ;

    var Adjust : integer ; // Amount to adjust Y offset by to align text baselines
        Dummy, Dummy1 : integer ;
        Height : integer ; { Current height }
        Index, Loop, M : integer ;
        Line : integer ; { Current line (0 based) }
        Original_Max_X, Original_X : integer ;
    {$IFNDEF NOVCL}
        Pen : TPen ;
    {$ENDIF}
        Saved : THandle ;
        Total_Height : integer ;
        X_Center, Y_Center : integer ;
        X_Offset, Y_Offset : integer ;
        Weight : integer ;
        Width : integer ; // Width of widest line (Result is the current position of the current line)

        procedure New_Line ;

        begin
            // Move to next line...
            Y := Y + Height - 1 ; // EAC, 10-Oct-2006: Added "- 1" because Copyright notice appears to have an extra pixel between lines
            inc( Line ) ;
            if( List <> nil ) then
            begin
                if( Draw ) then
                begin
                    Height := 0 ;
                    if( List.Count > Line ) then
                    begin
                        Height := List[ Line ] ;
                    end ;
                end else
                begin
                    List.Add( Height ) ;
                    Height := 0 ;
                end ;
            end ;
            Max_X := Original_Max_X ;
            Total_Height := Total_Height + Height ;//Max_Height ;
            Max_Height := 0 ;
            if( Result > Width ) then
            begin
                Width := Result ;
            end ;

            X := Original_X ;
            if( List_W <> nil ) then
            begin
                if( Draw ) then
                begin
                    Result := Original_X ;
                    if( Center and ( List_W.Count > Line ) ) then
                    begin
                        Result := ( Max_X - List_W[ Line ] ) div 2 ;
                        if( Result < Original_X ) then
                        begin
                            Result := Original_X ;
                        end ;
                    end ;
                end else
                begin
                    List_W.Add( Result ) ;
                    Result := Original_X ;
                end ;
                X := Result ;
            end ;
        end ; // .New_Line

    label Again ;

    begin
        { Setup... }
        Default_Font := TFont_Info.Create ;
        try
            fillchar( Size, sizeof( Size ), 0 ) ;
            Max_Height := 0 ;
            Height := 0 ;
            Total_Height := 0 ;
            Width := 0 ;
            Result := 0 ;
            Big_Count := 0 ;
            Italic_Count := 0 ;
            Bold_Count := 0 ;
            Strikeout_Count := 0 ;
            Superscript_Count := 0 ;
            Underline_Count := 0 ;
            This_Font := 0 ;
            Line := 0 ;
            Original_X := X ;
            Original_Max_X := Max_X ;
            Fonts := TList.Create ;
    {$IFNDEF NOVCL}
            Color := Font.Color ;
            Default_Font.Face := Font.Name ;
            Default_Font.Size := Pixels_To_Size( abs( Font.Height ) ) ;
    {$ENDIF}
            Default_Font.Color := Color ;
    //        Default_Font.Size := Font.Size div Font_Scaling ;
            if( Default_Font.Size < 1 ) then
            begin
                Default_Font.Size := 1 ;
            end ;
            fillchar( LogFont, sizeof( LogFont ), 0 ) ;
    {$IFNDEF NOVCL}
            GetObject( Font.Handle, sizeof( LogFont ), @LogFont ) ;
    {$ENDIF}
            LogFont.lfHeight := -Size_To_Pixels( Default_Font.Size ) ;
            Create_New_Font ;

            { Process text... }
            try
                if( ( List <> nil ) and ( List.Count > 0 ) and Draw ) then
                begin
                    Height := List[ 0 ] ;
                end ;
                if( ( List_W <> nil ) and ( List_W.Count > 0 ) and Draw ) then
                begin
                    Result := Original_X ;
                    if( Center ) then
                    begin
                        Result := ( Max_X - List_W[ 0 ] ) div 2 ;
                        if( Result < Original_X ) then
                        begin
                            Result := Original_X ;
                        end ;
                    end ;
                    X := Result ;
                end ;
                while( length( S ) > 0 ) do
                begin
                    if( copy( S, 1, 1 ) = '<' ) then { Tag }
                    begin
                        Dummy := pos( '>', S + '>' ) ;
                        Tag := copy( S, 1, Dummy ) ;
                        if( copy( Tag, length( Tag ), 1 ) = '>' ) then
                        begin
                            Tag := copy( Tag, 1, length( Tag ) - 1 ) ; { Trim trailing ">" }
                        end ;
                        S := copy( S, Dummy + 1, length( S ) ) ;
                        Tag := Edit( Tag, 16 or 32 or 256 ) ;
                        if( ( Tag = '<B' ) or ( Tag = '<STRONG' ) ) then
                        begin
                            Bold_Count := Bold_Count + 1 ;
                            LogFont.lfWeight := FW_BOLD ;
                            Create_New_Font ;
                        end else
                        if( ( Tag = '</B' ) or ( Tag = '</STRONG' ) ) then
                        begin
                            Bold_Count := Bold_Count - 1 ;
                            if( Bold_Count = 0 ) then
                            begin
                                LogFont.lfWeight := FW_NORMAL ;
                                Create_New_Font ;
                            end ;
                        end else
                        if( Tag = '<BIG' ) then
                        begin
                            Big_Count := Big_Count + 1 ;
                            LogFont.lfHeight := LogFont.lfHeight * 2 ;
                            Create_New_Font ;
                        end else
                        if( Tag = '</BIG' ) then
                        begin
                            Big_Count := Big_Count - 1 ;
                            if( Big_Count = 0 ) then
                            begin
                                LogFont.lfHeight := LogFont.lfHeight div 2 ;
                                Create_New_Font ;
                            end ;
                        end else
                        if( Tag = '<BR' ) then
                        begin
                            New_Line ;
                        end else
                        if( copy( Tag, 1, 5 ) = '<FONT' ) then
                        begin
                            P := TFont_Info.Create ;
                            if( Fonts.Count > 0 ) then
                            begin
                                // Default settings from previous font...
                                P.Color := TFont_Info( Fonts[ Fonts.Count - 1 ] ).Color ;
                                P.Size := TFont_Info( Fonts[ Fonts.Count - 1 ] ).Size ;
                                P.Face := TFont_Info( Fonts[ Fonts.Count - 1 ] ).Face ;
                            end else
                            begin
                                P.Color := Default_Font.Color ;
                                P.Face := Default_Font.Face ;
                                P.Size := Default_Font.Size ; 
                            end ;
                            Fonts.Add( P ) ;
                            Color := P.Color ;
                            Tag := copy( Tag, 6, length( Tag ) ) ; // Trim "FONT"
                            while( length( Tag ) > 0 ) do
                            begin
                                Tag := Edit( Tag, 8 or 128 ) ; // Trim leading/trailing spaces
                                Work := Parse_Parameter( ' ', Tag ) ;
                                if( copy( Work, 1, 6 ) = 'FACE="' ) then
                                begin
                                    Work := copy( Work, 7, length( Work ) ) ;
                                    if( copy( Work, length( Work ), 1 ) = '"' ) then
                                    begin
                                        Work := copy( Work, 1, length( Work ) - 1 ) ;
                                    end ;
                                    if( length( Work ) <= sizeof( LogFont.lfFaceName ) ) then
                                    begin
                                        fillchar( LogFont.lfFaceName, sizeof(LogFont.lfFaceName ), 0 ) ;
                                        move( Work[ 1 ], LogFont.lfFaceName, length( Work ) ) ;
                                        P.Face := Work ;
                                        Create_New_Font ;
                                    end ;
                                end else
                                if( copy( Work, 1, 5 ) = 'SIZE=' ) then
                                begin
                                    { Strip SIZE }
                                    Work := copy( Work, 6, length( Work ) ) ;

                                    { Strip quotes... }
                                    if( copy( Work, 1, 1 ) = '"' ) then
                                    begin
                                        Work := copy( Work, 2, length( Work ) - 1 ) ;
                                    end ;
                                    if( copy( Work, length( Work ), 1 ) = '"' ) then
                                    begin
                                        Work := copy( Work, 1, length( Work ) - 1 ) ;
                                    end ;

                                    if( copy( Work, 1, 1 ) = '-' ) then
                                    begin
                                        Work := copy( Work, 2, length( Work ) - 1 ) ;
                                        val( Work, Index, Dummy ) ;
                                        if( Dummy = 0 ) then
                                        begin
                                            P.Size := P.Size - Index ;
                                        end ;
                                    end else
                                    if( copy( Work, 1, 1 ) = '+' ) then
                                    begin
                                        Work := copy( Work, 2, length( Work ) - 1 ) ;
                                        val( Work, Index, Dummy ) ;
                                        if( Dummy = 0 ) then
                                        begin
                                            P.Size := P.Size + Index ;
                                        end ;
                                    end else
                                    begin
                                        val( Work, Index, Dummy ) ;
                                        if( Dummy = 0 ) then
                                        begin
                                            P.Size := Index ;
                                        end ;
                                    end ;
                                    if( P.Size > 7 ) then
                                    begin
                                        P.Size := 7 ;
                                    end ;
                                    LogFont.lfHeight := -Size_To_Pixels( P.Size ) ;
                                    Create_New_Font ;
                                end else
                                if( copy( Work, 1, 6 ) = 'COLOR=' ) then
                                begin
                                    Work := copy( Work, 7, length( Work ) ) ;
                                    if( copy( Work, 1, 4 ) = 'RGB(' ) then
                                    begin
                                        Work := copy( Work, 5, length( Work ) ) ;
                                        Dummy := pos( ')', Work + ')' ) ;
                                        Work := copy( Work, 1, Dummy - 1 ) ;
                                        Dummy := pos( ',', Work + ',' ) ;
                                        Color := NFS( copy( Work, 1, Dummy - 1 ) ) ;
                                        Work := copy( Work, Dummy + 1, length( Work ) ) ;
                                        Dummy := pos( ',', Work + ',' ) ;
                                        Color := color or ( NFS( copy( Work, 1, Dummy - 1 ) ) shl 8 ) ;
                                        Work := copy( Work, Dummy + 1, length( Work ) ) ;
                                        Color := color or ( NFS( Work ) shl 16 ) ;
                                        P.Color := Color ;
                                    end else
                                    begin
                                        if( copy( Work, 1, 1 ) = '"' ) then
                                        begin
                                            Work := copy( Work, 2, length( Work ) ) ;
                                            if( copy( Work, length( Work ), 1 ) = '"' ) then
                                            begin
                                                Work := copy( Work, 1, length( Work ) - 1 ) ;
                                            end ;
                                        end ;
                                        Decode_Color( Work, P.Color ) ;
                                    end ;
                                    Color := P.Color ;
                                end ;
                            end ; { while( length( Tag ) > 0 ) }
                        end else
                        if( Tag = '</FONT' ) then
                        begin
                            P := Default_Font ;
                            if( Fonts.Count > 0 ) then { Something to unnest }
                            begin
                                TFont_Info( Fonts[ Fonts.Count - 1 ] ).Free ;
                                Fonts[ Fonts.Count - 1 ] := nil ;
                                Fonts.Delete( Fonts.Count - 1 ) ;
                                if( Fonts.Count > 0 ) then { We are nested }
                                begin
                                    P := TFont_Info( Fonts[ Fonts.Count - 1 ] ) ;
                                end ;
                            end ;
                            Color := P.Color ;
                            if( P.Face <> '' ) then
                            begin
                                if( length( P.Face ) <= sizeof( LogFont.lfFaceName ) ) then
                                begin
                                    fillchar( LogFont.lfFaceName, sizeof( LogFont.lfFaceName ), 0 ) ;
                                    move( P.Face[ 1 ], LogFont.lfFaceName, length( P.Face ) ) ;
                                end ;
                            end ;
                            LogFont.lfHeight := -Size_To_Pixels( P.Size ) ;
                            Create_New_Font ;
                        end else
                        if( ( Tag = '<I' ) or ( Tag = '<ITALIC' ) ) then
                        begin
                            Italic_Count := Italic_Count + 1 ;
                            LogFont.lfItalic := 1 ;
                            Create_New_Font ;
                        end else
                        if( ( Tag = '</I' ) or ( Tag = '</ITALIC' ) ) then
                        begin
                            Italic_Count := Italic_Count - 1 ;
                            if( Italic_Count = 0 ) then
                            begin
                                LogFont.lfItalic := 0 ;
                                Create_New_Font ;
                            end ;
                        end else
                        if( ( Tag = '<S' ) or ( Tag = '<STRIKE' ) ) then
                        begin
                            Strikeout_Count := Strikeout_Count + 1 ;
                            LogFont.lfStrikeOut := 1 ;
                            Create_New_Font ;
                        end else
                        if( ( Tag = '</S' ) or ( Tag = '</STRIKE' ) ) then
                        begin
                            Strikeout_Count := Strikeout_Count - 1 ;
                            if( Strikeout_Count = 0 ) then
                            begin
                                LogFont.lfStrikeOut := 0 ;
                                Create_New_Font ;
                            end ;
                        end else
                        if( copy( Tag, 1, 8 ) = '<STDIMG ' ) then
                        begin
                            Tag := copy( Tag, 9, length( Tag ) ) ;
                            if( Draw ) then
                            begin
                                { Setup... }
    {$IFNDEF NOVCL}
                                Pen := TPen.Create ;
                                Pen.Color := Color ;
                                Saved := selectobject( DC, Pen.Handle ) ;
    {$ENDIF}

                                // Determine maximum segment length...
                                M := TM.tmAscent - TM.tmExternalLeading ;
                                if( TM.tmExternalLeading = 0 ) then
                                begin
                                    M := M - 1 ;
                                end ;
                                if( M > TM.tmMaxCharWidth - 2 ) then
                                begin
                                    M := TM.tmMaxCharWidth - 2 ;
                                end ;

                                // Determine x and Y offset and center points...
                                X_Offset := ( TM.tmMaxCharWidth - M ) div 2 ;
                                Y_Offset := ( TM.tmAscent - M ) div 2 ;
                                if( TM.tmExternalLeading = 0 ) then
                                begin
                                    Y_Offset := Y_Offset + 1 ;
                                end ;
                                X_Center := X_Offset + M div 2 ;
                                Y_Center := Y_Offset + M div 2 ;

                                // Determine number of lines for weight...
                                Weight := M div 2 ; // Maximum weight
                                Weight := Weight * LogFont.lfWeight div 1000 ;
                                if( Weight < 1 ) then
                                begin
                                    Weight := 1 ;
                                end ;
                                X_Center := X_Center - Weight div 2 ;
                                Y_Center := Y_Center - Weight div 2 ;

                                // Process specific control type
                                if( Tag = 'COLLAPSE' ) then
                                begin
                                    Y_Center := Y + Y_Center ;
                                    X_Offset := X + X_Offset ;
                                    for Loop := 0 to Weight - 1 do
                                    begin
                                        WinAPI.MoveToEx( DC, X_Offset, Y_Center + Loop, nil ) ;
                                        WinAPI.LineTo( DC, X_Offset + M, Y_Center + Loop ) ;
                                    end ;
                                end else
                                if( Tag = 'EXPAND' ) then
                                begin
                                    X_Center := X + X_Center ;
                                    Y_Center := Y + Y_Center ;
                                    X_Offset := X + X_Offset ;
                                    Y_Offset := Y + Y_Offset ;
                                    for Loop := 0 to Weight - 1 do
                                    begin
                                        WinAPI.MoveToEx( DC, X_Center + Loop, Y_Offset, nil ) ;
                                        WinAPI.LineTo( DC, X_Center + Loop, Y_Offset + M ) ;
                                        WinAPI.MoveToEx( DC, X_Offset, Y_Center + Loop, nil ) ;
                                        WinAPI.LineTo( DC, X_Offset + M, Y_Center + Loop ) ;
                                    end ;
                                end ;

                                // Cleanup...
    {$IFNDEF NOVCL}
                                selectobject( DC, Saved ) ;
                                Pen.Free ;
    {$ENDIF}

                                // Adjust position...
                                X := X + TM.tmMaxCharWidth ;
                                Max_X := Max_X - Size.cx ;
                                if( Max_X <= 0 ) then
                                begin
                                    Result := Result + TM.tmMaxCharWidth ;
                                    break ;
                                end ;
                            end ;
                            Result := Result + TM.tmMaxCharWidth ;
                        end else
                        if( Tag = '<SUP' ) then
                        begin
                            Superscript_Count := Superscript_Count + 1 ;
                        end else
                        if( Tag = '</SUP' ) then
                        begin
                            Superscript_Count := Superscript_Count - 1 ;
                        end else
                        if( Tag = '<U' ) then
                        begin
                            Underline_Count := Underline_Count + 1 ;
                            LogFont.lfUnderline := 1 ;
                            Create_New_Font ;
                        end else
                        if( Tag = '</U' ) then
                        begin
                            Underline_Count := Underline_Count - 1 ;
                            if( Underline_Count = 0 ) then
                            begin
                                LogFont.lfUnderline := 0 ;
                                Create_New_Font ;
                            end ;
                        end ;
                        continue ;
                    end ; { if( copy( S, 1, 1 ) = '<' ) }

                    { Normal text... }
                    Dummy := pos( '<', S + '<' ) ;
                    Dummy1 := pos( ' ', S + ' ' ) ;
                    if( Dummy1 = 1 ) then
                    begin
                        Dummy1 := instr( 2, S + ' ', ' ' ) ;
                    end ;
                    if( Dummy > Dummy1 ) then
                    begin
                        Dummy := Dummy1 ;
                    end ;
                    { At this point, Dummy is the end of some text to process.  This may
                      be the literal end of the text, or a space in the text, or the
                      start of a tag. }
Again:
                    //NLR Oct-16-2006 Changed options in following code to 1 - allowing processing of '&' tab text.
                    Temp := From_HTML( copy( S, 1, Dummy - 1 ), 1{0} ) ;
                    GetTextExtentPoint( DC, PAnsichar( Temp ), length( Temp ), Size ) ;
                    if( Superscript_Count > 0 ) then
                    begin
                        Size.cy := Size.cy + Size.cy div 3 ;
                    end ;
                    if( Size.cy > Max_Height ) then
                    begin
                        Max_Height := Size.cy ;
                    end ;
                    Dummy1 := Max_Position( DC, Temp, Max_X ) ;
                    if( ( Dummy1 > 0 ) and ( Dummy1 < length( Temp ) ) ) then // Some, but not all, of the text fits
                    begin
                        Dummy1 := Dummy ;
                        while( ( Dummy1 > 0 ) and ( S[ Dummy1 ] <> ' ' ) ) do
                        begin
                            dec( Dummy1 ) ;
                        end ;
                        if( Dummy1 > 1 ) then // Succeeded in trimming a word
                        begin
                            Dummy := Dummy1 ;
                            goto Again ;
                        end ;
                    end else // EAC, 27-Nov-2006, 1714: Added "( not Single_Line )" to next line
                    if( ( Dummy1 = 0 ) and ( not Single_Line ) ) then // None of the text fits on multi-line
                    begin
                        if( X = Original_X ) then // First item on line
                        begin
                            Dummy1 := pos( ' ', Temp + ' ' ) ; // Find end of first word
                            while(
                                   ( Dummy1 <= length( Temp ) )
                                   and
                                   ( Temp[ Dummy1 ] = ' ' )
                                 ) do
                            begin
                                inc( Dummy1 ) ; // Skip past all spaces
                            end ;
                            Dummy := Dummy1 ;
                            Temp := copy( Temp, 1, Dummy - 1 ) ;
                        end else
                        begin
                            New_Line ;
                            Dummy := 1 ;
                            Temp := '' ;
                            S := Edit( S, 8 ) ; // Strip leading spaces on new line
                        end ; // if( X = 0 )
(*
                    end else
                    begin
                        if( Dummy1 <= length( Temp ) ) then
                        begin
                            Dummy := Dummy1 + 1 ;
                        end ;
                        Temp := copy( Temp, 1, Dummy - 1 ) ;
*)
                    end ; // if( Dummy1 = 0 )

                    GetTextExtentPoint( DC, PAnsichar( Temp ), length( Temp ), Size ) ;
                    if( length( Temp ) > 0 ) then
                    begin
                        if( Superscript_Count > 0 ) then
                        begin
                            Size.cy := Size.cy + Size.cy div 3 ;
                        end ;

                        { Normalize baseline... }
//                        Size.cy := Size.cy - TM.tmDescent ;
                        Adjust := 0 ;
                        if( Height <> 0 ) then // Line height already determined
                        begin
                            // Adjust the top of the text by the difference between the text height and the line height
                            if( Superscript_Count <= 0 ) then
                            begin
                                Adjust := ( Height - Size.cy ) ;
                            end ;
(*
                            if( Size.cy > Height ) then
                            begin
                                Y := Y - ( Size.cy - Height ) ;
                            end else
                            if( Size.cy < Height ) then
                            begin
                                Y := Y + ( Height - Size.cy ) ;
                            end ;
*)
                        end ;
                        if( Size.cy {+ TM.tmDescent} > Height ) then
                        begin
                            Height := Size.cy {+ TM.tmDescent} ;
                        end ;

                        if( Draw ) then
                        begin
                            { Draw the text }
                            Write_Text( DC, This_Font, Color, X, Y + Adjust, Max_X, Temp ) ;
                        end ; { if( Draw ) }
                        X := X + Size.cx ;
                        Max_X := Max_X - Size.cx ;
                        if( ( Max_X <= 0 ) and Single_Line ) then
                        begin
                            Result := Result + Size.cx ;
                            break ;
                        end ;
                    end ; { if( length( Temp ) > 0 ) }
                    S := copy( S, Dummy, length( S ) ) ;
                    Result := Result + Size.cx ;
                end ; { while( length( S ) > 0 ) }
                if( ( List <> nil ) and ( not Draw ) ) then
                begin
                    List.Add( Height ) ;
                end ;
                if( ( List_W <> nil ) and ( not Draw ) ) then
                begin
                    List_W.Add( Result ) ;
                end ;

                // Restore original font
    {$IFNDEF NOVCL}
                selectobject( DC, Font.Handle ) ;
                SetTextColor( DC, ColorToRGB( Font.Color ) ) ;
    {$ENDIF}
            finally
                if( Result < Width ) then
                begin
                    Result := Width ;
                end ;
                Max_Height := Total_Height + Max_Height ;

                // Clean up...
                for Dummy := 0 to Fonts.Count - 1 do
                begin
                    TFont_Info( Fonts[ Dummy ] ).Free ;
                    Fonts[ Dummy ] := nil ;
                end ;
                Fonts.Free ;
                deleteobject( This_Font ) ;
            end ;
        finally
            Default_Font.Free ;
        end ;
    end ; { _Process_Text }


var List_H, List_W : TInteger_List ;
    Temp : integer ;

begin
    List_H := TInteger_List.Create ; // Line heights for each line
    List_W := TInteger_List.Create ; // Line widths for each line
    try
        // Dry run to determine width, wrapped lines, etc.
        Result := _Process_Text( DC, Font, S, False, X, Y, Max_X,
            Max_Height, List_H, List_W, Single_Line, Center ) ;

        // Now add together all the line heights
        Max_Height := 0 ;
        for Temp := 0 to List_H.Count - 1 do
        begin
            Max_Height := Max_Height + List_H[ Temp ] ;
        end ;
        if( Draw ) then
        begin
            _Process_Text( DC, Font, S, True, X, Y, Max_X, Temp, List_H,
                List_W, Single_Line, Center ) ;
        end ;
    finally
        List_H.Free ;
        List_W.Free ;
    end ;
end ; // THMTL.Process_Text
  {$ENDIF}
{$ENDIF}


{$IFDEF MSWINDOWS}
function Pixels_To_Size( X : integer ) : integer ;

var Loop : integer ;

begin
    for Loop := 1 to 7 do
    begin
        if( Size_To_Pixels( Loop ) > X ) then
        begin
            Result := Loop - 1 ;
            exit ;
        end ;
    end ;
    Result := 7 ;
end ;


function Size_To_Pixels( X : integer ) : integer ;

var DC : HDC ;

begin
    DC := GetDC( 0 ) ;
    try
        Result := GetDeviceCaps( DC, LOGPIXELSY ) ;
    finally
        releaseDC( 0, DC ) ;
    end ;
    if( X < 1 ) then
    begin
        X := 1 ;
    end ;
    if( X > 7 ) then
    begin
        X := 7 ;
    end ;
    case X of
        1 : Result := Result * 94 div 1000 ;
        2 : Result := Result * 125 div 1000 ;
        3 : Result := Result * 146 div 1000 ;
        4 : Result := Result * 167 div 1000 ;
        5 : Result := Result * 219 div 1000 ;
        6 : Result := Result * 292 div 1000 ;
        7 : Result := Result * 438 div 1000 ;
    end ;
    if( Result < 9 ) then
    begin
        Result := 9 ;
    end ;
end ;


function HTML_Text_Width( DC : HDC ; Font : TFont ; const S : Ansistring ) : integer ;

var Temp : integer ;

begin
    Result := Process_HTML_Text( DC, Font, S, False, 0, 0, 32767, Temp, True, False ) ;
end ;


function HTML_Text_Height( DC : HDC ; Font : TFont ; const S : Ansistring ) : integer ;

begin
    Process_HTML_Text( DC, Font, S, False, 0, 0, 32767, Result, True, False ) ;
end ;


function Process_HTML_Text( DC : HDC ; Font : TFont ; const S : Ansistring ;
     Draw : boolean ; X, Y, Max_X : integer ;
     var Max_Height : integer ; Single_Line, Center : boolean ) : integer ;

var H : THTML ;

begin
    H := THTML.Create ;
    try
        Result := H.Process_Text( DC, Font, S, Draw, X, Y, Max_X, Max_Height, Single_Line, Center ) ;
    finally
        H.Free ;
    end ;
end ;
{$ENDIF}



var Entities : array[ 0..255 ] of string ;

function From_HTML( X : Ansistring ; Options : longint ) : Ansistring ;

var Dummy, Index, Loop : integer ;
    Found : boolean ;
    Res, LTemp, S, Temp{, Work} : Ansistring ;

begin
    // Setup...
    Res := '' ;

    // Strip tags...
    Dummy := pos( '<', X ) ;
    while( Dummy > 0 ) do
    begin
        Loop := pos( '>', X ) ;
        if( Loop = 0 ) then
        begin
            Loop := length( X ) ;
        end ;
        if( Loop < Dummy ) then
        begin
            break ; // Not valid HTML, ignore tags
        end ;
        S := '' ;
        if( ( Options and 2 ) <> 0 ) then // Convert <br> to CRLF
        begin
            if( lowercase( copy( X, Dummy, 4 ) ) = '<br>' ) then
            begin
                S := CRLF ;
            end ;
        end ;
        if( ( Options and 4 ) <> 0 ) then // Convert <br> to CRLF
        begin
            if( lowercase( copy( X, Dummy, 2 ) ) = '<p' ) then
            begin
                S := CRLF ;
            end ;
        end ;
        X := copy( X, Low_String_Index, Dummy - Low_String_Index ) + S + copy( X, Loop + 1, length( X ) ) ;
        Dummy := pos( '<', X ) ;
    end ; // while( Dummy > 0 )

    // Process characters...
    while( length( X ) > 0 ) do
    begin
        Dummy := pos( '&', X + '&' ) ;
        Res := Res + copy( X, Low_String_Index, Dummy - Low_String_Index ) ;
        X := copy( X, Dummy, length( X ) ) ;
        Dummy := pos( ';', X ) ;
        if( Dummy = 0 ) then { No terminator - bad entity reference }
        begin
            if( ( Options and 1 ) <> 0 ) then { Keep bad entity values }
            begin
                Res := Res + X ;
            end ;
            X := '' ;
        end else
        if( Dummy < 4 ) then { "&;" or single-character entity }
        begin
            if( ( Options and 1 ) <> 0 ) then
            begin
                Res := Res + copy( X, 1, Dummy ) ;
            end ;
            X := copy( X, Dummy + 1, length( X ) ) ;
        end else
        begin
            Temp := copy( X, 2, Dummy - 2 ) ;
            X := copy( X, Dummy + 1, length( X ) ) ;
            if( ( copy( Temp, 1, 1 ) = '#' ) and ( length( Temp ) > 1 ) ) then
            begin
//                Work := copy( Temp, 2, length( Temp ) ) ;
                val( copy( Temp, 2, length( Temp ) ), Index, Dummy ) ;
                if( Dummy = 0 ) then { Valid number }
                begin
                    Res := Res + Ansichar( Index ) ;
                    continue ;
                end ;
            end ;
            Found := False ;
            for Loop := 0 to 255 do
            begin
                if( Entities[ Loop ] = Temp ) then
                begin
                    Res := Res + Ansichar( Loop ) ;
                    Found := True ;
                    break ;
                end ;
            end ;
            if( not Found ) then
            begin
                LTemp := lowercase( Temp ) ;
                for Loop := 0 to 255 do
                begin
                    if( Entities[ Loop ] = LTemp ) then
                    begin
                        Res := Res + Ansichar( Loop ) ;
                        Found := True ;
                        break ;
                    end ;
                end ;
            end ;
            if( not Found ) then
            begin
                if(
                    ( LTemp = 'spades' )
                    or
                    ( LTemp = 'clubs' )
                    or
                    ( LTemp = 'hearts' )
                    or
                    ( LTemp = 'diams' )
                    or
                    ( LTemp = 'oline' )
                    or
                    ( LTemp = 'larr' )
                    or
                    ( LTemp = 'rarr' )
                    or
                    ( LTemp = 'uarr' )
                    or
                    ( LTemp = 'darr' )
                  ) then
                begin
                    Found := True ;
                    // Cannot translate to a string
                end ;
            end ;
            if( ( not Found ) and ( ( Options and 1 ) <> 0 ) ) then
            begin
                Res := Res + '&' + Temp + ';' ;
            end ;
        end ; { if( Dummy = 0 )  }
    end ; { while( length( X ) > 0 )  }
    From_HTML := Res ;
end ; { From_HTML }


function To_HTML( const X : Ansistring ) : Ansistring ;

var C : byte ;
    Loop : integer ;
    S : Ansistring ;

begin
    S := '' ;
    for Loop := 1 to length( X ) do
    begin
        C := ord( X[ Loop ] ) ;
        if( length( Entities[ C ] ) > 1 ) then
        begin
            S := S + '&' + Entities[ C ] + ';' ;
        end else
        begin
            S := S + Entities[ C ] ;
        end ;
    end ;
    To_HTML := S ;
end ;


function To_HTML_Leave_Tags( const X : Ansistring ) : Ansistring ;

var C : byte ;
    Loop : integer ;
    S : Ansistring ;

begin
    S := '' ;
    for Loop := 1 to length( X ) do
    begin
        C := ord( X[ Loop ] ) ;
        if( ( X[ Loop ] = '<' ) or ( X[ Loop ] = '>' ) ) then
        begin
            S := S + X[ Loop ] ;
        end else
        if( length( Entities[ C ] ) > 1 ) then
        begin
            S := S + '&' + Entities[ C ] + ';' ;
        end else
        begin
            S := S + Entities[ C ] ;
        end ;
    end ;
    To_HTML_Leave_Tags := S ;
end ;


function Decode_Entities( X : Ansistring ; Options : integer ) : Ansistring ;

var Dummy, Index, Loop : integer ;
    Found : boolean ;
    Res, LTemp, Temp : Ansistring ;

begin
    // Setup...
    Res := '' ;

    // Process characters...
    while( length( X ) > 0 ) do
    begin
        Dummy := pos( '&', X + '&' ) ;
        Res := Res + copy( X, 1, Dummy - 1 ) ;
        X := copy( X, Dummy, length( X ) ) ;
        Dummy := pos( ';', X ) ;
        if( Dummy = 0 ) then { No terminator - bad entity reference }
        begin
            if( ( Options and 1 ) <> 0 ) then
            begin
                Res := Res + X ;
            end ;
            X := '' ;
        end else
        if( Dummy < 4 ) then { "&;" or single-character entity }
        begin
            if( ( Options and 1 ) <> 0 ) then
            begin
                Res := Res + copy( X, 1, Dummy ) ;
            end ;
            X := copy( X, Dummy + 1, length( X ) ) ;
        end else
        begin
            Temp := copy( X, 2, Dummy - 2 ) ;
            X := copy( X, Dummy + 1, length( X ) ) ;
            if( ( copy( Temp, 1, 1 ) = '#' ) and ( length( Temp ) > 1 ) ) then
            begin
//                Work := copy( Temp, 2, length( Temp ) ) ;
                val( copy( Temp, 2, length( Temp ) ), Index, Dummy ) ;
                if( Dummy = 0 ) then { Valid number }
                begin
                    Res := Res + Ansichar( Index ) ;
                    continue ;
                end ;
            end ;
            Found := False ;
            for Loop := 0 to 255 do
            begin
                if( Entities[ Loop ] = Temp ) then
                begin
                    Res := Res + Ansichar( Loop ) ;
                    Found := True ;
                    break ;
                end ;
            end ;
            if( not Found ) then
            begin
                LTemp := lowercase( Temp ) ;
                for Loop := 0 to 255 do
                begin
                    if( Entities[ Loop ] = LTemp ) then
                    begin
                        Res := Res + Ansichar( Loop ) ;
                        Found := True ;
                        break ;
                    end ;
                end ;
            end ;
            if( not Found ) then
            begin
                if(
                    ( LTemp = 'spades' )
                    or
                    ( LTemp = 'clubs' )
                    or
                    ( LTemp = 'hearts' )
                    or
                    ( LTemp = 'diams' )
                    or
                    ( LTemp = 'oline' )
                    or
                    ( LTemp = 'larr' )
                    or
                    ( LTemp = 'rarr' )
                    or
                    ( LTemp = 'uarr' )
                    or
                    ( LTemp = 'darr' )
                  ) then
                begin
                    Found := True ;
                    // Cannot translate to a string
                end ;
            end ;
            if( ( not Found ) and ( ( Options and 1 ) <> 0 ) ) then
            begin
                Res := Res + '&' + Temp + ';' ;
            end ;
        end ; { if( Dummy = 0 )  }
    end ; { while( length( X ) > 0 )  }
    Decode_Entities := Res ;
end ; { Decode_Entities }


procedure Initialize_Unit ;

var Loop : integer ;

begin
    // Note: Doesn't currently support certain entities which are UNICODE characters
    for Loop := 0 to 255 do
    begin
        Entities[ Loop ] := Ansichar( Loop ) ;
    end ;
    Entities[ 9 ] := '#TAB' ;
    Entities[ 10 ] := '#RS' ;
    Entities[ 13 ] := '#RE' ;
    Entities[ 38 ] := 'amp' ;
    Entities[ 60 ] := 'lt' ;
    Entities[ 62 ] := 'gt' ;
    Entities[ 128 ] := 'euro' ;
    Entities[ 130 ] := 'sbquo' ;
    Entities[ 131 ] := 'fnof' ;
    Entities[ 132 ] := 'dbquo' ;
    Entities[ 126 ] := 'tilde' ;
    Entities[ 133 ] := 'hellip' ;
    Entities[ 134 ] := 'dagger' ;
    Entities[ 135 ] := 'Dagger' ;
    Entities[ 136 ] := 'circ' ;
    Entities[ 137 ] := 'permil' ;
    Entities[ 138 ] := 'Scaron' ;
    Entities[ 139 ] := 'lsaquo' ;
    Entities[ 140 ] := 'OElig' ;
    Entities[ 142 ] := '#381' ;
    Entities[ 145 ] := 'lsquo' ;
    Entities[ 146 ] := 'rsquo' ;
    Entities[ 147 ] := 'ldquo' ; // 
    Entities[ 148 ] := 'rdquo' ;
    Entities[ 149 ] := 'bull' ;
    Entities[ 150 ] := 'ndash' ;
    Entities[ 151 ] := 'mdash' ;
    Entities[ 152 ] := 'tilde' ;
    Entities[ 153 ] := 'trade' ;
    Entities[ 154 ] := 'scaron' ;
    Entities[ 155 ] := 'rsaquo' ;
    Entities[ 156 ] := 'oelig' ;
    Entities[ 158 ] := '#382' ;
    Entities[ 159 ] := 'Yuml' ;
    Entities[ 160 ] := 'nbsp' ;
    Entities[ 161 ] := 'iexcl' ;
    Entities[ 162 ] := 'cent' ; // 
    Entities[ 163 ] := 'pound' ;
    Entities[ 164 ] := 'curren' ;
    Entities[ 165 ] := 'yen' ;
    Entities[ 166 ] := 'brvbar' ;
    Entities[ 167 ] := 'sect' ;
    Entities[ 168 ] := 'uml' ;
    Entities[ 169 ] := 'copy' ;
    Entities[ 170 ] := 'ordf' ;
    Entities[ 171 ] := 'laquo' ;
    Entities[ 172 ] := 'not' ;
    Entities[ 173 ] := 'shy' ;
    Entities[ 174 ] := 'reg' ;
    Entities[ 175 ] := 'macr' ;
    Entities[ 176 ] := 'deg' ;
    Entities[ 177 ] := 'plusmn' ;
    Entities[ 178 ] := 'sup2' ;
    Entities[ 179 ] := 'sup3' ;
    Entities[ 180 ] := 'acute' ;
    Entities[ 181 ] := 'micro' ;
    Entities[ 182 ] := 'para' ;
    Entities[ 183 ] := 'middot' ;
    Entities[ 184 ] := 'cedil' ;
    Entities[ 185 ] := 'supl' ;
    Entities[ 186 ] := 'ordm' ;
    Entities[ 187 ] := 'raquo' ;
    Entities[ 188 ] := 'frac14' ;
    Entities[ 189 ] := 'frac12' ;
    Entities[ 190 ] := 'frac34' ;
    Entities[ 191 ] := 'iquest' ;
    Entities[ 192 {ord( '' )} ] := 'Agrave' ;
    Entities[ 193 {ord( '' )} ] := 'Aacute' ;
    Entities[ 194 {ord( '' )} ] := 'Acirc' ;
    Entities[ 195 {ord( '' )} ] := 'Atilde' ;
    Entities[ 196 {ord( '' )} ] := 'Auml' ;
    Entities[ 197 {ord( '' )} ] := 'Aring' ;
    Entities[ 198 {ord( '' )} ] := 'AElig' ;
    Entities[ 199 {ord( '' )} ] := 'Ccedil' ;
    Entities[ 200 {ord( '' )} ] := 'Egrave' ;
    Entities[ 201 {ord( '' )} ] := 'Eacute' ;
    Entities[ 202 {ord( '' )} ] := 'Ecirc' ;
    Entities[ 203 {ord( '' )} ] := 'Euml' ;
    Entities[ 204 {ord( '' )} ] := 'Igrave' ;
    Entities[ 205 {ord( '' )} ] := 'Iacute' ;
    Entities[ 206 {ord( '' )} ] := 'Icirc' ;
    Entities[ 207 {ord( '' )} ] := 'Iuml' ;
    Entities[ 208 {ord( '' )} ] := 'ETH' ;
    Entities[ 209 {ord( '' )} ] := 'Ntilde' ;
    Entities[ 210 {ord( '' )} ] := 'Ograve' ;
    Entities[ 211 {ord( '' )} ] := 'Oacute' ;
    Entities[ 212 {ord( '' )} ] := 'Ocirc' ;
    Entities[ 213 {ord( '' )} ] := 'Otilde' ;
    Entities[ 214 {ord( '' )} ] := 'Ouml' ;
    Entities[ 215 ] := 'times' ;
    Entities[ 216 {ord( '' )} ] := 'Oslash' ;
    Entities[ 217 {ord( '' )} ] := 'Ugrave' ;
    Entities[ 218 {ord( '' )} ] := 'Uacute' ;
    Entities[ 219 {ord( '' )} ] := 'Ucirc' ;
    Entities[ 220 {ord( '' )} ] := 'Uuml' ;
    Entities[ 221 {ord( '' )} ] := 'Yacute' ;
    Entities[ 222 {ord( '' )} ] := 'thorn' ;
    Entities[ 223 {ord( '' )} ] := 'szlig' ;
    Entities[ 224 {ord( '' )} ] := 'agrave' ;
    Entities[ 225 {ord( '' )} ] := 'aacute' ;
    Entities[ 226 {ord( '' )} ] := 'acirc' ;
    Entities[ 227 {ord( '' )} ] := 'atilde' ;
    Entities[ 228 {ord( '' )} ] := 'auml' ;
    Entities[ 229 {ord( '' )} ] := 'aring' ;
    Entities[ 230 {ord( '' )} ] := 'aelig' ;
    Entities[ 231 {ord( '' )} ] := 'ccedil' ;
    Entities[ 232 {ord( '' )} ] := 'egrave' ;
    Entities[ 233 {ord( '' )} ] := 'eacute' ;
    Entities[ 234 {ord( '' )} ] := 'ecirc' ;
    Entities[ 235 {ord( '' )} ] := 'emul' ;
    Entities[ 236 {ord( '' )} ] := 'igrave' ;
    Entities[ 237 {ord( '' )} ] := 'iacute' ;
    Entities[ 238 {ord( '' )} ] := 'icirc' ;
    Entities[ 239 {ord( '' )} ] := 'iuml' ;
    Entities[ 240 ] := 'eth' ;
    Entities[ 241 {ord( '' )} ] := 'ntilde' ;
    Entities[ 242 {ord( '' )} ] := 'ograve' ;
    Entities[ 243 {ord( '' )} ] := 'oacute' ;
    Entities[ 244 {ord( '' )} ] := 'ocirc' ;
    Entities[ 245 {ord( '' )} ] := 'otilde' ;
    Entities[ 246 {ord( '' )} ] := 'ouml' ;
    Entities[ 247 ] := 'divide' ;
    Entities[ 248 {ord( '' )} ] := 'oslash' ;
    Entities[ 249 {ord( '' )} ] := 'ugrave' ;
    Entities[ 250 {ord( '' )} ] := 'uacute' ;
    Entities[ 251 {ord( '' )} ] := 'ucirc' ;
    Entities[ 252 {ord( '' )} ] := 'uuml' ;
    Entities[ 253 {ord( '' )} ] := 'yacute' ;
    Entities[ 254 {ord( '' )} ] := 'THORN' ;
    Entities[ 255 {ord( '' )} ] := 'yuml' ;
end ; // Initialize_Unit


{ TSHTML_Parser methods... }

{ Constructors and destructors... }

constructor TSHTML_Parser.Init ;

begin
    inherited Init ;

    Source.Init ;
    Preserve_Whitespace := False ;
end ;


destructor TSHTML_Parser.Done ;

begin
    Source.Done ;

    inherited Done ;
end ;


{ Internal utility routines... }

function TSHTML_Parser.EOF : boolean ;

begin
    EOF := ( Source.Length = 0 ) ;
end ;


function TSHTML_Parser.Get_Token : UStrings.PString ;

var Dummy : integer ;
    Res : PASCII_String ;
    Work : UStrings.PString ;

begin
    Dummy := Source.Instr( 1, Temp_String( '<' ) ) ;
    if( Dummy = 0 ) then
    begin
        Dummy := Source.Length + 1 ;
    end ;
    if( Dummy = 1 ) then
    begin
        Dummy := Source.Instr( 1, Temp_String( '>' ) ) ;
        if( Dummy = 0 ) then
        begin
            Dummy := Source.Length + 1 ;
        end else
        begin
            Dummy := Dummy + 1 ;
        end ;
    end ;

    { Extract token... }
    Res := new( PASCII_String, Init ) ;
    Work := Source.Copy( 1, Dummy - 1 ) ;
    Res.Assign( Work ) ;
    dispose( Work, Done ) ;
    Work := Source.copy( Dummy, Source.Length ) ;
    Source.Assign( Work ) ;
    dispose( Work, Done ) ;

    { Process EOL characters... }
    Dummy := Res.Instr( 1, Temp_String( CR ) ) ;
    while( Dummy > 0 ) do
    begin
        Res.At_Put( Dummy, 32 ) ;
        Dummy := Res.Instr( 1, Temp_String( CR ) ) ;
    end ;
    Dummy := Res.Instr( 1, Temp_String( LF ) ) ;
    while( Dummy > 0 ) do
    begin
        Res.At_Put( Dummy, 32 ) ;
        Dummy := Res.Instr( 1, Temp_String( LF ) ) ;
    end ;
    if( not Preserve_Whitespace ) then
    begin
        Work := Res.Edit( 16 ) ; { Multiple whitespace to single space }
        dispose( Res, Done ) ;
        Get_Token := Work ;
    end else
    begin
        Get_Token := Res ;
    end ;
end ; { TSHTML_Parser.Get_Token }


{ API... }

function TSHTML_Parser._Get_Section( S : Ansistring ; XML : boolean ) : UStrings.PString ;

var Dummy : integer ;
    Ending, Starting : Ansistring ;
    Nest_Count : integer ;
    Res : PASCII_String ;
    Temp : UStrings.PString ;
    Work : Ansistring ;

    function Match( Work, Ending : Ansistring ) : boolean ;

    begin
        if( copy( Work, 1, length( Ending ) ) = Ending ) then
        begin
            if(
                ( length( Work ) = length( Ending ) )
                or
                ( copy( Work, length( Ending ) + 1, 1 ) = ' ' )
                or
                ( copy( Work, length( Ending ) + 1, 1 ) = '>' )
              ) then
            begin
                Result := True ;
                exit ;
            end ;
        end ;
        Result := False ;
    end ;

begin
    { Normalize passed section name... }
    if( copy( S, 1, 1 ) = '<' ) then
    begin
        S := copy( S, 2, length( S ) ) ;
    end ;
    if( copy( S, length( S ), 1 ) = '>' ) then
    begin
        S := copy( S, 1, length( S ) - 1 ) ;
    end ;
    Dummy := pos( '=', S + '=' ) ;
    S := copy( S, 1, Dummy - 1 ) ;
    Dummy := pos( ' ', S + ' ' ) ;
    S := copy( S, 1, Dummy - 1 ) ;

    { Build ending tag to search for... }
    S := Edit( S, 512 ) ;
    if( XML ) then
    begin
        Ending := '<' + S + '/>' ;
    end else
    begin
        Ending := '</' + S ;
    end ;
    Starting := '<' + S ;

    { Setup }
    _Section_End := '' ;
    Nest_Count := 0 ;
    Res := new( PASCII_String, Init ) ;

    { Process section... }
    Temp := Token ;
    while( Temp.Length > 0 ) do
    begin
        Work := Edit( Temp.Text, 512 ) ;
        if( Match( Work, Ending ) ) then
        begin
            dec( Nest_Count ) ;
            if( Nest_Count < 0 ) then
            begin
                _Get_Section := Res ;
                _Section_End := Temp.Text ;
                dispose( Temp, Done ) ;
                exit ;
            end ;
        end ;
        Res.Append( Temp ) ;
        dispose( Temp, Done ) ;
        if( copy( Work, 1, 1 ) = '<' ) then
        begin
            Dummy := pos( '=', Work + '=' ) ;
            Work := copy( Work, 1, Dummy - 1 ) ;
            Dummy := pos( ' ', Work + ' ' ) ;
            Work := copy( Work, 1, Dummy - 1 ) ;
            if( copy( Work, length( Work ), 1 ) = '>' ) then
            begin
                Work := copy( Work, 1, length( Work ) - 1 ) ;
            end ;
            if( Work = Starting ) then
            begin
                inc( Nest_Count ) ;
            end ;
        end ;

        Temp := Token ;
    end ;
    _Get_Section := Res ; { Must have hit EOF before terminating tag }
    dispose( Temp, Done ) ;
end ; { TSHTML_Parser._Get_Section }


function TSHTML_Parser.Get_Section( const S : Ansistring ) : UStrings.PString ;

begin
    Result := _Get_Section( S, False ) ;
end ;


function TSHTML_Parser.Read_Next_Line : boolean ; { Read next line from source data }

var Dummy, Dummy1, Dummy2 : integer ;
    Check : Ansichar ;

begin
    Dummy1 := Source.Instr( 1, Temp_String( CR ) ) ;
    Dummy2 := Source.Instr( 1, Temp_String( LF ) ) ;
    if( ( Dummy1 = 0 ) and ( Dummy2 = 0 ) ) then // No match - get whole string
    begin
        Command.Assign( @Source ) ;
        Source.Set_Length( 0 ) ;
        exit ;
    end ;

    if( Dummy1 = 0 ) then
    begin
        Dummy := Dummy2 ;
    end else
    if( Dummy2 = 0 ) then
    begin
        Dummy := Dummy1 ;
    end else
    if( Dummy1 > Dummy2 ) then
    begin
        Dummy := Dummy2 ;
    end else
    begin
        Dummy := Dummy1 ;
    end ;
    if( Dummy = Dummy1 ) then
    begin
        Check := LF ;
    end else
    begin
        Check := CR ;
    end ;

    Command.Assign( Source.Copy( 1, Dummy - 1 ) ) ;
    if( Source.At( Dummy + 1 ) = ord( Check ) ) then
    begin
        inc( Dummy ) ; // Skip other line end
    end ;
    Source.Assign( Source.Copy( Dummy + 1, Source.Length ) ) ;
end ; { TSHTML_Parser.Read_Next_Line }


function TSHTML_Parser.Section_End : Ansistring ;

begin
    Result := _Section_End ;
end ;



{ TSXML_Parser methods... }

{ API... }

function TSXML_Parser.Get_Section( const S : Ansistring ) : UStrings.PString ;

var Dummy : integer ;
    Work : Ansistring ;

begin
    Work := Ansistring( S ) ;
    Dummy := pos( ' ', Work + ' ' ) ;
    Work := copy( Work, 1, Dummy - 1 ) ;
    if( copy( Work, length( Work ), 1 ) <> '>' ) then
    begin
        Work := Work + '>' ;
    end ;
    Result := _Get_Section( Work, True ) ;
end ;


function TSXML_Parser.Get_Section_String( const S : Ansistring ) : Ansistring ;

var Work : UStrings.PString ;

begin
    Work := Get_Section( S ) ;
    Get_Section_String := Work^.Text ;
    dispose( Work, Done ) ;
end ;


function TSXML_Parser.Get_Token_String : Ansistring ;

var Work : UStrings.PString ;

begin
    Work := Token ;
    Get_Token_String := Work^.Text ;
    dispose( Work, Done ) ;
end ;


{$IFDEF DYNAMIC_OBJECTS}
// THTML_Parser methods...

{ Constructors and destructors... }

constructor THTML_Parser.Create ;

begin
    inherited Create ;

    dispose( _Parser, Done ) ;
    _Parser := new( PSHTML_Parser, Init ) ;
end ;


function THTML_Parser.Get_Source : Ansistring ;

begin
    Result := PSHTML_Parser( _Parser )^.Source.Text ;
end ;


procedure THTML_Parser.Set_Source( const S : Ansistring ) ;

begin
    PSHTML_Parser( _Parser )^.Source.Set_String( S ) ;
end ;


function THTML_Parser.Get_Preserve_Whitespace : boolean ;

begin
    Result := PSHTML_Parser( _Parser )^.Preserve_Whitespace ;
end ;


procedure THTML_Parser.Set_Preserve_Whitespace( Value : boolean ) ;

begin
    PSHTML_Parser( _Parser )^.Preserve_Whitespace := Value ;
end ;


function THTML_Parser.Section_End : Ansistring ;

begin
    Result := PSHTML_Parser( _Parser )^.Section_End ;
end ;


{ Internal utility routines... }

function THTML_Parser._Get_Section( const S : Ansistring ; XML : boolean ) : Ansistring ;

var Temp : UStrings.PString ;

begin
    Temp := PSHTML_Parser( _Parser )^._Get_Section( S, XML ) ;
    Result := Temp.Text ;
    dispose( Temp, Done ) ;
end ;


{ API.. }

function THTML_Parser.EOF : boolean ;

begin
    EOF := PSHTML_Parser( _Parser )^.EOF ;
end ;


function THTML_Parser.Get_Section( const S : Ansistring ) : Ansistring ;

var Temp : UStrings.PString ;

begin
    Temp := PSHTML_Parser( _Parser )^.Get_Section( S ) ;
    Result := Temp.Text ;
    dispose( Temp, Done ) ;
end ;



// THTML_Tag methods...

constructor THTML_Tag.Create( S : Ansistring ) ;

var N, V : Ansistring ;

begin
    inherited Create ;

    Names := TStringList.Create ;
    Values := TStringList.Create ;

    // Parse tag...
    if( copy( S, 1, 1 ) = '<' ) then
    begin
        S := copy( S, 2, length( S ) ) ;
    end ;
    if( copy( S, length( S ), 1 ) = '>' ) then
    begin
        S := copy( S, 1, length( S ) - 1 ) ;
    end ;
    Name := Parse_Parameter( ' ', S ) ;

    while( length( S ) > 0 ) do
    begin
        V := Parse_Parameter( ' ', S ) ;
        N := Parse_Parameter( '=', V ) ;
        Trim_Quotes( V ) ;
        Names.Add( N ) ;
        Values.Add( V ) ;
    end ;
end ;


destructor THTML_Tag.Destroy ;

begin
    Names.Free ;
    Names := nil ;
    Values.Free ;
    Values := nil ;

    inherited Destroy ;
end ;


function THTML_Tag.Attribute_Name( Index : integer ) : Ansistring ;

begin
    Result := Names[ Index ] ;
end ;


function THTML_Tag.Attribute_Value( Index : integer ) : Ansistring ;

begin
    Result := Values[ Index ] ;
end ;


function THTML_Tag.Count : integer ;

begin
    Result := Names.Count ;
end ;



{ TXML_Parser methods... }

{ Constructors and destructors... }

constructor TXML_Parser.Create ;

begin
    inherited Create ;

    _Parser := THTML_Parser.Create ;
end ;


destructor TXML_Parser.Destroy ;

begin
    _Parser.Free ;
    _Parser := nil ;

    inherited Destroy ;
end ;


{ API... }

function TXML_Parser.Get_Current_Line : longint ;

begin
    Result := _Parser.Get_Current_Line ;
end ;


function TXML_Parser.Get_Non_Terminals : PAnsiChar ;

begin
    Temp := _Parser.Non_Terminals ;
    Result := PAnsichar( Temp ) ;
end ;


procedure TXML_Parser.Set_Non_Terminals( S : PAnsiChar ) ;

begin
    _Parser.Non_Terminals := Ansistring( S ) ;
end ;


function TXML_Parser.Get_Preserve_Whitespace : boolean ;

begin
    Result := _Parser.Preserve_Whitespace ;
end ;


procedure TXML_Parser.Set_Preserve_Whitespace( Value : boolean ) ;

begin
    _Parser.Preserve_Whitespace := Value ;
end ;


function TXML_Parser.Get_Source : PAnsiChar ;

begin
    Temp := _Parser.Get_Source ;
    Result := PAnsichar( Temp ) ;
end ;


procedure TXML_Parser.Set_Source( S : PAnsiChar ) ;

begin
    _Parser.Set_Source( S ) ;
end ;


function TXML_Parser.Get_Token_Line : integer ;

begin
    Result := _Parser.Token_Line ;
end ;


procedure TXML_Parser.Set_Token_Line( V : integer ) ;

begin
    _Parser.Token_Line := V ;
end ;


procedure TXML_Parser.Add_Special_Token( S : PAnsiChar ) ;

begin
    _Parser.Add_Special_Token( Ansistring( S ) ) ;
end ;


function TXML_Parser.EOF : boolean ;

begin
    Result := _Parser.EOF ;
end ;


function TXML_Parser.Get_Section( S : PAnsiChar ) : PAnsiChar ;

var Dummy : integer ;
    Work : Ansistring ;

begin
    Work := Ansistring( S ) ;
    Dummy := pos( ' ', Work + ' ' ) ;
    Work := copy( Work, 1, Dummy - 1 ) ;
    if( copy( Work, length( Work ), 1 ) <> '>' ) then
    begin
        Work := Work + '>' ;
    end ;
    Temp := _Parser._Get_Section( Work, True ) ;
    Result := PAnsichar( Temp ) ;
end ;


function TXML_Parser.Get_Token : PAnsiChar ;

begin
    Temp := _Parser.Get_Token ;
    Result := PAnsichar( Temp ) ;
end ;


function TXML_Parser.Previous_Token_Line : Integer ;

begin
    Result := _Parser.Previous_Token_Line ;
end ;


procedure TXML_Parser.Push_Token_Line ;

begin
    _Parser.Push_Token_Line ;
end ;


function TXML_Parser.Read_Next_Line : boolean ;

begin
    Result := _Parser.Read_Next_Line ;
end ;


function TXML_Parser.Grab_Line : PAnsiChar ;

begin
    Temp := _Parser.Grab_Line ;
    Result := PAnsichar( Temp ) ;
end ;


function TXML_Parser.Is_Class( N : PAnsiChar ) : boolean ;

var _N : Ansistring ;

begin
    _N := lowercase( Ansistring( N ) ) ;
    Result := ( _N = 'txml_parser' ) or ( _N = 'tcommon_com_interface' ) ;
end ;


function TXML_Parser.Last_Error : TUnified_Exception ;

begin
    Result := nil ;////~~~_Parser.Last_Error ;
end ;


procedure TXML_Parser.Put_Token( S : PAnsiChar ) ;

begin
    _Parser.Put_Token( Ansistring( S ) ) ;
end ;


function TXML_Parser.Token : PAnsiChar ;

begin
    Temp := _Parser.Token ;
    Result := PAnsichar( Temp ) ;
end ;


function TXML_Parser.Token_EOL : boolean ;

begin
    Result := _Parser.Token_EOL ;
end ;



{ TXML_Tag methods... }

{ Constructors and destructors... }

constructor TXML_Tag.Create( const S : Ansistring ) ;

begin
    inherited Create ;

    _Tag := THTML_Tag.Create( S ) ;
end ;


destructor TXML_Tag.Destroy ;

begin
    _Tag.Free ;
    _Tag := nil ;

    inherited Destroy ;
end ;


{ API... }

function TXML_Tag.Attribute_Name( Index : integer ) : PAnsiChar ;

begin
    Temp := _Tag.Attribute_Name( Index ) ;
    Result := PAnsichar( Temp ) ;
end ;


function TXML_Tag.Attribute_Value( Index : integer ) : PAnsiChar ;

begin
    Temp := _Tag.Attribute_Value( Index ) ;
    Result := PAnsichar( Temp ) ;
end ;


function TXML_Tag.Count : integer ;

begin
    Result := _Tag.Count ;
end ;


function TXML_Tag.Get_Name : PAnsiChar ;

begin
    Result := PAnsichar( _Tag.Name ) ;
end ;


procedure TXML_Tag.Set_Name( P : PAnsiChar ) ;

begin
    _Tag.Name := Ansistring( P ) ;
end ;

{$ENDIF}


function Extract_Tag_ID( const S : Ansistring ) : Ansistring ;

var Tag : THTML_Tag ;

begin
    Tag := THTML_Tag.Create( S ) ;
    try
        Result := Tag.Name ;
    finally
        Tag.Free ;
    end ;
end ;


function Remove_Tags( const S, Tag : string ) : string ;

var P, PE : integer ;

begin
    Result := S ;
    P := pos( '<' + Tag, Result ) ;
    while( P >= Low_String_Index ) do
    begin
        PE := posEx( '>', Result, P + 1 ) ;
        if( PE < Low_String_Index ) then
        begin
            exit ;
        end ;
        Result := copy( Result, Low_String_Index, P - Low_String_Index ) + copy( Result, PE + Low_String_Index, length( Result ) ) ;
        P := pos( '<' + Tag, Result ) ;
    end ;
end ; // Remove_Tags


procedure Search_HTML( Sub, Source : string ; var Starting, Ending : integer ;
    Options : integer ; Start : integer = -1 ) ;

var C : Char ;
    SubPos, SourcePos : integer ;
    Temp : string ;

begin
    // Setup...
{$IFDEF MOBILE}
    Ending := Low( Source ) - 1 ; // Assume failure
{$ELSE}
    Ending := 0 ;
{$ENDIF}
    Starting := Ending + 1 ;
    SubPos := Starting ;
    SourcePos := SubPos ;
    if( Sub = '' ) then // Null substring
    begin
        Starting := Ending ;
        exit ;
    end ;
    if( Start > Starting ) then
    begin
        Starting := Start ;
        SourcePos := Starting ;
    end ;

    while( SourcePos <= High_String_Index( Source ) ) do // Until we hit the end of the search string
    begin
        C := Source[ SourcePos ] ;
        if( C = '<' ) then // Opening of tag
        begin
            while( ( Source[ SourcePos ] <> '>' ) and ( SourcePos < High_String_Index( Source ) ) ) do
            begin
                inc( SourcePos ) ;
            end ;
            inc( SourcePos ) ;
            continue ;
        end ;
        if( C = '&' ) then
        begin
            Temp := '' ;
            while( ( Source[ SourcePos ] <> ';' ) and ( SourcePos < High_String_Index( Source ) ) ) do
            begin
                Temp := Temp + Source[ SourcePos ] ;
                inc( SourcePos ) ;
            end ;
            Temp := Decode_Entities( Temp + ';', Options ) ;
            if( length( Temp ) > 1 ) then
            begin
                SourcePos := SourcePos - length( Temp ) + 1 ;
            end else
            if( Temp = '' ) then
            begin
                inc( SourcePos ) ;
                continue ;
            end else
            begin
                C := Temp[ 1 ] ;
            end ;
        end ; // if( C = '&' )
        if( C <> Sub[ SubPos ] ) then // Mismatch
        begin
            SubPos := Low_String_Index ;
            Starting := SourcePos + 1 ;
        end else
        begin
            inc( SubPos ) ;
            if( SubPos > High_String_Index( Sub ) ) then // Match completed
            begin
                Ending := SourcePos ;
                if( copy( Source, Starting, 2 ) = '</' ) then // End of previous tag section
                begin
                    while( Source[ Starting ] <> '>' ) do
                    begin
                        inc( Starting ) ;
                    end ;
                    inc( Starting ) ;
                end ;
                exit ;
            end ;
        end ;
        inc( SourcePos ) ;
    end ; // while( SourcePos <= High( Source ) )
    Starting := Ending ; // Search failed
end ; // Search_HTML


{$IFDEF CC_Regression_Test}
procedure Unit_Test ;

begin
    // Test Search_HTML...
    Search_HTML( 'Alan', 'ABCD', s, e, 0 );
    if( s <> 0 ) then
    begin
    end ;
    Search_HTML( 'Alan', 'Alan', s, e, 0 );
    if( ( s <> 1 ) or ( e <> 4 ) ) then
    begin
    end ;
    Search_HTML( 'Alan', '', s, e, 0 );
    if( s <> 0 ) then
    begin
    end ;
    Search_HTML( '', 'ABCD', s, e, 0 );
    if( s <> 0 ) then
    begin
    end ;
    Search_HTML( 'Alan', '<p>Alan</p>', s, e, 0 );
    if( ( s <> 1 ) and ( e <> 7 ) ) then
    begin
    end ;
    Search_HTML( 'Alan', 'XYZ <p>Al<i>a</i>n</p>', s, e, 0 );
    if( ( s <> 5 ) and ( e <> 18 ) ) then
    begin
    end ;
    Search_HTML( '<', '<p>Alan<hr>&lt;</p>', s, e, 0 );
    if( ( s <> 8 ) and ( e <> 15 ) ) then
    begin
    end ;
    Search_HTML( '<', '<p>Alan</p>&lt;', s, e, 0 );
    if( ( s <> 12 ) and ( e <> 15 ) ) then
    begin
    end ;
    Search_HTML( 'Alan', '<p>Alan</p>&lt;Alan', s, e, 0, 10 );
    if( ( s <> 16 ) and ( e <> 19 ) ) then
    begin
    end ;
    Search_HTML( 'Alan', '<p>Alan</p>&lt;Alan', s, e, 0, 100 );
    if( s <> 0 ) then
    begin
    end ;

    {~~~}
end ;
{$ENDIF}


begin
    Initialize_Unit ;
end.
