{
        File Name    : Heap_Utils
        Program Name : Subroutine Library
        Package Name : Subroutine Library
        Purpose      : Utility classes for Stores
        Institution  : Conroy & Conroy Co.
        Date Written : 19-Jan-2017
        Written By   : Alan Conroy
        Version      : 1.0

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          This unit contains utility classes for use with heaps.
}

unit Heap_Utils ;

{$I EDEFINES.INC}

interface

uses // C&C...
     Allocation_Cluster_Manager, // TCOM_Allocation_Cluster_Manager64
     ComInter, // TCommon_COM_Object
     TypeDefs, // pInt64_Array
     _Types, // AnsiChar
     _UE, // TUnified_Exception

     // UOS...
     UOS_File_Heap, // TUOS_File_Heap
     UOS_Strings ; // TUOS_String

type TPrefix = packed array[ 0..2 ] of Ansichar ;
     TStore_String_Header = packed record
                               Prefix : TPrefix ; // "STR"
                               Version : byte ;
                               Length : longword ;
                               Data : cardinal ;
                            end ;
     TStore_List_Header = packed record
                              Prefix : TPrefix ; // "LIS" or "FLI"
                              Version : byte ;
                              Delta : longint ;
                              Capacity : longint ;
                              Max : longint ;
                              Data : cardinal ;
                           end ;
     TStore_String_Header64 = packed record
                               Prefix : TPrefix ; // "STR"
                               Version : byte ;
                               Length : longword ;
                               Flags : longint ; // reserved
                               RefCount : longint ; // reserved
                               Data : int64 ;
                            end ;
     TStore_List_Header64 = packed record
                              Prefix : TPrefix ; // "LIS" or "FLI"
                              Version : byte ;
                              Delta : longint ;
                              Capacity : longint ;
                              Max : longint ;
                              Data : int64 ;
                           end ;

type TStore_Object = class( TCommon_COM_Interface )
                         public // Constructors and destructors...
                             destructor Destroy ; override ;

                         protected // Instance data...
                             _Address : int64 ;
                             _Store : TUOS_File_Heap ;

                         public // API...
                             function Valid_Header : boolean ;
                                 virtual ; abstract ;

                         protected // Property handlers...
                             function Get_Address : int64 ; virtual ;
                             procedure Set_Address( _Value : int64 ) ;
                                 virtual ; abstract ;
                             procedure Set_Store( _Value : TUOS_File_Heap ) ;

                         public // Properties...
                             property Address : int64 // Header address
                                 read Get_Address
                                 write Set_Address ;
                             property Store : TUOS_File_Heap
                                 read _Store
                                 write Set_Store ;
                      end ; // TStore_Object

     TStore_String = class( TStore_Object )
                         private // Instance data...
                             Header : TStore_String_Header ;
                             S : Ansistring ; // Cached string from store

                         public // Overrides...
                             function Is_Class( Name : PAnsiChar ) : boolean ;
                                 override ; stdcall ;

                         protected // Property handlers...
                             procedure Set_Address( _Value : int64 ) ; override ;

                         public
                             function Get_Value : Ansistring ;
                             procedure Set_Value( _Value : Ansistring ) ;

                         public // Override...
                             function Valid_Header : boolean ; override ;

                         public // API...
                             procedure Delete ; virtual ;

                         public // Properties...
                             property Value : Ansistring
                                 read Get_Value
                                 write Set_Value ;
                     end ; // TStore_String

     TStore_List = class( TStore_Object )
                        public // Constructors and destructors...
                            destructor Destroy ; override ;

                        protected // Instance data...
                            ACM : TCOM_Allocation_Cluster_Manager64 ;
                            Buf : pInt64_Array ;
                            Header : TStore_List_Header ;
                            _Last_Error : TUnified_Exception ;

                        public // Property handlers...
                            function Get_Item( Index : integer ) : int64 ;
                            procedure Set_Item( Index : integer ; Value : int64 ) ;
                            procedure Set_Last_Error( E : TUnified_Exception ) ;

                        public // Override...
                            function Is_Class( Name : PAnsiChar ) : boolean ;
                                override ; stdcall ;
                            procedure Set_Address( _Value : int64 ) ; override ;
                            function Valid_Header : boolean ; override ;

                        public // API...
                            function Add( _Value : int64 ) : longint ; virtual ;
                            function Get_Capacity : longint ; virtual ;
                            procedure Set_Capacity( _Value : longint ) ; virtual ;
                            function Get_Count : longint ; virtual ;
                            procedure Set_Count( _Value : longint ) ; virtual ;
                            procedure Delete ; virtual ;
                            procedure Insert( Index : longint ; _Value : int64 ) ;

                        public // Properties...
                            property Capacity : longint
                                read Get_Capacity
                                write Set_Capacity ;
                            property Count : longint
                                read Get_Count
                                write Set_Count ;
                            property Items[ Index : integer ] : int64
                                read Get_Item
                                write Set_Item ;
                                default ;
                    end ; // TStore_List

     TStore_Fixed_List = class( TStore_Object )
                            public // Constructors and destructors...
                                destructor Destroy ; override ;

                            protected // Instance data...
                                Buf : pInt64_Array ;
                                _Last_Error : TUnified_Exception ;

                            public // Public for some optimized code
                                Header : TStore_List_Header ;

                            public // Property handlers...
                                function Get_Item( Index : integer ) : int64 ;
                                procedure Set_Item( Index : integer ; Value : int64 ) ;
                                procedure Set_Last_Error( E : TUnified_Exception ) ;

                            public // Override...
                                function Is_Class( Name : PAnsiChar ) : boolean ;
                                    override ; stdcall ;
                                procedure Set_Address( _Value : int64 ) ; override ;
                                function Valid_Header : boolean ; override ;

                            public // API...
                                function Add( _Value : int64 ) : longint ; virtual ;
                                function Get_Capacity : longint ; virtual ;
                                procedure Set_Capacity( _Value : longint ) ; virtual ;
                                function Get_Count : longint ; virtual ;
                                procedure Set_Count( _Value : longint ) ; virtual ;
                                procedure Delete ; virtual ;
                                procedure Insert( Index : longint ; _Value : int64 ) ;

                            public // Properties...
                                property Capacity : longint
                                    read Get_Capacity
                                    write Set_Capacity ;
                                property Count : longint
                                    read Get_Count
                                    write Set_Count ;
                                property Items[ Index : integer ] : int64
                                    read Get_Item
                                    write Set_Item ;
                                    default ;
                        end ; // TStore_Fixed_List

     TStore_String_List = class( TStore_List )
                              protected
                                  _Format : longint ;

                              public
                                  function Get_Item( Index : integer ) : Ansistring ;
                                  procedure Set_Item( Index : integer ;
                                      Value : Ansistring ) ;
                                  function Get_Format : longint ;
                                  procedure Set_Format( Value : longint ) ;

                              public // Override...
                                  function Is_Class( Name : PAnsiChar ) : boolean ;
                                      override ; stdcall ;
                                  function Valid_Header : boolean ; override ;

                              public // API...
                                  function Add( _Value : Ansistring ) : longint ;
                                      reintroduce ;
                                  procedure Delete ; override ;
                                  procedure Set_Count( _Value : longint ) ;
                                      override ;
                                  function IndexOf( _Value : Ansistring ;
                                      CI : boolean ) : longint ;

                              public // Properties...
                                  property Format : longint
                                      read Get_Format
                                      write Set_Format ;
                                  property Items[ Index : integer ] : Ansistring
                                      read Get_Item
                                      write Set_Item ;
                                      default ;
                          end ; // TStore_String_List

function Get_Store_String( Store : TUOS_File_Heap ; Address : int64 ) : TStore_String ;
procedure Delete_Store_String( Store : TUOS_File_Heap ; Address : int64  ) ;
function Create_Store_String( Store : TUOS_File_Heap ; S : Ansistring ) : TStore_String ;
function Get_Store_List( Store : TUOS_File_Heap ; Address : int64 ) : TStore_List ;
procedure Delete_Store_List( Store : TUOS_File_Heap ; Address : int64 ) ;
function Create_Store_List( Store : TUOS_File_Heap ; _Delta : integer ) : TStore_List ;
function Get_Store_Fixed_List( Store : TUOS_File_Heap ; Address : int64 ) : TStore_Fixed_List ;
procedure Delete_Store_Fixed_List( Store : TUOS_File_Heap ; Address : int64 ) ;
function Create_Store_Fixed_List( Store : TUOS_File_Heap ; _Delta : integer ) : TStore_Fixed_List ;
function Get_Store_String_List( Store : TUOS_File_Heap ; Address : int64 ;
    Format : longint = SF_UTF8 ) : TStore_String_List ;
procedure Delete_Store_String_List( Store : TUOS_File_Heap ; Address : int64 ) ;
function Create_Store_String_List( Store : TUOS_File_Heap ; _Delta : integer ) : TStore_String_List ;

const Store_String_Facility = 145 ;
      Store_StringErr_Success = 0 ;
      Store_StringErr_Invalid_Object = 1 ;

const Store_List_Facility = 146 ;
      Store_ListErr_Success = 0 ;
      Store_ListErr_Invalid_Object = 1 ;
      Store_ListErr_Index_Error = 2 ;

const Store_String_List_Facility = 147 ;
      Store_String_ListErr_Success = 0 ;
      Store_String_ListErr_Invalid_Object = 1 ;
      Store_String_ListErr_Index_Error = 2 ;
      Store_String_ListErr_Creation_Error = 3 ;


implementation

uses // Pascal...
     sysutils, // lowercase

     // C&C...
     CommonUt, // Low_String_Index
     UE ; // Create_Simple_Exception


const String_Prefix : TPrefix = ( 'S', 'T', 'R' ) ;
const List_Prefix : TPrefix = ( 'L', 'I', 'S' ) ;
const Fixed_List_Prefix : TPrefix = ( 'F', 'L', 'I' ) ;
const String_List_Prefix : TPrefix = ( 'S', 'L', 'I' ) ;


// TStore_Object methods...

// Constructors and destructors...

destructor TStore_Object.Destroy ;

begin
    Store := nil ;

    inherited Destroy ;
end ;


// API...

procedure TStore_Object.Set_Store( _Value : TUOS_File_Heap ) ;

begin
    if( _Value <> nil ) then
    begin
        _Value.Attach ;
    end ;
    if( _Store <> nil ) then
    begin
        _Store.Detach ;
    end ;
    _Store := _Value ;
end ;


function TStore_Object.Get_Address : int64 ;

begin
    Result := _Address ;
end ;



// TStore_String methods...

// Overrides...

function TStore_String.Is_Class( Name : PAnsiChar ) : boolean ;

var _N : Ansistring ;

begin
    _N := lowercase( Ansistring( Name ) ) ;
    Result := _N = 'tstore_string' ;
end ;


// Property handlers...

procedure TStore_String.Set_Address( _Value : int64 ) ;

var
{$IFDEF MOBILE}
    A : TBytes ;
    ByteString : RawByteString ;
    LEncoding, DestEncoding : TEncoding ;
    LOffset : Integer ;
    Loop : integer ;
    Success : boolean ;
    Work : string ;
{$ENDIF}
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( _Store <> nil ) then
    begin
        _Store.Read_Data( Header, _Value, sizeof( Header ), UEC ) ;
        if( UEC <> nil ) then
        begin
            Set_Last_Error( UEC ) ; { UNTESTED }
            exit ;
        end ;
        if( not Valid_Header ) then // Unknown version
        begin
            exit ;
        end ;
(*
        setlength( S, Header.Length ) ;
        if( Header.Length > 0 ) then
        begin
            Store.Read_Data( PAnsiChar( S )[ 0 ], Header.Data, Header.Length, UEC ) ;
            Set_Last_Error( UEC ) ;
        end ;
*)
{$IFDEF MOBILE}
        S := '' ;
        if( Header.Length > 0 ) then
        begin
            setlength( A, Header.Length ) ;
            Store.Read_Data( A[ 0 ], Header.Data, Header.Length, UEC ) ;
            Set_Last_Error( UEC ) ;

            // Convert buffer to string...
            Success := False ;
            try
                LEncoding := nil ;
                LOffset := TEncoding.GetBufferEncoding( A, LEncoding ) ;
                DestEncoding := TEncoding.ASCII ;
                A := LEncoding.Convert( LEncoding, DestEncoding, A,
                  LOffset, Length( A ) - LOffset ) ;
                LOffset := TEncoding.GetBufferEncoding( A, DestEncoding ) ;
                Work := DestEncoding.GetString( A, LOffset, Length( A ) - LOffset ) ;
                ByteString := AnsiToUtf8( Work ) ;
                S := UTF8ToString( ByteString ) ;
                Success := True ;
            except
            end ;
            if( not Success ) then
            begin
                for Loop := 0 to Header.Length - 1 do
                begin
                    S := S + chr( A[ Loop ] ) ;
                end ;
            end ;
        end ;
{$ELSE}
        setlength( S, Header.Length ) ;
        if( Header.Length > 0 ) then
        begin
            Store.Read_Data( PAnsiChar( S )[ 0 ], Header.Data, Header.Length, UEC ) ;
            Set_Last_Error( UEC ) ;
        end ;
{$ENDIF}
    end ;
    _Address := _Value ;
end ; // TStore_String.Set_Address


function TStore_String.Get_Value : Ansistring ;

begin
    Result := S ;
end ;


procedure TStore_String.Set_Value( _Value : Ansistring ) ;

var A : array of byte ;
    I : int64 ;
    Loop, Offset : Integer ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( _Value <> S ) then
    begin
        if( _Store <> nil ) then
        begin
            // Reallocate the storage for the value...
            I := Header.Data ;
            if( Header.Data = 0 ) then // Not already allocated
            begin
                if( length( _Value ) > 0 ) then // Non-null string
                begin
                    I := _Store.getmem( length( _Value ) ) ;
                    if( I = 0 ) then // Failure
                    begin
                        Set_Last_Error( _Store.Last_Error ) ; { UNTESTED }
                        exit ;
                    end ;
                end ;
            end else
            begin
                if( length( _Value ) = 0 ) then
                begin
                    I := 0 ;
                    _Store.freemem( Header.Data ) ;
                end else
                begin
                    I := _Store.Reallocmem( Header.Data, length( _Value ) ) ;
                    if( I = 0 ) then // Failure
                    begin
                        Set_Last_Error( _Store.Last_Error ) ; { UNTESTED }
                        exit ;
                    end ;
                end ;
            end ; // if( Header.Data = 0 )

            // Update the header...
            if( ( Header.Data <> I ) or ( Header.Length <> length( _Value ) ) ) then
            begin
                Header.Data := I ;
                Header.Length := length( _Value ) ;
                _Store.Write_Data( Header, Address, sizeof( Header ), UEC ) ;
                if( UEC <> nil ) then
                begin
                    Set_Last_Error( UEC ) ; { UNTESTED }
                    exit ;
                end ;
            end ; // if

            // Write the new value...
            if( Header.Data <> 0 ) then
            begin
(*
                _Store.Write_Data( PAnsiChar( _Value )[ 0 ], Header.Data, Header.Length, UEC ) ;
*)
                setlength( A, Header.Length ) ;
                Offset := 0 ;
                for Loop := Low_String_Index to High_String_Index( _Value ) do
                begin
                    A[ Offset ] := ord( _Value[ Loop ] ) ;
                    inc( Offset ) ;
                end ;
                _Store.Write_Data( A[ 0 ], Header.Data, Header.Length, UEC ) ;
                if( UEC <> nil ) then
                begin
                    Set_Last_Error( UEC ) ; { UNTESTED }
                    exit ;
                end ;
            end ;
        end ; // if( _Store <> nil )
        S := _Value ;
    end ; // if( _Value <> S )
end ; // TStore_String.Set_Value


// Override...

function TStore_String.Valid_Header : boolean ;

begin
    Result := False ;
    if( Header.Version >= 10 ) then // Unknown version
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_Facility, 1,
            Store_StringErr_Invalid_Object, UE_Error, 'Invalid object', '' ) ) ; { UNTESTED }
        exit ;
    end ;
    if( Header.Prefix <> String_Prefix ) then // Not a string header
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_Facility, 1,
            Store_StringErr_Invalid_Object, UE_Error, 'Invalid object', '' ) ) ;
        exit ;
    end ;
    if( Header.Data > _Store.Max_Storage ) then // Invalid pointer
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_Facility, 1,
            Store_StringErr_Invalid_Object, UE_Error, 'Invalid object', '' ) ) ; { UNTESTED }
        exit ;
    end ;
    Result := True ;
end ;


// API...

procedure TStore_String.Delete ;

var H : TStore_String_Header ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( _Store <> nil ) then
    begin
        fillchar( H, sizeof( H ), 0 ) ;
        _Store.Write_Data( H, Address, sizeof( H ), UEC ) ;
        _Store.freemem( Header.Data ) ;
        _Store.freemem( Address ) ;
    end ;
    Free ;
end ;



// TStore_List methods...

// Constructors and destructors...

destructor TStore_List.Destroy ;

begin
    if( ACM <> nil ) then
    begin
        ACM.Detach ;
        ACM := nil ;
    end ;
    freemem( Buf ) ;
    Buf := nil ;

    inherited Destroy ;
end ;


// Property handlers...

function TStore_List.Get_Item( Index : integer ) : int64 ;

var AC, I : int64 ;
    Offset : longint ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( ( Store = nil ) or ( Index < 0 ) or ( Index >= Header.Max ) or ( ACM = nil ) ) then
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Index_Error,
            UE_Error, 'Index error', '' ) ) ;
        Result := 0 ;
        exit ;
    end ;
    AC := Index * sizeof( int64 ) div ACM.Clustersize ;
    I := ACM.Get_Allocation_Cluster( AC ) ;
    Offset := Index * sizeof( int64 ) - AC * ACM.Clustersize ;
    ReAllocmem( Buf, ACM.Clustersize ) ;
    Store.Read_Data( Buf^[ 0 ], I, ACM.Clustersize, UEC ) ;
    move( Buf^[ Offset div sizeof( int64 ) ], Result, sizeof( Result ) ) ;
end ;


procedure TStore_List.Set_Item( Index : integer ; Value : int64 ) ;

var AC, I : int64 ;
    Offset : longint ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( ( Store = nil ) or ( Index < 0 ) or ( Index >= Header.Max ) or ( ACM = nil ) ) then
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Index_Error,
            UE_Error, 'Index error', '' ) ) ;
        exit ;
    end ;
    AC := Index * sizeof( int64 ) div ACM.Clustersize ;
    I := ACM.Get_Allocation_Cluster( AC ) ;
    Offset := Index * sizeof( int64 ) - AC * ACM.Clustersize ;
    ReAllocmem( Buf, ACM.Clustersize ) ;
    Store.Read_Data( Buf^[ 0 ], I, ACM.Clustersize, UEC ) ;
    move( Value, Buf^[ Offset div sizeof( int64 ) ], sizeof( Value ) ) ;
    Store.Write_Data( Buf^[ 0 ], I, ACM.Clustersize, UEC ) ;
end ;


procedure TStore_List.Set_Last_Error( E : TUnified_Exception ) ;

begin
    if( E <> _Last_Error ) then
    begin
        if( E <> nil ) then
        begin
            E.Attach ;
        end ;
        if( _Last_Error <> nil ) then
        begin
            _Last_Error.Detach ;
        end ;
        _Last_Error := E ;
    end ;
end ;


procedure TStore_List.Set_Address( _Value : int64 ) ;

var Old : TCOM_Allocation_Cluster_Manager64 ;
    UEC : TUnified_Exception ;

begin
    if( ( Store = nil ) or ( _Value = 0 ) ) then
    begin
        exit ;
    end ;
    Store.Read_Data( Header, _Value, sizeof( Header ), UEC ) ;
    if( UEC <> nil ) then
    begin
        exit ; {UNTESTED}
    end ;
    if( not Valid_Header ) then // Unknown version
    begin
        exit ; {UNTESTED}
    end ;
    _Address := _Value ;
    Old := ACM ;
    ACM := TCOM_Allocation_Cluster_Manager64.Create ;
    ACM.Attach ;
    if( Old <> nil ) then
    begin
        Old.Detach ; {UNTESTED}
    end ;
    ACM.Store := Store ;
    ACM.Clustersize := Header.Delta * sizeof( int64 ) ;
    if( Header.Data <> 0 ) then
    begin
        ACM.Set_Root_And_Size( Header.Data, Header.Capacity * sizeof( int64 ) ) ;
    end ;
end ; // TStore_List.Set_Address


// Override...

function TStore_List.Is_Class( Name : PAnsiChar ) : boolean ;

var _N : Ansistring ;

begin
    _N := lowercase( Ansistring( Name ) ) ;
    Result := _N = 'tstore_list' ;
end ;


function TStore_List.Valid_Header : boolean ;

begin
    Set_Last_Error( nil ) ;
    Result := False ;
    if( Header.Version >= 10 ) then // Unknown version
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ; { UNTESTED }
        exit ;
    end ;
    if( Header.Prefix <> List_Prefix ) then // Not a list header
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ;
        exit ;
    end ;
    Result := True ;
end ;


function TStore_List.Get_Capacity : longint ;

begin
    Result := Header.Capacity ;
end ;


procedure TStore_List.Set_Capacity( _Value : longint ) ;

var UEC : TUnified_Exception ;

begin
    if( Header.Delta = 0 ) then
    begin
        exit ;
    end ;
    _Value := ( _Value + Header.Delta - 1 ) div Header.Delta * Header.Delta ; // Round up by delta
    if( _Value = Capacity ) then
    begin
        exit ; // No change
    end ;
    if( _Value < Count ) then
    begin
        Count := _Value ;
    end ;
    if( ( _Store <> nil ) and ( ACM <> nil ) ) then
    begin
        ACM.Set_Size( _Value * sizeof( int64 ) ) ;
        Header.Capacity := _Value ;
        Header.Data := ACM.Get_Root ;
        Store.Write_Data( Header, Address, sizeof( Header ), UEC ) ;
    end ;
end ;


function TStore_List.Get_Count : longint ;

begin
    Result := Header.Max ;
end ;


procedure TStore_List.Set_Count( _Value : longint ) ;

var Index : integer ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( _Value = Count ) then
    begin
        exit ; // No change
    end ;
    if( Capacity < _Value ) then
    begin
        Capacity := _Value ;
    end ;
    if( _Store <> nil ) then
    begin
        Index := Count ;
        Header.Max := _Value ;
        while( _Value > Index ) do // Extending list
        begin
            Set_Item( Index, 0 ) ;
            inc( Index ) ;
        end ;
        Store.Write_Data( Header, Address, sizeof( Header ), UEC ) ;
        Set_Last_Error( UEC ) ;
    end ;
end ;


function TStore_List.Add( _Value : int64 ) : longint ;

begin
    Result := Count ;
    Count := Result + 1 ;
    Set_Item( Result, _Value ) ;
end ;


procedure TStore_List.Delete ;

var H : TStore_String_Header ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( ( _Store <> nil ) and ( ACM <> nil ) ) then
    begin
        ACM.Set_Size( 0 ) ;
        fillchar( H, sizeof( H ), 0 ) ;
        _Store.Write_Data( H, Address, sizeof( H ), UEC ) ;
        if( Header.Data <> 0 ) then
        begin
            _Store.freemem( Header.Data ) ;
        end ;
        _Store.freemem( Address ) ;
    end ;
    Free ;
end ;


procedure TStore_List.Insert( Index : longint ; _Value : int64 ) ;

var Loop : integer ;

begin
    // Handle out of range indexes...
    if( Index < 0 ) then
    begin
        Index := 0 ;
    end ;
    if( Index >= Count ) then
    begin
        Add( _Value ) ;
        exit ;
    end ;

    // Need to shift values up...
    Add( Items[ Count - 1 ] ) ; // Duplicate the last item, thus moving it up one position
    for Loop := Count - 2 downto Index + 1 do
    begin
        Items[ Loop ] := Items[ Loop - 1 ] ;
    end ;

    // Insert the new item
    Items[ Index ] := _Value ;
end ;


// TStore_Fixed_List methods...

// Constructors and destructors...

destructor TStore_Fixed_List.Destroy ;

begin
    freemem( Buf ) ;
    Buf := nil ;

    inherited Destroy ;
end ;


// Property handlers...

function TStore_Fixed_List.Get_Item( Index : integer ) : int64 ;

var UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( ( Store = nil ) or ( Index < 0 ) or ( Index >= Header.Max ) ) then
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Index_Error,
            UE_Error, 'Index error', '' ) ) ;
        Result := 0 ;
        exit ;
    end ;
    Store.Read_Data( Result, Header.Data + Index * sizeof( int64 ), sizeof( int64 ), UEC ) ;
end ;


procedure TStore_Fixed_List.Set_Item( Index : integer ; Value : int64 ) ;

var UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( ( Store = nil ) or ( Index < 0 ) or ( Index >= Header.Max ) ) then
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Index_Error,
            UE_Error, 'Index error', '' ) ) ;
        exit ;
    end ;
    Store.Write_Data( Value, Header.Data + Index * sizeof( int64 ), sizeof( int64 ), UEC ) ;
end ;


procedure TStore_Fixed_List.Set_Last_Error( E : TUnified_Exception ) ;

begin
    if( E <> _Last_Error ) then
    begin
        if( E <> nil ) then
        begin
            E.Attach ;
        end ;
        if( _Last_Error <> nil ) then
        begin
            _Last_Error.Detach ;
        end ;
        _Last_Error := E ;
    end ;
end ;


procedure TStore_Fixed_List.Set_Address( _Value : int64 ) ;

var UEC : TUnified_Exception ;

begin
    if( ( Store = nil ) or ( _Value = 0 ) ) then
    begin
        exit ;
    end ;
    Store.Read_Data( Header, _Value, sizeof( Header ), UEC ) ;
    if( UEC <> nil ) then
    begin
        exit ; {UNTESTED}
    end ;
    if( not Valid_Header ) then // Unknown version
    begin
        exit ; {UNTESTED}
    end ;
    _Address := _Value ;
end ; // TStore_List.Set_Address


// Override...

function TStore_Fixed_List.Is_Class( Name : PAnsiChar ) : boolean ;

var _N : Ansistring ;

begin
    _N := lowercase( Ansistring( Name ) ) ;
    Result := _N = 'tstore_fixed_list' ;
end ;


function TStore_Fixed_List.Valid_Header : boolean ;

begin
    Set_Last_Error( nil ) ;
    Result := False ;
    if( Header.Version >= 10 ) then // Unknown version
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ; { UNTESTED }
        exit ;
    end ;
    if( Header.Prefix <> Fixed_List_Prefix ) then // Not a list header
    begin
        Set_Last_Error( Create_Simple_UE( Store_List_Facility, 1, Store_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ;
        exit ;
    end ;
    Result := True ;
end ;


function TStore_Fixed_List.Get_Capacity : longint ;

begin
    Result := Header.Capacity ;
end ;


procedure TStore_Fixed_List.Set_Capacity( _Value : longint ) ;

var New : int64 ;
    UEC : TUnified_Exception ;

begin
    if( Header.Delta = 0 ) then
    begin
        exit ;
    end ;
    _Value := ( _Value + Header.Delta - 1 ) div Header.Delta * Header.Delta ; // Round up by delta
    if( _Value = Capacity ) then
    begin
        exit ; // No change
    end ;
    if( _Value < Count ) then
    begin
        Count := _Value ;
    end ;
    if( _Store <> nil ) then
    begin
        New := Store.Reallocmem( Header.Data, _Value * sizeof( int64 ) ) ;
        if( New <> 0 ) then // Successful
        begin
            Header.Data := New ;
            Header.Capacity := _Value ;
            Store.Write_Data( Header, Address, sizeof( Header ), UEC ) ;
        end ;
    end ;
end ;


function TStore_Fixed_List.Get_Count : longint ;

begin
    Result := Header.Max ;
end ;


procedure TStore_Fixed_List.Set_Count( _Value : longint ) ;

var Index : integer ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( _Value = Count ) then
    begin
        exit ; // No change
    end ;
    if( Capacity < _Value ) then
    begin
        Capacity := _Value ;
    end ;
    if( Capacity < _Value ) then // Attempt to extend failed
    begin
        exit ;
    end ;
    if( _Store <> nil ) then
    begin
        Index := Count ;
        Header.Max := _Value ;
        if( _Value > Index ) then // Extending list
        begin
            Store.Fill( Header.Data + Index * sizeof( int64 ), ( _Value - Index ) * sizeof( int64 ), 0 ) ;
        end ;
        Store.Write_Data( Header, Address, sizeof( Header ), UEC ) ;
        Set_Last_Error( UEC ) ;
    end ;
end ;


function TStore_Fixed_List.Add( _Value : int64 ) : longint ;

begin
    Result := Count ;
    Count := Result + 1 ;
    Set_Item( Result, _Value ) ;
end ;


procedure TStore_Fixed_List.Delete ;

var H : TStore_String_Header ;
    UEC : TUnified_Exception ;

begin
    Set_Last_Error( nil ) ;
    if( _Store <> nil ) then
    begin
        fillchar( H, sizeof( H ), 0 ) ;
        _Store.Write_Data( H, Address, sizeof( H ), UEC ) ; // Erase "FLI"
        _Store.freemem( Header.Data ) ;
        _Store.freemem( Address ) ;
    end ;
    Free ;
end ;


procedure TStore_Fixed_List.Insert( Index : longint ; _Value : int64 ) ;

var Loop : integer ;

begin
    // Handle out of range indexes...
    if( Index < 0 ) then
    begin
        Index := 0 ;
    end ;
    if( Index >= Count ) then
    begin
        Add( _Value ) ;
        exit ;
    end ;

    // Need to shift values up...
    Add( Items[ Count - 1 ] ) ; // Duplicate the last item, thus moving it up one position
    for Loop := Count - 2 downto Index + 1 do
    begin
        Items[ Loop ] := Items[ Loop - 1 ] ;
    end ;

    // Insert the new item
    Items[ Index ] := _Value ;
end ;



// TStore_String_List methods...

function TStore_String_List.Get_Item( Index : integer ) : Ansistring ;

var P : int64 ;
    S : TStore_String ;

begin
    Set_Last_Error( nil ) ;
    Result := '' ;
    if( ( Store = nil ) or ( Index < 0 ) or ( Index >= Header.Max ) or ( ACM = nil ) ) then
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Index_Error,
            UE_Error, 'Index error', '' ) ) ;
        exit ;
    end ;
    P := inherited Get_Item( Index ) ;
    if( P = 0 ) then // Null pointer
    begin
        exit ;
    end ;
    S := Get_Store_String( Store, P ) ;
    if( S = nil ) then // Not a valid string {UNTESTED}
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ;
        exit ;
    end ;
    try
        Result := S.Value ;
    finally
        S.Free ;
    end ;
end ;


procedure TStore_String_List.Set_Item( Index : integer ; Value : Ansistring ) ;

var P : int64 ;
    S : TStore_String ;

begin
    Set_Last_Error( nil ) ;
    if( ( Store = nil ) or ( Index < 0 ) or ( Index >= Header.Max ) or ( ACM = nil ) ) then
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Index_Error,
            UE_Error, 'Index error', '' ) ) ;
        exit ;
    end ;
    P := inherited Get_Item( Index ) ;
    if( P <> 0 ) then // Pre-existing string
    begin
        S := Get_Store_String( Store, P ) ;
        if( S = nil ) then {UNTESTED}
        begin
            Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Invalid_Object,
                UE_Error, 'Invalid object', '' ) ) ;
            exit ;
        end ;
        S.Value := Value ; // Change string
    end else
    begin
        S := Create_Store_String( Store, Value ) ; // Create string
        if( S = nil ) then {UNTESTED}
        begin
            Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Creation_Error,
                UE_Error, 'Creation error', '' ) ) ;
            exit ;
        end ;
        inherited Set_Item( Index, S.Address ) ;
    end ;
    S.Free ;
end ; // TStore_String_List.Set_Item


function TStore_String_List.Get_Format : longint ;

begin
    Result := _Format ;
end ;


procedure TStore_String_List.Set_Format( Value : longint ) ;

begin
    _Format := Value ;
end ;


// Overrides...

function TStore_String_List.Is_Class( Name : PAnsiChar ) : boolean ;

var _N : Ansistring ;

begin
    _N := lowercase( Ansistring( Name ) ) ;
    Result := _N = 'tstore_string_list' ;
end ;


function TStore_String_List.Valid_Header : boolean ;

begin
    Set_Last_Error( nil ) ;
    Result := False ;
    if( Header.Version >= 10 ) then // Unknown version
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ; { UNTESTED }
        exit ;
    end ;
    if( Header.Prefix <> String_List_Prefix ) then // Not a string list header
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Invalid_Object,
            UE_Error, 'Invalid object', '' ) ) ;
        exit ;
    end ;
    Result := True ;
end ;


function TStore_String_List.Add( _Value : Ansistring ) : longint ;

var S : TStore_String ;

begin
    Result := -1 ; // Assume failure
    S := Create_Store_String( Store, _Value ) ; // Create string
    if( S = nil ) then {UNTESTED}
    begin
        Set_Last_Error( Create_Simple_UE( Store_String_List_Facility, 1, Store_String_ListErr_Creation_Error,
            UE_Error, 'Creation error', '' ) ) ;
        exit ;
    end ;
    Result := inherited Add( S.Address ) ;
end ;


procedure TStore_String_List.Delete ;

begin
    Count := 0 ;
    inherited Delete ;
end ;


procedure TStore_String_List.Set_Count( _Value : longint ) ;

var Loop : longint ;
    I : int64 ;
    S : TStore_String ;

begin
    // Delete strings that we are removing...
    for Loop := Count - 1 downto _Value do
    begin
        I := inherited Get_Item( Loop ) ;
        if( I <> 0 ) then {ELSE UNTESTED}
        begin
            S := Get_Store_String( Store, I ) ;
            if( S <> nil ) then {ELSE UNTESTED}
            begin
                S.Delete ; // Delete and free
            end ; // if( S <> nil )
        end ; // if( I <> 0 )
        inherited Set_Item( Loop, 0 ) ;
    end ; // for Loop := Count - 1 downto _Value

    inherited Set_Count( _Value ) ;
end ;


function TStore_String_List.IndexOf( _Value : Ansistring ;
    CI : boolean ) : longint ;

var Loop : longint ;
    S : TStore_String ;
    U_S : TUOS_String ;

begin
    Result := -1 ; // Assume not count
    if( ( Store = nil ) or ( Count = 0 ) ) then
    begin
        exit ;
    end ;
    S := TStore_String.Create ;
    U_S := TUOS_String.Create ;
    try
        if( CI ) then
        begin
            U_S.Assign_From_String( _Value, SF_UTF8 ) ;
            U_S.Lowercase ;
            _Value := U_S.As_String ;
        end ;
        S.Store := Store ;
        for Loop := 0 to Count - 1 do
        begin
            S.Address := inherited Get_Item( Loop ) ;
            if( CI ) then
            begin
                U_S.Assign_From_String( S.Value, _Format ) ;
                U_S.Lowercase ;
                if( U_S.As_String = _Value ) then
                begin
                    Result := Loop ;
                    exit ;
                end ;
            end else
            if( S.Value = _Value ) then
            begin
                Result := Loop ;
                exit ;
            end ;
        end ;
    finally
        S.Free ;
        U_S.Free ;
    end ;
end ; // TStore_String_List.IndexOf



// Unit API...

function Get_Store_String( Store : TUOS_File_Heap ;
    Address : int64 ) : TStore_String ;

var Header : TStore_String_Header ;
    UEC : TUnified_Exception ;

begin
    Result := nil ;
    if( ( Store = nil ) or ( Address = 0 ) ) then
    begin
        exit ;
    end ;
    Store.Read_Data( Header, Address, sizeof( Header ), UEC ) ;
    if( UEC <> nil ) then
    begin
        exit ;
    end ;
    Result := TStore_String.Create ;
    Result.Header := Header ;
    Result.Store := Store ;
    if( not Result.Valid_Header ) then // Unknown version
    begin
        Result.Free ;
        Result := nil ;
        exit ;
    end ;
    Result.Address := Address ; // Force a read
end ; // Get_Store_String


procedure Delete_Store_String( Store : TUOS_File_Heap ; Address : int64 ) ;

var S : TStore_String ;

begin
    S := Get_Store_String( Store, Address ) ;
    if( S <> nil ) then
    begin
        S.Delete ;
    end ;
end ;


function Create_Store_String( Store : TUOS_File_Heap ;
    S : Ansistring ) : TStore_String ;

var I : int64 ;

begin
    Result := nil ;
    if( Store = nil ) then
    begin
        exit ;
    end ;
    I := Store.Getmem( sizeof( TStore_String_Header ) ) ;
    if( I = 0 ) then
    begin
        exit ;
    end ;
    Result := TStore_String.Create ;
    fillchar( Result.Header, sizeof( Result.Header ), 0 ) ;
    Result.Header.Prefix := String_Prefix ;
    Result.Header.Version := 1 ;
    Store.Write( I, sizeof( Result.Header ), Result.Header ) ;
    Result.Store := Store ;
    Result.Address := I ;
    Result.Value := S ;
end ;


function Get_Store_List( Store : TUOS_File_Heap ;
    Address : int64 ) : TStore_List ;

var Header : TStore_List_Header ;
    UEC : TUnified_Exception ;

begin
    Result := nil ;
    if( ( Store = nil ) or ( Address = 0 ) ) then
    begin
        exit ;
    end ;
    Store.Read_Data( Header, Address, sizeof( Header ), UEC ) ;
    if( UEC <> nil ) then
    begin
        exit ; { UNTESTED }
    end ;
    Result := TStore_List.Create ;
    Result.Header := Header ;
    if( not Result.Valid_Header ) then // Unknown version
    begin
        Result.Free ;
        Result := nil ;
        exit ;
    end ;
    Result.Store := Store ;
    Result.Address := Address ; // Force a read
end ; // Get_Store_List


procedure Delete_Store_List( Store : TUOS_File_Heap ; Address : int64 ) ;

var S : TStore_List ;

begin
    S := Get_Store_List( Store, Address ) ;
    if( S <> nil ) then
    begin
        S.Delete ;
    end ;
end ;


function Create_Store_List( Store : TUOS_File_Heap ;
    _Delta : integer ) : TStore_List ;

var I : int64 ;

begin
    Result := nil ;
    if( Store = nil ) then
    begin
        exit ;
    end ;
    I := Store.Getmem( sizeof( TStore_List_Header ) ) ;
    if( I = 0 ) then
    begin
        exit ; { UNTESTED }
    end ;
    Result := TStore_List.Create ;
    fillchar( Result.Header, sizeof( Result.Header ), 0 ) ;
    Result.Header.Prefix := List_Prefix ;
    Result.Header.Version := 1 ;
    _Delta := ( _Delta + Store.Min_Storage - 1 ) and ( not ( Store.Min_Storage - 1 ) ) ;
    Result.Header.Delta := _Delta ;
    Store.Write( I, sizeof( Result.Header ), Result.Header ) ;
    Result.Store := Store ;
    Result.Address := I ;
end ; // Create_Store_List



function Get_Store_Fixed_List( Store : TUOS_File_Heap ;
    Address : int64 ) : TStore_Fixed_List ;

var Header : TStore_List_Header ;
    UEC : TUnified_Exception ;

begin
    Result := nil ;
    if( ( Store = nil ) or ( Address = 0 ) ) then
    begin
        exit ;
    end ;
    Store.Read_Data( Header, Address, sizeof( Header ), UEC ) ;
    if( UEC <> nil ) then
    begin
        exit ; { UNTESTED }
    end ;
    Result := TStore_Fixed_List.Create ;
    Result.Header := Header ;
    if( not Result.Valid_Header ) then // Unknown version
    begin
        Result.Free ;
        Result := nil ;
        exit ;
    end ;
    Result.Store := Store ;
    Result.Address := Address ; // Force a read
end ; // Get_Store_Fixed_List


procedure Delete_Store_Fixed_List( Store : TUOS_File_Heap ; Address : int64 ) ;

var S : TStore_Fixed_List ;

begin
    S := Get_Store_Fixed_List( Store, Address ) ;
    if( S <> nil ) then
    begin
        S.Delete ;
    end ;
end ;


function Create_Store_Fixed_List( Store : TUOS_File_Heap ;
    _Delta : integer ) : TStore_Fixed_List ;

var I : int64 ;

begin
    Result := nil ;
    if( Store = nil ) then
    begin
        exit ;
    end ;
    I := Store.Getmem( sizeof( TStore_List_Header ) ) ;
    if( I = 0 ) then
    begin
        exit ; { UNTESTED }
    end ;
    Result := TStore_Fixed_List.Create ;
    fillchar( Result.Header, sizeof( Result.Header ), 0 ) ;
    Result.Header.Prefix := Fixed_List_Prefix ;
    Result.Header.Version := 1 ;
    _Delta := ( _Delta + Store.Min_Storage - 1 ) and ( not ( Store.Min_Storage - 1 ) ) ;
    Result.Header.Delta := _Delta ;
    Store.Write( I, sizeof( Result.Header ), Result.Header ) ;
    Result.Store := Store ;
    Result.Address := I ;
end ; // Create_Fixed_Store_List



function Get_Store_String_List( Store : TUOS_File_Heap ;
    Address : int64 ; Format : longint = SF_UTF8 ) : TStore_String_List ;

var Header : TStore_List_Header ;
    UEC : TUnified_Exception ;

begin
    Result := nil ;
    if( ( Store = nil ) or ( Address = 0 ) ) then
    begin
        exit ;
    end ;
    Store.Read_Data( Header, Address, sizeof( Header ), UEC ) ;
    if( UEC <> nil ) then
    begin
        exit ;
    end ;
    Result := TStore_String_List.Create ;
    Result.Header := Header ;
    if( not Result.Valid_Header ) then // Unknown version
    begin
        Result.Free ;
        Result := nil ;
        exit ;
    end ;
    Result.Format := Format ;
    Result.Store := Store ;
    Result.Address := Address ; // Force a read
end ; // Get_Store_String_List


procedure Delete_Store_String_List( Store : TUOS_File_Heap ; Address : int64 ) ;

var S : TStore_String_List ;

begin
    S := Get_Store_String_List( Store, Address ) ;
    if( S <> nil ) then
    begin
        S.Delete ;
    end ;
end ;


function Create_Store_String_List( Store : TUOS_File_Heap ;
    _Delta : integer ) : TStore_String_List ;

var I : int64 ;

begin
    Result := nil ;
    if( Store = nil ) then
    begin
        exit ;
    end ;
    I := Store.Getmem( sizeof( TStore_List_Header ) ) ;
    if( I = 0 ) then
    begin
        exit ; { UNTESTED }
    end ;
    Result := TStore_String_List.Create ;
    fillchar( Result.Header, sizeof( Result.Header ), 0 ) ;
    Result.Header.Prefix := String_List_Prefix ;
    Result.Header.Version := 1 ;
    _Delta := ( _Delta + Store.Min_Storage - 1 ) and ( not ( Store.Min_Storage - 1 ) ) ;
    Result.Header.Delta := _Delta ;
    Store.Write( I, sizeof( Result.Header ), Result.Header ) ;
    Result.Store := Store ;
    Result.Address := I ;
end ; // Create_Store_String_List


{$IFDEF CC_Regression}
type TUOS_Test_File = class( TUOS_File )
                          public // Constructors and destructors...
                              destructor Destroy ; override ;

                          private // Instance data...
                              _File : file ; // Data stream buffer
                              _Drive : integer ;

                          public // API (overrides)...
                              function Is_Class( Name : PAnsiChar ) : boolean ;
                                  override ;

                              function Create_Stream( Name : int64 ) : longint ;
                                  override ;

                              procedure Delete_Stream( Name : int64 ; Index : longint ) ;
                                  override ;

                              function Max_Stream : longint ; override ;

                              function Stream_Name( Index : longint ) : int64 ;
                                  override ;

                              function Read( Stream : longint ;
                                  Position : TStore_Address64 ;
                                  Length : TStore_Size64 ; var Buff ) : TStore_Size64 ;
                                  override ;

                              function Write( Stream : longint ;
                                  Position : TStore_Address64 ;
                                  Length : TStore_Size64 ; var Buff ) : TStore_Size64 ;
                                  override ;

                			        function Get_Stream_Size( Stream : longint ) : TStore_Size64 ;
                                  override ;

                		          procedure Set_Stream_Size( Stream : longint ;
                                  Value : TStore_Size64 ) ; override ;

                              function Get_Contiguous : boolean ; override ;

                              procedure Set_Contiguous( Value : boolean ) ;
                                  override ;

                              function Get_File_Size : int64 ;
                                  override ;

                              procedure Set_File_Size( Value : int64 ) ;
                                  override ;

                              function Read_Only : boolean ; override ;

                              function Write_Only : boolean ; override ;

                              function XSpaceAvail : int64 ; override ;

                              function Rebuild : boolean ;
                      end ;

destructor TUOS_Test_File.Destroy ;

begin
    closefile( _File ) ;

    inherited Destroy ;
end ;


function TUOS_Test_File.Is_Class( Name : PAnsiChar ) : boolean ;

begin
    Result := Name = 'tuos_test_file' ;
    if( not Result ) then
    begin
        Result := inherited Is_Class( Name ) ;
    end ;
end ;


function TUOS_Test_File.Create_Stream( Name : int64 ) : longint ;

begin
    Result := -1 ;
end ;


procedure TUOS_Test_File.Delete_Stream( Name : int64 ; Index : longint ) ;

begin
end ;


function TUOS_Test_File.Max_Stream : longint ;

begin
    Result := 0 ;
end ;


function TUOS_Test_File.Stream_Name( Index : longint ) : int64 ;

begin
    Result := 0 ;
end ;


function TUOS_Test_File.Read( Stream : longint ;
    Position : TStore_Address64 ;
    Length : TStore_Size64 ; var Buff ) : TStore_Size64 ;

begin
    if( Stream = 0 ) then
    begin
        seek( _File, Position ) ;
        blockread( _File, Buff, Length ) ;
        Result := Length ;
    end else
    begin
        Result := 0 ;
    end ;
end ;


function TUOS_Test_File.Write( Stream : longint ;
    Position : TStore_Address64 ;
    Length : TStore_Size64 ; var Buff ) : TStore_Size64 ;

begin
    if( Stream = 0 ) then
    begin
        seek( _File, Position ) ;
        blockwrite( _File, Buff, Length ) ;
    end else
    begin
        Result := 0 ;
    end ;
end ;


function TUOS_Test_File.Get_Stream_Size( Stream : longint ) : TStore_Size64 ;

begin
    if( Stream = 0 ) then
    begin
        Result := File_Size ;
    end else
    begin
        Result := 0 ;
    end ;
end ;


procedure TUOS_Test_File.Set_Stream_Size( Stream : longint ;
    Value : TStore_Size64 ) ;

begin
    if( Stream = 0 ) then
    begin
        Set_File_Size( Value ) ;
    end ;
end ;


function TUOS_Test_File.Get_Contiguous : boolean ;

begin
    Result := False ;
end ;


procedure TUOS_Test_File.Set_Contiguous( Value : boolean ) ;
                   
begin
end ;


function TUOS_Test_File.Get_File_Size : int64 ;

begin
    Result := filesize( _File ) ;
end ;


procedure TUOS_Test_File.Set_File_Size( Value : int64 ) ;

var Buffer : array[ 0..255 ] of byte ;
    I : integer ;

begin
    fillchar( Buffer, sizeof( Buffer ), 0 ) ;
    I := Value - filesize( _File ) ;
    seek( _File, filesize( _File ) ) ;
    while( I > 0 ) do
    begin
        if( I < sizeof( Buffer ) ) then
        begin
            blockwrite( _File, Buffer, I ) ;
        end else
        begin
            blockwrite( _File, Buffer, sizeof( Buffer ) ) ;
        end ;
        I := I - sizeof( Buffer ) ;
    end ;
end ;


function TUOS_Test_File.Read_Only : boolean ;

begin
    Result := False ;
end ;


function TUOS_Test_File.Write_Only : boolean ;

begin
    Result := False ;
end ;


function TUOS_Test_File.XSpaceAvail : int64 ;

begin
    Result := DiskFree( _Drive ) ;
end ;


function TUOS_Test_File.Rebuild : boolean ;

begin
    Result := False ;
end ;


procedure Error ;

begin
    halt ;
end ;


procedure Test_Store_String ;

var Test_File : TUOS_Test_File ;
    File_Heap : TUOS_File_Heap ;
    I, I1 : int64 ;
    SS : TStore_String ;

begin
    Test_File := TUOS_Test_File.Create ;
    assignfile( Test_File._File, 'c:\work\uos\test.fh' ) ;
    rewrite( Test_File._File, 1 ) ;
    Test_File._Drive := 3 ; // C:

    File_Heap := TUOS_File_Heap.Create ;
    File_Heap.Free ;
    File_Heap := TUOS_File_Heap.Create ;
    File_Heap.Attach ;
    File_Heap._File := Test_File ;

    SS := Create_Store_String( File_Heap, 'This is a test' ) ;
    if( SS.Value <> 'This is a test' ) then
    begin
        Error ;
    end ;
    SS.Value := 'New string' ;
    if( SS.Value <> 'New string' ) then
    begin
        Error ;
    end ;
    SS.Delete ;

    SS := TStore_String.Create ;
    SS.Address := 1 ;
    SS.Value := 'A' ;
    SS.Value := 'A' ;
    SS.Delete ;
    if( Create_Store_String( nil, 'Hi' ) <> nil ) then
    begin
        Error ;
    end ;
    if( Get_Store_String( nil, 1 ) <> nil ) then
    begin
        Error ;
    end ;

    SS := Create_Store_String( File_Heap, 'Test1' ) ;
    I := SS.Address ;
    SS.Free ;
    SS := Create_Store_String( File_Heap, 'Test2' ) ;
    I1 := SS.Address ;
    SS.Free ;
    SS := Get_Store_String( File_Heap, I ) ;
    if( SS.Value <> 'Test1' ) then
    begin
        Error ;
    end ;
    SS.Value := 'Test3' ;
    SS.Free ;
    SS := Get_Store_String( File_Heap, I ) ;
    if( SS.Value <> 'Test3' ) then
    begin
        Error ;
    end ;
    SS.Value := '1234567890abcdefghijklmnopqrstuvwxyz' ;
    SS.Free ;
    SS := Get_Store_String( File_Heap, I ) ;
    if( SS.Value <> '1234567890abcdefghijklmnopqrstuvwxyz' ) then
    begin
        Error ;
    end ;
    SS.Value := '' ;
    SS.Value := '' ;
    SS.Store := nil ;
    SS.Free ;
    SS := Get_Store_String( File_Heap, I + 16 ) ;
    if( SS <> nil ) then
    begin
        Error ;
    end ;

    File_Heap.Free ;
end ; // Test_Store_String


procedure Test_Store_List ;

var I : int64 ;
    Test_File : TUOS_Test_File ;
    File_Heap : TUOS_File_Heap ;
    List : TStore_List ;

begin
    Delete_Store_List( nil, 0 ) ;
    if( Get_Store_List( nil, 0 ) <> nil ) then
    begin
        Error ;
    end ;
    if( Create_Store_List( nil, 0 ) <> nil ) then
    begin
        Error ;
    end ;
    Test_File := TUOS_Test_File.Create ;
    assignfile( Test_File._File, 'c:\work\uos\test.fh' ) ;
    rewrite( Test_File._File, 1 ) ;
    Test_File._Drive := 3 ; // C:

    File_Heap := TUOS_File_Heap.Create ;
    File_Heap.Free ;
    File_Heap := TUOS_File_Heap.Create ;
    File_Heap.Attach ;
    File_Heap._File := Test_File ;

    List := Create_Store_List( File_Heap, 4 ) ;
    I := List.Address ;
    if( List.Count <> 0 ) then
    begin
        Error ;
    end ;
    List.Add( 0 ) ;
    if( List.Count <> 1 ) then
    begin
        Error ;
    end ;
    if( List[ 0 ] <> 0 ) then
    begin
        Error ;
    end ;
    List.Add( 1 ) ;
    List.Add( 2 ) ;
    List.Add( 3 ) ;
    List.Add( 4 ) ;
    if( List.Count <> 5 ) then
    begin
        Error ;
    end ;
    List.Count := List.Count ;
    List.Capacity := List.Capacity ;
    if( List[ 0 ] <> 0 ) then
    begin
        Error ;
    end ;
    if( List[ 1 ] <> 1 ) then
    begin
        Error ;
    end ;
    if( List[ 2 ] <> 2 ) then
    begin
        Error ;
    end ;
    if( List[ 3 ] <> 3 ) then
    begin
        Error ;
    end ;
    if( List[ 4 ] <> 4 ) then
    begin
        Error ;
    end ;

    List[ 0 ] := 1 ;
    if( List[ 0 ] <> 1 ) then
    begin
        Error ;
    end ;
    List[ 1 ] := 2 ;
    List[ 2 ] := 3 ;
    List[ 3 ] := 6 ;
    List[ 4 ] := 15 ;
    if( List[ 0 ] <> 1 ) then
    begin
        Error ;
    end ;
    if( List[ 1 ] <> 2 ) then
    begin
        Error ;
    end ;
    if( List[ 2 ] <> 3 ) then
    begin
        Error ;
    end ;
    if( List[ 3 ] <> 6 ) then
    begin
        Error ;
    end ;
    if( List[ 4 ] <> 15 ) then
    begin
        Error ;
    end ;
    List.Free ;
    List := Get_Store_List( File_Heap, I ) ;
    if( List = nil ) then
    begin
        Error ;
    end ;
    if( List[ 0 ] <> 1 ) then
    begin
        Error ;
    end ;
    if( List[ 1 ] <> 2 ) then
    begin
        Error ;
    end ;
    if( List[ 2 ] <> 3 ) then
    begin
        Error ;
    end ;
    if( List[ 3 ] <> 6 ) then
    begin
        Error ;
    end ;
    if( List[ 4 ] <> 15 ) then
    begin
        Error ;
    end ;
    if( List.Count <> 5 ) then
    begin
        Error ;
    end ;
    List.Add( 10 ) ;
    List.Add( 11 ) ;
    List.Add( 12 ) ;
    List.Add( 13 ) ;
    List.Add( 14 ) ;
    List.Add( 15 ) ;
    List.Add( 16 ) ;
    List.Add( 17 ) ;
    List.Add( 18 ) ;
    List.Add( 19 ) ;
    List.Add( 20 ) ;
    List.Add( 21 ) ;
    List.Capacity := 3 ;
    if( List.Count <> 3 ) then
    begin
        Error ;
    end ;
    List[ -1 ] ;
    List[ -1 ] := 0 ;
    List.Free ;
    Delete_Store_List( File_Heap, I ) ;
    List := Get_Store_List( File_Heap, I ) ;
    if( List <> nil ) then
    begin
        Error ;
    end ;

    List := TStore_List.Create ;
    List.Address := 0 ;
    List.Count := 1 ;
    List.Header.Delta := 1 ;
    List.Count := 1 ;
    List.Free ;

    File_Heap.Free ;
end ; // Test_Store_List


procedure Test_Store_String_List ;

var I : int64 ;
    Test_File : TUOS_Test_File ;
    File_Heap : TUOS_File_Heap ;
    List : TStore_String_List ;

begin
    Test_File := TUOS_Test_File.Create ;
    assignfile( Test_File._File, 'c:\work\uos\test.fh' ) ;
    rewrite( Test_File._File, 1 ) ;
    Test_File._Drive := 3 ; // C:

    File_Heap := TUOS_File_Heap.Create ;
    File_Heap.Free ;
    File_Heap := TUOS_File_Heap.Create ;
    File_Heap.Attach ;
    File_Heap._File := Test_File ;

    List := Create_Store_String_List( File_Heap, 4 ) ;
    if( List.Count <> 0 ) then
    begin
        Error ;
    end ;
    if( List.IndexOf( 'a', False ) <> -1 ) then
    begin
        Error ;
    end ;
    List.Add( 'Test' ) ;
    if( List.Count <> 1 ) then
    begin
        Error ;
    end ;
    if( List[ 0 ] = 'Test' ) then
    begin
        Error ;
    end ;
    List.Add( 'Two' ) ;
    if( List.Count <> 2 ) then
    begin
        Error ;
    end ;
    if( List[ 1 ] <> 'Two' ) then
    begin
        Error ;
    end ;
    List[ 1 ] := '2' ;
    if( List[ 1 ] <> '2' ) then
    begin
        Error ;
    end ;
    if( List.Indexof( '2', True ) <> 1 ) then
    begin
        Error ;
    end ;
    if( List.Indexof( '2', False ) <> 1 ) then
    begin
        Error ;
    end ;
    if( List.Indexof( '3', False ) <> -1 ) then
    begin
        Error ;
    end ;
    I := List.Address ;
    List[ -1 ] := List[ -1 ] ;
    List.Free ;
    List := Get_Store_String_List( File_Heap, I ) ;
    if( List = nil ) then
    begin
        Error ;
    end ;
    if( List.Count <> 2 ) then
    begin
        Error ;
    end ;
    if( List[ 1 ] <> '2' ) then
    begin
        Error ;
    end ;
    List.Count := 20 ;
    if( List[ 10 ] <> '' ) then
    begin
        Error ;
    end ;
    List[ 10 ] := 'Ten' ;
    List.Free ;
    if( Get_Store_String_List( nil, 1 ) <> nil ) then
    begin
        Error ;
    end ;
    if( Get_Store_String_List( File_Heap, 0 ) <> nil ) then
    begin
        Error ;
    end ;
    if( Get_Store_String_List( File_Heap, 1 ) <> nil ) then
    begin
        Error ;
    end ;
    if( Create_Store_String_List( nil, 4 ) <> nil ) then
    begin
        Error ;
    end ;
    Delete_Store_String_List( nil, 0 ) ;
    Delete_Store_String_List( File_Heap, I ) ;
    if( Get_Store_String_List( File_Heap, I ) <> nil ) then
    begin
        Error ;
    end ;

    File_Heap.Free ;
end ; // Test_Store_String_List
{$ENDIF}


end.
