{$IFNDEF FPC}
  {$N+}
  {$E+}
{$ENDIF}
{
        Program Name    : Language
        Package Name    : Subroutine Library - Layer 1
        Purpose         : Language-related data conversion routines
        Institution     : Conroy & Conroy Co.
        Date written    : 1-May-15
        Written by      : Alan Conroy
        Version         : 1.0

        Copyright (C) 2015 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

      ***************************************************************
      *                                                             *
      *      M O D I F I C A T I O N   H I S T O R Y                *
      *                                                             *
      ***************************************************************

          DATE        BY            REASON

      *******************************************************
      *                                                     *
      *          P R O G R A M   P U R P O S E              *
      *                                                     *
      *******************************************************

      This module contains routines for converting numeric data.
}

unit Language ;

{$I EDEFINES.INC}

interface

uses { Subroutine Library - Layer 0... }
     _Types, { Ansistring }

     { Subroutine Library - Layer 1... }
     _Languag ; { TBase_Delphi_Object }

type TSEnglish_Language = object( TSLanguage )
                              public
                                  { Converts a number to a specific numeric string. }
                                  function Numeric_To_String( Numeric : double ;
                                      Typ : integer ) : Ansistring ;
                                      virtual ;

                                  { Determines last position in S that is part of a
                                    number in English.  Returns < 1 if S doesn't start
                                    with a valid number in English. }
                                  function Numeric_Extent( S : Ansistring ) : integer ;
                                      virtual ;

                                  function To_Numeric( S : Ansistring ;
                                      var Numeric : double ) : boolean ;
                                      virtual ;

                                  { Converts a numeric string to a number and returns
                                    the type of the value.  The return is True if the
                                    passed numeric string is recognized or false
                                    otherwise. }
                                  function String_To_Numeric( S : Ansistring ;
                                      var Numeric : double ;
                                      var Typ : integer ) : boolean ;
                                      virtual ;

                                  { Converts a numeric string to a number and returns
                                    the type of the value.  The return is True if the
                                    passed numeric string is recognized or false
                                    otherwise.  This differs from String_To_Numeric in
                                    that it recognizes only strings containing valid
                                    prefix and suffixes, even if the specific
                                    combination is not a recognized English word.  For
                                    example, "centgram" would be recognized as a
                                    100-sided 2D object.  However, strings without both
                                    prefix and suffix are not recognized (such as "one",
                                    or "hex"). }
                                  function Generic_String_To_Numeric( S : Ansistring ;
                                      var Numeric : double ;
                                      var Typ : integer ) : boolean ;
                                      virtual ;
                          end ; { TSEnglish_Language }
     PSEnglish_Language = ^TSEnglish_Language ;


function Get_Language( Lang : Ansistring ) : PSLanguage ;
procedure Add_Language( Lang : Ansistring ; O : PSLanguage ) ;
procedure Remove_Language( O : PSLanguage ) ;


implementation

uses { Subroutine Library - Layer 0... }
     Collect, { TCollection }
     _Common, { Abstract_Error }

     { Subroutine Library - Layer 1... }
     CommonUt, { Edit }
     Num1s ; { Num1 }

var English_Language : PSLanguage ;

{ Unit internal routines... }

function Arabic_To_Roman( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
{$IFNDEF FPC_OR_DELPHI}
    if( Numeric > 247000 ) then
    begin
        Arabic_To_Roman := S ; { UNTESTED }
        exit ;
    end ;
{$ENDIF}
    Numeric := int( Numeric ) ; { Ignore fractions }
    while( Numeric >= 1000 ) do
    begin
        S := S + 'M' ;
        Numeric := Numeric - 1000 ;
    end ;
    if( Numeric >= 900 ) then
    begin
        S := S + 'CM' ;
        Numeric := Numeric - 900 ;
    end ;
    if( Numeric >= 500 ) then
    begin
        S := S + 'D' ;
        Numeric := Numeric - 500 ;
    end ;
    if( Numeric >= 400 ) then
    begin
        S := S + 'CD' ;
        Numeric := Numeric - 400 ;
    end ;
    while( Numeric >= 100 ) do
    begin
        S := S + 'C' ;
        Numeric := Numeric - 100 ;
    end ;

    if( Numeric >= 90 ) then
    begin
        S := S + 'XC' ;
        Numeric := Numeric - 90 ;
    end ;
    if( Numeric >= 50 ) then
    begin
        S := S + 'L' ;
        Numeric := Numeric - 50 ;
    end ;
    if( Numeric >= 40 ) then
    begin
        S := S + 'XL' ;
        Numeric := Numeric - 40 ;
    end ;
    while( Numeric > 9 ) do
    begin
        S := S + 'X' ;
        Numeric := Numeric - 10 ;
    end ;
    case trunc( Numeric ) of
        9 : S := S + 'IX' ;
        8 : S := S + 'VIII' ;
        7 : S := S + 'VII' ;
        6 : S := S + 'VI' ;
        5 : S := S + 'V' ;
        4 : S := S + 'IV' ;
        3 : S := S + 'III' ;
        2 : S := S + 'II' ;
        1 : S := S + 'I' ;
    end ;
    Arabic_To_Roman := S ;
end ;


function Num_To_English( Numeric : double ) : Ansistring ;

    function _Num( Base : double ; const Nam : Ansistring ) : Ansistring ;

    var S, R : Ansistring ;
        Temp : double ;

    begin
        if( Numeric >= Base ) then
        begin
            R := '' ;
            Temp := int( Numeric / Base ) ;
            Numeric := Numeric - Temp * Base ;
            S := Num1( Temp ) ;
            while( length( S ) < 3 ) do
            begin
                S := '0' + S ;
            end ;
            case S[ 1 ] of
                '1' : R := 'one' ;
                '2' : R := 'two' ;
                '3' : R := 'three' ;
                '4' : R := 'four' ;
                '5' : R := 'five' ;
                '6' : R := 'six' ;
                '7' : R := 'seven' ;
                '8' : R := 'eight' ;
                '9' : R := 'nine' ;
            end ;
            if( S[ 1 ] <> '0' ) then
            begin
                R := R + ' hundred' ;
            end ;
            case S[ 2 ] of
                '0', '1' : R := R + ' ' ;
                '2' : R := R + ' twenty' ;
                '3' : R := R + ' thirty' ;
                '4' : R := R + ' forty' ;
                '5' : R := R + ' fifty' ;
                '6' : R := R + ' sixty' ;
                '7' : R := R + ' seventy' ;
                '8' : R := R + ' eighty' ;
                '9' : R := R + ' ninety' ;
            end ;
            if( S[ 2 ] = '1' ) then
            begin
                case S[ 3 ] of
                    '0' : R := R + 'ten' ;
                    '1' : R := R + 'eleven' ;
                    '2' : R := R + 'twelve' ;
                    '3' : R := R + 'thirteen' ;
                    '4' : R := R + 'fourteen' ;
                    '5' : R := R + 'fifteen' ;
                    '6' : R := R + 'sixteen' ;
                    '7' : R := R + 'seventeen' ;
                    '8' : R := R + 'eighteen' ;
                    '9' : R := R + 'nineteen' ;
                end ;
            end else
            begin
                case S[ 3 ] of
                    '1' : R := R + 'one' ;
                    '2' : R := R + 'two' ;
                    '3' : R := R + 'three' ;
                    '4' : R := R + 'four' ;
                    '5' : R := R + 'five' ;
                    '6' : R := R + 'six' ;
                    '7' : R := R + 'seven' ;
                    '8' : R := R + 'eight' ;
                    '9' : R := R + 'nine' ;
                end ;
            end ; { if( S[ 2 ] = '1' ) }
            if( ( length( R ) > 0 ) and ( length( Nam ) > 0 ) ) then
            begin
                R := R + ' ' + Nam ;
            end ;
            _Num := Edit( R, 8 or 16 or 128 ) ; { Remove excess spaces }
        end else
        begin
            _Num := '' ;
        end ;
    end ; { Num_To_English._Num }

var Fraction : double ;
    S : Ansistring ;

begin { Num_To_English }
    if( Numeric > 1E+103 ) then { One google is the largest we know about }
    begin
        Num_To_English := '' ;
        exit ;
    end ;
    if( Numeric = 0 ) then
    begin
        Num_To_English := 'zero' ;
        exit ;
    end ;
    Fraction := Numeric - int( Numeric ) ;
    Numeric := int( Numeric ) ;
    S := '' ;
    if( Numeric <> 0 ) then
    begin
        if( Numeric < 0 ) then
        begin
            S := 'negative ' ;
            Numeric := -Numeric ;
        end ;
        S := S + _Num( 1E100, 'google' ) ;
        if( Numeric >= 1E+36 ) then { except for google, don't know above decillion }
        begin
            Num_To_English := '' ;
            exit ;
        end ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E33, 'decillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E30, 'nonillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E27, 'octillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E24, 'septillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E21, 'sextillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E18, 'quintillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E15, 'quadrillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E12, 'trillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E9, 'billion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E6, 'million' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1000, 'thousand' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1, '' ) ;
    end ; { if( Numeric <> 0 ) }
    if( Fraction <> 0 ) then
    begin
        S := S + ' point ' + copy( Num1( Fraction ), 3, 255 ) ;
    end ;
    Num_To_English := Edit( S, 8 or 16 or 128 ) ; { Remove excess spaces }
end ; { Num_To_English }


function Num_To_Ordinal( Numeric : double ) : Ansistring ;

var Ones : Ansichar ;
    S, Tens : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) ) then
    begin
        S := Num1( Numeric ) ;
        Ones := S[ length( S ) ] ;
        if( length( S ) > 1 ) then
        begin
            Tens := copy( S, length( S ) - 1, 1 ) ;
        end else
        begin
            Tens := '' ;
        end ;
        if( ( Tens = '1' ) or ( pos( Ones, '123' ) = 0 ) ) then
        begin
            S := S + 'th' ;
        end else
        begin
            case Ones of
                '1' : S := S + 'st' ;
                '2' : S := S + 'nd' ;
                '3' : S := S + 'rd' ;
            end ;
        end ;
    end ;
    Num_To_Ordinal := S ;
end ;


function Num_To_English_Ordinal( Numeric : double ) : Ansistring ;

    function _Num( Base : double ; Nam : Ansistring ) : Ansistring ;

    var S, R : Ansistring ;
        Temp : double ;

    begin
        if( Numeric >= Base ) then
        begin
            R := '' ;
            Temp := int( Numeric / Base ) ;
            Numeric := Numeric - Temp * Base ;
            S := Num1( Temp ) ;
            while( length( S ) < 3 ) do
            begin
                S := '0' + S ;
            end ;
            case S[ 1 ] of
                '1' : R := 'one' ;
                '2' : R := 'two' ;
                '3' : R := 'three' ;
                '4' : R := 'four' ;
                '5' : R := 'five' ;
                '6' : R := 'six' ;
                '7' : R := 'seven' ;
                '8' : R := 'eight' ;
                '9' : R := 'nine' ;
            end ;
            if( S[ 1 ] <> '0' ) then
            begin
                R := R + ' hundred' ;
            end ;
            case S[ 2 ] of
                '0', '1' : R := R + ' ' ;
                '2' : R := R + ' twenty' ;
                '3' : R := R + ' thirty' ;
                '4' : R := R + ' forty' ;
                '5' : R := R + ' fifty' ;
                '6' : R := R + ' sixty' ;
                '7' : R := R + ' seventy' ;
                '8' : R := R + ' eighty' ;
                '9' : R := R + ' ninety' ;
            end ;
            if( S[ 2 ] = '1' ) then
            begin
                case S[ 3 ] of
                    '0' : R := R + 'tenth' ;
                    '1' : R := R + 'eleventh' ;
                    '2' : R := R + 'twelfth' ;
                    '3' : R := R + 'thirteenth' ;
                    '4' : R := R + 'fourteenth' ;
                    '5' : R := R + 'fifteenth' ;
                    '6' : R := R + 'sixteenth' ;
                    '7' : R := R + 'seventeenth' ;
                    '8' : R := R + 'eighteenth' ;
                    '9' : R := R + 'nineteenth' ;
                end ;
            end else
            begin
                if( Base > 1 ) then
                begin
                    case S[ 3 ] of
                        '1' : R := 'one' ;
                        '2' : R := 'two' ;
                        '3' : R := 'three' ;
                        '4' : R := 'four' ;
                        '5' : R := 'five' ;
                        '6' : R := 'six' ;
                        '7' : R := 'seven' ;
                        '8' : R := 'eight' ;
                        '9' : R := 'nine' ;
                    end ;
                end else
                begin
                    case S[ 3 ] of
                        '0' :
                              begin
                                  if( R[ length( R ) ] = 'y' ) then { else UNTESTED - this appears to always be true }
                                  begin
                                      R[ length( R ) ] := 'i' ;
                                  end ;
                                  if( R[ length( R ) ] = ' ' ) then
                                  begin
                                      R := copy( R, 1, length( R ) - 1 ) ; { UNTESTED - This appears to never occur }
                                  end ;
                                  R := R + 'eth' ;
                              end ;
                        '1' : R := R + 'first' ;
                        '2' : R := R + 'second' ;
                        '3' : R := R + 'third' ;
                        '4' : R := R + 'fourth' ;
                        '5' : R := R + 'fifth' ;
                        '6' : R := R + 'sixth' ;
                        '7' : R := R + 'seventh' ;
                        '8' : R := R + 'eighth' ;
                        '9' : R := R + 'ninth' ;
                    end ;
                end ;
            end ; { if( S[ 2 ] = '1' ) }
            if( ( length( R ) > 0 ) and ( length( Nam ) > 0 ) ) then
            begin
                R := R + ' ' + Nam ;
            end ;
            _Num := Edit( R, 8 or 16 or 128 ) ; { Remove excess spaces }
        end else
        begin
            _Num := '' ;
        end ;
    end ; { Num_To_English_Ordinal._Num }

var S : Ansistring ;

begin { Num_To_English_Ordinal }
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) ) then
    begin
        if( Numeric > 1E+103 ) then { One google is the largest we know about }
        begin
            Num_To_English_Ordinal := '' ;
            exit ;
        end ;
        if( Numeric = 0 ) then
        begin
            Num_To_English_Ordinal := 'zeroeth' ;
            exit ;
        end ;
        S := S + _Num( 1E100, 'google' ) ;
        if( Numeric >= 1E+36 ) then { except for google, don't know above decillion }
        begin
            Num_To_English_Ordinal := '' ;
            exit ;
        end ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E33, 'decillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E30, 'nonillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E27, 'octillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E24, 'septillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E21, 'sextillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E18, 'quintillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E15, 'quadrillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E12, 'trillion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E9, 'billion' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1E6, 'million' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        S := S + _Num( 1000, 'thousand' ) ;
        if( length( S ) > 0 ) then
        begin
            S := S + ' ' ;
        end ;
        if( Numeric = 0 ) then
        begin
            S := Edit( S, 128 ) ;
            if( ( copy( S, length( S ), 1 ) = 'e' ) or ( copy( S, length( S ) - 2, 3 ) = 'ion' ) ) then
            begin
                S := S + 'th' ;
            end else
            begin
                S := S + 'eth' ;
            end ;
        end else
        begin
            S := S + _Num( 1, '' ) ;
        end ;
    end ; { if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) ) }
    Num_To_English_Ordinal := S ;
end ; { Num_To_English_Ordinal }


function Num_To_Metric_Prefix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( Numeric = 1000000.0 ) then
    begin
        S := 'Mega' ;
    end else
    if( Numeric = 1000000000.0 ) then
    begin
        S := 'Giga' ;
    end else
    if( Numeric = 1000000000000.0 ) then
    begin
        S := 'Tera' ;
    end else
    if( Numeric = 1000000000000000.0 ) then
    begin
        S := 'Peta' ;
    end else
    if( Numeric = 1000000000000000000.0 ) then
    begin
        S := 'Exa' ;
    end else
    if( Numeric = 1E+21 ) then
    begin
        S := 'Zetta' ;
    end else
    if( Numeric = 1E+24 ) then
    begin
        S := 'Yotta' ; { UNTESTED due to compiler bugs }
    end else
    if( Num1( Numeric ) = '0.1' ) then
    begin
        S := 'Deci' ;
    end else
    if( Num1( Numeric ) = '0.01' ) then
    begin
        S := 'Centi' ;
    end else
    if( Num1( Numeric ) = '0.001' ) then
    begin
        S := 'Milli' ;
    end else
    if( Num1( Numeric ) = '0.000001' ) then
    begin
        S := 'Micro' ;
    end else
    if( Num1( Numeric ) = '0.000000001' ) then
    begin
        S := 'Nano' ;
    end else
    if( Numeric = 0.000000000001 ) then
    begin
        S := 'Pico' ; { UNTESTED due to compiler bugs }
    end else
    if( Numeric = 0.000000000000001 ) then
    begin
        S := 'Femto' ; { UNTESTED due to compiler bugs }
    end else
    if( Numeric = 0.000000000000000001 ) then
    begin
        S := 'Atto' ; { UNTESTED due to compiler bugs }
    end else
    if( Numeric = 0.000000000000000000001 ) then
    begin
        S := 'Zepto' ; { UNTESTED due to compiler bugs }
    end else
    if( Numeric = 0.000000000000000000000001 ) then
    begin
        S := 'Yocto' ; { UNTESTED due to compiler bugs }
    end else
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 6 ) and ( Numeric <= 1000 ) ) then
    begin
        case trunc( Numeric ) of
            6 : S := 'Hexa' ;
            10 : S := 'Deca' ;
            100 : S := 'Hecto' ;
            1000 : S := 'Kilo' ;
        end ;
    end ;
    Num_To_Metric_Prefix := S ;
end ;


function Num_To_Radix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 16 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Unary' ;
            2 : S := 'Binary' ;
            3 : S := 'Ternary' ;
            4 : S := 'Quatrinary' ;
            5 : S := 'Quinary' ;
            7 : S := 'Septimal' ;
            8 : S := 'Octal' ;
            10 : S := 'Decimal' ;
            12 : S := 'Duodecimal' ;
            16 : S := 'Hexadecimal' ;
        end ;
    end ;
    Num_To_Radix := S ;
end ;


function Num_To_Alt_Radix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 7 ) and ( Numeric <= 8 ) ) then
    begin
        case trunc( Numeric ) of
            7 : S := 'Septenary' ;
            8 : S := 'Octinary' ;
            else S := Num_To_Radix( Numeric ) ;
        end ;
    end ;
    Num_To_Alt_Radix := S ;
end ;


function Num_To_Group0( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 8 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Solo' ;
            2 : S := 'Duet' ;
            3 : S := 'Trio' ;
            4 : S := 'Quartet' ;
            5 : S := 'Quintet' ;
            6 : S := 'Sextet' ;
            7 : S := 'Septet' ;
            8 : S := 'Octet' ;
        end ;
    end ;
    Num_To_Group0 := S ;
end ;


function Num_To_Group1( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 3 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Unity' ;
            2 : S := 'Duality' ;
            3 : S := 'Trinity' ;
        end ;
    end ;
    Num_To_Group1 := S ;
end ;


function Num_To_Group2( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 10 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Monad' ;
            2 : S := 'Dyad' ;
            3 : S := 'Triad' ;
            5 : S := 'Pentad' ;
            6 : S := 'Hexad' ;
            7 : S := 'Heptad' ;
            8 : S := 'Octad' ;
            10 : S := 'Decad' ;
        end ;
    end ;
    Num_To_Group2 := S ;
end ;


function Num_To_Group3( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 8 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Simplex' ;
            2 : S := 'Duplex' ;
            3 : S := 'Triplex' ;
            4 : S := 'Quadruplex' ;
            6 : S := 'Sextuplex' ;
        end ;
    end ;
    Num_To_Group3 := S ;
end ;


function Num_To_Prefix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( Numeric = 0.5 ) then
    begin
        S := 'Hemi' ;
    end ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 8 ) ) then
    begin
        case trunc( Numeric ) of
            0 : S := 'Non' ;
            1 : S := 'Uni' ;
            2 : S := 'Bi' ;
            3 : S := 'Tri' ;
            5 : S := 'Penta' ;
            6 : S := 'Sexta' ;
            7 : S := 'Septi' ;
            8 : S := 'Octa' ;
        end ;
    end ;
    Num_To_Prefix := S ;
end ;


function Num_To_Alt_Prefix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 8 ) ) then
    begin
        case trunc( Numeric ) of
            0 : S := 'A' ;
            1 : S := 'Mono' ;
            2 : S := 'Di' ;
            3 : S := 'Tri' ;
            4 : S := 'Quad' ;
            5 : S := 'Quint' ;
            6 : S := 'Hexa' ;
            7 : S := 'Hepta' ;
            8 : S := 'Octo' ;
            10 : S := 'Deci' ;
            12 : S := 'Dihexa' ;
            15 : S := 'Quindeca' ;
            16 : S := 'Hexadeci' ;
            24 : S := 'Tetrahexa' ;
            48 : S := 'Hexocta' ;
            200 : S := 'Bicent' ;
            300 : S := 'Tercent' ;
            600 : S := 'Sexcent' ;
        end ;
    end ;
    Num_To_Alt_Prefix := S ;
end ;


function Num_To_Greek_Prefix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 8 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Mono' ;
            2 : S := 'Di' ;
            3 : S := 'Tri' ;
            4 : S := 'Tetra' ;
            5 : S := 'Penta' ;
            6 : S := 'Hex' ;
            7 : S := 'Hepta' ;
            8 : S := 'Octa' ;
        end ;
    end ;
    Num_To_Greek_Prefix := S ;
end ;


function Num_To_Multiplier( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( Numeric = 1.0 / 6.0 ) then
    begin
        S := 'Subsextuple' ; { UNTESTED due to compiler bug }
    end else
    if( Numeric = 0.25 ) then
    begin
        S := 'Quarter' ;
    end else
    if( Numeric = 0.5 ) then
    begin
        S := 'Half' ;
    end else
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 100 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Single' ;
            2 : S := 'Double' ;
            3 : S := 'Triple' ;
            4 : S := 'Quadruple' ;
            5 : S := 'Quintuple' ;
            6 : S := 'Sextuple' ;
            7 : S := 'Septuple' ;
            8 : S := 'Octuple' ;
            10 : S := 'Decuple' ;
            100 : S := 'Centuple' ;
        end ;
    end ;
    Num_To_Multiplier := S ;
end ;


function Num_To_Math( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 6 ) ) then
    begin
        case trunc( Numeric ) of
            0 : S := 'Niladic' ;
            1 : S := 'Monadic' ;
            2 : S := 'Dyadic' ;
            3 : S := 'Triadic' ;
            6 : S := 'Hexadic' ;
        end ;
    end ;
    Num_To_Math := S ;
end ;


function Num_To_Positional( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 600 ) ) then
    begin
        case trunc( Numeric ) of
            1 : S := 'Primary' ;
            2 : S := 'Secondary' ;
            3 : S := 'Tertiary' ;
            7 : S := 'Septuary' ;
            8 : S := 'Octonal' ;
            12 : S := 'Duodenary' ;
            60 : S := 'Sexagenary' ;
            100 : S := 'Centenary' ;
            200 : S := 'Bicentenary' ;
            300 : S := 'Tercentenary' ;
            600 : S := 'Sexcentenary' ;
        end ;
    end ;
    Num_To_Positional := S ;
end ;


function Num_To_2D( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 32767 ) ) then
    begin
        case trunc( Numeric ) of
            0 : S := 'Point' ;
            1 : S := 'Line' ;
            3 : S := 'Triangle' ;
            4 : S := 'Tetragon' ;
            5 : S := 'Pentagon' ;
            6 : S := 'Hexagon' ;
            7 : S := 'Heptagon' ;
            8 : S := 'Octagon' ;
            12 : S := 'Dihexagon' ;
            15 : S := 'Quindecagon' ;
            else S := 'Polygon' ;
        end ;
    end ;
    Num_To_2D := S ;
end ;


function Num_To_3D( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 32767 ) ) then
    begin
        case trunc( Numeric ) of
            0,1, 3 : ;
            2 : S := 'Plane' ;
            4 : S := 'Cube' ;
            5 : S := 'Pentahedron' ;
            6 : S := 'Hexahedron' ;
            8 : S := 'Octahedron' ;
            20 : S := 'Eicosahedron' ;
            24 : S := 'Tetrahexahedron' ;
            48 : S := 'Hexoctahedron' ;
            else S := 'Polyhedron' ;
        end ;
    end ;
    Num_To_3D := S ;
end ;


function Num_To_Government( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 0 ) and ( Numeric <= 32767 ) ) then
    begin
        case trunc( Numeric ) of
            0 : S := 'Anarchy' ;
            1 : S := 'Monarchy' ;
            4 : S := 'Tetrarchy' ;
            5 : S := 'Pentarchy' ;
            6 : S := 'Hexarchy' ;
            7 : S := 'Heptarchy' ;
            8 : S := 'Octarchy' ;
            else S := 'Oligarchy' ;
        end ;
    end ;
    Num_To_Government := S ;
end ;


function Num_To_Binary_Prefix( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( Numeric = 1024.0 ) then
    begin
        S := 'Kibi' ;
    end else
    if( Numeric = 1048576.0 ) then
    begin
        S := 'Mebi' ;
    end else
    if( Numeric = 1048576.0 * 1024.0 ) then
    begin
        S := 'Gibi' ;
    end else
    if( Numeric = 1073741824.0 * 1024.0 * 1024.0 ) then
    begin
        S := 'Pebi' ;
    end else
    if( Numeric = 1073741824.0 * 1024.0 * 1024.0 * 1024.0 ) then
    begin
        S := 'Exbi' ;
    end ;
    Num_To_Binary_Prefix := S ;
end ;


function Num_To_English_Ordinal_Multiplier( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    case trunc( Numeric ) of
        1 : S := 'Once' ;
        2 : S := 'Twice' ;
        3 : S := 'Thrice' ;
    end ;
    Num_To_English_Ordinal_Multiplier := S ;
end ;


function Num_To_Metric_Ab( Numeric : double ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( Numeric = 1000000.0 ) then
    begin
        S := 'M' ;
    end else
    if( Numeric = 1000000000.0 ) then
    begin
        S := 'G' ;
    end else
    if( Numeric = 1000000000000.0 ) then
    begin
        S := 'T' ;
    end else
    if( Numeric = 1000000000000000.0 ) then
    begin
        S := 'P' ;
    end else
    if( Numeric = 1000000000000000000.0 ) then
    begin
        S := 'E' ;
    end else
    if( Numeric = 1E+21 ) then
    begin
        S := 'Z' ;
    end else
    if( Numeric = 1E+24 ) then
    begin
        S := 'Y' ; { UNTESTED due to compiler bug }
    end else
    if( Num1( Numeric ) = '0.1' ) then
    begin
        S := 'd' ;
    end else
    if( Num1( Numeric ) = '0.01' ) then
    begin
        S := 'c' ;
    end else
    if( Num1( Numeric ) = '0.001' ) then
    begin
        S := 'm' ;
    end else
    if( Num1( Numeric ) = '0.000001' ) then
    begin
        S := 'u' ;
    end else
    if( Num1( Numeric ) = '0.000000001' ) then
    begin
        S := 'n' ;
    end else
    if( Numeric = 0.000000000001 ) then
    begin
        S := 'p' ; { UNTESTED due to compiler bug }
    end else
    if( Numeric = 0.000000000000001 ) then
    begin
        S := 'f' ; { UNTESTED due to compiler bug }
    end else
    if( Numeric = 0.000000000000000001 ) then
    begin
        S := 'a' ; { UNTESTED due to compiler bug}
    end else
    if( Numeric = 0.000000000000000000001 ) then
    begin
        S := 'z' ; { UNTESTED due to compiler bug }
    end else
    if( Numeric = 0.000000000000000000000001 ) then
    begin
        S := 'y' ; { UNTESTED due to compiler bug }
    end else
    if( ( int( Numeric ) = Numeric ) and ( Numeric >= 6 ) and ( Numeric <= 1000 ) ) then
    begin
        case trunc( Numeric ) of
            10 : S := 'da' ;
            100 : S := 'h' ;
            1000 : S := 'k' ;
        end ;
    end ;
    Num_To_Metric_Ab := S ;
end ;


{ TSLanguage methods... }

function TSEnglish_Language.Numeric_To_String( Numeric : double ;
    Typ : integer ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    case Typ of
        NTS_Arabic: S := Num1( Numeric ) ;
        NTS_Roman: S := Arabic_To_Roman( Numeric ) ;
        NTS_English: S := Num_To_English( Numeric ) ;
        NTS_Arabic_Ordinal: S := Num_To_Ordinal( Numeric ) ;
        NTS_English_Ordinal: S := Num_To_English_Ordinal( Numeric ) ;
        NTS_Metric_Prefix: S := Num_To_Metric_Prefix( Numeric ) ;
        NTS_Radix: S := Num_To_Radix( Numeric ) ;
        NTS_Alt_Radix: S := Num_To_Alt_Radix( Numeric ) ;
        NTS_Group0: S := Num_To_Group0( Numeric ) ;
        NTS_Group1: S := Num_To_Group1( Numeric ) ;
        NTS_Group2: S := Num_To_Group2( Numeric ) ;
        NTS_Group3: S := Num_To_Group3( Numeric ) ;
        NTS_Prefix: S := Num_To_Prefix( Numeric ) ;
        NTS_Alt_Prefix: S := Num_To_Alt_Prefix( Numeric ) ;
        NTS_Greek_Prefix: S := Num_To_Greek_Prefix( Numeric ) ;
        NTS_Multiplier: S := Num_To_Multiplier( Numeric ) ;
        NTS_Math: S := Num_To_Math( Numeric ) ;
        NTS_Positional: S := Num_To_Positional( Numeric ) ;
        NTS_2D: S := Num_To_2D( Numeric ) ;
        NTS_3D: S := Num_To_3D( Numeric ) ;
        NTS_Government: S := Num_To_Government( Numeric ) ;
        NTS_Metric_Abbreviation: S := Num_To_Metric_Ab( Numeric ) ;
        NTS_Binary_Prefix: S := Num_To_Binary_Prefix( Numeric ) ;
        NTS_Ordinal_Multiplier: S := Num_To_English_Ordinal_Multiplier( Numeric ) ;
    end ;
    Numeric_To_String := S ;
end ; { TSEnglish_Language.Numeric_To_String }


type Number_String_Array = array[ 1..19 ] of string ;

const Number_Strings : Number_String_Array =
      ( 'ONE', 'TWO', 'THREE', 'FOUR', 'FIVE', 'SIX', 'SEVEN', 'EIGHT', 'NINE', 'TEN',
        'ELEVEN', 'TWELVE', 'THIRTEEN', 'FOURTEEN', 'FIFTEEN', 'SIXTEEN', 'SEVENTEEN',
        'EIGHTEEN', 'NINETEEN' ) ;
      Ordinal_String : Number_String_Array =
      ( 'FIRST', 'SECOND', 'THIRD', 'FOURTH', 'FIFTH', 'SIXTH', 'SEVENTH', 'EIGHTH', 'NINTH', 'TENTH',
        'ELEVENTH', 'TWELFTH', 'THIRTEENTH', 'FOURTEENTH', 'FIFTEENTH', 'SIXTEENTH', 'SEVENTEENTH',
        'EIGHTEENTH', 'NINETEENTH' ) ;
      Plural_Ordinal_String : Number_String_Array =
      ( '', '', 'THIRDS', 'FOURTHS', 'FIFTHS', 'SIXTHS', 'SEVENTHS', 'EIGHTHS', 'NINTHS', 'TENTHS',
        'ELEVENTHS', 'TWELFTHS', 'THIRTEENTHS', 'FOURTEENTHS', 'FIFTEENTHS', 'SIXTEENTHS', 'SEVENTEENTHS',
        'EIGHTEENTHS', 'NINETEENTHS' ) ;
      Tens_Number_String : Number_String_Array =
      ( 'TEN', 'TWENTY', 'THIRTY', 'FORTY', 'FIFTY', 'SIXTY', 'SEVENTY', 'EIGHTY', 'NINETY',
        '', '', '', '', '', '', '', '', '', '' ) ;
      Tens_Ordinal_String : Number_String_Array =
      ( 'TENTH', 'TWENTIETH', 'THIRTIETH', 'FOURTIETH', 'FIFTIETH', 'SIXTIETH', 'SEVENTIETH', 'EIGHTIETH', 'NINTIETH',
        '', '', '', '', '', '', '', '', '', '' ) ;
      Plural_Tens_Ordinal_String : Number_String_Array =
      ( 'TENTHS', 'TWENTIETHS', 'THIRTIETHS', 'FOURTIETHS', 'FIFTIETHS', 'SIXTIETHS', 'SEVENTIETHS', 'EIGHTIETHS', 'NINTIETHS',
        '', '', '', '', '', '', '', '', '', '' ) ;
      Group_Number_String : Number_String_Array =
      ( 'HUNDRED', 'THOUSAND', 'MILLION', 'BILLION', 'TRILLION', 'QUADRILLION', 'QUINTILLION', 'SEPTILLION', 'OCTILLION',
        'NONILLION', 'DECILLION', '', '', '', '', '', '', '', '' ) ;


function Non_Ordinal_Prefix( S : Ansistring ) : boolean ;

begin
    S := Edit( S, 2 ) ; // no spaces
    Result := ( S = 'ONE' )
              or
              ( S = 'TWO' )
              or
              ( S = 'THREE' )
              or
              ( S = 'FOUR' )
              or
              ( S = 'FIVE' )
              or
              ( S = 'SIX' )
              or
              ( S = 'SEVEN' )
              or
              ( S = 'EIGHT' )
              or
              ( S = 'NINE' )
              or
              ( S = 'TEN' )
              or
              ( S = 'ELEVEN' )
              or
              ( S = 'TWELVE' )
              or
              ( S = 'THIRTEEN' )
              or
              ( S = 'FOURTEEN' )
              or
              ( S = 'FIFTEEN' )
              or
              ( S = 'SIXTEEN' )
              or
              ( S = 'SEVENTEEN' )
              or
              ( S = 'EIGHTEEN' )
              or
              ( S = 'NINETEEN' ) ;
end ;


function TSEnglish_Language.Numeric_Extent( S : Ansistring ) : integer ;

var Dummy, Last_Pos : integer ;
    Found : boolean ;
    Original, This_Word : Ansistring ;

begin
    S := Edit( S, 8 or 16 or 32 or 128 ) ; // Trim/collapse spaces, uppercase
    if( S = 'ZERO' ) then
    begin
        Numeric_Extent := 4 ;
        exit ;
    end ;
    Original := S ;
    Last_Pos := 0 ;
    while( S <> '' ) do
    begin
        Dummy := pos( ' ', S + ' ' ) ;
        This_Word := copy( S, 1, Dummy - 1 ) ;
        S := copy( S, Dummy + 1, length( S ) ) ;
        Found := False ;
        for Dummy := Low( Number_Strings ) to High( Number_Strings ) do
        begin
            if( Number_Strings[ Dummy ] = This_Word ) then
            begin
                Last_Pos := Last_Pos + length( This_Word ) + 1 ;
                Found := True ;
                break ;
            end ;
        end ;
        if( not Found ) then
        begin
            for Dummy := Low( Tens_Number_String ) to High( Tens_Number_String ) do
            begin
                if( Tens_Number_String[ Dummy ] = This_Word ) then
                begin
                    Last_Pos := Last_Pos + length( This_Word ) + 1 ;
                    Found := True ;
                    break ;
                end ;
            end ;
        end ;
        if( not Found ) then
        begin
            for Dummy := Low( Group_Number_String ) to High( Group_Number_String ) do
            begin
                if( Group_Number_String[ Dummy ] = This_Word ) then
                begin
                    Last_Pos := Last_Pos + length( This_Word ) + 1 ;
                    Found := True ;
                    break ;
                end ;
            end ;
        end ;
        if( not Found ) then
        begin
            for Dummy := Low( Ordinal_String ) to High( Ordinal_String ) do
            begin
                if(
                    ( Ordinal_String[ Dummy ] = This_Word )
                    or
                    ( Plural_Ordinal_String[ Dummy ] = This_Word )
                  ) then
                begin
                    if(
                        ( ( This_Word = 'FIRST' ) or ( This_Word = 'SECOND' ) )
                        and
                        Non_Ordinal_Prefix( copy( Original, 1, Last_Pos ) )
                      ) then
                    begin
                        Numeric_Extent := Last_Pos - 1 ;
                    end else
                    begin
                        Numeric_Extent := Last_Pos + length( This_Word ) ;
                    end ;
                    exit ;
                end ;
            end ;
        end ;
        if( not Found ) then
        begin
            for Dummy := Low( Tens_Ordinal_String ) to High( Tens_Ordinal_String ) do
            begin
                if(
                    ( Tens_Ordinal_String[ Dummy ] = This_Word )
                    or
                    ( Plural_Tens_Ordinal_String[ Dummy ] = This_Word )
                  ) then
                begin
                    Numeric_Extent := Last_Pos + length( This_Word ) ;
                    exit ;
                end ;
            end ;
        end ;
        if( not Found ) then // Word is not numeric
        begin
            break ;
        end ;
    end ; { while( S <> '' ) }
    Numeric_Extent := Last_Pos - 1 ;
end ;


function TSEnglish_Language.To_Numeric( S : Ansistring ; var Numeric : double ) : boolean ;

    function Token : Ansistring ;

    var Dummy : integer ;

    begin
        Dummy := pos( ' ', S + ' ' ) ;
        Token := copy( S, 1, Dummy - 1 ) ;
        S := copy( S, Dummy + 1, length( S ) ) ;
    end ;

var Accumulated : double ;
    Dummy : integer ;
    Loop : integer ;
    N : boolean ; { True if negative }
    T : Ansistring ; { Current token }

begin
    { Setup... }
    Numeric := 0 ;
    N := false ;
    To_Numeric := True ; { Assume success }
    S := Edit( S, 32 ) ; { Uppercase }
    if( ( S = 'ZERO' ) or ( S = 'ZEROETH' ) ) then
    begin
        exit ;
    end ;
    Accumulated := 0 ;
    T := Token ;
    if( T = 'POINT' ) then
    begin
        if( length( S ) = 0 ) then
        begin
            To_Numeric := False ;
            exit ;
        end ;
        val( '0.' + S, Numeric, Dummy ) ;
        if( Dummy <> 0 ) then
        begin
            To_Numeric := False ;
        end ;
        exit ;
    end ;
    if( T = 'NEGATIVE' ) then
    begin
        N := True ;
        if( length( S ) = 0 ) then { "NEGATIVE" by itself is not valid }
        begin
            To_Numeric := False ;
            exit ;
        end ;
    end else
    begin
        if( length( S ) = 0 ) then
        begin
            S := T ;
        end else
        begin
            S := T + ' ' + S ; { Put the token back }
        end ;
    end ;

    while( length( S ) > 0 ) do
    begin
        T := Token ; { Get next word }
        for Loop := 1 to 9 do
        begin
            if( T = Tens_Ordinal_String[ Loop ] ) then
            begin
                Accumulated := Accumulated + Loop * 10 ;
                T := '' ;
                break ;
            end ;
            if( copy( T, 1, length( Tens_Number_String[ Loop ] ) ) = Tens_Number_String[ Loop ] ) then
            begin
                Accumulated := Accumulated + Loop * 10 ;
                T := copy( T, length( Tens_Number_String[ Loop ] ) + 1, length( T ) ) ;
                break ;
            end ;
        end ; { for Loop := 1 to 9 }
        for Loop := 1 to 19 do
        begin
            if(
                ( T = Number_Strings[ Loop ] )
                or
                ( T = Ordinal_String[ Loop ] )
              ) then
            begin
                T := '' ;
                if(
                    ( copy( S + ' ', 1, 8 ) = 'HUNDRED ' )
                    or
                    ( copy( S + ' ', 1, 10 ) = 'HUNDREDTH ' )
                  ) then
                begin
                    T := Token ; { Swallow the "HUNDRED*" }
                    T := '' ;
                    Accumulated := Accumulated + Loop * 100 ;
                end else
                begin
                    Accumulated := Accumulated + Loop ;
                end ;
                break ;
            end ;
        end ; { for Loop := 1 to 19 }
        if( T = 'POINT' ) then
        begin
            Numeric := Numeric + Accumulated ;
            val( '0.' + S, Accumulated, Dummy ) ;
            if( Dummy = 0 ) then
            begin
                Numeric := Numeric + Accumulated ;
            end else
            begin
                To_Numeric := False ;
            end ;
            exit ;
        end ;
        if( ( T = 'GOOGLE' ) or ( T = 'GOOGLETH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E100 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'DECILLION' ) or ( T = 'DECILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E33 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'NONILLION' ) or ( T = 'NONILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E30 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'OCTILLION' ) or ( T = 'OCTILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E27 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'SEPTILLION' ) or ( T = 'SEPTILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E24 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'SEXTILLION' ) or ( T = 'SEXTILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E21 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'QUINTILLION' ) or ( T = 'QUINTILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E18 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'QUADRILLION' ) or ( T = 'QUADRILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E15 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'TRILLION' ) or ( T = 'TRILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E12 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'BILLION' ) or ( T = 'BILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E9 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'MILLION' ) or ( T = 'MILLIONTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1E6 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( ( T = 'THOUSAND' ) or ( T = 'THOUSANDTH' ) ) then
        begin
            Numeric := Numeric + Accumulated * 1000.0 ;
            Accumulated := 0 ;
            T := '' ;
            continue ;
        end ;
        if( length( T ) > 0 ) then { Unresolved word }
        begin
            To_Numeric := False ;
            exit ;
        end ;
    end ; { while( length( S ) > 0 ) }
    if( Accumulated > 0 ) then
    begin
        Numeric := Numeric + Accumulated ;
    end ;
    if( N ) then { Negative }
    begin
        Numeric := -Numeric ;
    end ;
end ;


const Max_Prefix = 87 ;

{$IFDEF FPC_OR_DELPHI}
var Prefix_Count : integer = 0 ;
{$ELSE}
var Prefix_Count : integer ;
{$ENDIF}
var Prefixes : array[ 0..Max_Prefix ] of string ;
    Prefix_Values : array[ 0..Max_Prefix ] of double ;

procedure Build_Prefixes ;

    procedure Add_Prefix( const S : Ansistring ; Value : double ) ;

    begin
        Prefixes[ Prefix_Count ] := S ;
        Prefix_Values[ Prefix_Count ] := Value ;
        Prefix_Count := Prefix_Count + 1 ;
    end ;

begin
    Add_Prefix( 'ATTO', 0.000000000000000001 ) ;
    Add_Prefix( 'BI', 2 ) ;
    Add_Prefix( 'CENTA', 100 ) ;
    Add_Prefix( 'CENTI', 0.01 ) ;
    Add_Prefix( 'CENT', 100 ) ;
    Add_Prefix( 'DA', 10 ) ;
    Add_Prefix( 'DI', 2 ) ;
    Add_Prefix( 'DY', 2 ) ;
    Add_Prefix( 'DECA', 10 ) ;
    Add_Prefix( 'DECI', 10 ) ;
    Add_Prefix( 'DEC', 10 ) ;
    Add_Prefix( 'DUODECI', 12 ) ;
    Add_Prefix( 'DUA', 2 ) ;
    Add_Prefix( 'DUODEN', 2 ) ;
    Add_Prefix( 'DUODE', 2 ) ;
    Add_Prefix( 'DUOD', 2 ) ;
    Add_Prefix( 'DUO', 2 ) ;
    Add_Prefix( 'EICOSA', 20 ) ;
    Add_Prefix( 'EXA', 1000000000000000000.0 ) ;
    Add_Prefix( 'FEMTO', 0.000000000000001 ) ;
    Add_Prefix( 'GIGA', 1000000000.0 ) ;
    Add_Prefix( 'HECTO', 100 ) ;
    Add_Prefix( 'HECTA', 100 ) ;
    Add_Prefix( 'HECTI', 100 ) ;
    Add_Prefix( 'HECTE', 100 ) ;
    Add_Prefix( 'HEPTA', 7 ) ;
    Add_Prefix( 'HEPTI', 7 ) ;
    Add_Prefix( 'HEPTO', 7 ) ;
    Add_Prefix( 'HEPTE', 7 ) ;
    Add_Prefix( 'HEPT', 7 ) ;
    Add_Prefix( 'HEXADECI', 16 ) ;
    Add_Prefix( 'HEXA', 6 ) ;
    Add_Prefix( 'HEXI', 6 ) ;
    Add_Prefix( 'HEX', 6 ) ;
    Add_Prefix( 'ICOSA', 20 ) ;
    Add_Prefix( 'KILO', 1000 ) ;
    Add_Prefix( 'MEGA', 1000000.0 ) ;
    Add_Prefix( 'MICRO', 0.000001 ) ;
    Add_Prefix( 'MILLI', 0.001 ) ;
    Add_Prefix( 'MON', 1 ) ;
    Add_Prefix( 'NANO', 0.000000001 ) ;
    Add_Prefix( 'NILA', 0 ) ;
    Add_Prefix( 'NIL', 0 ) ;
    Add_Prefix( 'NONA', 0 ) ;
    Add_Prefix( 'NON', 0 ) ;
    Add_Prefix( 'OCTA', 8 ) ;
    Add_Prefix( 'OCTI', 8 ) ;
    Add_Prefix( 'OCTO', 8 ) ;
    Add_Prefix( 'OCT', 8 ) ;
    Add_Prefix( 'PECTA', 1000000000000000.0 ) ;
    Add_Prefix( 'PENTA', 5 ) ;
    Add_Prefix( 'PENT', 5 ) ;
    Add_Prefix( 'PICO', 0.000000000001 ) ;
    Add_Prefix( 'QUADRA', 4 ) ;
    Add_Prefix( 'QUADRI', 4 ) ;
    Add_Prefix( 'QUAD', 4 ) ;
    Add_Prefix( 'QUAR', 4 ) ;
    Add_Prefix( 'QUATRI', 4 ) ;
    Add_Prefix( 'QUATRO', 4 ) ;
    Add_Prefix( 'QUINDECA', 15 ) ;
    Add_Prefix( 'QUINTA', 5 ) ;
    Add_Prefix( 'QUINT', 5 ) ;
    Add_Prefix( 'QUIN', 5 ) ;
    Add_Prefix( 'SEPTI', 7 ) ;
    Add_Prefix( 'SEPTA', 7 ) ;
    Add_Prefix( 'SEPTO', 7 ) ;
    Add_Prefix( 'SEPTE', 7 ) ;
    Add_Prefix( 'SEPTU', 7 ) ;
    Add_Prefix( 'SEPT', 7 ) ;
    Add_Prefix( 'SEP', 7 ) ;
    Add_Prefix( 'SEXA', 6 ) ;
    Add_Prefix( 'SEXI', 6 ) ;
    Add_Prefix( 'SEXO', 6 ) ;
    Add_Prefix( 'SEX', 6 ) ;
    Add_Prefix( 'SEXTA', 6 ) ;
    Add_Prefix( 'SUBSEX', 1.0 / 6.0 ) ;
    Add_Prefix( 'TERA', 1000000000000.0 ) ;
    Add_Prefix( 'TERTI', 3 ) ;
    Add_Prefix( 'TERT', 3 ) ;
    Add_Prefix( 'TER', 3 ) ;
    Add_Prefix( 'TRI', 3 ) ;
    Add_Prefix( 'TETRA', 4 ) ;
    Add_Prefix( 'TETRO', 4 ) ;
    Add_Prefix( 'UN', 1 ) ;
    Add_Prefix( 'YOCTO', 0.000000000000000000000001 ) ;
    Add_Prefix( 'YOTTA', 1E+24 ) ;
    Add_Prefix( 'ZEPTO', 0.000000000000000000001 ) ;
    Add_Prefix( 'ZETTA', 1E+21 ) ;
end ; { Build_Prefixes }


type Roman_Array = array[ 1..7 ] of integer ;

const Roman_Digits = 'IVXLCDM' ;
      Roman_Values : Roman_Array = ( 1, 5, 10, 50, 100, 500, 1000 ) ;

function TSEnglish_Language.String_To_Numeric( S : Ansistring ;
  var Numeric : double ; var Typ : integer ) : boolean ;

var Digit, Pre_Digit : Ansichar ;
    Dummy : integer ;
    Found : boolean ;
    Index, Loop : integer ;
    Original_S : Ansistring ;
    Work : Ansistring ;

begin
    { Setup... }
    String_To_Numeric := True ; { Assume success }
    Original_S := Edit( S, 1 or 4 or 8 or 16 or 128 ) ;
    S := Edit( Original_S, 32 ) ;
    if( length( S ) = 0 ) then
    begin
        String_To_Numeric := False ;
        exit ;
    end ;

    { Try the descrete values... }
    if( S = 'ONCE' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Ordinal_Multiplier ;
        exit ;
    end ;
    if( S = 'TWICE' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Ordinal_Multiplier ;
        exit ;
    end ;
    if( S = 'THRICE' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Ordinal_Multiplier ;
        exit ;
    end ;
    if( S = 'MEGA' ) then
    begin
        Numeric := 1000000.0 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'MEGABINARY' ) then
    begin
        Numeric := 1048576.0 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'MEBI' ) then
    begin
        Numeric := 1048576.0 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'GIGA' ) then
    begin
        Numeric := 1000000000.0 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'GIGABINARY' ) then
    begin
        Numeric := 1073741824.0 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'GIBI' ) then
    begin
        Numeric := 1073741824.0 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'TERA' ) then
    begin
        Numeric := 1000000000000.0 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'TERABINARY' ) then
    begin
        Numeric := 1073741824.0 * 1024 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'TEBI' ) then
    begin
        Numeric := 1073741824.0 * 1024 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'PETABINARY' ) then
    begin
        Numeric := 1073741824.0 * 1024 * 1024 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'PEBI' ) then
    begin
        Numeric := 1073741824.0 * 1024 * 1024 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'PECTA' ) then
    begin
        Numeric := 1000000000000000.0 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'PETA' ) then
    begin
        Numeric := 1000000000000000.0 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'EXA' ) then
    begin
        Numeric := 1000000000000000000.0 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'EXABINARY' ) then
    begin
        Numeric := 1073741824.0 * 1024 * 1024 * 1024 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'EXBI' ) then
    begin
        Numeric := 1073741824.0 * 1024 * 1024 * 1024 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'ZETTA' ) then
    begin
        Numeric := 1E+21 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'YOTTA' ) then
    begin
        Numeric := 1E+24 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'DECI' ) then
    begin
        Numeric := 0.1 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'CENTI' ) then
    begin
        Numeric := 0.01 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'MILLI' ) then
    begin
        Numeric := 0.001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'MICRO' ) then
    begin
        Numeric := 0.000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'NANO' ) then
    begin
        Numeric := 0.000000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'PICO' ) then
    begin
        Numeric := 0.000000000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'FEMTO' ) then
    begin
        Numeric := 0.000000000000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'ATTO' ) then
    begin
        Numeric := 0.000000000000000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'ZEPTO' ) then
    begin
        Numeric := 0.000000000000000000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'YOCTO' ) then
    begin
        Numeric := 0.000000000000000000000001 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'HEXA' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'DECA' ) then
    begin
        Numeric := 10 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'HECTO' ) then
    begin
        Numeric := 100 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'KILO' ) then
    begin
        Numeric := 1000 ;
        Typ := NTS_Metric_Prefix ;
        exit ;
    end ;
    if( S = 'KILOBINARY' ) then
    begin
        Numeric := 1024.0 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'KIBI' ) then
    begin
        Numeric := 1024.0 ;
        Typ := NTS_Binary_Prefix ;
        exit ;
    end ;
    if( S = 'UNARY' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'BINARY' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'TERNARY' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'QUATRINARY' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'QUINARY' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'SEPTIMAL' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'OCTAL' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'DECIMAL' ) then
    begin
        Numeric := 10 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'DUODECIMAL' ) then
    begin
        Numeric := 12 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'HEXADECIMAL' ) then
    begin
        Numeric := 16 ;
        Typ := NTS_Radix ;
        exit ;
    end ;
    if( S = 'SEPTENARY' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Alt_Radix ;
        exit ;
    end ;
    if( S = 'OCTINARY' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Alt_Radix ;
        exit ;
    end ;
    if( S = 'SOLO' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'DUET' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'TRIO' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'QUARTET' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'QUINTET' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'SEXTET' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'SEPTET' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'OCTET' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Group0 ;
        exit ;
    end ;
    if( S = 'UNITY' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Group1 ;
        exit ;
    end ;
    if( S = 'DUALITY' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Group1 ;
        exit ;
    end ;
    if( S = 'TRINITY' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Group1 ;
        exit ;
    end ;
    if( S = 'MONAD' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'DYAD' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'TRIAD' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'PENTAD' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'HEXAD' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'HEPTAD' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'OCTAD' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'DECAD' ) then
    begin
        Numeric := 10 ;
        Typ := NTS_Group2 ;
        exit ;
    end ;
    if( S = 'SIMPLEX' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Group3 ;
        exit ;
    end ;
    if( S = 'DUPLEX' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Group3 ;
        exit ;
    end ;
    if( S = 'TRIPLEX' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Group3 ;
        exit ;
    end ;
    if( S = 'QUADRUPLEX' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Group3 ;
        exit ;
    end ;
    if( S = 'SEXTUPLEX' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Group3 ;
        exit ;
    end ;
    if( S = 'HEMI' ) then
    begin
        Numeric := 0.5 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'NON' ) then
    begin
        Numeric := 0 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'UNI' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'BI' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'TRI' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'PENTA' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'SEXTA' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'SEPTI' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'OCTA' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Prefix ;
        exit ;
    end ;
    if( S = 'SUBSEXTUPLE' ) then
    begin
        Numeric := 1.0 / 6.0 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'QUARTER' ) then
    begin
        Numeric := 0.25 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'HALF' ) then
    begin
        Numeric := 0.5 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'SINGLE' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'DOUBLE' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'TRIPLE' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'QUADRUPLE' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'QUINTUPLE' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'SEXTUPLE' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'SEPTUPLE' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'OCTUPLE' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'DECUPLE' ) then
    begin
        Numeric := 10 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'CENTUPLE' ) then
    begin
        Numeric := 100 ;
        Typ := NTS_Multiplier ;
        exit ;
    end ;
    if( S = 'NILADIC' ) then
    begin
        Numeric := 0 ;
        Typ := NTS_Math ;
        exit ;
    end ;
    if( S = 'MONADIC' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Math ;
        exit ;
    end ;
    if( S = 'DYADIC' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Math ;
        exit ;
    end ;
    if( S = 'TRIADIC' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Math ;
        exit ;
    end ;
    if( S = 'HEXADIC' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Math ;
        exit ;
    end ;
    if( S = 'PRIMARY' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'SECONDARY' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'TERTIARY' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'SEPTUARY' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'OCTONAL' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'DUODENARY' ) then
    begin
        Numeric := 12 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'SEXAGENARY' ) then
    begin
        Numeric := 60 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'CENTENARY' ) then
    begin
        Numeric := 100 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'BICENTENARY' ) then
    begin
        Numeric := 200 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'TERCENTENARY' ) then
    begin
        Numeric := 300 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'SEXCENTENARY' ) then
    begin
        Numeric := 600 ;
        Typ := NTS_Positional ;
        exit ;
    end ;
    if( S = 'POINT' ) then
    begin
        Numeric := 0 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'LINE' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'TRIANGLE' ) then
    begin
        Numeric := 3 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'SQUARE' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'TETRAGON' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'QUADRANGLE' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'PARALLELOGRAM' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'RHOMBUS' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'RHOMBOID' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'OBLONG' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'TRAPEZOID' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'TRAPEZIUM' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'PENTAGON' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'HEXAGON' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'HEPTAGON' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'OCTAGON' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'DIHEXAGON' ) then
    begin
        Numeric := 12 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'QUINDECAGON' ) then
    begin
        Numeric := 13 ;
        Typ := NTS_2D ;
        exit ;
    end ;
    if( S = 'PLANE' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'CUBE' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'PENTAHEDRON' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'HEXAHEDRON' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'OCTAHEDRON' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( ( S = 'EICOSAHEDRON' ) or ( S = 'ICOSAHEDRON' ) ) then
    begin
        Numeric := 20 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'TETRAHEXAHEDRON' ) then
    begin
        Numeric := 24 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'HEXOCTAHEDRON' ) then
    begin
        Numeric := 48 ;
        Typ := NTS_3D ;
        exit ;
    end ;
    if( S = 'ANARCHY' ) then
    begin
        Numeric := 0 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'MONARCHY' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'TETRARCHY' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'PENTARCHY' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'HEXARCHY' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'HEPTARCHY' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'OCTARCHY' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Government ;
        exit ;
    end ;
    if( S = 'A' ) then
    begin
        Numeric := 0 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'MONO' ) then
    begin
        Numeric := 1 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'DI' ) then
    begin
        Numeric := 2 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'QUAD' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'QUINT' ) then
    begin
        Numeric := 5 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'HEPTA' ) then
    begin
        Numeric := 7 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'OCTO' ) then
    begin
        Numeric := 8 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'HEXADECI' ) then
    begin
        Numeric := 16 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'BICENT' ) then
    begin
        Numeric := 200 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'TERCENT' ) then
    begin
        Numeric := 300 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'SEXCENT' ) then
    begin
        Numeric := 600 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'DIHEXA' ) then
    begin
        Numeric := 12 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'QUINDECA' ) then
    begin
        Numeric := 15 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'TETRAHEXA' ) then
    begin
        Numeric := 24 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'HEXOCTA' ) then
    begin
        Numeric := 48 ;
        Typ := NTS_Alt_Prefix ;
        exit ;
    end ;
    if( S = 'TETRA' ) then
    begin
        Numeric := 4 ;
        Typ := NTS_Greek_Prefix ;
        exit ;
    end ;
    if( S = 'HEX' ) then
    begin
        Numeric := 6 ;
        Typ := NTS_Greek_Prefix ;
        exit ;
    end ;
    if( S = 'G' ) then
    begin
        Numeric := 1000000000.0 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( S = 'Y' ) then
    begin
        Numeric := 1E+24 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'p' ) then
    begin
        Numeric := 0.000000000001 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'f' ) then
    begin
        Numeric := 0.000000000000001 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'z' ) then
    begin
        Numeric := 0.000000000000000000001 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'h' ) then
    begin
        Numeric := 100 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'k' ) then
    begin
        Numeric := 1000 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( S = 'T' ) then
    begin
        Numeric := 1000000000000.0 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( S = 'P' ) then
    begin
        Numeric := 1000000000000000.0 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( S = 'E' ) then
    begin
        Numeric := 1000000000000000000.0 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( S = 'Z' ) then
    begin
        Numeric := 1E+21 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'u' ) then
    begin
        Numeric := 0.000001 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'n' ) then
    begin
        Numeric := 0.000000001 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;
    if( Original_S = 'da' ) then
    begin
        Numeric := 10.0 ;
        Typ := NTS_Metric_Abbreviation ;
        exit ;
    end ;

    { Now try Roman numerals... }
    Found := True ;
    for Loop := 1 to length( S ) do
    begin
        Index := pos( S[ Loop ], Roman_Digits ) ;
        if( Index = 0 ) then
        begin
            Found := False ;
            break ;
        end ;
    end ; { for Loop := 1 to length( S ) }
    if( Found ) then
    begin
        Numeric := 0 ;
        Loop := 1 ;
        while( Loop <= length( S ) ) do
        begin
            Index := pos( S[ Loop ], Roman_Digits ) ;
            if( ( Loop = length( S ) ) or ( S[ Loop ] = copy( S, Loop + 1, 1 ) ) ) then
            begin
                Numeric := Numeric + Roman_Values[ Index ] ;
            end else
            begin
                if( Index > pos( S[ Loop + 1 ], Roman_Digits ) ) then
                begin
                    Numeric := Numeric + Roman_Values[ Index ] ;
                end else
                begin
                    Numeric := Numeric - Roman_Values[ Index ] ;
                    inc( Loop ) ;
                    Index := pos( S[ Loop ], Roman_Digits ) ;
                    Numeric := Numeric + Roman_Values[ Index ] ;
                end ;
            end ;
            inc( Loop ) ;
        end ;
        Typ := NTS_Roman ;
        exit ;
    end ; { if( Found ) }

    { Now try Arabic ordinal }
    if( length( S ) > 2 ) then
    begin
        Found := True ;
        for Loop := 1 to length( S ) - 2 do
        begin
            if( pos( S[ Loop ], '0123456789' ) = 0 ) then
            begin
                Found := False ;
                break ;
            end ;
        end ;
        Work := copy( S, length( S ) - 1, 2 ) ;
        if( ( Work <> 'ST' ) and ( Work <> 'RD' ) and ( Work <> 'ND' ) and ( Work <> 'TH' ) ) then
        begin
            Found := False ;
        end ;
        Digit := S[ length( S ) - 2 ] ;
        Pre_Digit := ' ' ;
        if( length( S ) > 3 ) then
        begin
            Pre_Digit := S[ length( S ) - 3 ] ;
        end ;
        if( ( Work = 'ST' ) and ( Digit <> '1' ) ) then
        begin
            Found := False ;
        end ;
        if( ( Work = 'ND' ) and ( Digit <> '2' ) ) then
        begin
            Found := False ;
        end ;
        if( ( Work = 'RD' ) and ( Digit <> '3' ) ) then
        begin
            Found := False ;
        end ;
        if(
            ( Work <> 'TH' )
            and
            ( ( pos( Digit, '123' ) = 0 ) or ( Pre_Digit = '1' ) )
          ) then
        begin
            Found := False ;
        end ;
        if( Found ) then
        begin
            val( copy( S, 1, length( S ) - 2 ), Numeric, Loop ) ;
            Typ := NTS_Arabic_Ordinal ;
            exit ;
        end ;
    end ; { if( length( S ) > 2 ) }

    { Now try English }
    if( To_Numeric( S, Numeric ) ) then
    begin
        if(
            ( pos( 'ETH', S ) = 0 )
            and
            ( pos( 'FIRST', S ) = 0 )
            and
            ( pos( 'SECOND', S ) = 0 )
            and
            ( pos( 'THIRD', S ) = 0 )
            and
            ( pos( 'FOURTH', S ) = 0 )
            and
            ( pos( 'FIFTH', S ) = 0 )
            and
            ( pos( 'SIXTH', S ) = 0 )
            and
            ( pos( 'EIGHTH', S ) = 0 )
            and
            ( pos( 'NTH', S ) = 0 )
            and
            ( pos( 'DTH', S ) = 0 )
          ) then
        begin
            Typ := NTS_English ;
        end else
        begin
            Typ := NTS_English_Ordinal ;
        end ;
        exit ;
    end ; { if( English_To_Numeric( S, Numeric ) ) }

    val( S, Numeric, Dummy ) ;
    if( Dummy = 0 ) then
    begin
        Typ := NTS_Arabic ;
        exit ;
    end ;
    String_To_Numeric := False ;
end ;


function TSEnglish_Language.Generic_String_To_Numeric( S : Ansistring ;
  var Numeric : double ; var Typ : integer ) : boolean ;

var Prefix{, Suffix} : Ansistring ;

    function Get_Suffix( const _S : Ansistring ) : boolean ;

    begin
        if( copy( S, length( S ) - length( _S ) + 1 , length( _S ) ) = _S ) then
        begin
            Get_Suffix := True ;
            Prefix := copy( S, 1, length( S ) - length( _S ) ) ;
//            Suffix := _S ;
        end else
        begin
            Get_Suffix := False ;
        end ;
    end ;

var Dummy : integer ;
    Found : boolean ;

begin
    { Setup... }
    Generic_String_To_Numeric := False ; { Assume failure }
    S := Edit( S, 1 or 4 or 8 or 16 or 32 or 128 ) ;
    if( length( S ) = 0 ) then
    begin
        exit ;
    end ;

    { Split into prefix/suffix... }
    if( Get_Suffix( 'GON' ) ) then
    begin
        Typ := NTS_2D ;
    end else
    if( Get_Suffix( 'GRAM' ) ) then
    begin
        Typ := NTS_2D ;
    end else
    if( Get_Suffix( 'ANGLE' ) ) then
    begin
        Typ := NTS_2D ;
    end else
    if( Get_Suffix( 'OID' ) ) then
    begin
        Typ := NTS_2D ;
    end else
    if( Get_Suffix( 'HEDRON' ) ) then
    begin
        Typ := NTS_3D ;
    end else
    if( Get_Suffix( 'BINARY' ) ) then
    begin
        Typ := NTS_Binary_Prefix ;
    end else
    if( Get_Suffix( 'ARCHY' ) ) then
    begin
        Typ := NTS_Government ;
    end else
    if( Get_Suffix( 'TET' ) ) then
    begin
        Typ := NTS_Group0 ;
    end else
    if( Get_Suffix( 'ITY' ) ) then
    begin
        Typ := NTS_Group1 ;
    end else
    if( Get_Suffix( 'ARY' ) ) then
    begin
        Typ := NTS_Radix ;
    end else
    if( Get_Suffix( 'AD' ) ) then
    begin
        Typ := NTS_Group2 ;
    end else
    if( Get_Suffix( 'PLEX' ) ) then
    begin
        Typ := NTS_Group3 ;
    end else
    begin
        exit ;
    end ;
    if( length( Prefix ) = 0 ) then
    begin
        exit ;
    end ;

    { Now calculate numeric value of prefix }
    if( Prefix_Count = 0 ) then
    begin
        Build_Prefixes ;
    end ;
    Numeric := 0 ;
    while( length( Prefix ) > 0 ) do
    begin
        Found := False ;
        for Dummy := 0 to Max_Prefix do
        begin
            S := Prefixes[ Dummy ] ;
            if( copy( Prefix, 1, length( S ) ) = S ) then
            begin
                Prefix := copy( Prefix, length( S ) + 1, length( Prefix ) ) ;
                Found := True ;
                if( Numeric = 0 ) then
                begin
                    Numeric := Prefix_Values[ Dummy ] ;
                end else
                begin
                    Numeric := Numeric * Prefix_Values[ Dummy ] ;
                end ;
                break ;
            end ; { if( copy( Prefix, 1, length( S ) ) = S ) }
        end ; { for Dummy := 0 to Max_Prefix }
        if( not Found ) then { Something is unrecognized }
        begin
            exit ;
        end ;
    end ; { while( length( Prefix ) > 0 ) }
    Generic_String_To_Numeric := True ;
end ;


{ API... }

var Languages : TString_Collection ;
    LObjects : TCollection ;

procedure Add_Language( Lang : Ansistring ; O : PSLanguage ) ;

begin
    if( ( Lang = '' ) or ( O = nil ) ) then
    begin
        exit ;
    end ;
    Languages.Insert( Lang ) ;
    LObjects.Insert( O ) ;
end ;


procedure Remove_Language( O : PSLanguage ) ;

var Loop : integer ;

begin
    for Loop := 0 to LObjects.Count - 1 do
    begin
        if( O = LObjects.At( Loop ) ) then
        begin
            Languages.AtDelete( Loop ) ;
            LObjects.AtDelete( Loop ) ;
            exit ;
        end ;
    end ;
end ;


function Get_Language( Lang : Ansistring ) : PSLanguage ;

var Loop : integer ;

begin
    Loop := Languages.Index_Of_Uppercase( Lang ) ;
    if( Loop <> -1 ) then
    begin
        Get_Language := PSLanguage( LObjects.At( Loop ) ) ;
        exit ;
    end ;

    { If we get here, we couldn't find the defined language... }
    if( English_Language = nil ) then
    begin
        English_Language := new( PSEnglish_Language, Init ) ;
    end ;
    Get_Language := English_Language ;
end ;


{$IFDEF CC_Regression_Test}
procedure Error ;

begin
    writeln( 'Error' ) ;
    halt ;
end ;


procedure Test_Numeric_To_String ;

begin
    if( Get_Language( '' ).Numeric_To_String( 1, NTS_Arabic ) <> '1' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 11, NTS_Arabic ) <> '11' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 19, NTS_Arabic ) <> '19' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 29, NTS_Arabic ) <> '29' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 111, NTS_Arabic ) <> '111' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Roman ) <> 'I' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 4, NTS_Roman ) <> 'IV' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 5, NTS_Roman ) <> 'V' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 6, NTS_Roman ) <> 'VI' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 9, NTS_Roman ) <> 'IX' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 10, NTS_Roman ) <> 'X' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 18, NTS_Roman ) <> 'XVIII' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 19, NTS_Roman ) <> 'XIX' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 29, NTS_Roman ) <> 'XXIX' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1900, NTS_Roman ) <> 'MCM' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 600, NTS_Roman ) <> 'DC' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 490, NTS_Roman ) <> 'CDXC' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 40, NTS_Roman ) <> 'XL' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 50, NTS_Roman ) <> 'L' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_English ) <> 'one' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 11, NTS_English ) <> 'eleven' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 19, NTS_English ) <> 'nineteen' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 29, NTS_English ) <> 'twentynine' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 111, NTS_English ) <> 'one hundred eleven' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1111.2, NTS_English ) <> 'one thousand one hundred eleven point 2' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E+104, NTS_English ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0, NTS_English ) <> 'zero' ) then
    begin
        Error ;
    end ;
  {$IFDEF INT64_CONSTANTS}
    if(Get_Language( '' ).Numeric_To_String( 1001001000000, NTS_English ) <> 'one trillion one billion one million' ) then
    begin
        Error ;
    end ;
  {$ENDIF}
    if(Get_Language( '' ).Numeric_To_String( -1, NTS_English ) <> 'negative one' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.1, NTS_English ) <> 'point 1' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E100, NTS_English ) <> 'one google' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E37, NTS_English ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E33, NTS_English ) <> 'one decillion' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E30, NTS_English ) <> 'one nonillion' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E27, NTS_English ) <> 'one octillion' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E24, NTS_English ) <> 'one septillion' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E21, NTS_English ) <> 'one sextillion' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E18, NTS_English ) <> 'one quintillion' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E15, NTS_English ) <> 'one quadrillion' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Arabic_Ordinal ) <> '1st' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( -1, NTS_Arabic_Ordinal ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 11, NTS_Arabic_Ordinal ) <> '11th' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 32, NTS_Arabic_Ordinal ) <> '32nd' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 29, NTS_Arabic_Ordinal ) <> '29th' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 111, NTS_Arabic_Ordinal ) <> '111th' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_English_Ordinal ) <> 'first' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 11, NTS_English_Ordinal ) <> 'eleventh' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 32, NTS_English_Ordinal ) <> 'thirtysecond' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 29, NTS_English_Ordinal ) <> 'twentyninth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 111, NTS_English_Ordinal ) <> 'one hundred eleventh' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 120, NTS_English_Ordinal ) <> 'one hundred twentieth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1120, NTS_English_Ordinal ) <> 'one thousand one hundred twentieth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E+104, NTS_English_Ordinal ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0, NTS_English_Ordinal ) <> 'zeroeth' ) then
    begin
        Error ;
    end ;
  {$IFDEF INT64_CONSTANTS}
    if(Get_Language( '' ).Numeric_To_String( 1001001000000, NTS_English_Ordinal ) <> 'one trillion one billion one millionth' ) then
    begin
        Error ;
    end ;
  {$ENDIF}
    if(Get_Language( '' ).Numeric_To_String( 1E100, NTS_English_Ordinal ) <> 'one googleth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E37, NTS_English_Ordinal ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E33, NTS_English_Ordinal ) <> 'one decillionth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E30, NTS_English_Ordinal ) <> 'one nonillionth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E27, NTS_English_Ordinal ) <> 'one octillionth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E24, NTS_English_Ordinal ) <> 'one septillionth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E21, NTS_English_Ordinal ) <> 'one sextillionth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E18, NTS_English_Ordinal ) <> 'one quintillionth' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E15, NTS_English_Ordinal ) <> 'one quadrillionth' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1000000.0, NTS_Metric_Prefix ) <> 'Mega' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000.0, NTS_Metric_Prefix ) <> 'Giga' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000000.0, NTS_Metric_Prefix ) <> 'Tera' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000000000.0, NTS_Metric_Prefix ) <> 'Pecta' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000000000000.0, NTS_Metric_Prefix ) <> 'Exa' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E+21, NTS_Metric_Prefix ) <> 'Zetta' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.0, NTS_Metric_Prefix ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.1, NTS_Metric_Prefix ) <> 'Deci' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.01, NTS_Metric_Prefix ) <> 'Centi' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.001, NTS_Metric_Prefix ) <> 'Milli' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.000001, NTS_Metric_Prefix ) <> 'Micro' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.000000001, NTS_Metric_Prefix ) <> 'Nano' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 6.0, NTS_Metric_Prefix ) <> 'Hexa' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Radix ) <> 'Unary' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Radix ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 7, NTS_Alt_Radix ) <> 'Septenary' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Alt_Radix ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Group0 ) <> 'Solo' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Group0 ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Group1 ) <> 'Unity' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Group1 ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Group2 ) <> 'Monad' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Group2 ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Group3 ) <> 'Simplex' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Group3 ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0, NTS_Prefix ) <> 'Non' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Prefix ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.5, NTS_Prefix ) <> 'Hemi' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0, NTS_Alt_Prefix ) <> 'A' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Alt_Prefix ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Greek_Prefix ) <> 'Mono' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Greek_Prefix ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0.25, NTS_Multiplier ) <> 'Quarter' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.5, NTS_Multiplier ) <> 'Half' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1.0, NTS_Multiplier ) <> 'Single' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0, NTS_Math ) <> 'Niladic' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 100, NTS_Math ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Positional ) <> 'Primary' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000, NTS_Positional ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0, NTS_Positional ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0, NTS_2D ) <> 'Point' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( -1, NTS_2D ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 2, NTS_3D ) <> 'Plane' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( -1, NTS_3D ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 0, NTS_Government ) <> 'Anarchy' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( -1, NTS_Government ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1000000.0, NTS_Metric_Abbreviation ) <> 'M' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.0, NTS_Metric_Abbreviation ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000.0, NTS_Metric_Abbreviation ) <> 'G' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000000.0, NTS_Metric_Abbreviation ) <> 'T' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000000000.0, NTS_Metric_Abbreviation ) <> 'P' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1000000000000000000.0, NTS_Metric_Abbreviation ) <> 'E' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1E+21, NTS_Metric_Abbreviation ) <> 'Z' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.1, NTS_Metric_Abbreviation ) <> 'd' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.01, NTS_Metric_Abbreviation ) <> 'c' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.001, NTS_Metric_Abbreviation ) <> 'm' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.000001, NTS_Metric_Abbreviation ) <> 'u' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0.000000001, NTS_Metric_Abbreviation ) <> 'n' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 10, NTS_Metric_Abbreviation ) <> 'da' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1, NTS_Ordinal_Multiplier ) <> 'Once' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 2, NTS_Ordinal_Multiplier ) <> 'Twice' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 3, NTS_Ordinal_Multiplier ) <> 'Thrice' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 13, NTS_Ordinal_Multiplier ) <> '' ) then
    begin
        Error ;
    end ;

    if(Get_Language( '' ).Numeric_To_String( 1024.0, NTS_Binary_Prefix ) <> 'Kibi' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 0, NTS_Binary_Prefix ) <> '' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1048576.0, NTS_Binary_Prefix ) <> 'Mebi' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1048576.0 * 1024.0, NTS_Binary_Prefix ) <> 'Gibi' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1073741824.0 * 1024.0 * 1024.0, NTS_Binary_Prefix ) <> 'Pebi' ) then
    begin
        Error ;
    end ;
    if(Get_Language( '' ).Numeric_To_String( 1073741824.0 * 1024.0 * 1024.0 * 1024.0, NTS_Binary_Prefix ) <> 'Exbi' ) then
    begin
        Error ;
    end ;
end ;


procedure Test_String_To_Numeric ;

var D : double ;
    I : integer ;

begin
    if( Get_Language( '' ).String_To_Numeric( '', D, I ) ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).String_To_Numeric( 'ZZZZZZZZZZ', D, I ) ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( '1', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '11', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 11 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '19', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 19 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '29', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 29 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '111', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 111 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'I', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'IV', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'V', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'VI', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'IX', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 9 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'X', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 10 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'XVIII', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 18 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'XIX', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 19 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'XXIX', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 29 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'MCM', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1900 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'DC', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 600 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'CDXC', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 490 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'XL', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 40 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'L', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 50 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Roman ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( '1st', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '11th', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 11 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '32nd', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 32 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '29th', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 29 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( '111th', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 111 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Arabic_Ordinal ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).String_To_Numeric( '4rd', D, I ) ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Once', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Ordinal_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Twice', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Ordinal_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Thrice', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Ordinal_Multiplier ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Mega', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Giga', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tera', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pecta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Peta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Exabinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 * 1024 * 1024 * 1024 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Exa', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Yotta', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler bug }
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Zetta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E+21 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Deci', D, I ) ) then
    begin
        Error ;
    end ;
    if( num1( D ) <> '0.1' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Centi', D, I ) ) then
    begin
        Error ;
    end ;
    if( num1( D ) <> '0.01' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Milli', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '0.001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Micro', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '0.000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Nano', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '0.000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexa', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pico', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Femto', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Atto', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Zepto', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Yocto', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000000000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Deca', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 10 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hecto', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 100 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Kilo', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Kilobinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Unary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Binary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Ternary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quatrinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Septimal', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octal', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Decimal', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 10 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Duodecimal', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 12 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexadecimal', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 16 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Radix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Septenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Radix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Solo', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Duet', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Trio', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quartet', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quintet', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sextet', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Septet', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octet', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group0 ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Unity', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group1 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Duality', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group1 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Trinity', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group1 ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Monad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Dyad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Triad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pentad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Heptad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Decad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 10.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group2 ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Simplex', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group3 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Duplex', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group3 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Triplex', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group3 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quadruplex', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group3 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sextuplex', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Group3 ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Non', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hemi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Uni', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Bi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tri', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Penta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sexta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Septi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'OCta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'A', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Mono', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Di', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tri', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quad', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quint', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hepta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octo', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tetra', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Greek_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hex', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Greek_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexadeci', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 16 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Bicent', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 200 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tercent', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 300 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sexcent', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 600 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Dihexa', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 12 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quindeca', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 15 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tetrahexa', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 24 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexocta', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 48 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Alt_Prefix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Quarter', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.25 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Subsextuple', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot test result due to compiler bug }
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Half', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Single', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Double', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Triple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quadruple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quintuple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sextuple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Septuple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octuple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Decuple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 10 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Centuple', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 100 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Multiplier ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Niladic', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Math ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Monadic', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Math ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Dyadic', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Math ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Triadic', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Math ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexadic', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Math ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Primary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Secondary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tertiary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Septuary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octonal', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Duodenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 12 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sexagenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 60 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Centenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 100 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Bicentenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 200 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tercentenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 300 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Sexcentenary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 600 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Positional ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Point', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Line', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Triangle', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 3 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Square', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pentagon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexagon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Heptagon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octagon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Dihexagon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 12 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quindecagon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 13 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tetragon', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Quadrangle', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Parallelogram', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Rhombus', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Rhomboid', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Oblong', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Trapezoid', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Trapezium', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_2D ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Plane', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 2.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Cube', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pentahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Icosahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 20 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Eicosahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 20 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexoctahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 48 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tetrahexahedron', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 24 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_3D ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Anarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Monarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tetrarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 4 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pentarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Hexarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Heptarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 7 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Octarchy', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 8 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Government ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'G', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'T', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'P', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'E', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000000000000000.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Z', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E+21 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'u', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '0.000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'n', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '0.000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'da', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 10.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Y', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler bug }
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'p', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'f', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'z', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num2( D ) <> '0.000000000000000000001' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'h', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 100 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'k', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Metric_Abbreviation ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'Kibi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Mebi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1048576.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Gigabinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Gibi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1048576.0 * 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Pebi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 * 1024.0 * 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Tebi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 * 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Terabinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 * 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Petabinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 * 1024.0 * 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Megabinary', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1048576.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'Exbi', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1073741824.0 * 1024.0 * 1024.0 * 1024.0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'one', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'eleven', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 11 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'nineteen', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 19 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'twentynine', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 29 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one hundred eleven', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 111 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'zero', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'zero', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one trillion one billion one million', D, I ) ) then
    begin
        Error ;
    end ;
  {$IFDEF INT64_CONSTANTS}
    if( D <> 1001001000000 ) then
    begin
        Error ;
    end ;
  {$ENDIF}
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one google', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler error }
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one decillion', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler error }
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one nonillion', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler error }
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one octillion', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler error }
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one septillion', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot check result due to compiler error }
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one sextillion', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E21 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one quintillion', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E18 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one quadrillion', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E15 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'point 1', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '0.1' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).To_Numeric( 'POINT', D ) ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).To_Numeric( 'POINT X', D ) ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).To_Numeric( 'ONE POINT X', D ) ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).String_To_Numeric( 'negative', D, I ) ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'negative one', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> -1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one thousand one hundred eleven point 2', D, I ) ) then
    begin
        Error ;
    end ;
    if( Num1( D ) <> '1111.2' ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English ) then
    begin
        Error ;
    end ;

    if( not Get_Language( '' ).String_To_Numeric( 'first', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'eleventh', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 11 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'thirtysecond', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 32 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'twentyninth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 29 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one hundred eleventh', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 111 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one hundred twentieth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 120 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one thousand one hundred twentieth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1120 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'zeroeth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 0 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one trillion one billion one millionth', D, I ) ) then
    begin
        Error ;
    end ;
  {$IFDEF INT64_CONSTANTS}
    if( D <> 1001001000000 ) then
    begin
        Error ;
    end ;
  {$ENDIF}
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one googleth', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot test result due to compiler bug }
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one decillionth', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot test result due to compiler bug }
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one nonillionth', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot test result due to compiler bug }
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one octillionth', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot test result due to compiler bug }
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one septillionth', D, I ) ) then
    begin
        Error ;
    end ;
    { Cannot test result due to compiler bug }
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one sextillionth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E21 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one quintillionth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E18 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).String_To_Numeric( 'one quadrillionth', D, I ) ) then
    begin
        Error ;
    end ;
    if( D <> 1E15 ) then
    begin
        Error ;
    end ;
    if( I <> NTS_English_Ordinal ) then
    begin
        Error ;
    end ;
end ;


procedure Test_Generic_String_To_Numeric ;

var D : double ;
    T : integer ;

begin
    if( Get_Language( '' ).Generic_String_To_Numeric( '', D, T ) ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).Generic_String_To_Numeric( 'GON', D, T ) ) then
    begin
        Error ;
    end ;
    if( Get_Language( '' ).Generic_String_To_Numeric( 'HEX', D, T ) ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'HEXAGON', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 6 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'NONAGRAM', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 0 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'BIANGLE', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'DECOID', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 10 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_2D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'QUINTAHEDRON', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 5 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_3D ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'DUOBINARY', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 2 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Binary_Prefix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'KILOARCHY', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Government ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'MEGATET', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 1000000 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Group0 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'DECITY', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 10 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Group1 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'QUADHEXARY', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 24 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Radix ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'CENTAD', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 100 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Group2 ) then
    begin
        Error ;
    end ;
    if( not Get_Language( '' ).Generic_String_To_Numeric( 'DUOHEXACENTPLEX', D, T ) ) then
    begin
        Error ;
    end ;
    if( D <> 1200 ) then
    begin
        Error ;
    end ;
    if( T <> NTS_Group3 ) then
    begin
        Error ;
    end ;
end ;


procedure Stress_Test_String_To_Numeric ;

var A : integer ;
    AR, R : double ;
    S : Ansistring ;
    T : integer ;

begin
    randomize ;
    while( True ) do
    begin
        A := random( 65535 ) + 1 ;
        S :=Get_Language( '' ).Numeric_To_String( A, NTS_Roman ) ;
        if( not Get_Language( '' ).String_To_Numeric( S, R, T ) ) then
        begin
            Error ;
        end ;
        if( T <> NTS_Roman ) then
        begin
            Error ;
        end ;
        if( A <> R ) then
        begin
            Error ;
        end ;
        AR := A ;
        AR := AR * 65536.0 + random( 65535 ) ;
        S :=Get_Language( '' ).Numeric_To_String( AR, NTS_English ) ;
        if( not Get_Language( '' ).String_To_Numeric( S, R, T ) ) then
        begin
            Error ;
        end ;
        if( T <> NTS_English ) then
        begin
            halt ;
        end ;
        if( AR <> R ) then
        begin
            Error ;
        end ;
        AR := -AR ;
        S :=Get_Language( '' ).Numeric_To_String( AR, NTS_English ) ;
        if( not Get_Language( '' ).String_To_Numeric( S, R, T ) ) then
        begin
            Error ;
        end ;
        if( T <> NTS_English ) then
        begin
            Error ;
        end ;
        if( AR <> R ) then
        begin
            Error ;
        end ;
    end ;
end ;


procedure Test_Unit ;

begin
    Test_Numeric_To_String ;
    Test_String_To_Numeric ;
    Test_Generic_String_To_Numeric ;
end ;
{$ENDIF}

begin
    Prefix_Count := 0 ;
    Languages.Init( 0, 0 ) ;
    LObjects.Init( 0, 0 ) ;
    English_Language := nil ;
end.
