{$A-}
{
              Program Name : LIB
              Package Name : LB
              Purpose      : Librarian routines
              Institution  : Conroy & Conroy Co.
              Date Written : 15-Dec-1988
              Written By   : Alan Conroy
              Version      : 1.0

              TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *********************************************************
        *                                                       *
        *        M O D I F I C A T I O N   H I S T O R Y        *
        *                                                       *
        *********************************************************

             DATE      BY             REASON

        *********************************************************
        *                                                       *
        *           P R O G R A M   P U R P O S E               *
        *                                                       *
        *********************************************************

        *********************************************************
        *                                                       *
        *            F I L E   D E S C R I P T I O N S          *
        *                                                       *
        *********************************************************

}

unit Lib ;

interface

{$I EDEFINES.INC}

uses
{$IFDEF FPC_OR_DELPHI}
     { Borland... }
     SysUtils,

     { C&C... }
     ComInter,
{$ENDIF}
     _UE, { TUnified_Exception }
     _Lib ;

type TLibrary_Header = record
                           OK : byte ; { OK = 0 }
                           Version : byte ; { 10 = V1.0 }
                           Source : longint ; { 2 = Graphics images }
                           First_Name_Block : array[ 0..2 ] of byte ;
                           Deleted : array[ 0..2 ] of byte ; { Deleted blocks }
                           File_Type : byte ; { 2 = library }
                           Creation_Day : byte ;
                           Creation_Month : byte ;
                           Creation_Year : byte ;
                           Creation_Hour : byte ;
                           Creation_Minute : byte ;
                           Update_Day : byte ;
                           Update_Month : byte ;
                           Update_Year : byte ;
                           Update_Hour : byte ;
                           Update_Minute : byte ;
                           Library_Type : byte ; { 0 = OLB, 1 = TLB }
                           Filler : array[ 21..255 ] of byte ;
                       end ;

     TName_Block = record
                       Name : string[ 15 ] ;
                       Back_Link : array[ 0..2 ] of byte ;
                       Forward_Link : array[ 0..2 ] of byte ;
                       Start : array[ 0..2 ] of byte ; { Module start block }
                       Size : array[ 0..2 ] of byte ; { Module size (in blocks) }
                       Iinsertion_Day : byte ;
                       Insertion_Month : byte ;
                       Insertion_Year : byte ;
                       Insertion_Hour : byte ;
                       Insertion_Minute : byte ;
                       Flags : byte ;
                       Filler : array[ 34..255 ] of byte ; { Reserved }
                   end ;

     TSLibrary = object( TSObject_Librarian )
                    public { Constructors and destructors... }
                        constructor Init ;
                        destructor Done ; virtual ;

                    private { Instance data... }
                        _Error : tSUnified_Exception ;
                        Libr : file of Block ;
                        Lib_Buffer : Block ;
                        Opened : boolean ;
                        Day : integer ; { Day of last change }
                        Month : integer ; { Month of last change }
                        Year : integer ; { Year of last change }
                        Hour : integer ; { Hour of last change }
                        Minute : integer ; { Minute of last change }
                        L, Back_Link, Forward_Link : Longint ; { File link }

                    private { Internal methods... }
                        function Get_Value( Offset : Integer ) : Longint ;
                        procedure Insertion ;
                        function Module_Pos( Index : integer ) : longint ;
                        procedure Update ;
                        procedure Stuff( Offset : Integer ; Value : Longint ) ;
                        procedure Get_Date ;
                        procedure Get_Time ;

                    public { API... }
                        procedure Create_New( Command : String ;
                            Language_Id, Typ : Integer) ; virtual ;
                        function Error : tsUnified_Exception ; virtual ;
                        procedure Extract( Module, Filename : string ) ;
                            virtual ;
                        procedure Extract_Index( Module : integer ;
                            Filename : string ; Start_Block : longint ) ;
                            virtual ;
                        procedure Open( X : string ) ; virtual ;
                        procedure Delete( Command : string ) ; virtual ;
                        procedure Insert( Command, Filename : string ) ;
                        virtual ;
                        procedure Get_Module( Start_Block, X : integer ;
                            var Buffer ) ; virtual ;
                        function Get_Module_Info( X : integer ;
                            var Info : TModule_Info ) : boolean ; virtual ;
                        function Get_Module_Info_By_Name( Command : string ;
                            var Info : TModule_Info ) : boolean ; virtual ;
                        function Size : longint ; virtual ;
                        function Get_Error : TsUnified_Exception ;
                end ; { TSLibrary }

     PSLibrary = ^TSLibrary ;

    {Library file format:
     Block    Description
     -----    -----------
       1      header block
       2+     name and data blocks

     Header block format:
         Byte(s)    Description
         -------    -----------
         0          =0 : Good, <>0 : Bad
         1          Version (10 = V1.0)
         2          0 (Adventure Builder), 1 (Sirius), 2 (Graphics images)
         3-5        First name block
         6-8        Number of deleted blocks
         9          File type (2=Library)
         10         Day of creation
         11         Month of creation
         12         Year of creation
         13         Hour of creation
         14         Minute of creation
         15         Day of last update
         16         Month of last update
         17         Year of last update
         18         Hour of last update
         19         Minute of last update
         20         Library type (0 = OLB, 1 = ILB)

     Name block format:
         Byte(s)    Description
         -------    -----------
           0-15     Module name
          16-18     Back Link
          19-21     Forward Link
          22-24     Module start block
          25-27     Module size (in blocks)
          28        Module insertion day
          29        Module insertion month
          30        Module insertion year
          31        Module insertion hour
          32        Module insertion minute
          33        Flags
          34-255    reserved}

implementation

uses { C&C... }
     ErrMastr, { Register_Error_Interface }
     Num1s, { Num1 }
     Filestd, { FileName }
     L10n, { Text_* }
     O_S, { OS }
     TypeDefs, { TByte_Array }
     UEHDefs, { TStandard_Error_Interface }

     // SVM32...
     _SVM ; // TSVM_Header

{ tLib_EI definition and methods... }

{$IFDEF DYNAMIC_CLASSES}
type tLib_EI = class( TStandard_Error_Interface )
                            public
                                function Get_Facility : longint ; override ; stdcall ;
                                function Get_Name : PChar ; override ; stdcall ;
                                function Translate_Error( Code : longint ) : PChar ; override ; stdcall ;
                                procedure Done ; override ; stdcall ;
                                function Facility_Version : longint ;
                                    override ; stdcall ;
                                function Version : longint ;
                                    override ; stdcall ;
                  end ;
     pLib_EI = tLib_EI ;
{$ELSE}
type tLib_EI = object( TSStandard_Error_Interface )
                            public
                                function Get_Facility : longint ; virtual ;
                                function Get_Name : PChar ; virtual ;
                                function Translate_Error( Code : longint ) : PChar ; virtual ;
                                function Facility_Version : longint ;
                                    virtual ;
                                function Version : longint ;
                                    virtual ;
                  end ;
     pLib_EI = ^tLib_EI ;
{$ENDIF}

function tLib_EI.Get_Facility : longint ;

begin
    Get_Facility := LibErr_Facility ;
end ;


const Lib_Facility_Name : Pchar = 'Lib' ;

function tLib_EI.Get_Name : PChar ;

begin
    Get_Name := @Lib_Facility_Name ;
end ;


var Lib_Error_Text : array[ 0..127 ] of char ;

function tLib_EI.Translate_Error( Code : longint ) : PChar ;

var S : string ;

begin
    case Code of
        LibErr_Success : S := Text_Success ;
        LibErr_Invalid_Library_File_Format : S := Text_LibErr_Invalid_Library_File_Format ;
        LibErr_Module_Not_Found : S := Text_LibErr_Module_Not_Found ;
        LibErr_Module_Already_Exists : S := Text_LibErr_Module_Already_Exists ;
        LibErr_Illegal_Module_Format : S := Text_LibErr_Illegal_Module_Format ;
    end ;
    S := S + #0 ;
    move( S[ 1 ], Lib_Error_Text[ 0 ], length( S ) ) ;
    Translate_Error := @Lib_Error_Text ;
end ;


function tLib_EI.Facility_Version : longint ;

begin
    Facility_Version := 0 ;
end ;


function tLib_EI.Version : longint ;

begin
    Version := 0 ;
end ;


{$IFDEF FPC_OR_DELPHI}
procedure tLib_EI.Done ;

begin
end ;
{$ENDIF}



{ Unit routines... }

function Cvtbl( X : Byte ) : Longint ; {Convert byte to long integer}

begin
    Cvtbl := X
end ;



{ TSLibrary methods... }

{ Constructors and destructors... }
constructor TSLibrary.Init ;

begin
    _Error.Code := 0 ;
    _Error.Facility := LibErr_Facility ;
    Opened := False ;
{$IFDEF DYNAMIC_STRINGS}
    Initialize( Version ) ;
{$ELSE}
    Version := '' ;
{$ENDIF}
end ;


destructor TSLibrary.Done ;

begin
    if( Opened ) then
    begin
        close( Libr ) ;
    end ;
{$IFDEF DYNAMIC_STRINGS}
    Finalize( Version ) ;
{$ENDIF}
end ;


{ Internal routines... }

function TSLibrary.Get_Value( Offset : Integer ) : Longint ;
{Get and unpack value from buffer}

begin
    Get_Value := Lib_Buffer[ Offset ] + Cvtbl( Lib_Buffer[ Offset + 1 ] ) * 256 +
        Lib_Buffer[ Offset + 2 ] * 65536
end ;


procedure TSLibrary.Stuff( Offset : Integer ; Value : Longint ) ;
{ Stuff a 3-byte value into the buffer }

begin
    Lib_Buffer[ Offset + 2 ] := Trunc( Value Div 65536 ) ;
    Value := Value - Lib_Buffer[ Offset + 2 ] * 65536 ;
    Lib_Buffer[ Offset + 1 ] := Trunc( Value Div 256 ) ;
    Value := Value - Cvtbl( Lib_Buffer[ Offset + 1 ] ) * 256 ;
    Lib_Buffer[ Offset ] := Trunc( Value )
end ;


function TSLibrary.Module_Pos( Index : integer ) : longint ;

var _L : longint ;

begin
    seek( Libr, 0 ) ;
    {$I-}
    read( Libr, Lib_Buffer ) ;
    {$I+}
    Module_Pos := -1 ;
    _Error.Facility := 2 ;
    _Error.Code := IOResult ;
    if( _Error.Code <> 0 ) then
    begin
        exit ;
    end ;
    _L := Get_Value( 3 ) ; { First name block }
    while( Index >= 0 ) do
    begin
        if( _L = 0 ) then
        begin
            _Error.Facility := LibErr_Facility ;
            _Error.Code := LibErr_Module_Not_Found ;
            exit ;
        end ;
        seek( Libr, _L ) ;
        {$I-}
        read( Libr, Lib_Buffer ) ;
        {$I+}
        _Error.Code := IOResult ;
        if( _Error.Code <> 0 ) then
        begin
            _Error.Facility := 2 ;
            exit ;
        end ;
        dec( Index ) ;
        if( Index >= 0 ) then
        begin
            _L := Get_Value( 19 ) ;
        end ;
    end ; { while( X >= 0 ) }
    Module_Pos := _L ;
end ; { TSLibrary.Module_Pos }


procedure TSLibrary.Update ;

begin
    Seek( Libr, 0 ) ;
    Read( Libr, Lib_Buffer ) ; { Read header }
    Seek( Libr, 0 ) ; { Reposition }
    Get_Date ;
    Get_Time ;
    Lib_Buffer[ 15 ] := Day ; { Day of last change }
    Lib_Buffer[ 16 ] := Month ; { Month of last change }
    Lib_Buffer[ 17 ] := Year ; { Year of last change }
    Lib_Buffer[ 18 ] := Hour ; { Hour of last change }
    Lib_Buffer[ 19 ] := Minute ; { Minute of last change }
    Modified_Day := Day ;
    Modified_Month := Month ;
    Modified_Year := Year ;
    Modified_Hour := Hour ;
    Modified_Minute := Minute ;
    Write( Libr, Lib_Buffer ) { Write updated header }
end ; { TSLibrary.Update }


{ API... }

procedure TSLibrary.Create_New( Command : String ; Language_Id, Typ : Integer ) ;

begin
    if( Opened ) then
    begin
        Opened := False ;
        close( Libr ) ;
    end ;
    Assign( Libr, Command ) ;
    {$I-}
    Rewrite( Libr ) ;
    {$I+}
    _Error.Code := Ioresult ;
    _Error.Facility := LibErr_Facility ;
    If( _Error.Code <> 0 ) then
    Begin
        Exit ;
    End ;
    Lib_Buffer[ 0 ] := 0 ; {Anything but 0 indicates a bad file}
    Lib_Buffer[ 1 ] := 10 ; {Version 1.0}
    Lib_Buffer[ 2 ] := Language_Id ;
    Lib_Buffer[ 3 ] := 0 ; {First name block}
    Lib_Buffer[ 4 ] := 0 ;
    Lib_Buffer[ 5 ] := 0 ;
    Lib_Buffer[ 6 ] := 0 ; {Unused blocks count}
    Lib_Buffer[ 7 ] := 0 ;
    Lib_Buffer[ 8 ] := 0 ;
    Lib_Buffer[ 9 ] := 2 ; {Library file}
    Get_Date ;
    Get_Time ;
    Lib_Buffer[ 10 ] := Day ; {Day of creation}
    Lib_Buffer[ 11 ] := Month ; {Month of creation}
    Lib_Buffer[ 12 ] := Year ; {Year of creation}
    Lib_Buffer[ 13 ] := Hour ; {Hour of creation}
    Lib_Buffer[ 14 ] := Minute ; {Minute of creation}
    Lib_Buffer[ 15 ] := Day ; {Day of last change}
    Lib_Buffer[ 16 ] := Month ; {Month of last change}
    Lib_Buffer[ 17 ] := Year ; {Year of last change}
    Lib_Buffer[ 18 ] := Hour ; {Hour of last change}
    Lib_Buffer[ 19 ] := Minute ; {Minute of last change}
    Lib_Buffer[ 20 ] := Typ ; { Library file type }
    {$I-}
    Write( Libr, Lib_Buffer ) ;
    {$I+}
    _Error.Code := IOResult ;
    _Error.Facility := LibErr_Facility ;
    Close( Libr ) ;
end ; { TSLibrary.Create_New }


function TSLibrary.Error : tsUnified_Exception ;

begin
    Error := _Error ;
    if( _Error <> nil ) then
    begin
        _Error := nil ;
    end ;
end ;


procedure TSLibrary.Extract( Module, Filename : string ) ;

var A : integer ;
    Inp : string[ 255 ] ;
    _L, Link : longint ;
    Output : file of block ;

begin
    _Error.Code := 0 ;
    _Error.Facility := LiBErr_Facility ;
    if( not Opened ) then
    begin
        _Error.Facility := 1 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    Seek( Libr, 0 ) ;
    Read( Libr, Lib_Buffer ) ;
    Link := Get_Value( 3 ) ; { Get first name link }
    while( Link <> 0 ) do
    begin
        Seek( Libr, Link ) ;
        Read( Libr, Lib_Buffer ) ;
        for A := 0 to 15 do
        begin
            Inp[ A ] := Chr( Lib_Buffer[ A ] ) ;
        end ;
        if( Inp > Module ) then { End early if module not found }
        begin
            _Error.Facility := LibErr_Facility ;
            _Error.Code := LibErr_Module_Not_Found ; { Module not found }
            exit ;
        end ;
        if( Inp = Module ) then { Found module }
        begin
            assign( Output, Filename ) ;
            {$I+}
            rewrite( Output ) ;
            {$I+}
            _Error.Code := IOResult ;
            if( _Error.Code <> 0 ) then
            begin
                _Error.Code := LibErr_Facility ;
                exit ;
            end ;

            _L := Get_Value( 25 ) ; { Get size of module }
            for A := 1 to Trunc( _L ) do
            begin
                Read( Libr, Lib_Buffer ) ;
                Write( Output, Lib_Buffer ) ;
            end ;
            close( Output ) ;
            exit ;
        end ;
        Link := Get_Value( 19 ) ; { Follow link to next module }
    end ;
    _Error.Facility := LibErr_Facility ;
    _Error.Code := LibErr_Module_Not_Found ; { Module not found }
end ; { TSLibrary.Extract }


procedure TSLibrary.Extract_Index( Module : integer ; Filename : string ;
    Start_Block : longint ) ;

var A : integer ;
    Inp : string[ 255 ] ;
    _L, Link : longint ;
    Output : file of block ;

begin
    _Error.Code := 0 ;
    _Error.Facility := LiBErr_Facility ;
    if( not Opened ) then
    begin
        _Error.Facility := 1 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    Seek( Libr, 0 ) ;
    Read( Libr, Lib_Buffer ) ;
    Link := Get_Value( 3 ) ; { Get first name link }
    while( Link <> 0 ) do
    begin
        Seek( Libr, Link ) ;
        Read( Libr, Lib_Buffer ) ;
        for A := 0 to 15 do
        begin
            Inp[ A ] := Chr( Lib_Buffer[ A ] ) ;
        end ;
        if( Module = 0 ) then { Found module }
        begin
            assign( Output, Filename ) ;
            {$I+}
            rewrite( Output ) ;
            {$I+}
            _Error.Code := IOResult ;
            if( _Error.Code <> 0 ) then
            begin
                _Error.Code := LibErr_Facility ;
                exit ;
            end ;

            _L := Get_Value( 25 ) ; { Get size of module }
            for A := 1 to Trunc( _L ) do
            begin
                Read( Libr, Lib_Buffer ) ;
                Write( Output, Lib_Buffer ) ;
            end ;
            close( Output ) ;
            exit ;
        end ;
        Link := Get_Value( 19 ) ; { Follow link to next module }
        dec( Module ) ;
    end ;
    _Error.Facility := LibErr_Facility ;
    _Error.Code := LibErr_Module_Not_Found ; { Module not found }
end ; { TSLibrary.Extract_Index }


procedure TSLibrary.Open( X : string ) ;

var Dummy : integer ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    assign( Libr, FileName( X ) ) ;
    {$I-}
    reset( Libr ) ;
    {$I+}
    _Error.Facility := 2 ;
    _Error.Code := IOResult ;
    if( _Error.Code = 0 ) then
    begin
        if( filesize( Libr ) < 1 ) then
        begin
            close( Libr ) ;
            _Error.Facility := LibErr_Facility ;
            _Error.Code := LibErr_Invalid_Library_File_Format ; { Invalid file format }
            exit ;
        end ;
        read( Libr, Lib_Buffer ) ;
        if( ( Lib_Buffer[ 0 ] <> 0 ) or ( Lib_Buffer[ 9 ] <> 2 ) ) then
        begin
            close( Libr ) ;
            _Error.Facility := LibErr_Facility ;
            _Error.Code := LibErr_Invalid_Library_File_Format ; { Invalid file format }
            exit ;
        end ;
        Opened := True ;
        Creation_Day := Lib_Buffer[ 10 ] ; { Day of creation }
        Creation_Month := Lib_Buffer[ 11 ] ; { Month of creation }
        Creation_Year := Lib_Buffer[ 12 ] ; { Year of creation }
        Creation_Hour := Lib_Buffer[ 13 ] ; { Hour of creation }
        Creation_Minute := Lib_Buffer[ 14 ] ; { Minute of creation }
        Modified_Day := Lib_Buffer[ 15 ] ; { Day of last change }
        Modified_Month := Lib_Buffer[ 16 ] ; { Month of last change }
        Modified_Year := Lib_Buffer[ 17 ] ; { Year of last change }
        Modified_Hour := Lib_Buffer[ 18 ] ; { Hour of last change }
        Modified_Minute := Lib_Buffer[ 19 ] ; { Minute of last change }
        Dummy := Lib_Buffer[ 1 ] ;
        Version := num1( Dummy div 10 ) + '.' +
            num1( Dummy - ( ( Dummy div 10 ) * 10 ) ) ;
        Deleted_Space := Get_Value( 6 ) ;
    end ; { if( _Error = 0 ) }
end ; { TSLibrary.Open }


procedure TSLibrary.Delete( Command : string ) ;

var A : Integer ;
    Inp : string[ 255 ] ;
    _Back_Link, _Forward_Link, _L, L1, Link : Longint ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    if( not Opened ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    Seek( Libr, 0 ) ;
    Read( Libr, Lib_Buffer ) ; { Get library header }
    Link := Get_Value( 3 ) ; { Get first name link }
    while( Link <> 0 ) do
    begin
        Seek( Libr, Link ) ;
        {$I-}
        Read( Libr, Lib_Buffer ) ;
        {$I+}
        _Error.Facility := 2 ;
        _Error.Code := IOResult ;
        if( _Error.Code <> 0 ) then
        begin
            exit ;
        end ;
        For A := 0 To 15 Do
        begin
            Inp[ A ] := Chr( Lib_Buffer[ A ] ) ;
        end ;
        _Forward_Link := Get_Value( 19 ) ;
        If( Inp = Command ) then {Found module}
        Begin
            _Back_Link := Get_Value( 16 ) ;
            L1 := Get_Value( 25 ) ; {Get module size}
            If( _Back_Link > 0 ) Then {A previous record}
            Begin
                Seek( Libr, _Back_Link ) ; {Get previous record}
                {$I-}
                Read( Libr, Lib_Buffer ) ;
                {$I+}
                _Error.Facility := 2 ;
                _Error.Code := IOResult ;
                if( _Error.Code <> 0 ) then
                begin
                    exit ;
                end ;
                Stuff( 19, _Forward_Link ) ;
                Seek( Libr, _Back_Link ) ;
                {$I-}
                Write( Libr, Lib_Buffer ) ; {Patch forward link}
                {$I+}
                _Error.Facility := 2 ;
                _Error.Code := IOResult ;
                if( _Error.Code <> 0 ) then
                begin
                    exit ;
                end ;
            end ;
            If( _Forward_Link > 0 ) then {If a following record}
            Begin
                Seek( Libr, _Forward_Link ) ; {Get next record}
                {$I-}
                Read( Libr, Lib_Buffer ) ;
                {$I+}
                _Error.Code := IOResult ;
                _Error.Facility := 2 ;
                if( _Error.Code <> 0 ) then
                begin
                    exit ;
                end ;
                Stuff( 16, _Back_Link ) ;
                Seek( Libr, _Forward_Link ) ;
                {$I-}
                Write( Libr, Lib_Buffer ) ; { Patch back link }
                {$I+}
                _Error.Facility := 2 ;
                _Error.Code := IOResult ;
                if( _Error.Code <> 0 ) then
                begin
                    exit ;
                end ;
            End ;
            Seek( Libr, 0 ) ;
            Read( Libr, Lib_Buffer ) ; {Get header record}
            _L := Get_Value( 6 ) + L1 + 10 ; {Add up deleted block count}
            Stuff( 6, _L ) ;
            If( _Back_Link = 0 ) then
            begin
                Stuff( 3, _Forward_Link ) ;
                { If deleted module was first module }
            end ;
            Seek( Libr, 0 ) ;
            {$I-}
            Write( Libr, Lib_Buffer ) ; {Update header}
            {$I+}
            _Error.Facility := 2 ;
            _Error.Code := IOResult ;
            if( _Error.Code <> 0 ) then
            begin
                exit ;
            end ;
            Update ;
            Exit ;
        End ;
        If( Inp > Command ) then
        begin
            Link := 0 ;
        end else
        begin
            Link := _Forward_Link ;
        end ;
    end ;
    _Error.Facility := LibErr_Facility ;
    _Error.Code := LibErr_Module_Not_Found ; { Module not found }
end ; { TSLibrary.Delete }


procedure TSLibrary.Insert( Command, Filename : String ) ;

Var A, Dummy : Integer ;
    Header : TSVM_Header ;
    Inp : String[ 255 ] ;
    _Back_Link, _Forward_Link, _L, L1, Last_Link, Link : Longint ;
    Object_File : file of Block ;
    O : Block ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    if( not Opened ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    assign( Object_File, Filename ) ;
    {$I-}
    reset( Object_File ) ;
    {$I+}
    _Error.Code := IOResult ;
    if( _Error.Code <> 0 ) then
    begin
        _Error.Facility := 2 ;
        exit ;
    end ;
    Read( Object_File, O ) ;
    move( O, Header, 256 ) ;
    if( ( Header.OK_Flag <> 0 ) or ( Header.Prefix <> 255 ) or ( Header.Format <> 0 ) ) then
    { File did not compile OK or isn't object formatted }
    begin
        _Error.Code := LibErr_Illegal_Module_Format ;
        Close( Object_File ) ;
        exit ;
    end ;

    Seek( Libr, 0 ) ;
    Read( Libr, Lib_Buffer ) ; { Read library header }
    Link := Get_Value( 3 ) ; { Get first name link }
    _L := Filesize( Libr ) ; { Get current library size }
    Dummy := pos( '\', Command ) ;
    while( Dummy > 0 ) do
    begin
        Command := copy( Command, Dummy + 1, length( Command ) ) ;
        Dummy := pos( '\', Command ) ;
    end ;
    Dummy := pos( ':', Command ) ;
    Command := copy( Command, Dummy + 1, length( Command ) ) ;
    _Back_Link := 0 ;
    _Forward_Link := 0 ;
    If( Link = 0 ) then { First module }
    Begin
        _Back_Link := 0 ;
        _Forward_Link := 0 ;
        Stuff( 3, _L ) ;
        Seek( Libr, 0 ) ;
        Write( Libr, Lib_Buffer ) ; { Update library header }
    End Else
    while( Link <> 0 ) do { Until end of list }
    begin
        Seek( Libr, Link ) ;
        Read( Libr, Lib_Buffer ) ;
        for A := 0 To 15 do Inp[ A ] := Chr( Lib_Buffer[ A ] ) ;
        If( Command = Inp ) then
        Begin
            _Error.Facility := LibErr_Facility ;
            _Error.Code := LibErr_Module_Already_Exists ; { Module already exists }
            close( Object_File ) ;
            Exit ;
        End ;
        _Back_Link := Link ;
        _Forward_Link := Link ;
        If( Inp > Command ) then { Insert new module just before this one }
        Begin
            If(
                ( Lib_Buffer[ 16 ] = 0 )
                and
                ( Lib_Buffer[ 17 ] = 0 )
                and
                ( Lib_Buffer[ 18 ] = 0 )
              ) then { No back link }
            Begin
                Stuff( 16, _L ) ;
                Seek( Libr, Link ) ;
                Write( Libr, Lib_Buffer ) ;
                _Forward_Link := Link ;
                _Back_Link := 0 ;
                Link := 0 ;
                Seek( Libr, 0 ) ;
                Read( Libr, Lib_Buffer ) ;
                Seek( Libr, 0 ) ;
                Stuff( 3, _L ) ;
                Write( Libr, Lib_Buffer ) ;
            End Else {Must insert this module between two others}
            Begin
                L1 := Get_Value( 16 ) ; {Get back link}
                Stuff( 16, _L ) ; {Set back link to new record}
                Seek( Libr, Link ) ;
                Write( Libr, Lib_Buffer ) ; {Update record}
                Seek( Libr, L1 ) ; {Get previous module}
                Read( Libr, Lib_Buffer ) ;
                Stuff( 19, _L ) ; {Set forward link}
                Seek( Libr, L1 ) ;
                Write( Libr, Lib_Buffer ) ; {Update previous record}
                _Forward_Link := Link ;
                _Back_Link := L1 ; {Set up links for new record}
                Link := 0 ; {Terminate loop}
            End ; { Follow back link }
        End Else
        Begin { Inp < Command }
            If(
                ( Lib_Buffer[ 19 ] = 0 )
                and
                ( Lib_Buffer[ 20 ] = 0 )
                and
                ( Lib_Buffer[ 21 ] = 0 )
              ) then { No forward link }
            Begin
                Stuff( 19, _L ) ;
                Seek( Libr, Link ) ;
                Write( Libr, Lib_Buffer ) ;
                _Back_Link := Link ;
                _Forward_Link := 0 ;
                Link := 0
            End Else
            Begin
                Last_Link := Link ; {Save current record number}
                L1 := Get_Value( 19 ) ; {Get forward link}
                If( L1 = Last_Link ) then
                Begin
                    _Forward_Link := Last_Link ;
                    _Back_Link := Link ;
                    Insertion ;
                    Link := 0
                End Else Link := L1
            End ;
            {Follow forward link}
        End ;
    end ;
    Seek( Libr, _L ) ;
    Inp := Command ;
    for A := 0 to 15 do Lib_Buffer[ A ] := Ord( Inp[ A ] ) ;
    Stuff( 16, _Back_Link ) ;
    Stuff( 19, _Forward_Link ) ;
    Stuff( 22, _L + 1 ) ;
    L1 := Filesize( Object_File ) ;
    Stuff( 25, L1 ) ;
    Get_Date ;
    Get_Time ;
    Lib_Buffer[ 28 ] := Day ; {Day of last change}
    Lib_Buffer[ 29 ] := Month ; {Month of last change}
    Lib_Buffer[ 30 ] := Year ; {Year of last change}
    Lib_Buffer[ 31 ] := Hour ; {Hour of last change}
    Lib_Buffer[ 32 ] := Minute ; {Minute of last change}
    Lib_Buffer[ 33 ] := 0 ;
    Write( Libr, Lib_Buffer ) ; {Write out name block}
    Dummy := FileMode ;
    FileMode := 0 ;
    {$I-}
    Reset( Object_File ) ;
    {$I+}
    FileMode := Dummy ;
    Dummy := IOResult ;
    if( Dummy <> 0 ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := IOResult ;
        exit ;
    end ;
    while( not Eof( Object_File ) ) do
    begin
        Read( Object_File, Lib_Buffer ) ;
        Write( Libr, Lib_Buffer ) ;
    end ;
    close( Object_File ) ;

    Update ;
end ; { TSLibrary.Insert }


procedure TSLibrary.Get_Module( Start_Block, X : integer ; var Buffer ) ;

var A, Dummy : integer ;
    Info : TModule_Info ;
    _L : longint ;
    _Size : integer ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    if( not Opened ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    if( not Get_Module_Info( X, Info ) ) then
    begin
        exit ;
    end ;
    seek( Libr, 0 ) ;
    {$I-}
    read( Libr, Lib_Buffer ) ;
    {$I+}
    _Error.Facility := 2 ;
    _Error.Code := IOResult ;
    if( _Error.Code <> 0 ) then
    begin
        exit ;
    end ;
    _L := Module_Pos( X ) ;
    if( _L <= 0 ) then
    begin
        exit ;
    end ;
    seek( Libr, _L ) ;
    {$I-}
    read( Libr, Lib_Buffer ) ;
    {$I+}
    _Error.Code := IOResult ;
    if( _Error.Code <> 0 ) then
    begin
        _Error.Facility := 2 ;
        exit ;
    end ;

    { Now return module... }
    _Size := Info.Size ;
    Dummy := 0 ;
    for A := 1 to _Size div 256 do
    begin
        if( ( A = 1 ) or ( A > Start_Block ) or ( ( Info.Flags and 1 ) = 0 ) )
            Then
                Read( Libr, Lib_Buffer )
            Else
                Fillchar( Lib_Buffer, 256, 0 ) ;
        move( Lib_Buffer, TByte_Array( Buffer )[ Dummy ], sizeof( Lib_Buffer ) ) ;
        Dummy := Dummy + sizeof( Lib_Buffer ) ;
    end ;
end ; // TSLibrary.Get_Module


function TSLibrary.Get_Module_Info( X : integer ; var Info : TModule_Info ) : boolean ;

var Dummy : integer ;
    _L : longint ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    Get_Module_Info := True ;
    if( not Opened ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := 103 ;
        Get_Module_Info := False ;
        exit ;
    end ;

    _L := Module_Pos( X ) ;
    if( _L < 0 ) then
    begin
        _Error.Facility := LibErr_Facility ;
        _Error.Code := LibErr_Module_Not_Found ; // Module not found
        Get_Module_Info := False ;
        exit ;
    end ;
    seek( Libr, _L ) ;

    { Now return information about the module }
    for Dummy := 0 To 15 Do
    begin
        Info.Name[ Dummy ] := Chr( Lib_Buffer[ Dummy ] ) ;
    end ;
    Info.Size := Get_Value( 25 ) * 256 ;
    Info.Day := Lib_Buffer[ 28 ] ;
    Info.Month := Lib_Buffer[ 29 ] ;
    Info.Year := Lib_Buffer[ 30 ] ;
    Info.Hour := Lib_Buffer[ 31 ] ;
    Info.Minute := Lib_Buffer[ 32 ] ;
    Info.Flags := Lib_Buffer[ 33 ] ;
end ; { TSLibrary.Get_Module }


function TSLibrary.Get_Module_Info_By_Name( Command : string ;
    var Info : TModule_Info ) : boolean ;

var A : Integer ;
    Inp : string[ 255 ] ;
    _Forward_Link, Link : Longint ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    Get_Module_Info_By_Name := False ;
    if( not Opened ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    Seek( Libr, 0 ) ;
    Read( Libr, Lib_Buffer ) ; { Get library header }
    Link := Get_Value( 3 ) ; { Get first name link }
    while( Link <> 0 ) do
    begin
        Seek( Libr, Link ) ;
        {$I-}
        Read( Libr, Lib_Buffer ) ;
        {$I+}
        _Error.Facility := 2 ;
        _Error.Code := IOResult ;
        if( _Error.Code <> 0 ) then
        begin
            exit ;
        end ;
        For A := 0 To 15 Do
        begin
            Inp[ A ] := Chr( Lib_Buffer[ A ] ) ;
        end ;
        _Forward_Link := Get_Value( 19 ) ;
        If( Inp = Command ) then {Found module}
        Begin
            Get_Module_Info_By_Name := Get_Module_Info( Link, Info ) ;
            exit ;
        end ;
        If( Inp > Command ) then
        begin
            Link := 0 ;
        end else
        begin
            Link := _Forward_Link ;
        end ;
    end ;
    _Error.Facility := LibErr_Facility ;
    _Error.Code := LibErr_Module_Not_Found ;
end ;


function TSLibrary.Size : longint ;

begin
    _Error.Facility := LibErr_Facility ;
    _Error.Code := 0 ;
    Size := 0 ;
    if( not Opened ) then
    begin
        _Error.Facility := 2 ;
        _Error.Code := 103 ;
        exit ;
    end ;

    Size := filesize( Libr ) * 256 ;
end ;


function TSLibrary.Get_Error : TSUnified_Exception ;

begin
    Get_Error := _Error ;
end ;


procedure TSLibrary.Get_Date ;

var Y, M, D : word ;

Begin
    OS^.Get_Date( Y, M, D ) ;
    Month := M ;
    Day := D ;
    Year := Y - 1900 ;
    While( Year > 99 ) do
    begin
        Year := Year - 100 ;
    end ;
End ;


Procedure TSLibrary.Get_Time ;

var H, M, S, MS : word ;

Begin
    OS^.Get_Time( H, M, S, MS ) ;
    Hour := H ;
    Minute := M ;
End ;


Procedure TSLibrary.Insertion ;
{Change links so as to insert this module (name).  L is the name record #}

Begin
    Seek( Libr, Back_Link ) ; {Get previous logical record}
    Read( Libr, Lib_Buffer ) ;
    Stuff( 19, L ) ; {Set forward link}
    Seek( Libr, Back_Link ) ;
    Write( Libr, Lib_Buffer ) ; {Update record}
    Seek( Libr, Forward_Link ) ; {Get next logical record}
    Read( Libr, Lib_Buffer ) ;
    Stuff( 16, L ) ; {Set back link}
    Seek( Libr, Forward_Link ) ; {Update record}
    Write( Libr, Lib_Buffer )
End ;


var Lib_EI : pLib_EI ;

procedure Unit_Initialization ;

begin
{$IFDEF FPC_OR_DELPHI}
    Lib_EI := tLib_EI.Create ;
    Register_Error_Interface( PSError_Interface( Lib_EI ) ) ;
{$ELSE}
    new( Lib_EI, Init ) ;
    Register_Error_Interface( @Lib_EI ) ;
{$ENDIF}
end ;


procedure Unit_Deallocation ;

begin
{$IFDEF FPC_OR_DELPHI}
    Unregister_Error_Interface( Lib_EI.Get_Facility ) ;
    Lib_EI.Free ;
{$ELSE}
    Unregister_Error_Interface( Lib_EI^.Get_Facility ) ;
    dispose( Lib_EI ) ;
{$ENDIF}
end ;


{$IFDEF UNIT_INIT_AND_FINAL}
initialization
{$ELSE}
begin
{$ENDIF}
    Unit_Initialization ;
{$IFDEF UNIT_INIT_AND_FINAL}

finalization
    Unit_Deallocation ;
{$ENDIF}
end.
