{$IFNDEF FPC}
  {$E+}
  {$N+}
{$ENDIF}
{
        Program Name : OS_Windows
        Package Name : Subroutine Library - Layer 1
        Purpose      : MS Windows Operating system interface
        Institution  : Conroy & Conroy Co.
        Date Written : 6-Nov-2002
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2002 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

        DATE          BY          REASON
        2-Dec-2002    EAC         Address string initialization.
        3-Jun-2010    EAC         Don't request mainform handle if not
                                  allocated.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

}

unit OS_Windows ;

interface

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

{$I EDEFINES.INC}

uses // Borland...
     SysUtils, // inttostr
{$IFDEF FPC}
     LCLType, // ID_Abort
{$ENDIF}
     Classes, // TList
     
     { Subroutine Library - Layer 0... }
{$IFNDEF MOBILE}
     _DebugIn, // TDebug_Interface
{$ENDIF}
     _Types, { int64 }
     _UE, { PSUnified_Exception }

     { Subroutine Library - Layer 1... }
     _DevMan, { PSDevice_Manager }
     _OS, { TOperating_System }
     _TPM ; { TSText_Presentation_Manager }

type TSWindows_Text_Presentation_Manager = object( TSText_Presentation_Manager )
                                               public { Constructors and destructors... }
                                                   constructor Init ;
                                                   destructor Done ; virtual ;

                                               private
                                                   _Numeric_Text_Presentation_Manager : PSNumeric_Text_Presentation_Manager ;
                                                   _Monetary_Text_Presentation_Manager : PSMonetary_Text_Presentation_Manager ;
                                                   _Date_Text_Presentation_Manager : PSDate_Text_Presentation_Manager ;

                                               public { API... }
                                                   function Numeric_Text_Presentation_Manager : PSNumeric_Text_Presentation_Manager ;
                                                       virtual ;

                                                   function Monetary_Text_Presentation_Manager : PSMonetary_Text_Presentation_Manager ;
                                                       virtual ;

                                                   function Date_Text_Presentation_Manager : PSDate_Text_Presentation_Manager ;
                                                       virtual ;
                                           end ;
     PSWindows_Text_Presentation_Manager = ^TSWindows_Text_Presentation_Manager ;

type TSOS_Windows = object( TSOperating_System )
                             public { Constructors and destructors... }
                                 constructor Init ;
                                 destructor Done ; virtual ;

                             private { Instance data... }
                                 Input_Buffer : Ansistring ;
                                 Internal_Clipboard : Ansistring ;
                                 _Device_Manager : PSDevice_Manager ;
                                 _Text_Presentation_Manager : PSWindows_Text_Presentation_Manager ;

                                 File_Mapping_List : TList ;

                             public { API... }
                                 function Name : Ansistring ; virtual ; { O/S name }
                                 function Version : Ansistring ; virtual ;
                                 { O/S version }

                                 function Application_Path : Ansistring ; virtual ;
                                 procedure Beep( Typ : integer ) ; virtual ;
                                 procedure Cancel_Typeahead( Handle : integer ) ;
                                     virtual ;
                                 function Command_Line : Ansistring ; virtual ;
{$IFDEF FPC_OR_DELPHI}
  {$IFNDEF MOBILE}
                                 function Debugger : PSDebug_Interface ;
                                     virtual ;
                                 { Returns a debugger for the operating system }
  {$ENDIF}
{$ENDIF}
                                 function Get_Char( Wait : boolean ;
                                     var Valid : boolean ; Handle : integer ) : Ansichar ;
                                     virtual ;
                                 function Get_Symbol( const X : AnsiString ) : Ansistring ;
                                     virtual ;
                                 procedure Get_Date( var Year, Month, Day : word ) ;
                                     virtual ;
                                 procedure Get_Time( var Hour, Minute, Second, Hundreths, NS : word ) ;
                                     virtual ;
                                 function Last_Error : PSUnified_Exception ;
                                     virtual ;
                                 function Error_Text( Code : longint ;
                                     const Language : AnsiString ) : Ansistring ;
                                     virtual ;
                                 function Logical_CPUs : integer ; virtual ;
                                 function Pending_Input( Handle : integer ) : boolean ;
                                     virtual ;
                                 function PID : longint ; virtual ;
                                 procedure Print( T : AnsiString ) ; virtual ;
                                 function Spawn( const Path, Parameters, Default_Dir : Ansistring ;
                                     Flags : integer ) : boolean ;
                                     virtual ;
                                 function Get_Temp_Path : Ansistring ; virtual ;
                                 function System_Path : Ansistring ; virtual ;
                                 function Text_Presentation_Manager : PSText_Presentation_Manager ;
                                     virtual ;
                                 function Swap_File_List : Ansistring ; virtual ;
                                 function Thread_ID : longint ; virtual ;
                                 function Timestamp : int64 ; virtual ;
                                 function Get_Clipboard( Path : AnsiString ) : Ansistring ;
                                     virtual ;
                                 procedure Set_Clipboard( Path, Value : AnsiString ) ;
                                     virtual ;
                                 function Max_File_Size : int64 ; virtual ;
                                 function User_Name : Ansistring ; virtual ;
                                 function Device_Manager : PSDevice_Manager ;
                                     virtual ;
                                 function Load_Library( Nam : AnsiString ) : longint ;
                                     virtual ;

                                 function Get_Symbol_Address( Lib : longint ;
                                     Nam : AnsiString ) : int64 ; virtual ;

                                 function Show_Message( Handle : longint ;
                                     Msg, Title : Ansistring ;
                                     Options : integer ) : integer ; virtual ;

                                 function Get_Country : Ansistring ; virtual ;

                                 { Memory mapping... }
                                 function Max_Memory_Map : int64 ; virtual ;

                                 function Memory_Map_File( _Name : Ansistring ;
                                     Writeable : boolean ;
                                     Start, Size : int64 ;
                                     Flag : boolean ) : integer ; virtual ;

                                 procedure Unmap_File_From_Memory( Handle : integer ) ;
                                     virtual ;

                                 function Memory_Mapped_File_Base( Handle : integer ) : pointer ;
                                     virtual ;

                                 function Memory_Mapped_File_Size( Handle : integer ) : int64 ;
                                     virtual ;

                                 { Memory... }
                                 function Total_Physical_Memory : int64 ;
                                     virtual ;

                                 function Available_Physical_Memory : int64 ;
                                     virtual ;

                                 function Total_Virtual_Memory : int64 ;
                                     virtual ;

                                 function Available_Virtual_Memory : int64 ;
                                     virtual ;

                                 function Application_Data_Path( Default_Root, Path : Ansistring ;
                                     Flags : integer ) : Ansistring ;
                                     virtual ;

                                 function Create_Path( Path : AnsiString ) : integer ;
                                     virtual ;
                         end ; { TSOS_Windows }
     PSOS_Windows = ^TSOS_Windows ;

implementation

uses { Borland... }
{$IFDEF FPC}
     Applic, // Application_Manager
{$ENDIF}
{$IFDEF NOVCL}
     FMX.Clipboard, FMX.Clipboard.Win, { Clipboard }
     FMX.Forms, { Application }
     FMX.Platform,
     FMX.Printer, { Printer }
     FMX.Types, { TTextAlign }
{$ELSE}
  {$IFDEF DELPHI10UP}
     VCL.Clipbrd, { Clipboard }
     VCL.Forms, // Application
     VCL.Graphics, { TBitmap }
     VCL.Printers, { Printer }
  {$ELSE}
     Clipbrd, { Clipboard }
     Forms, // Application
     Graphics, { TBitmap }
     Printers, { Printer }
  {$ENDIF}
{$ENDIF}
     Windows, { TOSVersionInfo }
     Messages, { WM_CHAR }
     Registry, { TRegistry }
     ShellApi, // ShellExecute
     ShlObj, { PItemIDList }

     { Subroutine Library - Layer 0... }
     _ASCII, { NUL }

     { Subroutine Library - Layer 1... }
     CommonUt, { Edit }
     Compatib, { SetLength }
     DebugInt, { Static_Debug_Wrapper }
{$IFDEF FPC_OR_DELPHI}
  {$IFDEF Internals}
     OSWinDeb, { TMS_Windows_Debugger }
  {$ENDIF}
{$ENDIF}
     UE,
     Win_UE, { Create_PSWindows_UE }

     { Subroutine Library - Layer 3... }
     Filestd,
     WinAPI ;

const IOCTL_STORAGE_CHECK_VERIFY = 2967552 ;

(*
procedure MSDOS( var Regs : TRegisters ) ;

begin
    asm
        PUSH EBP ;
        PUSH ESI ;
        PUSH EDI ;

        MOV AX, Regs._AX ;
        MOV BX, Regs._BX ;
        MOV CX, Regs._CX ;
        MOV DX, Regs._DX ;
        MOV BP, Regs._BP ;
        MOV SI, Regs._SI ;
        MOV DI, Regs._DI ;
        PUSH Regs.Flags ;
        POPF ;
        INT $21 ;
        PUSHF ;
        POP Regs.Flags ;
        MOV Regs._DI, DI ;
        MOV Regs._SI, SI ;
        MOV Regs._BP, BP ;
        MOV Regs._DX, DX ;
        MOV Regs._CX, CX ;
        MOV Regs._BX, BX ;
        MOV Regs._AX, AX ;

        POP EDI ;
        POP ESI ;
        POP EBP ; { Pop EBP }
    end ;
end ;
*)

{ TOS_Device_Manager... }

type TOS_Device_Manager = object( TSDevice_Manager )
                              public { API... }
                                  function Disk_Clusters( Drive : AnsiString ) : int64 ;
                                      virtual ;
                                  { Returns number of clusters on specified disk
                                    drive.  Returns -1 if an error or unknown. }

                                  function Disk_Sector_Size( Drive : AnsiString ) : longint ;
                                      virtual ;
                                  { Returns size of disk sectors, in bytes.
                                    Returns -1 if an error or unknown. }

                                  function Fixed_Device( Device : AnsiString ) : boolean ;
                                      virtual ;
                                  { Returns true if the passed device is fixed
                                    (non-removable) }

                                  function Max_Filename_Length( Device : AnsiString ) : longint ;
                                      virtual ;
                                      
                                  function Media_ID( Device : AnsiString ) : longint ;
                                      virtual ;
                                  { Return type of media in/on device.  -1 if
                                    error or unknown. }

                                  function Valid_Device( Device : AnsiString ) : boolean ;
                                      virtual ;
                                  { Returns true if the passed device is valid. }

                                  function Device_Name( Clas, Index : integer ) : Ansistring ;
                                      virtual ;

                                  function Current_Path( Device : AnsiString ) : Ansistring ;
                                      virtual ;

                                  function Media_Present( Device : AnsiString ) : boolean ;
                                      virtual ;
                          end ; // TOS_Device_Manager
     POS_Device_Manager = ^TOS_Device_Manager ;


{ API... }

function TOS_Device_Manager.Disk_Clusters( Drive : AnsiString ) : int64 ;

begin
    Disk_Clusters := -1 ;
end ;


function TOS_Device_Manager.Disk_Sector_Size( Drive : AnsiString ) : longint ;

begin
    Disk_Sector_Size := -1 ;
end ;


function TOS_Device_Manager.Fixed_Device( Device : AnsiString ) : boolean ;

begin
    Fixed_Device := False ;
    if(
        ( length( Device ) = 0 )
        or
        ( ( length( Device ) > 1 ) and ( Device[ 2 ] <> ':' ) )
      ) then
    begin
        exit ;
    end ;

    case GetDriveType( PChar( copy( Device, 1, 2 ) + '\' ) ) of
        DRIVE_FIXED: Result := True ;
    end ;
end ;


function TOS_Device_Manager.Max_Filename_Length( Device : AnsiString ) : longint ;

begin
    Result := -1 ;
    if(
        ( length( Device ) = 0 )
        or
        ( ( length( Device ) > 1 ) and ( Device[ 2 ] <> ':' ) )
      ) then
    begin
        exit ;
    end ;
    Result := 255 ;
end ;


function TOS_Device_Manager.Media_ID( Device : AnsiString ) : longint ;

var Typ : integer ;

begin
    Media_ID := -1 ;
    if(
        ( length( Device ) = 0 )
        or
        ( ( length( Device ) > 1 ) and ( Device[ 2 ] <> ':' ) )
      ) then
    begin
        exit ;
    end ;

    Typ := GetDriveType( PChar( copy( Device, 1, 2 ) + '\' ) ) ;
    case Typ of
        DRIVE_REMOVABLE: Result := DMI_Removable ;
        DRIVE_FIXED: Result := DMI_Fixed ;
        DRIVE_REMOTE: Result := DMI_Remote ;
        DRIVE_CDROM: Result := DMI_CDROM ;
        DRIVE_RAMDISK: Result := DMI_RAM ;
    end ;
end ;


function TOS_Device_Manager.Valid_Device( Device : AnsiString ) : boolean ;

begin
    Valid_Device := ( Media_ID( Device ) <> -1 ) ;
end ;


function TOS_Device_Manager.Device_Name( Clas, Index : integer ) : Ansistring ;

var Loop : integer ;

begin
    Device_Name := '' ;
    if( ( Clas = DC_Unknown ) or ( Clas = DC_Disk ) ) then
    begin
        for Loop := 1 to 26 do
        begin
            if( Valid_Device( chr( Loop + 64 ) + ':' ) ) then
            begin
                dec( Index ) ;
                if( Index < 0 ) then
                begin
                    Device_Name := chr( Loop + 64 ) + ':' ;
                    exit ;
                end ;
            end ;
        end ;
    end ;
end ;


function TOS_Device_Manager.Current_Path( Device : AnsiString ) : Ansistring ;

begin
    Current_Path := '' ;
    if( Device = '' ) then
    begin
        setlength( Result, 127 ) ;
        GetCurDir( PAnsiChar( Result ), 0 ) ;
        setlength( Result, strlen( PAnsiChar( Result ) ) ) ;
    end else
    if( Valid_Device( Device ) ) then
    begin
        setlength( Result, 127 ) ;
        GetCurDir( PAnsiChar( Result ), ord( Device[ 1 ] ) - 64 ) ;
        setlength( Result, strlen( PAnsiChar( Result ) ) ) ;
    end ;
end ;


function TOS_Device_Manager.Media_Present( Device : AnsiString ) : boolean ;

var D : string ;
    Dummy : DWORD ;
    Handle : THandle ;
    Info : OSVERSIONINFO ;
    T : integer ;

begin
    // Setup...
    if( not Valid_Device( Device ) ) then
    begin
        Media_Present := False ;
        exit ; // Invalid device
    end ;
    Result := True ;

    Info.dwOSVersionInfoSize := sizeof( Info ) ;
    GetVersionEx( Info ) ;
    if( Info.dwPlatformId = VER_PLATFORM_WIN32_NT ) then // Can only do this check on NT/2000/XP
    begin
        D := copy( Device, 1, 2 ) + '\' ;
        T := Media_ID( PChar( D ) ) ;
        if( ( T = DMI_CDROM ) or ( T = DMI_Removable ) ) then // Removable media
        begin
            D := '\\.\' + copy( D, 1, 2 ) ;
            Handle := CreateFile( PChar( D ), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0 ) ;
            if( Handle = INVALID_HANDLE_VALUE ) then // Not a valid device
            begin
                Result := False ;
                exit ;
            end ;
            try
                Result := DeviceIoControl( Handle, IOCTL_STORAGE_CHECK_VERIFY, nil, 0, nil, 0, Dummy, nil ) ;
            finally
                CloseHandle( Handle ) ;
            end ;
        end ;
    end ; // if(info.dwPlatformId == VER_PLATFORM_WIN32_NT)
end ; // TOS_Device_Manager.Media_Present


const HWD_MT = 1 shl 28 ;
const Number_Of_LP = $FF0000 ;
const Num_Cores = $FC000000 ;

{$IFDEF ASM_CPUID}
function _Get_CPUID : integer ;

asm
    xor eax, eax
    cpuid
    mov Result, eax
end ;
{$ENDIF}


function CPUID_Supported : boolean ;

var Max_Value : longint ;

begin
{$IFNDEF ASM_CPUID}
    Result := False ;
{$ELSE}
    Max_Value := 0 ;
    try
        Max_Value := _Get_CPUID ;
        Result := ( Max_Value <> 0 ) ;
    except
        Result := False ;
    end ;
{$ENDIF}
end ;


{$IFDEF ASM_CPUID}
function _Get_HWMT : longint ;

asm
    mov eax,1
    cpuid
    mov Result, edx
end ;
{$ENDIF}


function Is_HWMT_Supported : boolean ;

var _EDX : longint ;

begin
    if( not CPUID_Supported ) then
    begin
        Result := False ;
        exit ;
    end ;
    _EDX := 0 ;
{$IFDEF ASM_CPUID}
    _EDX := _Get_HWMT ;
{$ENDIF}
    Result := ( ( _EDX and HWD_MT ) <> 0 ) ;
end ;


{$IFDEF ASM_CPUID}
function _Get_LP : longint ;

asm
    mov eax,1
    cpuid
    mov Result, ebx
end ;
{$ENDIF}


function Number_Of_Logical_Processors : integer ;

var _EBX : longint ;

begin
    if( not Is_HWMT_Supported ) then
    begin
        Result := 1 ;
        exit ;
    end ;
    _EBX := 0 ;
{$IFDEF ASM_CPUID}
    _EBX := _Get_LP ;
{$ENDIF}
    Result := ( _EBX and Number_of_LP ) shr 16 ;
end ;


{$IFDEF ASM_CPUID}
function _Get_Max_Cores : integer ;

label Single_Core, Multi_Core ;

asm
    mov eax, 0
    cpuid
    cmp eax, 4
    jl Single_Core
    mov eax, 4
    mov ecx, 0
    cpuid
    mov Result, eax
    jmp Multi_Core
Single_Core:
    mov Result, 0
Multi_Core:
end ;
{$ENDIF}


function Get_Max_Cores : integer ;

var _EAX : longint ;

label Single_Core, Multi_Core ;

begin
    if( not Is_HWMT_Supported ) then
    begin
        Result := 1 ;
        exit ;
    end ;
    _EAX := 0 ;
{$IFDEF ASM_CPUID}
    _EAX := _Get_Max_Cores ;
{$ENDIF}
    Result := ( ( _EAX and Num_Cores ) shr 26 ) + 1 ;
end ;



{ TOS_Windows methods... }

{ Constructors and destructors... }

constructor TSOS_Windows.Init ;

begin
    inherited Init ;

    Initialize( Input_Buffer ) ;
    Initialize( Internal_Clipboard ) ;
    _Device_Manager := nil ;
end ;


destructor TSOS_Windows.Done ;

var Loop : integer ;

begin
    _Device_Manager.Free ;
    _Device_Manager := nil ;
    if( _Text_Presentation_Manager <> nil ) then
    begin
        dispose( _Text_Presentation_Manager, Done ) ;
        _Text_Presentation_Manager := nil ;
    end ;
    finalize( Input_Buffer ) ;
    finalize( Internal_Clipboard ) ;

    if( File_Mapping_List <> nil ) then
    begin
        for Loop := File_Mapping_List.Count - 1 downto 0 do
        begin
            Unmap_File_From_Memory( Loop + 1 ) ;
        end ;
        File_Mapping_List.Free ;
        File_Mapping_List := nil ;
    end ;

    inherited Done ;
end ;


{ API... }

function TSOS_Windows.Name : Ansistring ; { O/S name }

var X : TOSVersionInfo ;

begin
    X.dwOSVersionInfoSize := sizeof( X ) ;
    GetVersionEx( X ) ;
    case X.dwPlatformId of
        VER_PLATFORM_WIN32s:
            if( Version = '4.0' ) then
            begin
                Name := 'Windows 95' ;
            end else
            if( Version = '4.01' ) then
            begin
                Name := 'Windows 95B OSR2' ;
            end else
            if( Version = '4.1' ) then
            begin
                Name := 'Windows 98' ;
            end else
            if( Version = '4.9' ) then
            begin
                Name := 'Windows ME' ;
            end else
            begin
                Name := 'Windows' ;
            end ;
        VER_PLATFORM_WIN32_NT:
            if( Version = '5.0' ) then
            begin
                Name := 'Windows 2000' ;
            end else
            if( Version = '5.1' ) then
            begin
                Name := 'Windows XP' ;
            end else
            if( Version = '5.2' ) then
            begin
                Name := 'Windows 2003' ;
            end else
            if( Version = '6.0' ) then
            begin
                Name := 'Windows Vista' ;
            end else
            if( Version = '6.1' ) then
            begin
                Name := 'Windows 7' ;
            end else
            begin
                Name := 'Windows NT' ;
            end ;
        else Name := 'WIN32' ;
    end ;
end ; // TOS_Windows.Name


function TSOS_Windows.Version : Ansistring ; { O/S version }

var X : TOSVersionInfo ;

begin
    X.dwOSVersionInfoSize := sizeof( X ) ;
    GetVersionEx( X ) ;
    case X.dwPlatformId of
        VER_PLATFORM_WIN32s: Version := '3.1' ;
        VER_PLATFORM_WIN32_WINDOWS,
            VER_PLATFORM_WIN32_NT: Version := inttostr( X.dwMajorVersion ) + '.' + inttostr( X.dwMinorVersion ) ;
        else Version := '' ;
    end ;
end ;


function TSOS_Windows.Application_Path : Ansistring ;

var Dummy : integer ;
    S : string ;

begin
    S := Paramstr( 0 ) ;
    Dummy := length( S ) ;
    while( S[ Dummy ] <> '\' ) do
    begin
        dec( Dummy ) ;
    end ;
    S := copy( S, 1, Dummy ) ;
    if( length( S ) = 0 ) then
    begin
        S := GetCurrentDir ;
    end ;
    Application_Path := S ;
end ;


procedure TSOS_Windows.Cancel_Typeahead( Handle : integer ) ;

var Msg : TMsg ;

begin
    Input_Buffer := '' ;
    if( Handle = 0 ) then // Default
    begin
        try
            FlushConsoleInputBuffer( GetStdHandle( STD_INPUT_HANDLE ) ) ;
        except
        end ;
        while( PeekMessage( Msg, 0, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE ) ) do ;
    end else
    begin
        if( IsWindow( Handle ) ) then
        begin
            while( PeekMessage( Msg, 0, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE ) ) do ;
        end else
        begin
{$IFDEF CONSOLE}
            FlushConsoleInputBuffer( Handle ) ;
{$ENDIF}
        end ;
    end ;

    inherited Cancel_Typeahead( Handle ) ;
end ;


function Parse_Parameter( Separator : Ansistring ; var Command : AnsiString ) : Ansistring ;
{ Return next parameter - have a local copy of this routine to ensure that this
  unit is initialized before any others that might reference it (Parse.pas
  includes too many other units). }

var Dummy : Integer ;
    Quote : AnsiChar ;

begin
    Quote := ' ' ; { No quote yet }
    if Length( Separator ) = 0 then Separator := ' ' ; { Default to space }
    for Dummy := 1 to Length( Command ) do
    begin
        if( Quote <> ' ' ) { if within a quote }
            then
                if( Command[ Dummy ] = Quote ) then Quote := ' ' else
            else { Not within a quote }
                if( ( Command[ Dummy ] = '"' ) or ( Command[ Dummy ] = Chr( 39 ) ) )
                    then Quote := Command[ Dummy ] ; { Check for quotes }
        if( Command[ Dummy ] = Separator[ 1 ] ) then if( Quote = ' ' ) then
        begin
            Parse_Parameter := Copy( Command, 1, Dummy - 1 ) ; { Return parameter }
            Command := Copy( Command, Dummy + 1, length( Command ) ) ; { Strip from command line }
            Exit ; { Done }
        end ;
    end ;
    { At this point, the entire command line is a parameter }
    Parse_Parameter := Command ; { Return next parameter }
    Command := '' ; { Strip from command line }
end ; { Parse_Parameter }


function TSOS_Windows.Command_Line : Ansistring ;

begin
    Result := GetCommandLine ;
    Parse_Parameter( ' ', Result ) ; { Trim program name }
end ;


{$IFDEF FPC_OR_DELPHI}
function TSOS_Windows.Debugger : PSDebug_Interface ;

begin
  {$IFDEF Internals}
    Debugger := Static_Debug_Wrapper( TMS_Windows_Debugger.Create ) ;
  {$ELSE}
    Debugger := nil ;
  {$ENDIF}
end ;
{$ENDIF}


function VK_Code( Code : integer ) : boolean ;

begin
    case Code of
        VK_LBUTTON,
        VK_RBUTTON,
        VK_CANCEL,
        VK_MBUTTON,
        VK_CLEAR,
        VK_SHIFT,
        VK_CONTROL,
        VK_MENU,
        VK_PAUSE,
        VK_CAPITAL,
        VK_KANA,
        VK_JUNJA,
        VK_FINAL,
        VK_HANJA,
        VK_CONVERT,
        VK_NONCONVERT,
        VK_ACCEPT,
        VK_MODECHANGE,
        VK_PRIOR,
        VK_NEXT,
        VK_END,
        VK_HOME,
        VK_LEFT,
        VK_UP,
        VK_RIGHT,
        VK_DOWN,
        VK_SELECT,
        VK_PRINT,
        VK_EXECUTE,
        VK_SNAPSHOT,
        VK_INSERT,
        VK_DELETE,
        VK_HELP,
        VK_LWIN,
        VK_RWIN,
        VK_APPS,
        VK_F1,
        VK_F2,
        VK_F3,
        VK_F4,
        VK_F5,
        VK_F6,
        VK_F7,
        VK_F8,
        VK_F9,
        VK_F10,
        VK_F11,
        VK_F12,
        VK_F13,
        VK_F14,
        VK_F15,
        VK_F16,
        VK_F17,
        VK_F18,
        VK_F19,
        VK_F20,
        VK_F21,
        VK_F22,
        VK_F23,
        VK_F24,
        VK_NUMLOCK,
        VK_SCROLL,
        VK_PROCESSKEY,
        VK_ATTN,
        VK_CRSEL,
        VK_EXSEL,
        VK_EREOF,
        VK_PLAY,
        VK_ZOOM,
        VK_NONAME,
        VK_PA1,
        VK_OEM_CLEAR : Result := True ;
        else Result := False ;
    end ;
end ;


function TSOS_Windows.Get_Char( Wait : boolean ; var Valid : boolean ;
    Handle : integer ) : Ansichar ;

var Dummy : DWORD ;
    Count : integer ;
    _Handle : integer ;
    Input_Data : TInputRecord ;
    Msg : TMsg ;

    function Do_Window : boolean ;

    label Loop ;

    begin
        Result := False ;
Loop:
        if( PeekMessage( Msg, 0, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE ) ) then
        begin
            if( ( Msg.Message <> WM_CHAR ) and ( Msg.Message <> WM_SYSCHAR ) ) then
            begin
                if( not VK_Code( Msg.WParam ) ) then
                begin
                    if( TranslateMessage( Msg ) ) then
                    begin
                        DispatchMessage( Msg ) ;
                        goto Loop ;
                    end ;
                end ;
            end ;
            if( ( Msg.message = WM_KEYUP ) or ( Msg.Message = WM_SYSKEYUP ) ) then
            begin
                goto Loop ;
            end ;
            Msg.LParam := ( Msg.LParam and $FFFF ) - 1 ; // Mask off the count
            if(
                (
                  ( Msg.Message = WM_KEYDOWN )
                  or
                  ( Msg.Message = WM_SYSKEYDOWN )
                )
                and
                ( VK_Code( Msg.WParam ) )
              ) then
            begin
                Get_Char := ESC ;
                Input_Buffer := 'W' + chr( Msg.WParam ) ;
                Result := True ;
                exit ;
            end ;
            Get_Char := Ansichar( Msg.WParam ) ;

            // Handle repeats...
            Dummy := length( Input_Buffer ) + Msg.LParam ;
            setlength( Input_Buffer, Dummy ) ;
            while( Msg.LParam > 0 ) do
            begin
                Input_Buffer[ Dummy ] := AnsiChar( Msg.WParam ) ;
                dec( Dummy ) ;
                dec( Msg.LParam ) ;
            end ;

            Result := True ;
        end else
        begin
            Application.ProcessMessages ;
        end ; // if( PeekMessage( Msg, 0, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE ) )
    end ; // .Do_Window


    function Do_Console : boolean ;

    label Loop ;

    var C : AnsiChar ;
        Msg : TMsg ;

    begin
        Result := False ;
    Loop:
        Get_Char := NUL ;
        GetNumberOfConsoleInputEvents( _Handle, Dummy ) ;
        if( Wait ) then
        begin
            while( Dummy = 0 ) do
            begin
                Application.ProcessMessages ;
                GetNumberOfConsoleInputEvents( _Handle, Dummy ) ;
                if( ( Dummy = 0 ) and ( Handle = 0 ) ) then
                begin
                    if( PeekMessage( Msg, 0, WM_KEYFIRST, WM_KEYLAST, PM_NOREMOVE ) ) then
                    begin
                        Result := Do_Window ;
                        exit ;
                    end ;
                end ;
            end ;
        end ;
        if( Dummy > 0 ) then
        begin
            { Get the data... }
            ReadConsoleInput( _Handle, Input_Data, 1, Dummy ) ;
            if( Input_Data.EventType <> KEY_EVENT ) then
            begin
                goto Loop ;
            end ;
            if( not Input_Data.Event.KeyEvent.bKeyDown ) then
            begin
                goto Loop ;
            end ;

            C := ansichar( Input_Data.Event.KeyEvent.AsciiChar ) ;
            Get_Char := C ;

            { Handle repeats... }
            Count := Input_Data.Event.KeyEvent.wRepeatCount ;
            Dummy := length( Input_Buffer ) + Count ;
            setlength( Input_Buffer, Dummy ) ;
            while( Count > 0 ) do
            begin
                Input_Buffer[ Dummy ] := Ansichar( C ) ;
                dec( Dummy ) ;
                dec( Count ) ;
            end ;
        end ;
    end ;

label Loop ;

var C : Ansichar ;
    X : integer ;

begin
Loop:
    if( length( Input_Buffer ) > 0 ) then
    begin
        Get_Char := ansichar( Input_Buffer[ 1 ] ) ;
        Input_Buffer := copy( Input_Buffer, 2, length( Input_Buffer ) ) ;
        exit ;
    end ;
    _Handle := Handle ;
    if( Handle = 0 ) then
    begin
//{$IFDEF CONSOLE}
        X := 0 ;
        if( not GetConsoleMode( GetStdHandle( STD_INPUT_HANDLE ), Dummy ) ) then
        begin
            X := GetLastError ;
        end ;
        if( X = 6 ) then // Redirected input
        begin
            read( C ) ;
            Get_Char := C ;
            if( C = CR )then
            begin
{$HINTS OFF}
{$WARNINGS OFF}
                read( C ) ; // Swallow LF
{$WARNINGS ON}
{$HINTS ON}
            end ;
            exit ;
        end else
        begin
            _Handle := GetStdHandle( STD_INPUT_HANDLE ) ;
            if( Do_Console ) then
            begin
                exit ;
            end ;
        end ;
(*
{$ELSE}
        if( Do_Window ) then
        begin
            exit ;
        end ;
{$ENDIF}
*)
    end else
    begin
        if( IsWindow( Handle ) ) then
        begin
            if( Do_Window ) then
            begin
                exit ;
            end ;
        end else
        begin
            X := 0 ;
            if( not GetConsoleMode( Handle, Dummy ) ) then
            begin
                X := GetLastError ;
            end ;
            if( X = 6 ) then // Redirected input
            begin
                read( C ) ;
                Get_Char := C ;
                exit ;
            end else
            begin
                if( Do_Console ) then
                begin
                    exit ;
                end ;
            end ;
        end ;
    end ;

    { No input waiting }
    if( Wait ) then { Wait }
    begin
        goto Loop ;
    end ;
    Get_Char := NUL ;
end ; // TOS_Windows.Get_Char


procedure TSOS_Windows.Beep( Typ : integer ) ;

begin
    MessageBeep( 0 ) ;
end ;


function TSOS_Windows.Get_Symbol( const X : AnsiString ) : Ansistring ;

var Y : Ansistring ;

begin
    if( GetEnvironmentVariable( X, Y, 127 ) = 0 ) then
    begin
        Get_Symbol := '' ; { Not defined }
    end else
    begin
        Get_Symbol := Y ;
    end ;
end ;


procedure TSOS_Windows.Get_Date( var Year, Month, Day : word ) ;

begin
    DecodeDate( Date, Year, Month, Day ) ;
end ;


var Last_S : word ;
    Last_H : word ;
    Last_NS : word ;

procedure TSOS_Windows.Get_Time( var Hour, Minute, Second, Hundreths, NS : word ) ;

var I : int64 ;

begin
    DecodeTime( Time, Hour, Minute, Second, Hundreths ) ;
    NS := 0 ;
    if( QueryPerformanceCounter( I ) ) then
    begin
        while( I >= 1280000 ) do
        begin
            I := I shr 8 ;
        end ;
        while( I >= 10000 ) do
        begin
            I := I shr 1 ;
        end ;
        NS := I ;
    end ;
    if( ( Last_S = Second ) and ( Last_H = Hundreths ) and ( NS < Last_NS ) ) then
    begin
        // Make sure we don't go backwards in time (in ns)
        NS := Last_NS + 1 ;
        if( NS >= 10000 ) then
        begin
            NS := Last_NS ;
        end ;
    end ;
    Last_S := Second ;
    Last_H := Hundreths ;
    Last_NS := NS ;
end ;


function TSOS_Windows.Logical_CPUs : integer ;

begin
    Logical_CPUs := Number_Of_Logical_Processors ;
end ;


function TSOS_Windows.Pending_Input( Handle : integer ) : boolean ;

var Msg : TMsg ;

begin
    Application.ProcessMessages ;
    Pending_Input := PeekMessage( Msg, 0, WM_CHAR, WM_CHAR, PM_NOREMOVE ) ;
end ;


function TSOS_Windows.PID : longint ;

begin
    PID := GetCurrentProcessId ;
end ;


procedure TSOS_Windows.Print( T : AnsiString ) ;

var Dummy : integer ;
{$IFDEF NOVCL}
    H : single ;
    Rect : TRect ;
{$ELSE}
    H : integer ;
{$ENDIF}
    I : integer ;

begin
    Printer.BeginDoc ;
    I := 0 ;
    H := Printer.Canvas.TextHeight( T ) ;
    while( length( T ) > 0 ) do
    begin
        Dummy := pos( CR, T ) ;
        if( Dummy = 0 ) then
        begin
            Dummy := length( T ) + 1 ;
        end ;

{$IFDEF NOVCL}
        Rect.Top := 0 ;
        Rect.Left := 0 ;
        Rect.Bottom := trunc( int( H ) ) ;
        Rect.Right := Printer.Canvas.Width ;
        Printer.Canvas.FillText( rect, copy( T, 1, Dummy - 1 ), False, 1.0, [], TTextAlign.Trailing ) ;
{$ELSE}
        Printer.Canvas.TextOut( 0, I, copy( T, 1, Dummy - 1 ) ) ;
{$ENDIF}
        T := copy( T, Dummy + 1, length( T ) ) ;
        I := I + trunc( int( H ) ) ;
    end ;
    Printer.EndDoc ;
end ;


function TSOS_Windows.Last_Error : PSUnified_Exception ;

begin
    Result := Create_PSWindows_UE( GetLastError, '' ) ;
end ;


function TSOS_Windows.Error_Text( Code : longint ;
    const Language : AnsiString ) : Ansistring ;

var Dummy : integer ;
    S : Ansistring ;

begin
    if( Code <> 0 ) then
    begin
        //TODO: Convert passed Language to a language ID
        setlength( S, 255 ) ;
        FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER or FORMAT_MESSAGE_FROM_SYSTEM,
            nil, Code, 0, S, length( S ), nil ) ;
        Dummy := pos( #0, S + #0 ) ;
        Error_Text := copy( S, 1, Dummy - 1 ) ;
    end else
    begin
        Error_Text := '' ;
    end ;
end ;


function TSOS_Windows.Get_Temp_Path : Ansistring ;

var Res : Ansistring ;

begin
    SetLength( Res, 255 ) ;
    SetLength( Res, GetTempPath( length( Res ), Res ) ) ;
    Get_Temp_Path := Res ;
end ;


function TSOS_Windows.System_Path : Ansistring ;

var L : integer ;

begin
    L := GetSystemDirectory( Result, 32767 ) ;
end ;


function TSOS_Windows.Text_Presentation_Manager : PSText_Presentation_Manager ;

begin
    if( _Text_Presentation_Manager = nil ) then
    begin
        _Text_Presentation_Manager := new( PSWindows_Text_Presentation_Manager, Init ) ;
    end ;
    Text_Presentation_Manager := _Text_Presentation_Manager ;
end ;


function TSOS_Windows.Swap_File_List : Ansistring ;

var Buffer : Ansistring ;
    Dir, Nam, S, Temp, Work : Ansistring ;
    Dummy : integer ;
    Reg : TRegistry ;

begin
    // Determine default system directory for paging files...
    Dummy := GetSystemDirectory( Buffer, 32767 ) ;
    setlength( Dir, Dummy ) ;
    move( Buffer, PChar( string( Dir ) )[ 0 ], Dummy ) ;
    Dummy := length( Dir ) - 1 ;
    while( ( Dummy > 0 ) and ( Dir[ Dummy ] <> '\' ) ) do // Find parent of system directory
    begin
        dec( dummy ) ;
    end ;
    Dir := copy( Dir, 1, Dummy ) ;

    // Build the list...
    S := '' ;
    if( copy( Version, 1, 1 ) < '5' ) then // Windows 95/98/ME
    begin
        Nam := Dir + 'system.ini' ;
        if( GetPrivateProfileString( '386enh', 'PagingFile', '', Buffer, 32767, Nam ) > 0 ) then
        begin
            S := Buffer ;
            if( copy( S, 2, 1 ) <> ':' ) then // Missing a drive specification
            begin
                if( GetPrivateProfileString( '386enh', 'PagingDrive', '', Buffer, 32767, Nam ) > 0 ) then
                begin
                    S := Buffer[ 1 ] + ':' + S ;
                end else
                begin
                    S := copy( Nam, 1, 2 ) + S ; // Use default drive
                end ;
            end ;
        end else
        if( GetPrivateProfileString( '386enh', 'PagingDrive', '', Buffer, 32767, Nam ) > 0 ) then
        begin
            S := Buffer[ 1 ] + ':\win386.swp' ;
        end else
        begin
            S := Dir + 'win386.swp' ;
        end ;
    end else
    begin
        // Windows NT/2000/XP...

        // Create the list...
        Reg := TRegistry.Create ;
        try
            Reg.RootKey := HKEY_LOCAL_MACHINE ;
            Reg.OpenKeyReadOnly( 'SYSTEM\CurrentControlSet\Control\Session Manager\Memory Management' ) ;
            if( Reg.ValueExists( 'PagingFiles' ) ) then
            begin
                try
                    { TRegistry doesn't read REG_MULTI_SZ as a string (gives an
                      exception), so we have to read it as Binary. }
                    Dummy := Reg.ReadBinaryData( 'PagingFiles', Buffer, 32767 ) ;
                    setlength( S, Dummy - 1 ) ;
                    move( Buffer, PAnsiChar( S )[ 0 ], Dummy - 1 ) ;
                except
                    try
                        S := Reg.ReadString( 'PagingFiles' ) ; // Just in case they change the above behavior
                    except
                        S := Dir + 'Pagefile.sys' ;
                    end ;
                end ;
                Dummy := pos( NUL, S ) ;
                while( Dummy > 0 ) do
                begin
                    S[ Dummy ] := ';' ;
                    Dummy := pos( NUL, S ) ;
                end ;
                Temp := S ;
                S := '' ;
                while( length( Temp ) > 0 ) do
                begin
                    Dummy := pos( ';', Temp + ';' ) ;
                    Work := copy( Temp, 1, Dummy - 1 ) ;
                    Temp := copy( Temp, Dummy + 1, length( Temp ) ) ;
                    Dummy := pos( ' ', Work + ' ' ) ;
                    if( length( S ) > 0 ) then // Not first item in list
                    begin
                        S := S + ';' ;
                    end ;
                    if( copy( Work, 1, 2 ) = '?:' ) then
                    begin
                        S := S + Dir[ 1 ] + copy( Work, 2, Dummy - 1 ) ;
                    end else
                    begin
                        S := S + copy( Work, 1, Dummy - 1 ) ;
                    end ;
                end ; // while( length( Temp ) > 0 )
            end else
            begin
                S := Dir + 'Pagefile.sys' ;
            end ; // if( Reg.ValueExists( 'PagingFiles' ) )
        finally
            Reg.Free ;
        end ;
    end ; // if( copy( Version, 1, 1 ) < '5' )
    Swap_File_List := S ;
end ; // TOperating_System.Swap_File_List


function TSOS_Windows.Spawn( const Path, Parameters, Default_Dir : Ansistring ;
    Flags : integer ) : boolean ;

var Exit_Code : longword ;
    PDir : AnsiString ;
    P_I : TProcessInformation ;
    SI : TStartupInfo ;
//    Temp : string ;

begin
    // Setup...
    Result := False ;
    fillchar( SI, sizeof( SI ), 0 ) ;
    SI.cb := sizeof( SI ) ;
    SI.dwFlags := STARTF_USESHOWWINDOW ;
    if( ( Flags and Spawn_Quiet ) = 0 ) then
    begin
        SI.wShowWindow := SW_Show ;
    end else
    begin
        SI.wShowWindow := SW_Hide ;
    end ;
    if( Path = '' ) then
    begin
        PDir := '' ;
    end else
    begin
        PDir := Default_Dir ;
    end ;

    if( Parameters = '' ) then
    begin
        if( not CreateProcess( '', Path, nil, nil, false, 0, nil, PDir, SI, P_I ) ) then
        begin
{$IFDEF NOVCL}
            Result := integer( ShellExecute( 0, 'open', Path,
                '', PDir, SW_SHOWNORMAL ) ) > 32 ;
{$ELSE}
            Result := integer( ShellExecute( Application.MainForm.handle, 'open', Path,
                '', PDir, SW_SHOWNORMAL ) ) > 32 ;
{$ENDIF}
            exit ;
        end ;
    end else
    if( not CreateProcess( Path, Parameters, nil, nil, false,
        0, nil, PDir, SI, P_I ) ) then
    begin
{$IFDEF NOVCL}
        Result := integer( ShellExecute( 0, 'open', Path,
            Parameters, PDir, SW_SHOWNORMAL ) ) > 32 ;
{$ELSE}
        Result := integer( ShellExecute( Application.MainForm.handle, 'open', Path,
            Parameters, PDir, SW_SHOWNORMAL ) ) > 32 ;
{$ENDIF}
        exit ;
    end ;
//    Temp := Path + ' ' + Parameters ;
//    WinExec( PChar( Temp ), SW_SHOW ) ;
    if( ( Flags and Spawn_Asynchronous ) = 0 ) then
    begin
        WaitForInputIdle( P_I.hProcess, 5000 ) ;
        Result := True ;
        try
            Exit_Code := STILL_ACTIVE ;
            while( Exit_Code = STILL_ACTIVE ) do
//            while( GetProcessVersion( P_I.dwProcessId ) <> 0 ) do
            begin
                try
                    Application.ProcessMessages ; // Wait until process is done
                except
                end ;
                GetExitCodeProcess( P_I.hProcess, Exit_Code ) ;
            end ;
        except
        end ;
    end ;
    CloseHandle( P_I.hProcess ) ;
    CloseHandle( P_I.hThread ) ;
end ; // Spawn


function TSOS_Windows.Thread_ID : longint ;

begin
    Thread_ID := GetCurrentThreadId ;
end ;


function TSOS_Windows.Timestamp : int64 ;

begin
    Timestamp := 0 ;
end ;


function TSOS_Windows.Get_Clipboard( Path : AnsiString ) : Ansistring ;

var Handle : THandle ;
    P : pointer ;
{$IFDEF NOVCL}
    Clipboard : IFMXClipboardService ;
{$ENDIF}

begin
    if( Path = '' ) then
    begin
        Get_Clipboard := Internal_Clipboard ;
        exit ;
    end ;
{$IFDEF NOVCL}
    if TPlatformServices.Current.SupportsPlatformService( IFMXClipboardService, Clipboard ) then
    begin
        if( not Clipboard.GetClipboard.IsEmpty ) then
        begin
            if( Path = 'TEXT' ) then
            begin
                if( Clipboard.GetCLipboard.IsType<string> ) then
                begin
                    Result := Clipboard.GetClipboard.ToString ;
                    exit ;
                end ;
            end ;
        end ;
    end ;
{$ELSE}
    Path := Edit( Path, -1 ) ;
    if( Path = 'TEXT' ) then
    begin
        Get_Clipboard := Clipboard.AsText ;
        exit ;
    end ;
    if( Path = 'RTF' ) then
    begin
        if( not Clipboard.HasFormat( CF_TEXT ) ) then
        begin
            Get_Clipboard := '' ;
            exit ;
        end ;

        OpenClipboard( Application.MainForm.Handle ) ;
        try
            Handle := GetClipboardData( RegisterClipboardFormat( 'Rich Text Format' ) ) ;
            if( Handle = 0 ) then
            begin
                Get_Clipboard := '' ;
                exit ;
            end ;

            setlength( Result, GlobalSize( Handle ) ) ;
            try
                P := GlobalLock( Handle ) ;
                try
                    move( P^, PChar( string( Result ) )^, length( Result ) ) ;
                finally
                    GlobalUnlock( Handle ) ;
                end ;
            except
                GlobalFree( Handle ) ;
            end ;
        finally
            CloseClipboard ;
        end ;
        exit ;
    end ;
    if( Path = 'BITMAP' ) then
    begin
        if( not Clipboard.HasFormat( CF_BITMAP ) ) then
        begin
            Get_Clipboard := '' ;
            exit ;
        end ;

        OpenClipboard( Application.MainForm.Handle ) ;
        try
            Handle := GetClipboardData( CF_BITMAP ) ;
            if( Handle = 0 ) then
            begin
                Get_Clipboard := '' ;
                exit ;
            end ;

            setlength( Result, GlobalSize( Handle ) ) ;
            try
                P := GlobalLock( Handle ) ;
                try
                    move( P^, PChar( string( Result ) )^, length( Result ) ) ;
                finally
                    GlobalUnlock( Handle ) ;
                end ;
            except
                GlobalFree( Handle ) ;
            end ;
        finally
            CloseClipboard ;
        end ;
        exit ;
    end ;
{$ENDIF}
    Get_Clipboard := '' ;
end ; { TSOS_Windows.Get_Clipboard }


procedure TSOS_Windows.Set_Clipboard( Path, Value : AnsiString ) ;

var Handle : THandle ;
    P : pointer ;
{$IFDEF NOVCL}
    Clipboard : IFMXClipboardService ;
{$ENDIF}

begin
    if( Path = '' ) then
    begin
        Internal_Clipboard := Value ;
        exit ;
    end ;
{$IFDEF NOVCL}
    if TPlatformServices.Current.SupportsPlatformService( IFMXClipboardService, Clipboard ) then
    begin
        if( not Clipboard.GetClipboard.IsEmpty ) then
        begin
            if( Path = 'TEXT' ) then
            begin
                Clipboard.SetClipboard( Value ) ;
                exit ;
            end ;
        end ;
    end ;
{$ELSE}
    if( Path = 'TEXT' ) then
    begin
        Clipboard.AsText := Value ;
        exit ;
    end ;
    if( Path = 'RTF' ) then
    begin
        OpenClipboard( Application.MainForm.Handle ) ;
        try
            Handle := GlobalAlloc( GMEM_MOVEABLE, length( Value ) + 1 ) ;
            try
                P := GlobalLock( Handle ) ;
                try
                    move( PChar( string( Value ) )^, P^, length( Value ) + 1 ) ;
                    SetClipboardData( RegisterClipboardFormat( 'Rich Text Format' ), Handle ) ;
                finally
                    GlobalUnlock( Handle ) ;
                end ;
            except
                GlobalFree( Handle ) ;
            end ;
        finally
            CloseClipboard ;
        end ;
        exit ;
    end ;
    if( Path = 'BITMAP' ) then
    begin
        OpenClipboard( Application.MainForm.Handle ) ;
        try
            Handle := GlobalAlloc( GMEM_MOVEABLE, length( Value ) + 1 ) ;
            try
                P := GlobalLock( Handle ) ;
                try
                    move( PChar( string( Value ) )^, P^, length( Value ) + 1 ) ;
                    SetClipboardData( CF_BITMAP, Handle ) ;
                finally
                    GlobalUnlock( Handle ) ;
                end ;
            except
                GlobalFree( Handle ) ;
            end ;
        finally
            CloseClipboard ;
        end ;
        exit ;
    end ;
{$ENDIF}
end ; { TSOS_Windows.Set_Clipboard }


function TSOS_Windows.Max_File_Size : int64 ;

begin
    Max_File_Size := $3FFFFFFF ;
end ;


function TSOS_Windows.User_Name : Ansistring ;

var Len : dword ;

begin
    Len := 254 ;
    SetLength( Result, Len ) ;
    GetUserName( Result, Len ) ;
    if( Len > 0 ) then
    begin
        SetLength( Result, Len - 1 ) ;
    end else
    begin
        Result := '' ;
    end ;
end ;


function TSOS_Windows.Device_Manager : PSDevice_Manager ;

begin
    if( _Device_Manager = nil ) then
    begin
        _Device_Manager := new( POS_Device_Manager, Init ) ;
    end ;
    Device_Manager := _Device_Manager ;
end ;


function TSOS_Windows.Load_Library( Nam : AnsiString ) : longint ;

begin
    Load_Library := LoadLibrary( PChar( string( Nam ) ) ) ;
end ;


function TSOS_Windows.Get_Symbol_Address( Lib : longint ; Nam : AnsiString ) : int64 ;

begin
    Result := integer( GetProcAddress( Lib, PChar( string( Nam ) ) ) ) ;
end ;


function TSOS_Windows.Show_Message( Handle : longint ; Msg, Title : Ansistring ;
    Options : integer ) : integer ;

var O : integer ;

begin
    case ( Options and 7 ) of
        MBOF_OK : O := MB_OK ;
        MBOF_OKCancel : O := MB_OKCANCEL ;
        MBOF_AbortRetryIgnore : O := MB_ABORTRETRYIGNORE ;
        MBOF_YesNoCancel : O := MB_YESNOCANCEL ;
        MBOF_YesNo : O := MB_YESNO ;
        else O := MB_RETRYCANCEL ;
    end ;
    Result := MessageBox( Handle, PChar( string( Msg ) ), PChar( string( Title ) ), O ) ;
    case Result of
        ID_Abort : Result := MBR_Abort ;
        ID_Cancel : Result := MBR_Cancel ;
        ID_No : Result := MBR_No ;
        ID_OK : Result := MBR_OK ;
        ID_Retry : Result := MBR_Retry ;
        ID_Yes : Result := MBR_Yes ;
    end ;
end ; { TSOS_Windows.Show_Message }


function TSOS_Windows.Get_Country : Ansistring ;

var Buff : Ansistring ;

begin
     GetProfileString( 'Intl', 'sCountry', '', Buff, sizeof( Buff ) ) ;
     Result := Buff ;
end ;


function TSOS_Windows.Max_Memory_Map : int64 ;

var X : OSVERSIONINFO ;

begin
    X.dwOSVersionInfoSize := sizeof( X ) ;
    GetVersionEx( X ) ;
    if( X.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS ) then // Windows 95/98
    begin
        Result := 1073741824 ; // 1 Gb max
    end else
    begin
        Result := 2147483648 ; // 2 Gb max
    end ;
end ;


type TFile_Mapping_Info = class
                              public // API...
                                  Handle : THandle ;
                                  Mapping : THandle ;
                                  Base : pointer ;
                                  Size : int64 ;
                                  Writeable : boolean ;
                          end ;

function TSOS_Windows.Memory_Map_File( _Name : Ansistring ; Writeable : boolean ;
    Start, Size : int64 ; Flag : boolean ) : integer ;

var _Mapping : THandle ;
    _Base : pointer ;
    Info : TFile_Mapping_Info ;
    Temp : int64 ;

begin
    // Open the file...
    if( Writeable ) then
    begin
        Result := CreateFile( PChar( string( _Name ) ), GENERIC_READ or GENERIC_WRITE,
            FILE_SHARE_READ, nil, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0 ) ;
    end else
    begin
        Result := CreateFile( PChar( string( _Name ) ), GENERIC_READ, FILE_SHARE_READ, nil,
            OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0 ) ;
    end ;
		if( Result = integer( INVALID_HANDLE_VALUE ) ) then
    begin
        Result := 0 ;
        exit ;
    end ;

    if( Size = 0 ) then // No size specified - map entire file
    begin
        Temp := 0 ;
        Size := GetFileSize( Result, @Temp ) ;
        Temp := Temp shl 32 ;
        Size := Temp or Size ;
    end ;

    // Create a file-mapping handle...
    if( Writeable ) then
    begin
        _Mapping := CreateFileMapping( Result, nil, PAGE_READWRITE,
            Size shr 32, Size and $FFFFFFFFFFFFFFFF, '' ) ;
    end else
    begin
        _Mapping := CreateFileMapping( Result, nil, PAGE_READONLY,
            Size shr 32, Size and $FFFFFFFFFFFFFFFF, '' ) ;
    end ;
    if( _Mapping = 0 ) then
    begin
        CloseHandle( Result ) ;
        Result := 0 ;
        exit ;
    end ;

    // Map the file...
    if( Writeable ) then
    begin
        _Base := MapViewOfFile( _Mapping, FILE_MAP_ALL_ACCESS,
            Start shr 32, Start and $FFFFFFFFFFFFFFF, Size ) ;
    end else
    begin
        _Base := MapViewOfFile( _Mapping, FILE_MAP_READ,
            Start shr 32, Start and $FFFFFFFFFFFFFFF, Size ) ;
    end ;
    if( _Base = nil ) then
    begin
        CloseHandle( Result ) ;
        CloseHandle( _Mapping ) ;
        Result := 0 ;
        exit ;
    end ;

    // Save the information...
    if( File_Mapping_List = nil ) then
    begin
        File_Mapping_List := TList.Create ;
    end ;
    Info := TFile_Mapping_Info.Create ;
    File_Mapping_List.Add( Info ) ;
    Info.Handle := Result ;
    Info.Mapping := _Mapping ;
    Info.Base := _Base ;
    Info.Size := Size ;
    Info.Writeable := Writeable ;
    Result := File_Mapping_List.Count ;
end ; // TSOS_Windows.Memory_Map_File


procedure TSOS_Windows.Unmap_File_From_Memory( Handle : integer ) ;

var Info : TFile_Mapping_Info ;

begin
    if( ( Handle > 0 ) and ( Handle <= File_Mapping_List.Count ) ) then
    begin
        Info := TFile_Mapping_Info( File_Mapping_List[ Handle - 1 ] ) ;
        if( Info = nil ) then
        begin
            exit ;
        end ;
        if( Info.Writeable ) then
        begin
            FlushViewOfFile( Info.Base, 0 ) ;
        end ;
        try
            UnmapViewOfFile( Info.Base ) ;
            CloseHandle( Info.Mapping ) ;
            CloseHandle( Info.Handle ) ;
        except
            // Don't let exception end program
        end ;
        File_Mapping_List[ Handle - 1 ] := nil ;
        Info.Free ;
    end ;
end ;


function TSOS_Windows.Memory_Mapped_File_Base( Handle : integer ) : pointer ;

var Info : TFile_Mapping_Info ;

begin
    Result := nil ;
    if( ( Handle > 0 ) and ( Handle <= File_Mapping_List.Count ) ) then
    begin
        Info := TFile_Mapping_Info( File_Mapping_List[ Handle - 1 ] ) ;
        if( Info = nil ) then
        begin
            exit ;
        end ;
        Result := Info.Base ;
    end ;
end ;


function TSOS_Windows.Memory_Mapped_File_Size( Handle : integer ) : int64 ;

var Info : TFile_Mapping_Info ;

begin
    Result := 0 ;
    if( ( Handle > 0 ) and ( Handle <= File_Mapping_List.Count ) ) then
    begin
        Info := TFile_Mapping_Info( File_Mapping_List[ Handle - 1 ] ) ;
        if( Info = nil ) then
        begin
            exit ;
        end ;
        Result := Info.Size ;
    end ;
end ;


function TSOS_Windows.Total_Physical_Memory : int64 ;

var Status : TMemoryStatus ;

begin
    GlobalMemoryStatus( Status ) ;
    Result := Status.dwTotalPhys ;
end ;


function TSOS_Windows.Available_Physical_Memory : int64 ;

var Status : TMemoryStatus ;

begin
    GlobalMemoryStatus( Status ) ;
    Result := Status.dwTotalPhys ;
end ;


function TSOS_Windows.Total_Virtual_Memory : int64 ;

var Status : TMemoryStatus ;

begin
    GlobalMemoryStatus( Status ) ;
    Result := Status.dwTotalVirtual ;
end ;


function TSOS_Windows.Available_Virtual_Memory : int64 ;

var Status : TMemoryStatus ;

begin
    GlobalMemoryStatus( Status ) ;
    Result := Status.dwAvailVirtual ;
end ;


function TSOS_Windows.Application_Data_Path( Default_Root, Path : Ansistring ;
    Flags : integer ) : Ansistring ;
//      ADPF_Create = 2 ; { Create path if it doesn't exist }

var Dummy : integer ;
    ItemIDList : PItemIDList ;
    Loop : integer ;
    P : Ansistring ;
    S : string ;

    procedure Default ;

    begin
        // Prior to Windows 98...
        S := paramstr( 0 ) ;
        if( S = '' ) then
        begin
            Result := inherited Application_Data_Path( Default_Root, Path, Flags ) ;
            { Default path }
            exit ;
        end ;
        Dummy := length( S ) ;
        while( ( Dummy > 0 ) and ( S[ Dummy ] <> '\' ) and ( S[ Dummy ] <> ':' ) ) do
        begin
            dec( Dummy ) ;
        end ;
        S := copy( S, 1, Dummy ) ;
        if( ( length( S ) < 4 ) and ( copy( S, 2, 1 ) = ':' ) ) then { Only a device }
        begin
            if( copy( S, length( S ), 1 ) = '\' ) then
            begin
                S := copy( S, 1, length( S ) - 1 ) ; { Trim backslash }
            end ;
            Application_Data_Path :=
                S + inherited Application_Data_Path( Default_Root, Path, Flags ) ;
            exit ;
        end ;
        if( Path <> '' ) then
        begin
            if( ( copy( Path, 1, 1 ) <> '\' ) and ( pos( ':', S ) = 0 ) ) then
            begin
                S := '\' + S ;
            end ;
            S := S + Path ;
        end ;
        if( copy( S, length( S ), 1 ) <> '\' ) then
        begin
            S := S + '\' ;
        end ;
        Result := S ;
    end ; { TSOS_Windows.Application_Data_Path.Default }


    procedure Determine_Path ;

    var Dummy : integer ;
        H : THandle ;
        V : string ;

    begin
        V := Version ;
        if( ( Name = 'WIN32' ) or ( strtofloat( V ) < 4.1 ) ) then
        begin
            Default ;
            exit ;
        end ;

{$IFDEF NOVCL}
        H := 0 ;
{$ELSE}
        if(
            ( Application = nil )
            or
            ( Application.MainForm = nil )
            or
            ( not Application.MainForm.HandleAllocated ) // EAC, 3-Jun-2010
          ) then
        begin
            H := 0 ;
        end else
        begin
            H := Application.MainForm.Handle ;
        end ;
{$ENDIF}
        if( ( strtofloat( V ) < 4.71 ) or ( ( Flags and ADPF_Public ) = 0 ) ) then // Windows 98 or private
        begin
            if( SHGetSpecialFolderLocation( H, CSIDL_APPDATA, ItemIDList ) = NOERROR ) then
            begin
                SHGetPathFromIDList( ItemIDList, P ) ;
                S := string( P ) ;
                Loop := pos( ';', S + ';' ) ;
                S := copy( S, 1, Loop - 1 ) ; // I don't think this will ever happen, but let's be safe
                if( copy( S, length( S ), 1 ) <> '\' ) then
                begin
                    S := S + '\' ;
                end ;
                Result := S + Default_Root + Path ;
                if( not DirectoryExists( Result ) ) then
                begin
                    if( ( Flags and ADPF_Create ) = 0 ) then
                    begin
                        Default ;
                        exit ;
                    end ;
                end ;
                exit ;
            end ;
        end ;

        // XP (or later) and public
        if( SHGetSpecialFolderLocation( H, $23{CSIDL_COMMON_APPDATA}, ItemIDList ) = NOERROR ) then
        begin
            SHGetPathFromIDList( ItemIDList, P ) ;
            S := string( P ) ;
            Loop := pos( ';', S + ';' ) ;
            S := copy( S, 1, Loop - 1 ) ; // I don't think this will ever happen, but let's be safe
            if( copy( S, length( S ), 1 ) <> '\' ) then
            begin
                S := S + '\' ;
            end ;
            Result := S + Default_Root + Path ;
            S := Result ;
            if( copy( S, length( S ), 1 ) = '\' ) then
            begin
                setlength( S, length( S ) - 1 ) ;
            end ;
            if( DirectoryExists( S ) or ( ( Flags and ADPF_Create ) <> 0 ) ) then
            begin
                exit ;
            end ;
        end ;

        Default ;

        // Remove double separators...
        Dummy := pos( '\\', Result ) ;
        while( Dummy > 0 ) do
        begin
            Result := copy( Result, 1, Dummy ) + copy( Result, Dummy + 2, length( Result ) ) ;
            Dummy := pos( '\\', Result ) ;
        end ;
    end ; { TSOS_Windows.Application_Data_Path.Determine_Path }


begin { TSOS_Windows.Application_Data_Path }
    // Normalize paths...
    if( copy( Path, 1, 1 ) = '\' ) then
    begin
        Path := copy( Path, 2, length( Path ) ) ;
    end ;
    if( copy( Default_Root, 1, 1 ) = '\' ) then
    begin
        Default_Root := copy( Default_Root, 2, length( Default_Root ) ) ;
    end ;
    if( Path <> '' ) then
    begin
        if( copy( Path, length( Path ), 1 ) <> '\' ) then
        begin
            Path := Path + '\' ;
        end ;
    end ;
    if( Default_Root <> '' ) then
    begin
        if( copy( Default_Root, length( Default_Root ), 1 ) <> '\' ) then
        begin
            Default_Root := Default_Root + '\' ;
        end ;
    end ;

    // Now determine path...
    Determine_Path ;
    S := Result ;
    if( copy( S, length( S ), 1 ) = '\' ) then
    begin
        setlength( S, length( S ) - 1 ) ;
    end ;
    if( not DirectoryExists( S ) and ( ( Flags and ADPF_Create ) <> 0 ) ) then
    begin
        // Missing directory and caller wants to create missing directories...
        if( Create_Path( S ) <> 0 ) then
        begin
            Result := '' ;
        end ;
    end ;
end ; { TSOS_Windows.Application_Data_Path }


function TSOS_Windows.Create_Path( Path : AnsiString ) : integer ;

var Dummy : integer ;
    So_Far : Ansistring ;

begin
    So_Far := '' ;
    while( Path <> '' ) do
    begin
        Dummy := pos( '\', Path + '\' ) ;
        So_Far := So_Far + copy( Path, 1, Dummy - 1 ) ;
        Path := copy( Path, Dummy + 1, length( Path ) ) ;
        if( copy( So_Far, length( So_Far ), 1 ) <> ':' ) then
        begin
            {$I-}
            MkDir( So_Far ) ;
            {$I+}
            Dummy := IOResult ;
            if( ( Dummy <> 0 ) and ( Dummy <> 183 ) and ( Dummy <> 5 ) ) then // Not okay and not already exists
            begin
                Create_Path := Dummy ;
                exit ;
            end ;
        end ; // if( copy( So_Far, length( So_Far ), 1 ) <> ':' )
        So_Far := So_Far + '\' ;
    end ; // while( Path <> '' )
    Result := 0 ;
end ; // TSOS_Windows.Create_Path


function Get_Locale_Info( ID : integer ) : Ansistring ;

{$IFNDEF NOVCL}
var Size : integer ;
{$ENDIF}

begin
{$IFDEF NOVCL}
    Result := '' ;
{$ELSE}
    Size := 32767 ;
    GetLocaleInfo( LOCALE_USER_DEFAULT, ID, Result, Size ) ;
{$ENDIF}
end ;



{ PSWindows_Numeric_Text_Presentation_Manager... }

type TSWindows_Numeric_Text_Presentation_Manager = object( TSNumeric_Text_Presentation_Manager )
                                                       public
                                                          function Decimal : Ansistring ;
                                                              virtual ;

                                                          function Group : Ansistring ;
                                                              virtual ;

                                                          function Negative : Ansistring ;
                                                              virtual ;

                                                          function Positive : Ansistring ;
                                                              virtual ;
                                                   end ;
     PSWindows_Numeric_Text_Presentation_Manager = ^TSWindows_Numeric_Text_Presentation_Manager ;


function TSWindows_Numeric_Text_Presentation_Manager.Decimal : Ansistring ;

begin
    Decimal := Edit( Get_Locale_Info( LOCALE_SDECIMAL ), 4 ) ;
    if( Result = '' ) then
    begin
        Result := '.' ;
    end ;
end ;


function TSWindows_Numeric_Text_Presentation_Manager.Group : Ansistring ;

begin
    Group := Get_Locale_Info( LOCALE_STHOUSAND ) ;
end ;


function TSWindows_Numeric_Text_Presentation_Manager.Negative : Ansistring ;

begin
    Negative := Get_Locale_Info( LOCALE_SNEGATIVESIGN ) ;
end ;


function TSWindows_Numeric_Text_Presentation_Manager.Positive : Ansistring ;

begin
    Positive := Get_Locale_Info( LOCALE_SPOSITIVESIGN ) ;
end ;



{ TSWindows_Monetary_Text_Presentation_Manager... }

type TSWindows_Monetary_Text_Presentation_Manager = object( TSMonetary_Text_Presentation_Manager )
                                                       public { API... }
                                                           function Decimal : Ansistring ;
                                                               virtual ;

                                                           function Group : Ansistring ;
                                                               virtual ;

                                                           function Money_Symbol : Ansistring ;
                                                               virtual ;

                                                           function Internationl_Money_Symbol : Ansistring ;
                                                               virtual ;
                                                    end ;
     PSWindows_Monetary_Text_Presentation_Manager = ^TSWindows_Monetary_Text_Presentation_Manager ;


function TSWindows_Monetary_Text_Presentation_Manager.Decimal : Ansistring ;

begin
    Decimal := Get_Locale_Info( LOCALE_SMONDECIMALSEP ) ;
end ;


function TSWindows_Monetary_Text_Presentation_Manager.Group : Ansistring ;

begin
    Group := Get_Locale_Info( LOCALE_SMONTHOUSANDSEP ) ;
end ;


function TSWindows_Monetary_Text_Presentation_Manager.Money_Symbol : Ansistring ;

begin
    Money_Symbol := Get_Locale_Info( LOCALE_SCURRENCY ) ;
end ;


function TSWindows_Monetary_Text_Presentation_Manager.Internationl_Money_Symbol : Ansistring ;

begin
    Internationl_Money_Symbol := Get_Locale_Info( LOCALE_SINTLSYMBOL ) ;
end ;



{ TSWindows_Date_Text_Presentation_Manager... }

type TSWindows_Date_Text_Presentation_Manager = object( TSDate_Text_Presentation_Manager )
                                                   public { API... }
                                                       function AM : Ansistring ;
                                                           virtual ;

                                                       function Date_Separator : Ansistring ;
                                                           virtual ;

                                                       function Days( M : integer ) : Ansistring ;
                                                           virtual ;

                                                       function Months( M : integer ) : Ansistring ;
                                                           virtual ;

                                                       function PM : Ansistring ;
                                                           virtual ;

                                                       function Short_Days( M : integer ) : Ansistring ;
                                                           virtual ;

                                                       function Short_Months( M : integer ) : Ansistring ;
                                                           virtual ;

                                                       function Time_Separator : Ansistring ;
                                                           virtual ;
                                                end ;
     PSWindows_Date_Text_Presentation_Manager = ^TSWindows_Date_Text_Presentation_Manager ;


function TSWindows_Date_Text_Presentation_Manager.AM : Ansistring ;

begin
    AM := Get_Locale_Info( LOCALE_S1159 ) ;
end ;


function TSWindows_Date_Text_Presentation_Manager.Date_Separator : Ansistring ;

begin
    Date_Separator := Get_Locale_Info( LOCALE_SDATE ) ;
end ;


function TSWindows_Date_Text_Presentation_Manager.Days( M : integer ) : Ansistring ;

begin
    case M of
        1 : Days := Get_Locale_Info( LOCALE_SDAYNAME1 ) ;
        2 : Days := Get_Locale_Info( LOCALE_SDAYNAME2 ) ;
        3 : Days := Get_Locale_Info( LOCALE_SDAYNAME3 ) ;
        4 : Days := Get_Locale_Info( LOCALE_SDAYNAME4 ) ;
        5 : Days := Get_Locale_Info( LOCALE_SDAYNAME5 ) ;
        6 : Days := Get_Locale_Info( LOCALE_SDAYNAME6 ) ;
        7 : Days := Get_Locale_Info( LOCALE_SDAYNAME7 ) ;
        else Days := '' ;
    end ;
end ;


function TSWindows_Date_Text_Presentation_Manager.Months( M : integer ) : Ansistring ;

begin
    case M of
        1 : Months := Get_Locale_Info( LOCALE_SMONTHNAME1 ) ;
        2 : Months := Get_Locale_Info( LOCALE_SMONTHNAME2 ) ;
        3 : Months := Get_Locale_Info( LOCALE_SMONTHNAME3 ) ;
        4 : Months := Get_Locale_Info( LOCALE_SMONTHNAME4 ) ;
        5 : Months := Get_Locale_Info( LOCALE_SMONTHNAME5 ) ;
        6 : Months := Get_Locale_Info( LOCALE_SMONTHNAME6 ) ;
        7 : Months := Get_Locale_Info( LOCALE_SMONTHNAME7 ) ;
        8 : Months := Get_Locale_Info( LOCALE_SMONTHNAME8 ) ;
        9 : Months := Get_Locale_Info( LOCALE_SMONTHNAME9 ) ;
        10 : Months := Get_Locale_Info( LOCALE_SMONTHNAME10 ) ;
        11 : Months := Get_Locale_Info( LOCALE_SMONTHNAME11 ) ;
        12 : Months := Get_Locale_Info( LOCALE_SMONTHNAME12 ) ;
        13 : Months := Get_Locale_Info( LOCALE_SMONTHNAME13 ) ;
        else Months := '' ;
    end ;
end ;


function TSWindows_Date_Text_Presentation_Manager.PM : Ansistring ;

begin
    PM := Get_Locale_Info( LOCALE_S2359 ) ;
end ;


function TSWindows_Date_Text_Presentation_Manager.Short_Days( M : integer ) : Ansistring ;

begin
    case M of
        1 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME1 ) ;
        2 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME2 ) ;
        3 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME3 ) ;
        4 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME4 ) ;
        5 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME5 ) ;
        6 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME6 ) ;
        7 : Short_Days := Get_Locale_Info( LOCALE_SABBREVDAYNAME7 ) ;
        else Short_Days := '' ;
    end ;
end ;


function TSWindows_Date_Text_Presentation_Manager.Short_Months( M : integer ) : Ansistring ;

begin
    case M of
        1 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME1 ) ;
        2 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME2 ) ;
        3 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME3 ) ;
        4 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME4 ) ;
        5 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME5 ) ;
        6 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME6 ) ;
        7 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME7 ) ;
        8 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME8 ) ;
        9 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME9 ) ;
        10 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME10 ) ;
        11 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME11 ) ;
        12 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME12 ) ;
        13 : Short_Months := Get_Locale_Info( LOCALE_SABBREVMONTHNAME13 ) ;
        else Short_Months := '' ;
    end ;
end ;


function TSWindows_Date_Text_Presentation_Manager.Time_Separator : Ansistring ;

begin
    Time_Separator := Get_Locale_Info( LOCALE_STIME ) ;
end ;



{ TSWindows_Text_Presentation_Manager methods... }

{ Constructors and destructors... }

constructor TSWindows_Text_Presentation_Manager.Init ;

begin
    inherited Init ;

    _Numeric_Text_Presentation_Manager := nil ;
    _Monetary_Text_Presentation_Manager := nil ;
    _Date_Text_Presentation_Manager := nil ;
end ;


destructor TSWindows_Text_Presentation_Manager.Done ;

begin
    if( _Numeric_Text_Presentation_Manager <> nil ) then
    begin
        dispose( _Numeric_Text_Presentation_Manager, Done ) ;
        _Numeric_Text_Presentation_Manager := nil ;
    end ;
    if( _Monetary_Text_Presentation_Manager <> nil ) then
    begin
        dispose( _Monetary_Text_Presentation_Manager, Done ) ;
        _Monetary_Text_Presentation_Manager := nil ;
    end ;
    if( _Date_Text_Presentation_Manager <> nil ) then
    begin
        dispose( _Date_Text_Presentation_Manager, Done ) ;
        _Date_Text_Presentation_Manager := nil ;
    end ;

    inherited Done ;
end ;


{ API... }

function TSWindows_Text_Presentation_Manager.Numeric_Text_Presentation_Manager : PSNumeric_Text_Presentation_Manager ;

begin
    if( _Numeric_Text_Presentation_Manager = nil ) then
    begin
        _Numeric_Text_Presentation_Manager := new( PSWindows_Numeric_Text_Presentation_Manager, Init ) ;
    end ;
    Numeric_Text_Presentation_Manager := _Numeric_Text_Presentation_Manager ;
end ;


function TSWindows_Text_Presentation_Manager.Monetary_Text_Presentation_Manager : PSMonetary_Text_Presentation_Manager ;

begin
    if( _Monetary_Text_Presentation_Manager = nil ) then
    begin
        _Monetary_Text_Presentation_Manager := new( PSWindows_Monetary_Text_Presentation_Manager, Init ) ;
    end ;
    Monetary_Text_Presentation_Manager := _Monetary_Text_Presentation_Manager ;
end ;


function TSWindows_Text_Presentation_Manager.Date_Text_Presentation_Manager : PSDate_Text_Presentation_Manager ;

begin
    if( _Date_Text_Presentation_Manager = nil ) then
    begin
        _Date_Text_Presentation_Manager := new( PSWindows_Date_Text_Presentation_Manager, Init ) ;
    end ;
    Date_Text_Presentation_Manager := _Date_Text_Presentation_Manager ;
end ;



begin
    Last_S := 0 ;
    Last_H := 0 ;
    Last_NS := 0 ;
end.
