{
        Program Name : ResMon
        Package Name : Subroutine Library - Layer 1
        Purpose      : Monitor resources
        Institution  : Conroy & Conroy Co.
        Date Written : 14-April-2016
        Written By   : Alan Conroy
        Version      : 1.0

	    Copyright (C) 2016 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          The unit is used to monitor resource usage.
}

unit ResMon ;

interface

{$I EDEFINES.INC}

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

{$IFDEF CC_Track_Resources}
const RT_Unused = 0 ;
      RT_Memory = 1 ;
      RT_Object = 2 ;
      RT_Handle = 3 ;

function Add_Resource( Typ, Value : integer ) : integer ;
procedure Remove_Resource( Typ, Value : integer ) ;
procedure Dump_Resources( Name : string ) ;
procedure Clear_Resources ;
{$ENDIF}


implementation

{$IFDEF CC_Track_Resources}
type TResRec = packed record
                   Typ : integer ;
                   Value : integer ;
                   Serial : integer ;
               end ;

var Next_Serial_Number : integer ;
    _Opened : boolean ;
    ResFile : file of TResRec ;
    Seek_Start : integer ;

function Add_Resource( Typ, Value : integer ) : integer ;

var ResRec : TResRec ;

begin
    Result := 0 ;
    if( ( Typ = RT_Memory ) and ( Value = 0 ) ) then
    begin
        exit ; // Don't add nil
    end ;
    if( not _Opened ) then
    begin
        assignfile( ResFile, 'resources.va' ) ;
        rewrite( ResFile ) ;
        _Opened := True ;
    end ;

    // Find unused entry...
    seek( ResFile, Seek_Start ) ;
    while( not eof( ResFile ) ) do
    begin
        read( ResFile, ResRec ) ;
        if( ResRec.Typ = RT_Unused ) then
        begin
            seek( ResFile, filepos( ResFile ) - 1 ) ;
            ResRec.Typ := Typ ;
            ResRec.Value := Value ;
            ResRec.Serial := Next_Serial_Number ;
            Result := Next_Serial_Number ;
            inc( Next_Serial_Number ) ;
            write( ResFile, ResRec ) ;
            exit ;
        end ;
    end ; // while( not eof( ResFile ) )

    // Write new record at end...
    Seek_Start := filepos( Resfile ) ;
    ResRec.Typ := Typ ;
    ResRec.Value := Value ;
    ResRec.Serial := Next_Serial_Number ;
    Result := Next_Serial_Number ;
    inc( Next_Serial_Number ) ;
    write( ResFile, ResRec ) ;
end ; // Add_Resource


procedure Remove_Resource( Typ, Value : integer ) ;

var ResRec : TResRec ;

begin
    if( ( Typ = RT_Memory ) and ( Value = 0 ) ) then
    begin
        exit ; // Didn't add nil
    end ;
    if( not _Opened ) then
    begin
        exit ;
    end ;

    // Find entry...
    seek( ResFile, 0 ) ; // Start at beginning of file
    while( not eof( ResFile ) ) do
    begin
        read( ResFile, ResRec ) ;
        if( ( ResRec.Typ = Typ ) and ( ResRec.Value = Value ) ) then // Found it
        begin
            if( Seek_Start > filepos( ResFile ) - 1 ) then
            begin
                Seek_Start := filepos( ResFile ) ;
            end ;
            seek( ResFile, filepos( ResFile ) - 1 ) ;
            fillchar( ResRec, sizeof( ResRec ), 0 ) ;
            write( ResFile, ResRec ) ;
            exit ;
        end ;
    end ; // while( not eof( ResFile ) )
end ;


procedure Dump_Resources( Name : string ) ;

var F : text ;
    ResRec : TResRec ;
    
begin
    assignfile( F, Name ) ;
    rewrite( F ) ;
    try
        if( not _Opened ) then
        begin
            exit ;
        end ;
        seek( ResFile, 0 ) ; // Start at beginning of file
        while( not eof( ResFile ) ) do
        begin
            read( ResFile, ResRec ) ;
            if( ResRec.Typ <> RT_Unused ) then
            begin
                case ResRec.Typ of
                    RT_Memory: write( F, 'Memory, ' ) ;
                    RT_Object: write( F, 'Object, ' ) ;
                    RT_Handle: write( F, 'Handle, ' ) ;
                end ;
                writeln( F, '@', ResRec.Value, ',  sn=', ResRec.Serial ) ;
            end ;
        end ; // while( not eof( ResFile ) )
    finally
        closefile( F ) ;
    end ;
end ;


procedure Clear_Resources ;

begin
    if( _Opened ) then
    begin
        rewrite( ResFile ) ;
        Next_Serial_Number := 0 ;
        Seek_Start := 0 ;
    end ;
end ;


begin
    _Opened := False ;
    Next_Serial_Number := 0 ;
    Seek_Start := 0 ;
{$ENDIF}
end.
