unit scu32int;

interface

uses _SCU ; // T_SCU

{$I EDEFINES.INC}

const SCU_ServerFacility = 74 ;
      SCU_ServerErr_Invalid_Operation = 1 ; // Null or unknown operation
      SCU_ServerErr_Invalid_Handle = 2 ;
      SCU_ServerErr_Invalid_Format = 3 ; // Header had bad format

const SCU_Operation_Null = 0 ;
const SCU_Operation_Init = 1 ;
const SCU_Operation_Version = 2 ;
const SCU_Operation_Write = 3 ;
const SCU_Operation_Read = 4 ;
const SCU_Operation_Read_Size = 5 ;
const SCU_Operation_Read_Type = 6 ;
const SCU_Operation_Exists = 7 ;
const SCU_Operation_Key_Exists = 8 ;
const SCU_Operation_Lookup = 9 ;
const SCU_Operation_Delete = 10 ;
const SCU_Operation_Create = 11 ;
const SCU_Operation_Create_And_Write = 12 ;
const SCU_Operation_Rename = 13 ;
const SCU_Operation_Set_Value = 14 ;
const SCU_Operation_Get_Value = 15 ;
const SCU_Operation_Value_Size = 16 ;
const SCU_Operation_Value_Type = 17 ;
const SCU_Operation_Value_Exists = 18 ;
const SCU_Operation_Delete_Value = 19 ;
const SCU_Operation_Create_Value = 20 ;
const SCU_Operation_Verify = 21 ;
const SCU_Operation_Debugger = 22 ;
const SCU_Operation_Read_Value = 23 ;
const SCU_Operation_Init_And_Extend = 24 ;
const SCU_Operation_Delete_With_Callback = 25 ;
const SCU_Operation_Shutdown = 26 ;

type TSCU_Message_Header = record
                               Size : longint ; // Size of header, in bytes
                               Format : longint ; // Format of header (always 0)
                               Context : longint ; // Context
                               Operation : longint ; { SCU operation being requested:
                               			                   0 = null operation
                                                           1 = Init( Name : PChar ) : TUEC
                                                           2 = Version : longint
                                                           3 = Write( Path : PChar ; Data : pointer ;
                                                               Size : longint ) : TUEC
                                                           4 = Read( Path : PChar ; Data : pointer ;
                                                               Size : longint ) : TUEC
                                                           5 = Read_Size( Path : PChar ) : longint
                                                           6 = Read_Type( Path : PChar ) : longint
                                                           7 = Exists( Path : PChar ) : boolean
                                                           8 = Key_Exists( Path : PChar ) : boolean
                                                           9 = Lookup( Path : PChar ; DT : longint ) : PChar
                                                           10 = Delete( Path : PChar ) : TUEC
                                                           11 = Create( Path : PChar ; DT : longint ) : TUEC
                                                           12 = Create_And_Write( Path : PChar ; DT : longint ;
                                                                Data : pointer ; Size : longint ) : TUEC
                                                           13 = Rename( Old, New : PChar ) : TUEC
                                                           14 = Set_Value( User, Path : PChar ; Data : pointer ;
                                                                Size : longint ) : TUEC
                                                           15 = Get_Value( User, Path : PChar ; Data : pointer ;
                                                                Size : longint ) : TUEC
                                                           16 = Value_Size( User, Path : PChar ) : longint
                                                           17 = Value_Type( User, Path : PChar ) : longint
                                                           18 = Value_Exists( User, Path : PChar ) : boolean
                                                           19 = Delete_Value( User, Path : PChar ) : TUEC
                                                           20 = Create_Value( User, Path : PChar ;
                                                                DT : longint ) : TUEC
                                                           21 = Verify : TUEC
                                                           22 = Debugger : PDebug_Interface
                                                           23 = Read_Value( User, Path : PChar ;
                                                                var DT, Size : longint ; var Buffer : PChar) : TUEC
                                                           24 = Init_And_Extend( Name : PChar ;
                                                                Size : longint ) : TUEC
                                                           25 = Delete_With_Callback( Path : PChar ;
                                                                Logger : PLogger ) : TUEC
                                                           26 = Shutdown }
                           end ;

{$IFDEF Server}
function Create_SCU( Context : string ) : T_SCU ;
{$ENDIF}

implementation

uses // Borland...
     Classes, // TList
     Forms, // Application
     SysUtils, // inttostr
     Windows, // CreateNamedPipe

     // Other...
     _DebugIn, // PDebug_Interface
     _Loggers, // PLogger,
{$IFDEF Server}
     MailSlot, // TMessenger
{$ENDIF}
     RLoggers, // TRemote_Logger
     SCULib, // TSCU
     Standard, // Standard
     _UEHDefs, // TUEC
     _UE ; // TUnified_exception

// Unit internal utility functions...

function Decode_Integer( const S : string ; Index : longint ) : longint ;

begin
    move( PChar( S )[ Index - 1 ], Result, sizeof( Result ) ) ;
end ;


function Encode_Integer( I : longint ) : string ;

begin
    setlength( Result, sizeof( longint ) ) ;
    move( I, PChar( Result )^, length( Result ) ) ;
end ;


function Decode_String( const S : string ; Index : integer ) : string ;

var Dummy : integer ;

begin
    if( ( Index < 1 ) or ( Index + sizeof( longint ) > length( S ) ) ) then
    begin
        Decode_String := '' ; {UNTESTED}
        exit ;
    end ;
    Dummy := 0 ;
    move( PChar( S )[ Index - 1 ], Dummy, sizeof( longint ) ) ;
    setlength( Result, Dummy ) ;
    move( PChar( S )[ Index - 1 + sizeof( longint ) ], PChar( Result )^, Dummy ) ;
end ;


function Encode_String( const S : string ) : string ;

var Dummy : integer ;

begin
    { This function returns a string whose first 4 bytes are an integer length
      and where that many bytes follow, starting at offset 4, containing the
      contents of S. }
    Dummy := length( S ) ;
    setlength( Result, Dummy + sizeof( integer ) ) ;
    move( Dummy, PChar( Result )^, sizeof( integer ) ) ;
    move( PChar( S )^, PChar( Result )[ sizeof( integer ) ], Dummy ) ;
end ;


function Encode_UEC( Facility, Code : longint ) : string ;

var UEC : TUEC ;

begin
    UEC.Facility := Facility ;
    UEC.Code := Code ;
    setlength( Result, sizeof( UEC ) ) ;
    move( UEC, PChar( Result )^, length( Result ) ) ;
end ;


{$IFDEF Server}
type TSCU_Client = class( TSCU )
                       private // Instance data...
                           Reference_Count : integer ;
                           Mail_Slot : TMail_Slot ;
                           Got_Message : boolean ; // True if a message was sent to us
                           Last_Error : PSUnified_Exception ;
                           Last_Message : string ;
                           Context : longint ; // Our context from the server
                           Temp_Read_Value : string ; // For returning a PChar

                       public // Constructors and destructors...
                           constructor _Create( S : string ) ;
                           destructor Destroy ; override ;

                       protected // Internal utility routines...
                           procedure CB_Message( Sender : TObject ;
                               const Origin, Name, Data : string ) ;
                           procedure Send_Message( Operation : integer ;
                               const Data : string ) ;
                           procedure Wait_For_Response ;

                       public // API...
                           procedure Attach ; override ; stdcall ;
                           procedure Detach ; override ; stdcall ;
                           function Initialize : PSUnified_Exception ; override ;
                           procedure Terminate ; override ;

                           function Init( Name : PChar ) : PSUnified_Exception ;
                               override ;
                           function Version : longint ; override ;

                           function Write( Path : PChar ; Data : pointer ;
                               Size : longint ) : PSUnified_Exception ; override ;
                           function Read( Path : PChar ; Data : pointer ;
                               Size : longint ) : PSUnified_Exception ; override ;
                           function Read_Size( Path : PChar ) : longint ;
                               override ;
                           function Read_Type( Path : PChar ) : longint ;
                               override ;
                           function Exists( Path : PChar ) : boolean ;
                               override ;
                           function Key_Exists( Path : PChar ) : boolean ;
                               override ;

                           function Lookup( Path : PChar ; DT : longint ) : PChar ;
                               override ;

                           function Delete( Path : PChar ) : PSUnified_Exception ;
                               override ;
                           function Create_( Path : PChar ; DT : longint ) : PSUnified_Exception ;
                               override ;
                           function Create_And_Write( Path : PChar ; DT : longint ;
                               Data : pointer ; Size : longint ) : PSUnified_Exception ;
                               override ;
                           function Rename( Old, New : PChar ) : PSUnified_Exception ;
                               override ;

                           function Set_Value( User, Path : PChar ; Data : pointer ;
                               Size : longint ) : PSUnified_Exception ; override ;
                           function Get_Value( User, Path : PChar ; Data : pointer ;
                               Size : longint ) : PSUnified_Exception ; override ;
                           function Value_Size( User, Path : PChar ) : longint ;
                               override ;
                           function Value_Type( User, Path : PChar ) : longint ;
                               override ;
                           function Value_Exists( User, Path : PChar ) : boolean ;
                               override ;
                           function Delete_Value( User, Path : PChar ) : PSUnified_Exception ;
                               override ;
                           function Create_Value( User, Path : PChar ;
                               DT : longint ) : PSUnified_Exception ; override ;
                           function Verify : PSUnified_Exception ; override ;
                           function Debugger : PDebug_Interface ;
                               override ;

                           function Read_Value( User, Path : PChar ;
                               var DT, Size : longint ; var Buffer : PChar) : PSUnified_Exception ;
                               override ;
                           { Reads the specified value.  If result indicates a success,
                             DT is the type of the data, Size is the actual data size,
                             and Buffer is a pointer to a buffer containing the data.
                             DT, Size, and Buffer are ignored on call, and set on
                             return.

                             This function serves as a single call to: 1) determine if
                             the value exists, 2) determine what type it is, and
                             3) read the value's data. }

                           function Init_And_Extend( Name : PChar ;
                               Size : longint ) : PSUnified_Exception ; override ;
                           { Initializes access to the specified file.  If the file
                             doesn't exist, it is created and pre-extended to the
                             specified size, in bytes. }

{$IFDEF FPC_OR_DELPHI}
                           function Delete_With_Callback( Path : PChar ;
                               Logger : TCOM_Logger ) : PSUnified_Exception ;
                               override ;
{$ELSE}
                           function Delete_With_Callback( Path : PChar ;
                               Logger : PSLogger ) : PSUnified_Exception ;
                               override ; stdcall ;
{$ENDIF}

                           procedure Shutdown ; override ;
                   end ; // TSCU_Client


// Constructors and destructors...

constructor TSCU_Client._Create( S : string ) ;

begin
    if( S = '' ) then
    begin
        S := 'SCU' ;
    end ;
    Mail_Slot := TMail_Slot.Create( S, False ) ;
    Mail_Slot.On_Message := CB_Message ;
    Mail_Slot.Activate ;
end ;


destructor TSCU_Client.Destroy ;

begin
    Mail_Slot.Free ;
    Mail_Slot := nil ;

    inherited Destroy ;
end ;


// Internal utility routines...

procedure TSCU_Client.CB_Message( Sender : TObject ;
    const Origin, Name, Data : string ) ;

begin
    move( PChar( Data )^, Last_Error, sizeof( TUEC ) ) ;
    setlength( Last_Message, length( Data ) - sizeof( TUEC ) ) ;
    move( PChar( Data )[ sizeof( TUEC ) ], PChar( Last_Message )^, length( Last_Message ) ) ;
    Got_Message := True ;
end ;


procedure TSCU_Client.Send_Message( Operation : integer ; const Data : string ) ;

var Header : TSCU_Message_Header ;
    S : string ;

begin
    Got_Message := False ;
    setlength( S, sizeof( Header ) ) ;
    Header.Size := sizeof( Header ) ;
    Header.Format := 0 ;
    Header.Context := Context ;
    Header.Operation := Operation ;
    move( Header, PChar( S )^, sizeof( Header ) ) ;
    Mail_Slot.Send( '.', S + Data ) ;
end ;


procedure TSCU_Client.Wait_For_Response ;

begin
    while( not Got_Message ) do
    begin
        Sleep( 0 ) ;
        Application.ProcessMessages ;
    end ;
end ;


function Encode_Strings( const S : string ; Data : pointer ; Len : integer ) : string ;

var Dummy : integer ;

begin
    { This function returns a string whose first 4 bytes are an integer length
      and where that many bytes follow, starting at offset 4, containing the
      contents of S.  If Data <> nil and Len <> 0, then following that data is
      another 4 byte integer and Len bytes from Data. }
    Result := Encode_String( S ) ;
    if( ( Data <> nil ) and ( Len > 0 ) ) then
    begin
        Dummy := length( Result ) ;
        setlength( Result, Dummy + Len + sizeof( integer ) ) ;
        move( Len, PChar( Result )[ Dummy ], sizeof( integer ) ) ;
        move( Data^, PChar( Result )[ Dummy + sizeof( integer ) ], Len ) ;
    end ;
end ;


// API...

procedure TSCU_Client.Attach ;

begin
    inc( Reference_Count ) ;
end ;


procedure TSCU_Client.Detach ;

begin
    dec( Reference_Count ) ;
    if( Reference_Count <= 0 ) then
    begin
        Terminate ;
    end ;
end ;


function TSCU_Client.Initialize : PSUnified_Exception ;

begin
    fillchar( Result, sizeof( Result ), 0 ) ;
    // This routine intentionally left blank
end ;


procedure TSCU_Client.Terminate ;

begin
    Free ;
end ;


function TSCU_Client.Init( Name : PChar ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( Name ) ;
    Send_Message( SCU_Operation_Init, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
    Context := Decode_Integer( Last_Message, 1 ) ;
end ;


function TSCU_Client.Version : longint ;

begin
    Send_Message( SCU_Operation_Version, '' ) ;
    Wait_For_Response ;
    Result := Decode_Integer( Last_Message, 1 ) ;
end ;


function TSCU_Client.Write( Path : PChar ; Data : pointer ;
   Size : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_Strings( Path, Data, Size ) ;
    Send_Message( SCU_Operation_Write, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Read( Path : PChar ; Data : pointer ;
   Size : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( Path ) + Encode_Integer( Size ) ;
    Send_Message( SCU_Operation_Read, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
    move( PChar( Last_Message )^, Data^, Size ) ;
end ;


function TSCU_Client.Read_Size( Path : PChar ) : longint ;

var S : string ;

begin
    S := Encode_String( Path ) ;
    Send_Message( SCU_Operation_Read_Size, S ) ;
    Wait_For_Response ;
    Result := Decode_Integer( Last_Message, 1 ) ;
end ;


function TSCU_Client.Read_Type( Path : PChar ) : longint ;

var S : string ;

begin
    S := Encode_String( Path ) ;
    Send_Message( SCU_Operation_Read_Type, S ) ;
    Wait_For_Response ;
    Result := Decode_Integer( Last_Message, 1 ) ;
end ;


function TSCU_Client.Exists( Path : PChar ) : boolean ;

var S : string ;

begin
    S := Encode_String( Path ) ;
    Send_Message( SCU_Operation_Exists, S ) ;
    Wait_For_Response ;
    Result := boolean( Decode_Integer( Last_Message, 1 ) ) ;
end ;


function TSCU_Client.Key_Exists( Path : PChar ) : boolean ;

var S : string ;

begin
    S := Encode_String( Path ) ;
    Send_Message( SCU_Operation_Key_Exists, S ) ;
    Wait_For_Response ;
    Result := boolean( Decode_Integer( Last_Message, 1 ) ) ;
end ;


function TSCU_Client.Lookup( Path : PChar ; DT : longint ) : PChar ;

var S : string ;

begin
    S := Encode_String( Path ) + Encode_Integer( DT ) ;
    Send_Message( SCU_Operation_Lookup, S ) ;
    Wait_For_Response ;
    Result := PChar( Last_Message ) ;
end ;


function TSCU_Client.Delete( Path : PChar ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( Path ) ;
    Send_Message( SCU_Operation_Delete, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Create_( Path : PChar ; DT : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( Path ) + Encode_Integer( DT ) ;
    Send_Message( SCU_Operation_Create, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Create_And_Write( Path : PChar ; DT : longint ;
   Data : pointer ; Size : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_Strings( Path, Data, Size ) + Encode_Integer( DT ) ;
    Send_Message( SCU_Operation_Create_And_Write, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Rename( Old, New : PChar ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( Old ) + Encode_String( New ) ;
    Send_Message( SCU_Operation_Rename, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Set_Value( User, Path : PChar ; Data : pointer ;
   Size : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_Strings( Path, Data, Size ) ;
    Send_Message( SCU_Operation_Set_Value, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Get_Value( User, Path : PChar ; Data : pointer ;
   Size : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) + Encode_Integer( Size ) ;
    Send_Message( SCU_Operation_Get_Value, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
    move( PChar( Last_Message )^, Data^, Size ) ;
end ;


function TSCU_Client.Value_Size( User, Path : PChar ) : longint ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) ;
    Send_Message( SCU_Operation_Value_Size, S ) ;
    Wait_For_Response ;
    Result := Decode_Integer( Last_Message, 1 ) ;
end ;


function TSCU_Client.Value_Type( User, Path : PChar ) : longint ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) ;
    Send_Message( SCU_Operation_Value_Type, S ) ;
    Wait_For_Response ;
    Result := Decode_Integer( Last_Message, 1 ) ;
end ;


function TSCU_Client.Value_Exists( User, Path : PChar ) : boolean ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) ;
    Send_Message( SCU_Operation_Value_Exists, S ) ;
    Wait_For_Response ;
    Result := boolean( Decode_Integer( Last_Message, 1 ) ) ;
end ;


function TSCU_Client.Delete_Value( User, Path : PChar ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) ;
    Send_Message( SCU_Operation_Delete_Value, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Create_Value( User, Path : PChar ;
   DT : longint ) : PSUnified_Exception ;

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) + Encode_Integer( DT ) ;
    Send_Message( SCU_Operation_Create_Value, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Verify : PSUnified_Exception ;

begin
    Send_Message( SCU_Operation_Verify, '' ) ;
    Wait_For_Response ;
    Result := Last_Error ;
end ;


function TSCU_Client.Debugger : PDebug_Interface ;

begin
Result := nil ; // Not supported yet
(*
    Send_Message( SCU_Operation_Debugger, '' ) ;
    Wait_For_Response ;
*)
end ;


function TSCU_Client.Read_Value( User, Path : PChar ;
   var DT, Size : longint ; var Buffer : PChar) : PSUnified_Exception ;
{ Reads the specified value.  If result indicates a success,
 DT is the type of the data, Size is the actual data size,
 and Buffer is a pointer to a buffer containing the data.
 DT, Size, and Buffer are ignored on call, and set on
 return.

 This function serves as a single call to: 1) determine if
 the value exists, 2) determine what type it is, and
 3) read the value's data. }

var S : string ;

begin
    S := Encode_String( User ) + Encode_String( Path ) ;
    Send_Message( SCU_Operation_Read_Value, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
    DT := Decode_Integer( Last_Message, 1 ) ;
    Size := Decode_Integer( Last_Message, 1 + sizeof( longint ) ) ;
    setlength( Temp_Read_Value, Size ) ;
    move( PChar( Last_Message )[ sizeof( longint ) * 2 ], PChar( Temp_Read_Value )^, Size ) ;
    Buffer := PChar( Temp_Read_Value ) ;
end ;


function TSCU_Client.Init_And_Extend( Name : PChar ;
   Size : longint ) : PSUnified_Exception ;
{ Initializes access to the specified file.  If the file
 doesn't exist, it is created and pre-extended to the
 specified size, in bytes. }

var S : string ;

begin
    S := Encode_String( Name ) ;

    Send_Message( SCU_Operation_Init_And_Extend, S ) ;
    Wait_For_Response ;
    Result := Last_Error ;
    Context := Decode_Integer( Last_Message, 1 + sizeof( longint ) ) ;
end ;


{$IFDEF FPC_OR_DELPHI}
function TSCU_Client.Delete_With_Callback( Path : PChar ;
   Logger : TCOM_Logger ) : PSUnified_Exception ;
{$ELSE}
function TSCU_Client.Delete_With_Callback( Path : PChar ;
   Logger : PSLogger ) : PSUnified_Exception ;
{$ENDIF}

var _Logger : TRemote_Logger ;
    Name : string ;

begin
    Name := inttostr( Standard.PID ) ;
    _Logger := TRemote_Logger.Create ;
    _Logger.Initialize_Logger( Name, True, Logger ) ;
    Send_Message( SCU_Operation_Delete_With_Callback, Encode_String( Path ) + Encode_String( Name ) ) ;
    Wait_For_Response ;
    Result := Last_Error ;
    _Logger.Free ;
end ;


procedure TSCU_Client.Shutdown ;

begin
    Send_Message( SCU_Operation_Shutdown, '' ) ;
end ;



// Unit API...

function Create_SCU( Context : string ) : T_SCU ;

var Mutex : integer ;

begin
    // Setup...
    if( length( Context ) > 0 ) then
    begin
        Context := '_' + Context ;
    end ;

    // First, try to start the SCU server...
    Mutex := CreateMutex( nil, False, PChar( 'SCU' + Context ) ) ;
    if( GetLastError <> ERROR_ALREADY_EXISTS ) then // Server does not exist
    begin
        closehandle( Mutex ) ;
        WinExec( PChar( 'SCUSRV' + Context + '.EXE' ), SW_SHOWNA ) ;
        Sleep( 1000 ) ;
    end else
    begin
        closehandle( Mutex ) ;
    end ;

    // Now see if server has started...
    Mutex := CreateMutex( nil, False, PChar( 'SCU' + Context ) ) ;
    if( GetLastError = ERROR_ALREADY_EXISTS ) then // Server exists
    begin
        closehandle( Mutex ) ;
        Result := TSCU_Client._Create( 'SCU' + Context ) ;
    end else
    begin
        closehandle( Mutex ) ;
        Result := TSCU.Construct ;
    end ;
end ; // Create_SCU
{$ENDIF}


end.

