{
        Program Name : SCU_32
        Package Name : SCU
        Purpose      : System Configuration Utility 32-bit interface
        Institution  : Conroy & Conroy Co.
        Date Written : 16-November-2001
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2001 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

	  SCU32 is the 32-bit DLL interface to the SCU functionality.
}

unit SCU_32 ;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}
{$UNDEF Server}

interface

uses { C&C Subroutine Library... }
     ErrMastr, { Get_Error_Text }
     SCU,
{$IFDEF Server}
     SCU32Int,
{$ENDIF}
     SCULib,
     Sysutils ; // strlen

function Create_SCU_Interface : T_SCU ; stdcall ;
procedure Free_SCU_Interface( X : T_SCU ) ; stdcall ;
function SCU_Get_Message : PAnsiChar ; stdcall ;
procedure SCU_Set_Path( X : PAnsiChar ) ; stdcall ;
function SCU_Delete_Value( X : PAnsiChar ) : integer ; stdcall ;
function SCU_Set_Value( X : PAnsiChar ; Typ : integer ; Value : PAnsiChar ;
    Size : integer ) : integer ; stdcall ;
function SCU_Get_Value( X : PAnsiChar ; var Typ : integer ;
    var Size : integer ) : PAnsiChar ; stdcall ;
function Version : longint ; stdcall ;


implementation

uses { C&C Subroutine Library... }
     _UEHDefs, { TUEC }
     UStrings ; { String_To_Float }


// COM interface...

function Create_SCU_Interface : T_SCU ; stdcall ;

begin
{$IFDEF Server}
    Result := Create_SCU ;
{$ELSE}
    Result := T_SCU( TSCU.Construct ) ;
{$ENDIF}
end ;


procedure Free_SCU_Interface( X : T_SCU ) ; stdcall ;

begin
    X.Free ;
end ;


// Non-COM interface...

var SCU_Path : Ansistring ;
    UEC : TUEC ;
    Get_Message_Result : Ansistring ;

procedure Create_Path( SCU : T_SCU ; X : PAnsiChar ) ;

var C, Work : Ansistring ;
    Dummy : integer ;
    Path : Ansistring ;

begin
    Path := X ;
    if( copy( Path, 1, 1 ) = '\' ) then
    begin
        Path := copy( Path, 2, length( Path ) ) ;
    end ;
    Dummy := length( Path ) ;
    while( ( Dummy > 0 ) and ( Path[ Dummy ] <> '\' ) ) do
    begin
        dec( Dummy ) ;
    end ;
    Path := copy( Path, 1, Dummy - 1 ) ;
    Work := '' ;
    while( length( Path ) > 0 ) do
    begin
        Dummy := pos( '\', Path + '\' ) ;
        C := copy( Path, 1, Dummy - 1 ) ;
        Path := copy( Path, Dummy + 1, length( Path ) ) ;
        Work := Work + '\' + C ;
        SCU.Create_( PAnsiChar( Work ), SCUDT_Key ) ;
    end ;
end ;


function SCU_Get_Message : PAnsiChar ; stdcall ;

begin
    Get_Message_Result := Get_Error_Text( UEC, -1 ) ;
    Result := PAnsiChar( Get_Message_Result ) ;
end ;


procedure SCU_Set_Path( X : PAnsiChar ) ; stdcall ;

begin
    SCU_Path := X ;
    fillchar( UEC, sizeof( UEC ), 0 ) ;
end ;


function SCU_Delete_Value( X : PAnsiChar ) : integer ; stdcall ;

var SCU : T_SCU ;

begin
    SCU := TSCU.Construct ;
    if( length( SCU_Path ) = 0 ) then
    begin
        SCU.Initialize ;
    end else
    begin
        SCU.Init( PAnsiChar( SCU_Path ) ) ;
    end ;
    UEC := SCU.Delete( PAnsiChar( X ) ) ;
    SCU.Terminate ;
    SCU.Free ;
    if( UEC.Code = 0 ) then
    begin
        Result := 0 ; // Success
    end else
    begin
        Result := -1 ;
    end ;
end ;


function SCU_Set_Value( X : PAnsiChar ; Typ : integer ; Value : PAnsiChar ;
    Size : integer ) : integer ; stdcall ;

var B : boolean ;
    F : double ;
    I : integer ;
    SCU : T_SCU ;
    Temp : Ansistring ;

begin
    SCU := TSCU.Construct ;
    if( length( SCU_Path ) = 0 ) then
    begin
        SCU.Initialize ;
    end else
    begin
        SCU.Init( PAnsiChar( SCU_Path ) ) ;
    end ;
    Create_Path( SCU, PAnsiChar( X ) ) ;
    UEC := SCU.Create_( X, Typ ) ;
    if( ( UEC.Code = 0 ) and ( Typ <> SCUDT_Key ) )  then
    begin
        if( ( Typ = SCUDT_Binary ) or ( Typ = SCUDT_String ) ) then
        begin
            if( ( Size < 0 ) and ( Typ <> SCUDT_Binary ) ) then
            begin
                Size := strlen( Value ) ;
            end ;
            UEC := SCU.Write( X, Value, Size ) ;
        end else
        begin
            Temp := Value ;
            try
                case Typ of
                    SCUDT_Integer:
                        begin
                            I := strint( Temp ) ;
                            Size := sizeof( I ) ;
                            setlength( Temp, Size ) ;
                            move( I, Temp[ 1 ], Size ) ;
                        end ;
                    SCUDT_Float:
                        begin
                            F := string_to_float( Temp ) ;
                            Size := sizeof( F ) ;
                            setlength( Temp, Size ) ;
                            move( F, Temp[ 1 ], Size ) ;
                        end ;
                    SCUDT_Boolean:
                        begin
                            Temp := uppercase( Temp ) ;
                            B := ( copy( 'TRUE', 1, length( Temp ) ) = Temp ) ;
                            Size := sizeof( B ) ;
                            setlength( Temp, Size ) ;
                            move( B, Temp[ 1 ], Size ) ;
                        end ;
                end ;
                UEC := SCU.Write( X, PAnsiChar( Temp ), Size ) ;
            except
                UEC.Facility := SCUErr_Facility ;
		            UEC.Code := SCUErr_Bad_Data ;
            end ;
        end ;
    end ;
    SCU.Terminate ;
    SCU.Free ;
    if( UEC.Code = 0 ) then
    begin
        Result := 0 ; // Success
    end else
    begin
        Result := -1 ;
    end ;
end ; // SCU_Set_Value


var Get_Value_Result : Ansistring ;

function SCU_Get_Value( X : PAnsiChar ; var Typ : integer ;
    var Size : integer ) : PAnsiChar ; stdcall ;

var B : boolean ;
    F : double ;
    I : integer ;
    SCU : T_SCU ;

begin
    SCU := TSCU.Construct ;
    if( length( SCU_Path ) = 0 ) then
    begin
        SCU.Initialize ;
    end else
    begin
        SCU.Init( PAnsiChar( SCU_Path ) ) ;
    end ;
    Typ := SCU.Read_Type( X ) ;
    if( Typ = SCUDT_Key ) then
    begin
        fillchar( UEC, sizeof( UEC ), 0 ) ;
        Size := -1 ;
        Get_Value_Result := '' ;
        Result := PAnsiChar( Get_Value_Result ) ;
        exit ;
    end ;
    if( not SCU.Exists( X ) ) then
    begin
        UEC.Facility := SCUErr_Facility ;
        UEC.Code := SCUErr_Value_Not_Found ;
	    Typ := 0 ;
        Size := -1 ;
        Get_Value_Result := '' ;
        Result := PAnsiChar( Get_Value_Result ) ;
        exit ;
    end ;
    Size := SCU.Read_Size( X ) ;
    if( Size >= 0 ) then
    begin
        setlength( Get_Value_Result, Size ) ;
    end ;
    UEC := SCU.Read( X, PAnsiChar( Get_Value_Result ), Size ) ;
    if( UEC.Code = 0 ) then
    begin
	case Typ of
	    SCUDT_Boolean:
                begin
                    B := False ;
                    if( Size > sizeof( B ) ) then
                    begin
                        Size := sizeof( B ) ;
                    end ;
                    move( Get_Value_Result[ 1 ], B, Size ) ;
                    if( B ) then
                    begin
                        Get_Value_Result := 'True' ;
                    end else
                    begin
                        Get_Value_Result := 'False' ;
                    end ;
                end ;
            SCUDT_Float:
                begin
                    F := 0.0 ;
                    if( Size > sizeof( F ) ) then
                    begin
                        Size := sizeof( F ) ;
                    end ;
                    move( Get_Value_Result[ 1 ], F, Size ) ;
                    Get_Value_Result := floattostr( F ) ;
                end ;
            SCUDT_Integer:
                begin
                    I := 0 ;
                    if( Size > sizeof( I ) ) then
                    begin
                        Size := sizeof( I ) ;
                    end ;
                    move( Get_Value_Result[ 1 ], I, Size ) ;
                    Get_Value_Result := inttostr( I ) ;
                end ;
        end ;
    end ;
    SCU.Terminate ;
    SCU.Free ;
    if( UEC.Code <> 0 ) then
    begin
        Get_Value_Result := '' ;
    end ;
    Result := PAnsiChar( Get_Value_Result ) ;
end ; // SCU_Get_Value


function Version : longint ; stdcall ;

begin
    Result := 11 ;
end ;


end.
