{$V-}
{
          Program Name : SMU_Pane
          Package Name : Subroutine Library - Layer 5
          Purpose      : Pane used for SMU
          Institution  : Conroy & Conroy
          Date Written : 27-Jul-2004
          Written By   : Alan Conroy
          Version      : 1.0

          Copyright (C) 2004 by Alan Conroy.  Released to the public domain.

          TO THE GLORY of GOD THROUGH JESUS CHRIST

	*********************************************************
	*                                                       *
	*        M O D I F I C A T I O N   H I S T O R Y        *
	*                                                       *
	*********************************************************

	     DATE      BY             REASON

    *********************************************************
	*                                                       *
	*           P R O G R A M   P U R P O S E               *
	*                                                       *
	*********************************************************

	  These routines provide complete screen management.
}

unit SMU_Pane ;

interface

uses // Borland...
     Classes, // TNotifyEvent
     Controls, // TCustomControl

     // C&C Subroutine Library - Layer 1...
     TPanels, // TTransparent_Panel

     // C&C Subroutine Library - Layer 4...
     SMU ; // PWindow

type TSMU_Pane = class( TTransparent_Panel )
                     public // Constructors and destructors...
                         constructor Create( Owner : TComponent ;
                             Handle : PWindow ) ; reintroduce ;
                         destructor Destroy ; override ;

                     private // Instance data...
                         _SMU_Handle : PWindow ;

                     public // API...
                         On_Resize : TNotifyEvent ;

                         function SMU_Handle : PWindow ;
                         procedure Paint ; override ;
                         procedure DoEnter ; override ;
                         procedure Resize ; override ;

                         procedure Clear ;
                         function Columns : integer ;
                         function KB_Buffer : string ;
                         procedure Map_Key( const Source, Translation : string ) ;
                         procedure Unmap_Key( const Source, Translation : string ) ;
                         function Read : string ;
                         function Rows : integer ;
                         procedure Set_Window( Row, Column, Flag,
		                     Attributes : Integer ) ;
                         procedure Text_Out( const S : string ) ;
                 end ;


implementation

constructor TSMU_Pane.Create( Owner : TComponent ; Handle : PWindow ) ;

begin
    inherited Create( Owner ) ;

    _SMU_Handle := Handle ;
end ;


destructor TSMU_Pane.Destroy ;

begin
    if( _SMU_Handle <> nil ) then
    begin
        Delete_Window( _SMU_Handle ) ;
        _SMU_Handle := nil ;
    end ;

    inherited Destroy ;
end ;


// API...

function TSMU_Pane.SMU_Handle : PWindow ;

begin
    if( _SMU_Handle = nil ) then
    begin
        _SMU_Handle := Create_Window_Wrapper( Handle ) ;
    end ;
    Result := _SMU_Handle ;
end ;


procedure TSMU_Pane.Paint ;

var Saved : Window_Record ;

begin
    if( assigned( OnPaint ) ) then
    begin
        OnPaint( self ) ;
    end else
    begin
        SMU.Get_Default_Window( Saved ) ;
        SMU.Select_Window( SMU_Handle ) ;
        SMU.Invalidate_Window( SMU_Handle ) ;
        SMU.Update_Window( SMU_Handle ) ;
        SMU.Set_Default_Window( Saved ) ;
    end ;
end ;


procedure TSMU_Pane.DoEnter ;

begin
    SMU.Clear_Cursor( 0, 0 ) ;
    SMU.Select_Window( SMU_Handle ) ;
end ;


procedure TSMU_Pane.Resize ;

var R, C : longint ;

begin
    if( _SMU_Handle <> nil ) then
    begin
        SMU.Calculate_Window_Extent( SMU_Handle, R, C ) ;
        SMU.Resize( _SMU_Handle, R, C ) ;
    end ;
    if( assigned( On_Resize ) ) then
    begin
        On_Resize( self ) ;
    end ;
end ;


procedure TSMU_Pane.Clear ;

begin
    SMU.Clear_Window( SMU_Handle ) ;
    Invalidate ;
end ;


function TSMU_Pane.Columns : integer ;

begin
    Result := SMU_Handle.Columns ;
end ;


function TSMU_Pane.KB_Buffer : string ;

begin
    Result := SMU_Handle.KB_Buffer ;
end ;


procedure TSMU_Pane.Map_Key( const Source, Translation : string ) ;

begin
    SMU.Map_Key( SMU_Handle, Source, Translation ) ;
end ;


procedure TSMU_Pane.Unmap_Key( const Source, Translation : string ) ;

begin
    SMU.Unmap_Key( SMU_Handle, Source, Translation ) ;
end ;


function TSMU_Pane.Read : string ;

var Saved : Window_Record ;

begin
    SMU.Get_Default_Window( Saved ) ;
    Select_Window( SMU_Handle ) ;
    SMU.Window_Read_Input( Handle, 0 ) ; //.Window_Read
    Result := KB_Buffer ;
    SMU.Set_Default_Window( Saved ) ;
end ;


function TSMU_Pane.Rows : integer ;

begin
    Result := SMU_Handle^.Rows ;
end ;


procedure TSMU_Pane.Set_Window( Row, Column, Flag, Attributes : Integer ) ;

begin
    SMU.Set_Window( SMU_Handle, Row, Column, Flag, Attributes ) ;
end ;


procedure TSMU_Pane.Text_Out( const S : string ) ;

var Saved : Window_Record ;

begin
    SMU.Get_Default_Window( Saved ) ;
    Select_Window( SMU_Handle ) ;
    SMU.Text_Out( SMU_Handle, S, length( S ) ) ;
    Invalidate ;
    SMU.Set_Default_Window( Saved ) ;
end ;


end.

