{
        Program Name : StringMa
        Package Name : Subroutine Library
        Purpose      : String math
        Institution  : Conroy & Conroy Co.
        Date Written : 20-Sep-2015
        Written By   : Alan Conroy
        Version      : 1.0

	    Copyright (C) 2015 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

        DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

}

unit StringMa ;

interface

type TNSS = packed record
                Negative : boolean ;
                Whole : string ;
                Fractional : string ;
                Flags : longint ; // and 1 = E format
            end ;

function Normalize_Number( const Value : string ; Precision, Flags : integer ) : string ;
function Numeric_String_Scan( Value : string ; var Res : TNSS ) : boolean ;
function SM_Add( V1, V2 : string ) : string ;


implementation

uses { Borland... }
     SysUtils ; { trystrotint }

function Normalize_Number( const Value : string ; Precision, Flags : integer ) : string ;
{ Precision = max number of digits after the decimal: 0 = none, but include decimal
  point, -1 = no fractional part and no decimal point.  Flags = processing flags:
    and 1 = Use E-notation
    and 2+ = reserved
}

var E : integer ;
    NSS : TNSS ;
    S : string ;

begin
    // Setup and parse the number
    Result := '' ;
    if( not Numeric_String_Scan( Value, NSS ) ) then
    begin
        exit ;
    end ;

    // Handle E-notation
    if( ( Flags and 1 ) <> 0 ) then
    begin
        S := NSS.Whole ;
        if( ( length( S ) = 1 ) and ( S <> '0' ) ) then
        begin
            Result := Copy( NSS.Whole + '.' + NSS.Fractional, 1, Precision + 2 ) ;
            exit ;
        end ;
        E := length( S ) - 1 ;
        S := S + NSS.Fractional ;
        if( E <= 0 ) then
        begin
            while( copy( S, 1, 1 ) = '0' ) do
            begin
                E := E - 1 ;
                S := copy( S, 2, length( S ) ) ;
            end ;
        end ;
        S := copy( S, 1, 1 ) + '.' + copy( S, 2, length( S ) ) ;
        S := copy( S, 1, Precision + 2 ) + 'E' ;
        if( E > 0 ) then
        begin
            S := S + '+' ;
        end ;
        Result := S + inttostr( E ) ;
        exit ;
    end ; { if( ( Flags and 1 ) <> 0 ) }

    { Normal processing... }
    Result := NSS.Whole ;
    if( Precision >= 0 ) then
    begin
        Result := Result + '.' + copy( NSS.Fractional, 1, Precision )
    end ;
    if( NSS.Negative ) then
    begin
        Result := '-' + Result ;
    end ;
end ; { Normalize_Number }


function Numeric_String_Scan( Value : string ; var Res : TNSS ) : boolean ;

var C : char ;
    Dummy : integer ;
    Have_Decimal : boolean ;
    S : string ; { Exponent }

begin
    // Initialize data...
    Result := True ;
    Res.Negative := False ;
    Res.Whole := '0' ;
    Res.Fractional := '' ;
    Res.Flags := 0 ;
    Value := trim( Value ) ;
    if( Value = '' ) then
    begin
        exit ;
    end ;

    // Determine sign...
    for Dummy := 1 to length( Value ) do
    begin
        C := Value[ Dummy ] ;
        if( C = '-' ) then
        begin
            Res.Negative := not Res.Negative ;
            if( Dummy = length( Value ) ) then
            begin
                Value := '' ;
            end ;
        end else
        if( C <> '+' ) then
        begin
            Value := copy( Value, Dummy + 1, length( Value ) ) ;
            break ;
        end ;
    end ;
    if( Value = '' ) then // Only sign(s)
    begin
        Result := False ;
        exit ;
    end ;

    // Process actual numeric value...
    Res.Whole := '0' ;
    Have_Decimal := False ;
    S := '' ;
    for Dummy := 1 to length( Value ) do
    begin
        C := Value[ Dummy ] ;
        case C of
            'E' : begin
                      S := copy( Value, Dummy + 1, length( Value ) ) ;
                      break ;
                  end ;
            '.' : if( Have_Decimal ) then // Multiple decimals = invalid syntax
                  begin
                      Result := False ;
                      exit ;
                  end else
                  begin
                      Have_Decimal := True ;
                  end ;
            '0'..'9' : if( Have_Decimal ) then
                       begin
                           Res.Fractional := Res.Fractional + C ;
                       end else
                       begin
                           Res.Whole := Res.Whole + C ;
                       end ;
            else // Invalid character
                begin
                    Result := False ;
                    exit ;
                end ;
        end ; { case C of }
        if( S <> '' ) then // Have an exponent
        begin
            if( Trystrtoint( S ) ) then
            begin
                Result := False ;
                exit ;
            end ;
            Dummy := strtoint( S ) ;
            if( Dummy > 0 ) then
            begin
                // Positive E...
                while( Dummy > 0 ) do
                begin
                    if( Res.Fractional = '' ) then
                    begin
                        Res.Whole := Res.Whole + '0' ;
                    end else
                    begin
                        Res.Whole := Res.Whole + Res.Fractional[ 1 ] ;
                        Res.Fractional := copy( Res.Fractional, 2, length( Res.Fractional ) ) ;
                    end ;
                    dec( Dummy ) ;
                end ;
            end else
            begin
                // Negative E...
                while( Dummy < 0 ) do
                begin
                    if( Res.Whole = '' ) then
                    begin
                        Res.Fractional := '0' + Res.Fractional ;
                    end else
                    begin
                        Res.Fractional := Res.Whole[ length( Res.Whole ) ] + Res.Fractional ;
                        setlength( Res.Whole, length( Res.Whole ) - 1 ) ;
                    end ;
                    inc( Dummy ) ;
                end ;
            end ;
        end ;
    end ; { for Dummy := 1 to length( Value ) }

    { Final processing... }
    while( ( copy( Res.Whole, 1, 1 ) = '0' ) and ( Res.Whole <> '' ) ) do
    begin
        Res.Whole := copy( Res.Whole, 2, length( Res.Whole ) ) ; { Remove leading zeroes }
    end ;
    if( Res.Whole = '' ) then
    begin
        Res.Whole := '0' ;
    end ;
    while( ( Res.Fractional <> '' ) and ( copy( Res.Fractional, length( Res.Fractional ), 1 ) = '0' ) ) do
    begin
        setlength( Res.Fractional, length( Res.Fractional ) - 1 ) ; { Remove trailing zeroes}
    end ;
end ; { Numeric_String_Scan }


procedure _Normalize( var V1, V2 : string ) ;

var D1, D2, P1, P2 : integer ;

begin
    // Normalize numbers relative to each other...
    P1 := pos( '.', V1 ) ;
    P2 := pos( '.', V2 ) ;
    if( P1 = 0 ) then
    begin
        V1 := V1 + '.' ;
    end ;
    if( P2 = 0 ) then
    begin
        V2 := V2 + '.' ;
    end ;
    P1 := pos( '.', V1 ) ;
    P2 := pos( '.', V2 ) ;
    D1 := length( V1 ) - P1 ;
    D2 := length( V2 ) - P2 ;
    while( D1 < D2 ) do
    begin
        V1 := V1 + '0' ;
        inc( D1 ) ;
    end ;
    while( D1 > D2 ) do
    begin
        V2 := V2 + '0' ;
        inc( D2 ) ;
    end ;
end ;


function _Subtract( V1, V2 : string ) : string ;

begin
    { Setup... }
    Result := '' ;
    if( ( V1 = '' ) or ( V2 = '' ) ) then
    begin
        exit ;
    end ;
    ~~~
end ;


function _Add( V1, V2 : string ; Carry : integer ) : string ;

var D1, D2, P1, P2 : integer ;
    Result_Negative : boolean ;

begin
    { Setup... }
    Result := '' ;
    if( ( V1 = '' ) or ( V2 = '' ) ) then
    begin
        exit ;
    end ;
    Result_Negative := False ;
    if( ( V1[ 1 ] = '-' ) and ( V2[ 1 ] = '-' ) ) then
    begin
        Result_Negative := True ;
        V1 := copy( V1, 2, length( V1 ) ) ;
        V2 := copy( V2, 2, length( V2 ) ) ;
    end else
    if( V1[ 1 ] = '-' ) then
    begin
        V1 := copy( V1, 2, length( V1 ) ) ;
        Result := _Subtract( V2, V1 ) ;
        exit ;
    end else
    if( V2[ 1 ] = '-' ) then
    begin
        V2 := copy( V2, 2, length( V2 ) ) ;
        Result := _Subtract( V1, V2 ) ;
        exit ;
    end ;

    _Normalize( V1, V2 ) ;

    { Do the add... }
    Result := '' ;
    P1 := length( V1 ) ;
    P2 := length( V2 ) ;
    while( P1 + P2 > 0 ) do
    begin
        D1 := 0 ;
        D2 := 0 ;
        if( copy( V1, P1, 1 ) = '.' ) then
        begin
            Result := '.' + Result ;
        end else
        begin
            if( P1 > 0 ) then
            begin
                D1 := ord( V1[ P1 ] ) - 64 ;
            end else
            begin
                D1 := 0 ;
            end ;
            if( P2 > 0 ) then
            begin
                D2 := ord( V2[ P2 ] ) - 64 ;
            end else
            begin
                D2 := 0 ;
            end ;
            D1 := D1 + D2 + Carry ;
            Carry := 0 ;
            if( D1 > 9 ) then
            begin
                Carry := 1 ;
                D1 := D1 - 10 ;
            end ;
            Result := chr( D1 + 64 ) + Result ;
        end ; // if
    end ; // while( P1 + P2 > 0 )
end ; { _Add }


function SM_Add( V1, V2 : string ) : string ;

begin
    Result := '' ;
    V1 := Normalize_Number( V1, length( V1 ), 0 ) ;
    V2 := Normalize_Number( V2, length( V2 ), 0 ) ;
    if( ( V1 = '' ) or ( V2 = '' ) ) then
    begin
        exit ;
    end ;
    Result := _Add( V1, V2, 0 ) ;
end ;


end.

