{
        Program Name : Template
        Package Name : Template_Main
        Purpose      : Main form for circuit simulator
        Institution  : Conroy & Conroy Co.
        Date Written :
        Written By   : Alan Conroy
        Version      : 1.0

	Copyright (C) 2016 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

        DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

}

unit Template_Main ;

{$mode objfpc}{$H+}

interface

uses // Pascal...
     Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
     Menus, ComCtrls;

const Application_Title = 'Template' ;

type TTab_Object = class
                       public // API...
                           Dirty : boolean ; // True if unsaved changes
                           Filename : string ;

                       public // API...
                           function Can_Close : boolean ; virtual ;
                           function Can_Save : boolean ; virtual ;
                           function Load( const Name : string ) : boolean ;
                               virtual ; abstract ;
                           function Save : boolean ; virtual ; abstract ;
                           function Default_Extension : string ;
                               virtual ; abstract ;
                   end ;


type

{ TForm1 }

 TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem19: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem20: TMenuItem;
    MenuItem3: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControl1: TPageControl;
    Panel1: TPanel;
    SaveDialog1: TSaveDialog;
    procedure MenuItem12Click(Sender: TObject);
    procedure MenuItem14Click(Sender: TObject);
    procedure MenuItem15Click(Sender: TObject);
    procedure MenuItem17Click(Sender: TObject);
    procedure MenuItem18Click(Sender: TObject);
    procedure MenuItem19Click(Sender: TObject);
    procedure MenuItem20Click(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
  private // Internal utility routines...
      function Close_Tab( C : TTabSheet ) : TModalResult ;
      function Save_Tab( C : TTab_Object ) : TModalResult ;
      function Save( C : TTab_Object ) : TModalResult ;
      function Save_As( C : TTab_Object ) : TModalResult ;
      function Save_All : TModalResult ;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

uses // Pascal...
     Windows ; // MessageBox

{$R *.lfm}

// TTab_Object methods...

// API...

function TTab_Object.Can_Close : boolean ;

begin
    Result := True ;
end ;


function TTab_Object.Can_Save : boolean ;

begin
    Result := True ;
end ;


type TCircuit = class( TTab_Object )
                    public // Constructors and destructors...
                        constructor Create ;
                        destructor Destroy ; override ;

                    public // API...
                        Attributes : TStringList ;

                    public // Overrides...
                        function Default_Extension : string ; override ;
                        function Load( const Name : string ) : boolean ; override ;
                        function Save : boolean ; override ;
                end ; // TCircuit

// Constructors and destructors...

constructor TCircuit.Create ;

begin
    inherited Create ;

    Dirty := True ;
    Attributes := TStringList.Create ;
end ;


destructor TCircuit.Destroy ;

begin
    Attributes.Free ;

    inherited Destroy ;
end ;


function TCircuit.Default_Extension : string ;

begin
    Result := 'cir' ;
end ;


function TCircuit.Save : boolean ;

begin
    Result := True ;
    //~~~
end ;


function TCircuit.Load( const Name : string ) : boolean ;

begin
    Result := True ;
    //~~~
end ;



function Extract_Name( S : string ) : string ;

var Dummy : integer ;

begin
    if( S = '' ) then
    begin
        Result := 'New' ;
    end ;
    Dummy := pos( '\', S ) ;
    while( Dummy > 0 ) do
    begin
        S := copy( S, Dummy + 1, length( S ) ) ;
        Dummy := pos( '\', S ) ;
    end ;
    Dummy := pos( ':', S ) ;
    S := copy( S, Dummy + 1, length( S ) ) ;
    if( pos( '.', S ) > 0 ) then
    begin
        Dummy := length( S ) ;
        while( S[ Dummy ] <> '.' ) do
        begin
            dec( Dummy ) ;
        end ;
        S := copy( S, 1, Dummy - 1 ) ;
    end ;
    Result := S ;
end ;


{ TForm1 }

function TForm1.Close_Tab( C : TTabSheet ) : TModalResult ;

var This_Tab : TTab_Object ;

begin
    This_Tab := TTab_Object( C.Tag ) ;
    Result := mrOK ;
    if( ( This_Tab.Filename = '' ) or This_Tab.Dirty ) then
    begin
        case MessageBox( 0, 'Do you want to save the changes?', PChar( C.Caption + ' has unsaved changes' ), mb_YesNoCancel ) of
           idYes : Result := Save_Tab( This_Tab ) ;
           idCancel : Result := mrCancel ;
        end ;
    end ;
    if( Result = mrOK ) then
    begin
        This_Tab.Free ;
        C.Tag := 0 ;
        C.Free ;
        PageControl1Change( nil ) ;
    end ;
end ;


function TForm1.Save_Tab( C : TTab_Object ) : TModalResult ;

begin
    Result := mrOK ;
    if( C.Filename = '' ) then // New circuit
    begin
        Result := Save_As( C ) ;
    end else
    if( C.Dirty ) then
    begin
        Result := Save( C ) ;
    end ;
end ;


function TForm1.Save( C : TTab_Object ) : TModalResult ;

begin
    Result := mrOK ;
    if( not C.Save ) then
    begin
        Result := mrCancel ;
    end else
    begin
        C.Dirty := False ;
    end ;
end ;


function TForm1.Save_As( C : TTab_Object ) : TModalResult ;

begin
    SaveDialog1.DefaultExt := C.Default_Extension ;
    if( not SaveDialog1.Execute ) then
    begin
        Result := mrCancel ;
        exit ;
    end ;
    Result := mrOK ;
    C.Filename := SaveDialog1.FileName ;
    if( not C.Save ) then
    begin
        Result := mrCancel ;
    end else
    begin
        C.Dirty := False ;
    end ;
end ;


function TForm1.Save_All : TModalResult ;

var Loop : integer ;
    TabSheet : TTabSheet ;
    Tab_Object : TTab_Object ;

begin
    Result := mrOK ;
    for Loop := PageControl1.PageCount - 1 downto 0 do
    begin
        TabSheet := PageControl1.Pages[ Loop ] ;
        Tab_Object := TTab_Object( TabSheet.Tag ) ;
        Result := Save_Tab( Tab_Object ) ;
        if( Result = mrCancel ) then
        begin
            exit ;
        end ;
        TabSheet.Caption := Extract_Name( Tab_Object.Filename ) ;
    end ;
end ;


procedure TForm1.MenuItem2Click(Sender: TObject); // Exit

begin
    if( Save_All = mrCancel ) then
    begin
        exit ;
    end ;

    Close ;
end ;


procedure TForm1.MenuItem12Click(Sender: TObject); // New

var T : TTabSheet ;

begin
    T := PageControl1.AddTabSheet ;
    T.Caption := 'New' ;
    T.Tag := integer( TCircuit.Create ) ;
    PageControl1Change( nil ) ;
end ;


procedure TForm1.MenuItem14Click(Sender: TObject); // Save

begin
    Save( TTab_Object( PageControl1.ActivePage.Tag ) ) ;
    PageControl1Change( nil ) ;
end ;


procedure TForm1.MenuItem15Click(Sender: TObject) ;

var Tab_Object : TTab_Object ;
    TabSheet : TTabSheet ;

begin
    TabSheet := PageControl1.ActivePage ;
    Tab_Object := TTab_Object( TabSheet.Tag ) ;
    if( Save_As( Tab_Object ) = mrOK ) then
    begin
        MenuItem14.Enabled := True ; // Save
        TabSheet.Caption := Extract_Name( Tab_Object.Filename ) ;
        PageControl1Change( nil ) ;
    end ;
end ;


procedure TForm1.MenuItem17Click(Sender: TObject); // Close

begin
    Close_Tab( PageControl1.ActivePage ) ;
    PageControl1Change( nil ) ;
end ;


procedure TForm1.MenuItem18Click(Sender: TObject); // Close all

var Loop : integer ;

begin
    try
        for Loop := PageControl1.PageCount - 1 downto 0 do
        begin
            if( Close_Tab( PageControl1.Pages[ Loop ] ) = mrCancel ) then
            begin
                exit ;
            end ;
        end ;
    finally
        PageControl1Change( nil ) ;
    end ;
end ;


procedure TForm1.MenuItem19Click(Sender: TObject);

begin
    Save_All ;
    PageControl1Change( nil ) ;
end ;


procedure TForm1.MenuItem20Click(Sender: TObject);

var C : TCircuit ;
    T : TTabSheet ;

begin
    OpenDialog1.DefaultExt := 'cir' ;
    if( OpenDialog1.Execute ) then
    begin
        C := TCircuit.Create ;
        if( not C.Load( OpenDialog1.Filename ) ) then
        begin
            C.Free ;
            exit ;
        end ;
        C.Filename := OpenDialog1.Filename ;
        C.Dirty := False ;
        T := TTabSheet.Create( PageControl1 ) ;
        T.Tag := integer( C ) ;
        T.Caption := Extract_Name( OpenDialog1.Filename ) ;
        T.Parent := PageControl1 ;
        PageControl1.ActivePageIndex := PageControl1.PageCount - 1 ;
        PageControl1Change( T ) ;
    end ;
end ;


procedure TForm1.PageControl1Change(Sender: TObject);

var Any_Close : boolean ;
    Any_Need_Save : boolean ;
    Loop : integer ;
    This_Tab : TTab_Object ;

begin
    if( PageControl1.PageCount = 0 ) then // No more tabs open
    begin
        MenuItem17.Enabled := False ; // Close
        MenuItem18.Enabled := False ; // Close all
        MenuItem19.Enabled := False ; // Save all
        MenuItem15.Enabled := False ; // Save as
        MenuItem14.Enabled := False ; // Save
        Caption := Application_Title ;
    end else
    begin
        Any_Close := false ;
        Any_Need_Save := false ;
        for Loop := 0 to PageControl1.PageCount - 1 do
        begin
            This_Tab := TTab_Object( PageControl1.Pages[ Loop ].Tag ) ;
            Any_Close := Any_Close or This_Tab.Can_Close ;
            Any_Need_Save := Any_Need_Save or ( This_Tab.Can_Save and ( ( This_Tab.Filename <> '' ) or This_Tab.Dirty ) ) ;
        end ;
        MenuItem18.Enabled := Any_Close ; // Close all
        MenuItem19.Enabled := Any_Need_Save ; // Save all
        This_Tab := TTab_Object( PageControl1.ActivePage.Tag ) ;
        MenuItem17.Enabled := This_Tab.Can_Close ; // Close
        MenuItem15.Enabled := This_Tab.Can_Save ; // Save as
        MenuItem14.Enabled := This_Tab.Can_Save and ( ( This_Tab.Filename <> '' ) or This_Tab.Dirty ) ; // Save
        Caption := Application_Title + ' - ' + This_Tab.Filename ;
    end ;
end ;



end.

