{$N+}
{
        Program Name : Tables
        Package Name : Subroutine Library - Layer 4
        Purpose      : Manipulate tables
        Institution  : Conroy & Conroy Co.
        Date Written : 14-Jan-1994
        Written By   : Alan Conroy
        Version      : 1.3A

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

          31-May-1995	EAC         Added ability for table lists.
          01-Jun-1995   EAC         Added multiplier ability.
          20-Jul-1995	EAC	    Allow roll equations.
          13-Apr-2007   EAC         Updated to work in Delphi 32.
          21-Sep-2015   EAC         Various updates.

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

        TSTable is used to generate random results from tables in a file.  The
        table file is a text file with the following format:
        <table definition>
        <one or more blank lines>
        <table deinition>
        .
        .
        .

        Each table definition has the following layout:
        <table number><tab><table name/description>
        <expression>
        <selector>
        <selector>
        .
        .
        .

        where "<expression"> is an expression with the format: <count>D<basis>
              "<count>" and "<basis>" are integer values.  The basis indicates the
              largest random value to generate (the lowest value will be 1).
              The count indicates how many random values to generate and add
              together.  For instance "1D6" will generate a value from 1 to 6.
              "3D10" will generate a value from 3 to 30 (1 to 10, 3 times). A
              count of 1 will generate a linear distribution of possibilities
              while a count greater than 1 will generate a standard distribution
              curve of possible results.
        "<selector>" has the format:
              <value><tab><table expression>
              where "<value>" indicates a single number, or a range specified as
              <low>-<high> (for instance "3-10").  An inverted range will never
              match any generated value.
              "<table expression>" is a list of comma-separated terms that are
              evaluated, concated together, and returned to the caller.  General
              format:
              <term>{,<term>...}
{             or
              <term>{;<term>...}
{             The type of term is determined by the first character of the term.
              The following terms are supported:
              first character     Description
              ---------------     -----------
              " or '              Literal text.  Note that the term must end
                                  with a matching quote if any further terms are
                                  on the line, otherwise the term is considered
                                  to be the rest of the line.  Note that one
                                  kind of quote can be included inside the other
                                  as part of the literal text.
                                  Examples:
                                           "This is text"
                                           'Say "hello" to me'
              #                   Numeric expression.  The expression is
                                  calculated and the result is returned.
                                  Examples:
                                           #1d8*10
              @                   Indirection.  This indicates another table.
                                  The number of the table follows the at-sign.
                                  Optionally, a multiplier can be included after
                                  the table number to indicate how many times a
                                  value from that table is returned.  The
                                  general format for indirection is:
                                          @<table number>{x<expression>}
{                                 Examples:
                                          @1200
                                          @70x1d6
                                  The first example generates a result from
                                  table number 1200.  The second generates 1-6
                                  results from table number 70.  When a
                                  multiplier is included, each result from the
                                  table is separated by a space.
              !                   Comment.  Entire line (including !) is
                                  returned as-is.

              Any other value indicates that the term is literal text that
              extends to the end of the line.
              
              Example table:
              10      Table 10: Slot machine options
              1d10
              1       Cherry
              2-4     Lemon
              5       Apple
              6       Banana
              7       Peach
              8       Pear
              9       $
              10      @11

        When a value is requested from a specific table number, the table is
        looked up in the file, a random value is generated from the expression,
        and the table is searched for a value that matches the generated random
        value.  If the table is found, and contains the generated value, the
        table expression is evaluated, and the resulting string is returned to
        the caller.

        The comma (,) and semicolon (;) separators have slightly different
        meanings.  The semicolon is a simple delimiter with no side-effects.
        However, the comma also inserts a ", " in its place if there was
        previous term that returned something non-null.

        Note: Table 0 is considered a special case.  Any references to that
        table always returns the value of Special_Case_Text.
}

unit Tables ;

interface

{$I EDEFINES.INC}

uses { Borland... }
{$IFNDEF FPC_OR_DELPHI}
     Objects, { TObject }
{$ENDIF}
     { Subroutine Library - Layer 0... }
     CommBase, { TBase_Object }
     _Types, { Small_String }

     { Subroutine Library - Layer 3... }
     Filestd ;

{$IFDEF DELPHI}
type TRedirection_Notify_Event = function( Table : integer ) : Ansistring ;
{$ENDIF}

type PEntry_Record = ^TEntry_Record ;
     TEntry_Record = record
                         Low, High : Integer ;
                         Next : PEntry_Record ;
                         Txt : Small_String ;
                     end ;
     PTable_Record = ^TTable_Record ;
     TTable_Record = record
                         Table_ID : Longint ;
                         Roll : PAnsiChar ;
                         Indirection : Integer ;
                         First_Entry : PEntry_Record ;
                         Next : PTable_Record ;
                     end ;

     PSTable = ^TSTable ;
     TSTable = object( TBase_Object )
                   public { Instance data... }
                       Minimum_Heap : Longint ;
                       Root : PTable_Record ;
                       Special_Case_Text : Ansistring ;
{$IFDEF DELPHI}
                       Redirection_Notification : TRedirection_Notify_Event ;
{$ENDIF}

                   public { Constructors and destructors... }
                       constructor Init( MinFree : Longint ) ;
                       destructor Done ; virtual ;

                   public { API... }
                       procedure Find_Table( Table : Longint ;
                           Inp : AnsiString ; var Current : PTable_Record ) ;
                           virtual ;
                       function Get_Entry( Table, Ent : Longint;
                           Inp : Ansistring ) : Ansistring ; virtual ;
                       function Generate_Entry( Table : Longint;
                           Inp : Ansistring ) : Ansistring ; virtual ;
                       procedure Free_Memory ; virtual ;
               end ; { TSTable }


implementation

uses { Borland... }
{$IFDEF FPC_OR_DELPHI}
     SysUtils, { StrPCopy }
{$ELSE}
     Strings, { StrPCopy }
{$ENDIF}
     { C&C Subroutine Library - Layer 0... }
     _UE, { PSUnified_Exception }

     { C&C Subroutine Library - Layer 1... }
     _CacheMa, // Cache_Manager
     CommonUt, { Edit }
     _Files, { PSFile }
     _FiP, { FSOM_Text }
     NUM1S,

     { C&C Subroutine Library - Layer 3... }
     CCStandard, { MaxAvail }

     { C&C Subroutine Library - Layer 4... }
     Express ;

procedure Append_String( var Result, Temp : Ansistring ) ;
{ Append Temp to Result, using ellipses for overflow.  Ignore if Temp is null }

begin
    if( ( Temp <> ' ' ) and ( length( edit( Temp, -1 ) ) = 0 ) ) then { Nothing to append }
    begin
        exit ;
    end ;
{$IFNDEF DYNAMIC_STRINGS}
    if( length( Result ) + length( Temp ) > 255 ) then { Result is too large }
    begin
        if( length( Result ) < 253 ) then
        begin
            Result := Result + '...' ; { Indicate overflow }
        end ;
    end else
{$ENDIF}
    begin
        Result := Result + Temp ;
    end ;
end ;


function Die( Count, Size : integer ) : integer ;
{ Generate a random number }

var Res : integer ;

begin
    Res := 0 ;
    while Count > 0 do
    begin
        Res := Res + random( Size ) + 1 ;
        Count := Count - 1 ;
    end ;
    Die := Res ;
end ;


function Do_Die( X : Ansistring ) : integer ;

var Dummy, Dummy0, Dummy1, Dummy2 : integer ;

begin
    Do_Die := -1 ;
    Fedit( X, X, -1 ) ;
    Dummy := pos( 'D', X ) ;
    val( copy( X, 1, Dummy - 1 ), Dummy1, Dummy0 ) ;
    if( Dummy0 <> 0 ) then
    begin
        exit ;
    end ;
    val( copy( X, Dummy + 1, 255 ), Dummy2, Dummy0 ) ;
    if( Dummy0 <> 0 ) then
    begin
        exit ;
    end ;
    Do_Die := Die( Dummy1, Dummy2 ) ;
end ;



{TSTable methods}

constructor TSTable.Init( MinFree : Longint ) ;

begin
    Minimum_Heap := MinFree ;
    Root := nil ;
{$IFDEF DYNAMIC_STRINGS}
    Initialize( Special_Case_Text ) ;
{$ENDIF}
    Special_Case_Text := 'use Standard Encounter Tables' ;
end ;


destructor TSTable.Done ;

begin
    while( Root <> nil ) do
    begin
        Free_Memory ; { Free memory until done }
    end ;
{$IFDEF DYNAMIC_STRINGS}
    finalize( Special_Case_Text ) ;
{$ENDIF}

    inherited Done ;
end ;


procedure TSTable.Find_Table( Table : Longint ; Inp : Ansistring ;
    var Current : PTable_Record ) ;
{ Return record for specified table number and file }

var Dummy : Longint ;
    Dummy1 : Integer ;
    Current_Entry : PEntry_Record ;
    F : PSFile ;
    R : TEntry_Record ;
    S : Ansistring ;
    T : TTable_Record ;
    UEC : PSUnified_Exception ;
    Tx : Ansistring ;

begin
    Current := Root ;
    while( Current <> nil ) do
    begin
        if( Table = Current^.Table_ID ) then { Found the entry in memory }
        begin
            exit ;
        end ;
        Current := Current^.Next ; { Follow link }
    end ;

    { If we get here, the entry is not in memory, so find it in file }
    Current_Entry := nil ;
    S := iNum( Table ) + AnsiChar( 9 ) ;
    F := Open( Inp, FSOM_Text, UEC ) ;
    while( not F^.EOF ) do { Until end of file is reached }
    begin
        F^.Readln( Tx ) ;
        R.Txt := Tx ;
        if( copy( R.Txt, 1, length( S ) ) = S ) then { Found table }
        begin
            Current := Root ;
            if( Current <> nil ) then
            begin
                while( Current^.Next <> nil ) do
                begin
                    Current := Current^.Next ; {Find last item in list}
                end ;
            end ;
            T.Table_ID := Table ;
            T.Next := nil ;
            F^.Readln( Tx ) ;
            R.TXT := Tx ;
            R.Txt := Edit( Tx, -1 ) ;
            T.Indirection := 0 ;
            T.Roll := nil ;
            if( R.TXT[ 1 ] = '@' ) then { Table redirection }
            begin
                T.First_Entry := nil ;
                Val( copy( R.TXT, 2, 254 ), T.Indirection, Dummy1 ) ;
                if( Root = nil ) then
                begin
                    New( Root ) ;
                    Root^ := T ;
                    Current := Root ;
                end else
                begin
                    if( MaxAvail < sizeof( T ) ) then
                    begin
                        Cache_Manager^.Free_Cache( sizeof( T ), $4FFFFFFF, nil ) ;
                    end ;
                    while( ( Root <> nil ) and ( MaxAvail < sizeof( T ) ) ) do
                    begin
                        Free_Memory ;
                    end ;
                    New( Current^.Next ) ;
                    Current^.Next^ := T ;
                end ;
                dispose( F, Done ) ;
                exit ;
            end ; // if( R.TXT[ 1 ] = '@' )

            getmem( T.Roll, length( R.TXT ) + 1 ) ;
            strPCopy( T.Roll, R.TXT ) ;
            T.First_Entry := nil ;
            if( MaxAvail < sizeof( T ) ) then
            begin
                Cache_Manager^.Free_Cache( sizeof( T ), $4FFFFFFF, nil ) ;
            end ;
            while( ( Root <> nil ) and ( MaxAvail < sizeof( T ) ) ) do
            begin
                Free_Memory ;
            end ;
            if( Root = nil ) then { First table in memory }
            begin
                New( Root ) ;
                Root^ := T ;
                Current := Root ;
            end else
            begin
                New( Current^.Next ) ; { Add new table to end of list }
                Current^.Next^ := T ;
                Current := Current^.Next ;
            end ;
            { At this point, Current points to the table header }

            while( length( R.TXT ) <> 0 ) do { Until end of table }
            begin
                if( F^.EOF ) then { End of File is also end of table }
                begin
                    R.TXT := '' ;
                end else
                begin
                    F^.Readln( Tx ) ;
                    R.TXT := Tx ;
                    if( length( R.TXT ) > 0 ) then { Not end of table }
                    begin
                        Dummy := pos( AnsiChar( 9 ), R.TXT ) ;
                        S := copy( R.TXT, 1, Dummy - 1 ) ;
                        R.TXT := Copy( R.TXT, Dummy + 1, 254 ) ;
                        Dummy := pos( '-', S ) ;
                        Val( copy( S, Dummy + 1, 254 ), R.High, Dummy1 ) ;
                        if( Dummy = 0 )
                            then
                                R.Low := R.High
                            else
                                Val( copy( S, 1, Dummy - 1 ), R.Low, Dummy1 ) ;
                        if( R.High = 0 ) then
                        begin
                            Val( '1' + copy( S, Dummy + 1, 254 ), R.High,
                                Dummy1 ) ;
                        end ;
                        R.Next := nil ;
                        if( Current^.First_Entry = nil ) then { First entry }
                        begin
                            GetMem( Current^.First_Entry, sizeof( pointer ) + sizeof( Integer ) * 2 + 1 +
                                length( R.TXT ) ) ;
                            Current^.First_Entry^.Next := nil ;
                            Current^.First_Entry^.Low := R.Low ;
                            Current^.First_Entry^.High := R.High ;
                            Current^.First_Entry^.Txt := R.Txt ;
                            Current_Entry := Current^.First_Entry ;
                        end else
                        begin
                            GetMem( Current_Entry^.Next, sizeof( pointer ) + sizeof( Integer ) * 2 + 1 +
                                length( R.TXT ) ) ;
                            Current_Entry := Current_Entry^.Next ;
                            Current_Entry^.Next := nil ;
                            Current_Entry^.Low := R.Low ;
                            Current_Entry^.High := R.High ;
                            Current_Entry^.Txt := R.Txt ;
                        end ; { if Current^.First_Entry = nil }
                    end ; { if length( R.TXT ) > 0 }
                end ; { if F.EOF }
            end ; { while length( R.TXT ) <> 0 }
            dispose( F, Done ) ;
            exit ;
        end ; {if copy( R.Txt, 1, length( S ) ) = S}
        {Skip down to end of this table, since it is not a match}
        while ( not F^.EOF ) and ( length( R.Txt ) > 0 ) do
        begin
            F^.Readln( Tx ) ;
            R.Txt := Tx ;
        end ;
    end ; {while not F.EOF do}
    Current := nil ; {Table not found}
    dispose( F, Done ) ;
end ; { TSTable.Find_Table }


function TSTable.Get_Entry( Table, Ent : Longint ; Inp : Ansistring ) : Ansistring ;
{ Get a specific entry from a table in the specified input file }

var C : Ansichar ;
    Current : PTable_Record ;
    E, E1 : extended ;
    Dummy : Longint ;
    Dummy1, Dummy2, Dummy3, Dummy4 : Integer ;
    Entry : PEntry_Record ;
    Mult, Res, Temp, Work : Ansistring ;

begin
    { Setup... }
    Get_Entry := ' ' ;
    Find_Table( Table, Inp, Current ) ;
    if( Current = nil ) then
    begin
        exit ;
    end ;
    Work := '' ;

    { Look up the value... }
    Entry := Current^.First_Entry ;
    if( Entry = nil ) then { Table redirection }
    begin
        if( Current^.Indirection = 0 ) then
        begin
            Get_Entry := Special_Case_Text ;
            exit ;
        end ;
        Work := Generate_Entry( Current^.Indirection, Inp ) ;
    end else
    while( Entry <> nil ) do
    begin
        if ( Entry^.Low <= Ent ) and ( Entry^.High >= Ent ) then
        begin
            if Entry^.Txt = '@0' then {Special case}
            begin
                Get_Entry := Special_Case_Text ;
                exit ;
            end ;
            Work := Entry^.TXT ;
            break ;
        end ;
        Entry := Entry^.Next ; { Follow link to next entry }
    end ;

    { Process table expression... }
    if( Work <> '' ) then
    begin
        Res := '' ;
        while( Work <> '' ) do { While expression to evaluates }
        begin
            case( Work[ 1 ] ) of
                '!' : { Comment }
                      begin
                          Res := Res + Work ;
                          Work := '' ;
                      end ;
                '@' : { Indirection }
                      begin
                          { Extract indirection... }
                          Dummy2 := pos( ',', Work + ',' ) ;
                          Dummy4 := pos( ';', Work + ';' ) ;
                          if( Dummy4 < Dummy2 ) then
                          begin
                              Dummy2 := Dummy4 ;
                          end ;
                          Temp := copy( Work, 2, Dummy2 - 2 ) ;
                          Work := copy( Work, Dummy2 + 1, length( Work ) ) ;

                          { Find/extract multiplier... }
                          Val( Temp, Dummy, Dummy1 ) ;
                          if( Dummy1 > 1 ) then
                          begin
                              if(
                                  ( Dummy1 < length( Work ) )
                                  and
                                  ( upcase( Work[ Dummy1 + 1 ] ) = 'X' )
                                ) then
                              begin
                                  Mult := copy( Temp, Dummy1 + 2, length( Temp ) ) ;
                                  Eval( Mult, E, E1, Dummy2, Dummy3 ) ;
                                  Dummy2 := trunc( E ) ;
                              end else
                              begin
                                  Dummy2 := 1 ;
                              end ;
                          end else
                          begin
                              Dummy2 := 1 ;
                          end ;

                          { Generate string from indirection... }
                          while( Dummy2 > 0 ) do
                          begin
{$IFDEF DELPHI}
                              if( assigned( Redirection_Notification ) ) then
                              begin
                                  Temp := Redirection_Notification( Dummy ) ;
                              end else
{$ENDIF}
                              begin
                                  Temp := Generate_Entry( Dummy, Inp ) ;
                              end ;
                              if( Dummy2 > 1 ) then
                              begin
                                  Temp := Temp + ' ' ;
                              end ;
                              Append_String( Res, Temp ) ;
                              Dummy2 := Dummy2 - 1 ;
                          end ;
                      end ;
                '#' : { Expression }
                      begin
                          Work := copy( Work, 2, length( Work ) ) ;
                          Dummy := pos( ',', Work + ',' ) ;
                          Dummy4 := pos( ';', Work + ';' ) ;
                          if( Dummy4 < Dummy ) then
                          begin
                              Dummy := Dummy4 ;
                          end ;
                          Eval( copy( Work, 1, Dummy - 1 ), E, E1, Dummy2, Dummy3 ) ;
                          Temp := iNum( E ) ;
                          Append_String( Res, Temp ) ;
                          Work := copy( Work, Dummy + 1, length( Work ) ) ;
                      end ;
                '"', #39 : { Text }
                           begin
                               C := Work[ 1 ] ;
                               Work := copy( Work, 2, length( Work ) ) ;
                               Dummy := pos( C, Work + C ) ;
                               Temp := copy( Work, 1, Dummy - 1 ) ;
                               Append_String( Res, Temp ) ;
                               Work := copy( Work, Dummy + 1, length( Work ) ) ;
                           end ;
                ',' : begin
                          if( Res <> '' ) then
                          begin
                              Res := Res + ', ' ;
                          end ;
                          Work := copy( Work, 2, length( Work ) ) ; { Separation }
                      end ;
                ';' : Work := copy( Work, 2, length( Work ) ) ; { Separation }
                else { Anything else is literal text }
                     begin
                         Append_String( Res, Work ) ;
                         Work := '' ;
                     end ;
            end ; { case( Work[ 1 ] ) of }
        end ; { while( Work <> '' ) }

        Get_Entry := Res ;
        exit ;
    end ; { if( Work <> '' ) }
    Get_Entry := ' ' ; { Entry not found }
end ; { TSTable.Get_Entry }


function TSTable.Generate_Entry( Table : Longint; Inp : Ansistring ) : Ansistring ;
{ Get a random entry from a table }

var Current : PTable_Record ;
    Dummy, Dummy1 : extended ;
    Dummy2, Dummy3 : integer ;

begin
    Find_Table( Table, Inp, Current ) ;
    if( Current = nil ) then
    begin
        Generate_Entry := '' ; { Not found }
    end else
    begin
        Eval( strpas( Current^.Roll ), Dummy, Dummy1, Dummy2, Dummy3 ) ;
        Generate_Entry := Get_Entry( Table, trunc( Dummy ), Inp ) ;
    end ;
end ;


procedure TSTable.Free_Memory ; { Free first table in list }

var Current, Temp : PEntry_Record ;

begin
    if( Root <> nil ) then
    begin
        strdispose( Root^.Roll ) ;
        Current := Root^.First_Entry ;
        while( Current <> nil ) do
        begin
            Temp := Current ;
            Current := Current^.Next ;
            dispose( Temp ) ; { TODO: Need Free due to morphable size (see getmem usage) }
        end ;
        Root := Root^.Next ;
    end ;
end ;



end.
