{
        Program Name : UOS_File_Store
        Package Name : UOS
        Purpose      : UOS File store
        Institution  : Conroy & Conroy Co.
        Date Written : 16-Jul-2017
        Written By   : Alan Conroy
        Version      : 1.0

	    Copyright (C) 2017 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

        DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

        UOS file as a store class.
}

unit UOS_File_Store ;

interface

uses // Subroutine Library..
     _Cache, // TCOM_Cache64
     _Stores, // TCOM_Store64
     _UE,  // TUnified_Exception

     // UOS...
     _UOS_File_System, // TUOS_File
     UOS_MStore ; // TUOS_Managed_Store

type TUOS_Native_File_Store = class( TCOM_Store64 )
                                  public // Instance data...
                                      __File : TUOS_File ;

                                      _Cache : TCOM_Cache64 ;
                                      _Read_Only : boolean ;
                                      _Write_Only : boolean ;
                                      _Resolution : longint ;
                                      _Fixed : boolean ;

                                      _Bytes_Read : longint ;
                                      _Bytes_Written : longint ;
                                      _Reads : longint ;
                                      _Writes : longint ;
                                      _Error_Count : longint ;

                                  public // Constructors and destructors...
                                      destructor Destroy ; override ;

                                  public // Overrides...
                                      function Is_Class( Name : PChar ) : boolean ;
                                          override ; stdcall ;

                                  public // API...
                                      function Read_Data( var Data ; Address, _Size : TStore_Address64 ;
                                          var UEC : TUnified_Exception ) : TStore_Address64 ;
                                          override ;

                                      function Write_Data( var Data ; Address, _Size : TStore_Address64 ;
                                          var UEC : TUnified_Exception ) : TStore_Address64 ;
                                          override ;

                                      function Max_Storage : TStore_Size64 ;
                                          override ;

                                      function Min_Storage : TStore_Address64 ;
                                          override ;

                                      function Extend( Amount : TStore_Address64 ) : TStore_Address64 ;
                                          override ;

                                      function Get_Read_Only : boolean ;
                                          override ;

                                      function Get_Write_Only : boolean ;
                                          override ;

                                      procedure Format ; override ;

                                      function Get_Name : PChar ;
                                          override ;

                                      function Get_Cache : TCOM_Cache64 ;
                                          override ;

                                      procedure Set_Cache( Value : TCOM_Cache64 ) ;
                                          override ;

                                      function Contiguous_Store : boolean ;
                                          override ;

                                      procedure Set_Max_Storage( Value : TStore_Address64 ;
                                          var Res : TUnified_Exception ) ;
                                          override ;

                                      function Extended_Size : TStore_Address64 ;
                                          override ;

                                      function Get_Bytes_Read : longint ;
                                          override ;
                                      function Get_Bytes_Written : longint ;
                                          override ;
                                      function Get_Reads : longint ;
                                          override ;
                                      function Get_Writes : longint ;
                                          override ;
                                      function Get_Error_Count : longint ;
                                          override ;
                                      procedure Set_Bytes_Read( Value : longint ) ;
                                          override ;
                                      procedure Set_Bytes_Written( Value : longint ) ;
                                          override ;
                                      procedure Set_Reads( Value : longint ) ;
                                          override ;
                                      procedure Set_Writes( Value : longint ) ;
                                          override ;
                                      procedure Set_Error_Count( Value : longint ) ;
                                          override ;
                                      procedure Set_Read_Only( Value : boolean ) ;
                                          override ;
                                      procedure Set_Write_Only( Value : boolean ) ;
                                          override ;
                                      function Get_File : TUOS_File ;
                                          virtual ;
                                      procedure Set_File( Value : TUOS_File ) ;
                                          virtual ;

                                  published
                                      property _File : TUOS_File
                                          read Get_File
                                          write Set_File ;
                                      property Resolution : longint
                                          read _Resolution
                                          write _Resolution ;
                              end ; // TUOS_Native_File_Store


implementation

uses // Pascal...
     Sysutils, // lowercase
     
     // Subroutine Library...
     UE ; // Create_Error

const Err_UOS_Native_File_Store_Facility = 143 ;
      Err_Write_Only = 1 ;
      Err_Read_Only = 2 ;

// TUOS_Native_File_Store methods...

// Constructors and destructors...

destructor TUOS_Native_File_Store.Destroy ;

begin
    inherited Destroy ;
end ;


// Overrides...

function TUOS_Native_File_Store.Is_Class( Name : PChar ) : boolean ;

var _N : string ;

begin
    _N := lowercase( string( Name ) ) ;
    Result := _N = 'tuos_native_file_store' ;
end ;


// API...

function TUOS_Native_File_Store.Read_Data( var Data ; Address, _Size : TStore_Address64 ;
    var UEC : TUnified_Exception ) : TStore_Address64 ;

begin
    if( Write_Only ) then
    begin
        Result := 0 ;
        UEC := Create_Simple_UE( Err_UOS_Native_File_Store_Facility, 10, Err_Write_Only, UE_Error, 'File is write-only', '' ) ;
        exit ;
    end ;
    Result := _File.Read( 0, Address, _Size, Data ) ;
    UEC := _File.Last_Error ;
end ;


function TUOS_Native_File_Store.Write_Data( var Data ; Address, _Size : TStore_Address64 ;
    var UEC : TUnified_Exception ) : TStore_Address64 ;

var P : int64 ;

begin
    if( Read_Only ) then
    begin
        Result := 0 ;
        UEC := Create_Simple_UE( Err_UOS_Native_File_Store_Facility, 10, Err_Read_Only, UE_Error, 'File is read-only', '' ) ;
        exit ;
    end ;
    P := Address - _File.File_Size ;
    if( P > 0 ) then
    begin
        while( P > 0 ) do
        begin
            if( P >= sizeof( P ) ) then
            begin
                P := P - sizeof( P ) ;
                _File.Write( 0, _File.File_Size, sizeof( P ), P )
            end else
            begin
                _File.Write( 0, _File.File_Size, P, P ) ;
                P := 0 ;
            end ;
        end ;
    end ;
    Result := _File.Write( 0, Address, _Size, Data ) ;
    UEC := _File.Last_Error ;
end ;


function TUOS_Native_File_Store.Max_Storage : TStore_Size64 ;

begin
    Result := _File.File_Size ;
end ;


function TUOS_Native_File_Store.Min_Storage : TStore_Address64 ;

begin
    Result := _Resolution ;
end ;


function TUOS_Native_File_Store.Extend( Amount : TStore_Address64 ) : TStore_Address64 ;

var Original_Size : int64 ;

begin
    if( Amount = 0 ) then // Check for extendability
    begin
        if( Read_Only or _Fixed ) then
        begin
            Result := -1 ; // Cannot extend
        end else
        begin
            Result := 0 ; // Can extend
        end ;
        exit ;
    end ;
    if( Read_Only or _Fixed ) then
    begin
        Result := 0 ;
        exit ;
    end ;

    // Do the extend operation...
    Original_Size := _File.File_Size ;
    _File.File_Size := Original_Size + Amount ;
    Result := _File.File_Size - Original_Size ; // Amount actually extended
end ;


function TUOS_Native_File_Store.Get_Read_Only : boolean ;

begin
    Result := _Read_Only or _File.Read_Only ;
end ;


function TUOS_Native_File_Store.Get_Write_Only : boolean ;

begin
    Result := _Write_Only or _File.Write_Only ;
end ;


procedure TUOS_Native_File_Store.Format ;

begin
    // This does nothing
end ;


function TUOS_Native_File_Store.Get_Name : PChar ;

begin
    Result := nil ;
end ;


function TUOS_Native_File_Store.Get_Cache : TCOM_Cache64 ;

begin
    Result := _Cache ;
end ;


procedure TUOS_Native_File_Store.Set_Cache( Value : TCOM_Cache64 ) ;

begin
    if( Value <> nil ) then
    begin
        Value.Attach ;
    end ;
    if( _Cache <> nil ) then
    begin
        _Cache.Detach ;
    end ;
    _Cache := Value ;
end ;


function TUOS_Native_File_Store.Contiguous_Store : boolean ;

begin
    Result := True ;
end ;


procedure TUOS_Native_File_Store.Set_Max_Storage( Value : TStore_Address64 ;
    var Res : TUnified_Exception ) ;

begin
    if( Read_Only ) then
    begin
        Res := Create_Simple_UE( Err_UOS_Native_File_Store_Facility, 10, Err_Read_Only, UE_Error, 'File is read-only', '' ) ;
        exit ;
    end ;
    _File.Set_Stream_Size( 0, Value ) ;
    Res := _File.Last_Error ;
end ;


function TUOS_Native_File_Store.Extended_Size : TStore_Address64 ;

begin
    Result := Max_Storage ;
    if( not Read_Only ) then
    begin
        Result := Result + _File.XSpaceAvail ;
    end ;
end ;


function TUOS_Native_File_Store.Get_Bytes_Read : longint ;

begin
    Result := _Bytes_Read ;
end ;


function TUOS_Native_File_Store.Get_Bytes_Written : longint ;

begin
    Result := _Bytes_Written ;
end ;


function TUOS_Native_File_Store.Get_Reads : longint ;

begin
    Result := _Reads ;
end ;


function TUOS_Native_File_Store.Get_Writes : longint ;

begin
    Result := _Writes ;
end ;


function TUOS_Native_File_Store.Get_Error_Count : longint ;

begin
    Result := _Error_Count ;
end ;


procedure TUOS_Native_File_Store.Set_Bytes_Read( Value : longint ) ;

begin
    _Bytes_Read := Value ;
end ;


procedure TUOS_Native_File_Store.Set_Bytes_Written( Value : longint ) ;

begin
    _Bytes_Written := Value ;
end ;


procedure TUOS_Native_File_Store.Set_Reads( Value : longint ) ;

begin
    _Reads := Value ;
end ;


procedure TUOS_Native_File_Store.Set_Writes( Value : longint ) ;

begin
    _Writes := Value ;
end ;


procedure TUOS_Native_File_Store.Set_Error_Count( Value : longint ) ;

begin
    _Error_Count := Value ;
end ;


procedure TUOS_Native_File_Store.Set_Read_Only( Value : boolean ) ;

begin
    _Read_Only := True ;
end ;


procedure TUOS_Native_File_Store.Set_Write_Only( Value : boolean ) ;

begin
    _Write_Only := True ;
end ;


function TUOS_Native_File_Store.Get_File : TUOS_File ;

begin
    Result := __File ;
end ;


procedure TUOS_Native_File_Store.Set_File( Value : TUOS_File ) ;

begin
    if( Value <> nil ) then
    begin
        Value.Attach ;
    end ;
    if( _File <> nil ) then
    begin
        _File.Detach ;
    end ;
    __File := Value ;
end ;


end.
