{
        Program Name : UOS_Parse
        Package Name : UOS
        Purpose      : UOS Support Library: Parse
        Institution  : Conroy & Conroy Co.
        Date Written : 8-Nov-2017
        Written By   : Alan Conroy
        Version      : 1.0

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          This is a library of parsing-related services for UOS application use.
}

unit UOS_Parse ;

interface

uses // Pascal...
     Classes, // TStringList

     // C&C...
     Compatib, // Ansistring
     CommonUt ; // TInteger_List

type TParser = class
                   private { Instance data... }
                       _Current_Line : integer ; { Current line number }
                       _Leading_Whitespace : boolean ; { True if current line had leading whitespace }
                       _Quotes : Ansistring ;

                       Returned_Tokens : Integer ;
                       { Number of returned tokens waiting for reads }

                       Token_Stack : TInteger_List ;
                       { Stack of token lines }

                       Tokens : TStringList ; { Special tokens }

                   public { Constructors and destructors... }
                       constructor Create ;
                       destructor Destroy ; override ;

                   protected { Internal utility routines... }
                       function Get_Token : AnsiString ; virtual ;
                       { Return next token from source file }

                       function Pop_Token_Line : Integer ; { Return previous token line}
                       procedure Push_Token_Line ; { Push current token line }
                       function Read_Next_Line : boolean ; virtual ; { Read next line from source data }

                   public { Property handlers... }
                       function Get_Current_Line : longint ;
                       function Get_Token_Line : longint ;

                   public { API... }
                       Command : AnsiString ; { Current input line }
                       Non_Terminals : AnsiString ;
                       Token_Line : Integer ; { Line number of current token }
                       Case_Sensitive : boolean ; { True if case-sensitive outside of quotes }

                       procedure Add_Special_Token( S : AnsiString ) ;
                       function Grab_Line : AnsiString ;
                       function Token : AnsiString ;
                       function Token_EOL : boolean ;
                       procedure Put_Token( Value : AnsiString ) ; virtual ;
                       function Line_Text : AnsiString ; virtual ;
                       function Leading_Whitespace : boolean ; virtual ;
                       procedure Set_Quotes( Value : AnsiString ) ; virtual ;
                       function Peek : string ; virtual ;
               end ; // TParser

implementation

uses // C&C...
     _ASCII, // HT
     UStrings ; // Left

{ TSParser methods... }

{ Constructors and destructors... }

constructor TParser.Create ;

begin
    inherited Create ;

    Tokens := TStringList.Create ;
    Token_Stack := TInteger_List.Create ;
    Non_Terminals := '$_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789' ;
    _Quotes := '"'#39 ;
end ;


destructor TParser.Destroy ;

begin
    Tokens.Free ;
    Tokens := nil ;
    Token_Stack.Free ;
    Token_Stack := nil ;

    inherited Destroy ;
end ;


{ Internal utility routines... }

function TParser.Get_Token : AnsiString ; { Return next token from source }

var Aa : integer ; { Changed from byte to support lines > 255 bytes in length }
    Bb : AnsiChar ;
    C : AnsiString ;
    E : integer ;
    Quote : Boolean ;
    Quote_Type : AnsiChar ;

Label Next, Token_End ;

begin
    Quote_Type := ' ' ;

Next:
    if( Command = '' ) then { No more text in buffer }
    begin
        if( not Read_Next_Line ) then
        begin
            Get_Token := '' ;
            exit ;
        end ;
        Inc( _Current_Line ) ; { Increment line count }
        Token_Line := Get_Current_Line ; { Token line is now same as current line }

        _Leading_Whitespace := ( Left( Command, 1 ) = ' ' ) or ( Left( Command, 1 ) = HT ) ;
        E := 8700 ;
        if( Case_Sensitive ) then
        begin
            E := E - 32 ;  { No uppercase }
        end ;
        Command := Edit( Command, E ) ; { Format line }
        if( Command = '' ) then
        begin
            goto Next ; { Ignore blank lines }
        end ;
    end ;
    Quote := False ;
    C := '' ;
    while( ( Left( Command, 1 ) = ' ' ) or ( Left( Command, 1 ) = HT ) ) do // Trim leading spaces/tabs
    begin
        Command := copy( Command, 1 + Low_String_Index, length( Command ) ) ;
    end ;
    for Aa := Low_String_Index to high_string_Index( Command ) do
    begin
        Bb := Ansichar( Upcase( Command[ Aa ] ) ) ; { Get current character }
        if Quote then { Within a quoted token }
        begin
            if( Bb = Quote_Type ) then { End of quoted token }
            begin
                C := Left_Of( Command, Aa + 1 ) ;
                Command := Right( Command, Aa + 1 ) ;
                goto Token_End ;
            end ;
        end else
        if( Aa = Low_String_Index ) then { First character of token }
        begin
            if( pos( BB, _Quotes ) >= Low_String_Index ) then { if a quote }
            begin
                Quote := True ; { Now within quotes }
                Quote_Type := Bb { Save quote type }
            end else
            if( pos( BB, Non_Terminals ) < Low_String_Index ) then
            begin
                C := Left( Command, 1 ) ;
                Command := Right( Command, 1 + Low_String_Index ) ;
                goto Token_End ;
            end ;
        end else { After first character }
        if( pos( BB, Non_Terminals ) < Low_String_Index ) then
        begin
            C := Left_Of( Command, Aa ) ; // Everything before terminal
            Command := Right( Command, Aa ) ;
            goto Token_End ;
        end ;
    end ; { for Aa := Low_String_Index to High_String_Index( Command ) }
    C := Command ; { Token runs to end of line }
    Command := '' ;
Token_End:
    if( pos( Left( C, 1 ), _Quotes ) < Low_String_Index ) then
    begin
//        C := Edit( C, -1 ) ; { Format token }
    end ;
    if( C = '' ) then
    begin
        goto Next ; { Ignore spaces, etc }
    end ;
    if( Returned_Tokens > 0 ) then
    begin
        Token_Line := Pop_Token_Line ;
        Returned_Tokens := Returned_Tokens - 1 ;
    end ;
    Get_Token := C ; { Return value }
end ; { TParser.Get_Token }


function TParser.Pop_Token_Line : Integer ; { Return previous token line }

begin
    if( Token_Stack.Count = 0 ) then
    begin
        Pop_Token_Line := 0 ;
    end else
    begin
        Pop_Token_Line := Token_Stack[ Token_Stack.Count - 1 ] ;
        Token_Stack.Delete( Token_Stack.Count - 1 ) ;
    end ;
end ;


procedure TParser.Push_Token_Line ; { Push current token line }

begin
    Token_Stack.Add( Token_Line ) ;
end ;


function TParser.Read_Next_Line : boolean ;

begin
    Command := '' ;
    Read_Next_Line := False ;
end ;


{ Property handlers... }

function TParser.Get_Token_Line : longint ;

begin
    Get_Token_Line := Token_Line ;
end ;


function TParser.Get_Current_Line : longint ;

begin
    Get_Current_Line := _Current_Line ;
end ;


{ API... }

procedure TParser.Add_Special_Token( S : AnsiString ) ;

begin
    Tokens.Add( S ) ;
end ;


function TParser.Grab_Line : AnsiString ;

var E : integer ;

label Next ;

begin
    if( Token_Stack.Count > 0 ) then
    begin
        Token_Line := Token_Stack[ Token_Stack.Count - 1 ] ;
    end ;
    Returned_Tokens := 0 ;
    while( Token_Stack.Count > 0 ) do
    begin
        Token_Stack.Delete( Token_Stack.Count - 1 ) ;
    end ;
    if( Command = '' ) then { No more text in buffer }
    begin
Next:
        if( Read_Next_Line ) then
        begin
            Inc( _Current_Line ) ; { Increment line count }
            Token_Line := Get_Current_Line ; { Token line is now same as current line }

            _Leading_Whitespace := ( Left( Command, 1 ) = ' ' ) or ( Left( Command, 1 ) = HT ) ;
            E := 8700 ;
            if( Case_Sensitive ) then
            begin
                E := E - 32 ;  { No uppercase }
            end ;
            Command := Edit( Command, E ) ; { Format line }
            if( Command = '' ) then
            begin
                goto Next ; { Ignore blank lines }
            end ;
        end ;
    end ;
    Grab_Line := Command ;
    Command := '' ;
end ; { TParser.Grab_Line }


function TParser.Token : AnsiString ;

var Dummy : integer ;
    S, T : AnsiString ; { Accumulated token }
    Temp : AnsiString ;

begin
    T := Get_Token ; //NOTE: Handle quoted data differently (ie skip the following)
    if( ( Tokens.Count > 0 ) and ( T <> '' ) ) then
    begin
        { A list of special tokens is present and we have a token }
        for Dummy := 0 to Tokens.Count - 1 do
        begin
            S := Tokens[ Dummy ] ;
            if( T[ 1 ] = S[ 1 ] ) then { A potential match }
            begin
                Temp := Left( Command, length( S ) - 1 ) ;
                if( T + Temp = S ) then { Found a match }
                begin
                    while( length( T ) < length( S ) ) do
                    begin
                        T := T + Get_Token ;
                    end ;
                    Token := T ;
                    exit ;
                end ;
            end ; //  if( T[ 1 ] = S[ 1 ] )
        end ; // for Dummy := 0 to Tokens.Count - 1
    end ; // if( ( Tokens.Count > 0 ) and ( T <> '' ) )
    Token := T ; { Return result }
end ; { TParser.Token }


procedure TParser.Put_Token( Value : AnsiString ) ;

var Ch : AnsiChar ;
    Temp : AnsiString ;

begin
    if( length( Value ) > 0 ) then
    begin
        Inc( Returned_Tokens ) ;
        if( Command = '' ) then
        begin
            Command := Value ;
        end else
        begin
            Temp := Command ;
            Ch := Command[ 1 ] ;
            if(
                ( Length( Command ) > 0 )
                and
                ( pos( Ch, Non_Terminals ) >= Low_String_Index )
              ) then
            begin
                Command := Value + ' ' + Temp ;
            end else
            begin
                Command := Value + Temp ;
            end ;
        end ;
        Push_Token_Line ;
{        Token_Line := Token_Stack.At( Token_Stack.Count - 1 ) ;}
    end ;
end ; { TParser.Put_Token }


function TParser.Leading_Whitespace : boolean ;

begin
    Leading_Whitespace := _Leading_Whitespace ;
end ;


procedure TParser.Set_Quotes( Value : AnsiString ) ;

begin
    _Quotes := Value ;
end ;


function TParser.Peek : string ;

begin
    Result := Get_Token ;
    Put_Token( Result ) ;
end ;


function TParser.Line_Text : AnsiString ;

var C : AnsiString ;
    E : integer ;
    Temp : AnsiString ;

begin
    if( Command = '' ) then { No more text in buffer }
    begin
        if( not Read_Next_Line ) then
        begin
            Line_Text := '' ;
            exit ;
        end ;
        Inc( _Current_Line ) ; { Increment line count }
        Token_Line := Get_Current_Line ; { Token line is now same as current line }

        _Leading_Whitespace := ( Left( Command, 1 ) = ' ' ) or ( Left( Command, 1 ) = HT ) ;
        E := 8700 ; { Remove leading/trailing spaces }
        if( Case_Sensitive ) then
        begin
            E := E - 32 ;  { No uppercase }
        end ;
        Temp := Edit( Command, E ) ; { Format line }
        Command := Temp ;
    end ;
    C := Command ;
    Line_Text := C ;
end ;


function TParser.Token_EOL : boolean ;

begin
    Token_EOL := ( Command = '' ) ;
end ;



end.
