{
        Program Name : UOS_Strings
        Package Name : Subroutine Library
        Purpose      : UOS String routines
        Institution  : Conroy & Conroy Co.
        Date Written : 2-Sep-2017
        Written By   : Alan Conroy
        Version      : 1.0

    	Copyright (C) 2017 by Alan Conroy.  All rights reserved.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

        DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

        UOS Strings...
}

unit UOS_Strings ;

interface

// String types...
const SF_ASCII = 1 ;
      SF_UTF16 = 2 ;
      SF_UTF8 = 3 ;
      SF_UTF32 = 4 ;


type TUOS_String = class
{$IFDEF CC_Track_Resources}
                       public // Constructors and destructors...
                           constructor Create ;
                           destructor Destroy ; override ;
{$ENDIF}

                       private // Instance data...
                           Contents : array of cardinal ;

                       protected // Property handlers...
                           // Return length of our contents...
                           function Get_Length : integer ;
                           procedure Set_Length( Value : integer ) ;

                       public // API...
                           function As_String : string ;

                           // Assign our contents from a UTF8 string...
                           procedure Assign_From_String( const S : string ;
                               Format : integer ) ;

                           // Create...
                           function Copy( Start, Len : integer ) : TUOS_String ;

                           // Return true if our contents are equal to the match
                           function Equal( Match : TUOS_String ) : boolean ;

                           // Insert character at given position
                           procedure Insert( Position : integer ;
                               Value : cardinal ) ;

                           // Convert our characters to lowercase...
                           procedure Lowercase ;

                           // Position of substring...
                           function Pos( const Value : string ;
                               Start : integer = 1 ) : integer ; overload ;

                           function Pos( const Value : TUOS_String ;
                               Start : integer = 1 ) : integer ; overload ;

                           // Return rightmost instance of Value
                           function RPos( Value : char ) : integer ;

                       public // Properties...
                           property Length : integer
                               read Get_Length
                               write Set_Length ;
                   end ; // TUOS_String


{$IFDEF CC_Regression_Test}
procedure Test_Unit ;
{$ENDIF}


implementation

uses CommonUt // Edit
     ,_Unicode // Foldings
{$IFDEF CC_Track_Resources}
     ,ResMon
{$ENDIF}
{$IFDEF CC_Regression_Test}
     ,sysutils
     ,strutils
     ,windows
     ,dialogs
{$ENDIF}
     ;

// TUOS_String methods...

{$IFDEF CC_Track_Resources}
// Constructors and destructors...

constructor TUOS_String.Create ;

begin
    inherited Create ;

    Add_Resource( RT_Object, integer( self ) ) ;
end ;


destructor TUOS_String.Destroy ;

begin
    Remove_Resource( RT_Object, integer( self ) ) ;

    inherited Destroy ;
end ;
{$ENDIF}


// API...

function TUOS_String.As_String : string ;

var Dummy, Loop : integer ;

begin
    System.setlength( Result, Length ) ;
    for Loop := 0 to Length - 1 do
    begin
        Dummy := Contents[ Loop ] ;
        if( Dummy > 127 ) then
        begin
            Dummy := Dummy or 128 ;
        end ;
        Result[ Loop + 1 ] := chr( Dummy ) ;
    end ;
end ;


procedure TUOS_String.Assign_From_String( const S : string ;
    Format : integer ) ;

var Index, Size, Mask : integer ;
    Value : cardinal ;

begin
    Index := 1 ; // Index in Spec
    setlength( Contents, 0 ) ;
    if( Format = SF_UTF8 ) then // UTF8
    begin
        while( Index <= system.length( S ) ) do
        begin
            Value := 0 ;
            if( S[ Index ] > #$FC ) then
            begin
                Size := 6 ;
                Mask := 1 ;
            end else
            if( S[ Index ] > #$F8 ) then
            begin
                Size := 5 ;
                Mask := 3 ;
            end else
            if( S[ Index ] > #$F0 ) then
            begin
                Size := 4 ;
                Mask := 7 ;
            end else
            if( S[ Index ] > #$E0 ) then
            begin
                Size := 3 ;
                Mask := $F ;
            end else
            if( S[ Index ] > #$C0 ) then
            begin
                Size := 2 ;
                Mask := $1F ;
            end else
            begin
                Size := 1 ;
                Mask := $7F ;
            end ;
            while( Size > 0 ) do
            begin
                dec( Size ) ;
                Value := Value or ( ord( S[ Index ] ) and Mask ) ;
                if( Size > 0 ) then
                begin
                    Value := Value shl 6 ;
                end ;
                Mask := $3F ;
                inc( Index ) ;
            end ;
            setlength( Contents, system.length( Contents ) + 1 ) ;
            Contents[ system.length( Contents ) - 1 ] := Value ;
        end ; // while( Index < system.length( S ) )
    end else
    begin
        Value := 0 ;
        Index := 1 ; // Index in S
        while( Index <= system.length( S ) ) do
        begin
            move( PChar( S )[ Index - 1 ], Value, Format ) ;
            Index := Index + Format ;
            setlength( Contents, system.length( Contents ) + 1 ) ;
            Contents[ system.length( Contents ) - 1 ] := Value ;
        end ;
    end ; // if( Format = 3 )
end ; // TUOS_String.Assign_From_String


function TUOS_String.Get_Length : integer ;

begin
    Result := system.length( Contents ) ;
end ;


procedure TUOS_String.Set_Length( Value : Integer ) ;

begin
    setlength( Contents, Value ) ;
end ;


function lowcase( V : cardinal ; var _Folding_Index : integer ) : cardinal ;

var L, H, Index : integer ;

begin
    Result := V ;
    _Folding_Index := -1 ; // Indicates no translation
    if( ( V < $41 ) or ( V > $118BF ) ) then // Not within range of our table
    begin
        exit ;
    end ;
    L := 0 ;
    H := high( Foldings ) ;
    while( L < H ) do
    begin
        Index := L + ( ( H - L ) div 2 ) ;
        if( V = Foldings[ Index, 0 ] ) then
        begin
            if( Foldings[ Index, 2 ] > 0 ) then // Multiple outputs
            begin
                Result := 0 ;
                _Folding_Index := Index ;
                exit ;
            end ;
            Result := Foldings[ Index, 1 ] ;
            exit ;
        end else
        if( V < Foldings[ Index, 0 ] ) then
        begin
            H := Index ;
        end else
        begin
            L := Index + 1 ;
        end ;
        if( L >= H ) then
        begin
            exit ;
        end ;
    end ; // while( L < H )
end ; // lowcase


function TUOS_String.Copy( Start, Len : integer ) : TUOS_String ;

begin
    // Setup...
    Result := TUOS_String.Create ;
    if( Start > Length ) then
    begin
        exit ;
    end ;
    if( Start < 1 ) then
    begin
        Start := 1 ;
    end ;
    if( Start + Len - 1 > Length ) then
    begin
        Len := Length - Start + 1 ;
    end ;

    Result.Length := Len ;
    move( Contents[ Start - 1 ], Result.Contents[ 0 ], Len * sizeof( cardinal ) ) ;
end ; // TUOS_String.Copy


function TUOS_String.Equal( Match : TUOS_String ) : boolean ;

var Loop : integer ;

begin
    Result := False ;
    if( Length <> Match.Length ) then
    begin
        exit ;
    end ;
    for Loop := 0 to Length - 1 do
    begin
        if( ( Contents[ Loop ] <> Match.Contents[ Loop ] ) ) then
        begin
            exit ;
        end ;
    end ;
    Result := True ;
end ; // TUOS_String.Equal


procedure TUOS_String.Insert( Position : integer ; Value : cardinal ) ;

begin
    if( Position < 0 ) then
    begin
        exit ;
    end ;
    setlength( Contents, system.length( Contents ) + 1 ) ;
    move( Contents[ Position ], Contents[ Position + 1 ], ( System.length( Contents ) - Position ) * sizeof( cardinal ) ) ;
     Contents[ Position ] := Value ;
end ;


procedure TUOS_String.Lowercase ;

var Dummy, V : integer ;
    _Folding_Index : integer ;

begin
    Dummy := 0 ;
    while( Dummy < Length ) do
    begin
        V := lowcase( Contents[ Dummy ], _Folding_Index ) ;
        if( ( V = 0 ) and ( _Folding_Index >= 0 ) ) then
        begin
            Contents[ Dummy ] := Foldings[ _Folding_Index, 1 ] ;
            for V := 2 to 3 do
            begin
                if( Foldings[ _Folding_Index, V ] <> 0 ) then
                begin
                    Insert( Dummy + 1, Foldings[ _Folding_Index, V ] ) ;
                    inc( Dummy ) ;
                end ;
            end ;
        end else
        begin
            Contents[ Dummy ] := V ;
        end ;
        inc( Dummy ) ;
    end ;
end ; // TUOS_String.Lowercase


function TUOS_String.Pos( const Value : TUOS_String ;
    Start : integer = 1 ) : integer ;

var Dummy, Dummy1 : integer ;
    Found : boolean ;

begin
    Result := 0 ;
    if( Start > Length ) then
    begin
        exit ;
    end ;
    if( Value.Length > Length - Start + 1 ) then
    begin
        exit ; // Substring is longer than our contents
    end ;
    for Dummy := Start to Length - Value.Length + 1 do
    begin
        Found := True ;
        for Dummy1 := 1 to Value.Length do
        begin
            if( Value.Contents[ Dummy1 ] <> Contents[ Dummy1 + Dummy - 1 ] ) then
            begin
                Found := False ;
                break ;
            end ;
        end ; // for Dummy1
        if( Found ) then
        begin
            Result := Dummy ;
            exit ;
        end ;
    end ; // for Dummy
end ; // TUOS_String.Pos


function TUOS_String.Pos( const Value : string ;
    Start : integer = 1 ) : integer ;

var Dummy, Dummy1 : integer ;
    Found : boolean ;

begin
    Result := 0 ;
    if( Start > Length ) then
    begin
        exit ;
    end ;
    if( System.Length( Value ) > Length - Start + 1 ) then
    begin
        exit ; // Substring is longer than our contents
    end ;
    for Dummy := Start - 1 to Length - system.length( Value ) do
    begin
        Found := True ;
        for Dummy1 := 1 to system.length( Value ) do
        begin
            if( ord( Value[ Dummy1 ] ) <> Contents[ Dummy1 + Dummy - 1 ] ) then
            begin
                Found := False ;
                break ;
            end ;
        end ; // for Dummy1
        if( Found ) then
        begin
            Result := Dummy + 1 ;
            exit ;
        end ;
    end ; // for Dummy
end ; // TUOS_String.Pos


function TUOS_String.RPos( Value : char ) : integer ;

var Loop, V : cardinal ;

begin
    V := ord( Value ) ;
    for Loop := Length - 1 downto 0 do
    begin
        if( Contents[ Loop ] = V ) then
        begin
            Result := Loop + 1 ;
            exit ;
        end ;
    end ;
    Result := 0 ;
end ;



{$IFDEF CC_Regression_Test}
procedure Error ;

begin
    halt ;
end ;


procedure Test_TUOS_String ;

var S : string[ 255 ] ;
    X, Y : TUOS_String ;

begin
    X := TUOS_String.Create ;
    if( X.As_String <> '' ) then
    begin
        Error ;
    end ;
    if( X.Length <> 0 ) then
    begin
        Error ;
    end ;
    X.Assign_From_String( 'abcdef', 1 ) ;
    if( X.As_String <> 'abcdef' ) then
    begin
        Error ;
    end ;
    if( X.Length <> 6 ) then
    begin
        Error ;
    end ;
    if( X.Pos( 'abcdef' ) <> 1 ) then
    begin
        Error ;
    end ;
    X.Length := 3 ;
    if( X.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( X.As_String <> 'abc' ) then
    begin
        Error ;
    end ;
    X.Assign_From_String( 'ABCDEF!b', 1 ) ;
    X.Lowercase ;
    if( X.As_String <> 'abcdef!b' ) then
    begin
        Error ;
    end ;
    X.Insert( -1, 33 ) ;
    if( X.As_String <> 'abcdef!b' ) then
    begin
        Error ;
    end ;
    Y := X.Copy( 2, 3 ) ;
    if( Y.As_String <> 'bcd' ) then
    begin
        Error ;
    end ;
    if( X.Equal( Y ) ) then
    begin
        Error ;
    end ;
    Y.Assign_From_String( X.As_String, 1 ) ;
    if( not X.Equal( Y ) ) then
    begin
        Error ;
    end ;
    if( X.RPos( 'X' ) <> 0 ) then
    begin
        Error ;
    end ;
    if( X.RPos( 'b' ) <> 8 ) then
    begin
        Error ;
    end ;
    if( X.Pos( 'X' ) <> 0 ) then
    begin
        Error ;
    end ;
    if( X.Pos( 'b' ) <> 2 ) then
    begin
        Error ;
    end ;
    if( X.Pos( 'b', 10 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( X.Pos( 'bbbbbbbbbbb' ) <> 0 ) then
    begin
        Error ;
    end ;
    if( X.Pos( X ) <> 1 ) then
    begin
        Error ;
    end ;
    Y.Assign_From_String( 'a?cde?!b', 1 ) ;
    Y.Assign_From_String( 'a?cde?!x', 1 ) ;
    Y.Free ;
    Y := X.Copy( 100, 2 ) ;
    if( Y.Length <> 0 ) then
    begin
        Error ;
    end ;
    Y.Free ;
    Y := X.Copy( -1, 100 ) ;
    if( Y.As_String <> 'abcdef!b' ) then
    begin
        Error ;
    end ;
    S := #1#0#0#0#$DF#0#0#0 ;
    X.Assign_From_String( S, 4 ) ;
    if( X.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 0 ] <> 1 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 1 ] <> $DF ) then
    begin
        Error ;
    end ;
    X.Lowercase ;
    if( X.Length <> 3 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 0 ] <> 1 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 1 ] <> $73 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 2 ] <> $73 ) then
    begin
        Error ;
    end ;
    S := #1#0#$DF#0 ;
    X.Assign_From_String( S, 2 ) ;
    if( X.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 0 ] <> 1 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 1 ] <> $DF ) then
    begin
        Error ;
    end ;
    S := #$90#3#1#0 ;
    X.Assign_From_String( S, 2 ) ;
    if( X.Length <> 2 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 0 ] <> $390 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 1 ] <> 1 ) then
    begin
        Error ;
    end ;
    X.Lowercase ;
    if( X.Length <> 4 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 0 ] <> $3B9 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 1 ] <> $308 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 2 ] <> $301 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 3 ] <> 1 ) then
    begin
        Error ;
    end ;
    S := X.As_String ;
    if( S <> #185#136#129#1 ) then
    begin
        Error ;
    end ;
    Y.Length := X.Length ;
    if( X.Equal( Y ) ) then
    begin
        Error ;
    end ;
    if( X.Pos( Y, 100 ) <> 0 ) then
    begin
        Error ;
    end ;
    Y.Assign_From_String( '1234567890', 1 ) ;
    if( X.Pos( Y, 1 ) <> 0 ) then
    begin
        Error ;
    end ;
    Y.Length := 3 ;
    if( X.Pos( Y, 1 ) <> 0 ) then
    begin
        Error ;
    end ;
    if( X.Pos( #1 ) <> 4 ) then
    begin
        Error ;
    end ;
    S := ' ' + #$C1#33 + #$E1#1#34 + #$F1#1#1#35 + #$F9#1#1#1#36 + #$FD#1#1#1#1#37 ;
    X.Assign_From_String( S, 3 ) ;
    if( X.Length <> 6 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 0 ] <> 32 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 1 ] <> 33 or $40 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 2 ] <> 34 or $1040 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 3 ] <> 35 or $41040 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 4 ] <> 36 or $1041040 ) then
    begin
        Error ;
    end ;
    if( X.Contents[ 5 ] <> 37 or $41041040 ) then
    begin
        Error ;
    end ;
    X.Free ;
    Y.Free ;
end ;



(*
var Loop : integer ;
    _NS, _US, NS, US : cardinal ;
*)

procedure Test_Unit ;

begin
    Test_TUOS_String ;

(*
_NS := gettickcount ;
for Loop := 1 to 100000 do
begin
  Test_Native_String ;
end ;
NS := gettickcount - _NS ;
_US := gettickcount ;
for Loop := 1 to 100000 do
begin
  Test_Wildcard ;
end ;
US := gettickcount - _US ;
showmessage(inttostr(NS));
showmessage(inttostr(US));
*)
end ;


begin
    Test_Unit ;
{$ENDIF}
end.

