{
        Program Name : TCommon_COM_Interface
        Package Name : Subroutine Library - Layer 0
        Purpose      : Abstract virtual common COM interface class
        Institution  : Conroy & Conroy Co.
        Date Written : 21-March-2002
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2002-2015 by Alan Conroy.  Released to the Public Domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TBase_COM_Interface is an abstract base class definition for COM
        objects used in the Application Foundation.
}

unit _COMInter ;

{$I EDEFINES.INC}

interface

{$IFDEF DYNAMIC_OBJECTS}
  {$IFDEF STDCALL_SUPPORTED}

uses { Subroutine Library - layer 0... }
{$IFNDEF MOBILE}
     _DebugIn, { TDebug_Interface }
{$ENDIF}
     _Types, { PAnsiChar }
     _UE ; { TUnified_Exception }

type TBase_COM_Interface = class
                               public { API... }
                                   { Initialize the object.  Returns result of
                                     initialization attempt. }
                                   function Initialize : TUnified_Exception ;
                                       virtual ; stdcall ; abstract ;

                                   { Terminate the use of the object.  Returns
                                     result of termination.  This generally
                                     should not be used - instead, use the
                                     Detach method. }
                                   function Terminate : TUnified_Exception ;
                                       virtual ; stdcall ; abstract ;

                                   { Increments the reference count for the
                                     object. }
                                   procedure Attach ; virtual ;
                                       stdcall ; abstract ;

                                   { Decrements the reference count for the
                                     object.  When count reaches 0, the object
                                     is destroyed.  Returns true if object is
                                     destroyed, false otherwise. }
                                   function Detach : boolean ; virtual ;
                                       stdcall ; abstract ;

                                   { Returns True if this class is the passed
                                     class name. }
                                   function Is_Class( Name : PAnsiChar ) : boolean ;
                                       virtual ; stdcall ; abstract ;

                                   { Version of the COM interface for this
                                     object. }
                                   function Interface_Version : longint ;
                                       virtual ; stdcall ; abstract ;

                                   { Facility ID for the facility represented
                                     by this object.  Returns -1 if no
                                     facility assigned. }
                                   function Facility_ID : longint ; virtual ;
                                       stdcall ; abstract ;

                                   { Facility version.  Result only has
                                     meaning if Facility_ID doesn't return
                                     -1. }
                                   function Facility_Version : longint ;
                                       virtual ; stdcall ; abstract ;

    {$IFNDEF MOBILE}
                                   { Returns a debugging interface for the
                                     object.  Returns nil if not supported. }
                                   function Debugger : TDebug_Interface ;
                                       virtual ; stdcall ; abstract ;
    {$ENDIF}

                                   { Returns last error. }
                                   function Last_Error : TUnified_Exception ;
                                       virtual ; stdcall ; abstract ;

                                   { Returns a pointer to an object with
                                     extended common COM methods for object.
                                     Always returns nil for now. }
                                   function Extension : pointer ; virtual ;
                                       stdcall ; abstract ;
                           end ; { TBase_COM_Interface }
  {$ENDIF}
{$ENDIF}

implementation


end.
