{
        Program Name : _DebugIn
        Package Name : Subroutine Library - Layer 0
        Purpose      : Debugger Interface definitions
        Institution  : Conroy & Conroy Co.
        Date Written : 11-Jan-2001
        Written By   : Alan Conroy
        Version      : 1.0

	    Copyright (C) 2001-2005 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TDebug_Interface is used when debugging code by providing a way of
        examining the internals of the software component at run-time (for
        production versions of software, this is often not supported).  The
        interface assumes a hierarchical form with the root of a component
        returning children representing internal objects.  Each of those objects
        may have children, and so forth.
}

unit _DebugIn ;

interface

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

{$I EDEFINES.INC}

uses { Subroutine Library... }
     _Common, { TObject }
     Compatib ; { PAnsiChar }

{$IFDEF DYNAMIC_OBJECTS}
type TCOM_Debug_Interface = class
                                public { API... }
                                    function Count : longint ; virtual ;
                                        stdcall ; abstract ;
                                    { Returns the number of child components under
                                      this level.  If this is a terminal (leaf)
                                      node, the method returns 0. }

                                    function Get_Title : PAnsiChar ; virtual ;
                                        stdcall ; abstract ;
                                    { Returns text representing this level (such as
                                      "Cached data"). }

                                    function Has_Children : boolean ; virtual ;
                                        stdcall ; abstract ;
                                    { Returns whether or not there are child components under
                                      this level.  If this is a terminal (leaf)
                                      node, the method returns false. }

                                    procedure Set_Title( Value : PAnsiChar ) ;
                                        virtual ;
                                        stdcall ; abstract ;
                                    { Sets text representing this level. }

                                    procedure Activate ; virtual ;
                                        stdcall ; abstract ;
                                    { Indicates that the user wished to activate
                                      this level.  The behavior is undefined and up
                                      to the implementor.  Typically, this does
                                      nothing unless this level is a terminal
                                      representing some complex data which is not
                                      viewable in a hierarchical form. }

                                    function Can_Change : boolean ; virtual ;
                                        stdcall ; abstract ;
                                    { Returns True if this item can be modified. }

                                    function Child( Index : longint ) : TCOM_Debug_Interface ;
                                        virtual ;
                                        stdcall ; abstract ;
                                    { Returns a debug interface for the specified
                                      child level (first child level is 0). }

                                    procedure Kill ; virtual ;
                                        stdcall ; abstract ;
                                    { Tells object to destroy itself }

                                    function Modify( Data : PAnsiChar ; Size : integer ) : PAnsiChar ;
                                        virtual ;
                                        stdcall ; abstract ;
                                    { Modifies data.  If Size is -1, Data is assumed
                                      to be null-terminated and its length is
                                      automatically determined.  The result will be
                                      nil or a pointer to a null string to indicate
                                      success and error text otherwise. }

                                    function Get_Context : longint ; virtual ;
                                        stdcall ; abstract ;

                                    procedure Set_Context( Value : longint ) ;
                                        virtual ;
                                        stdcall ; abstract ;

                                    function Recursive : boolean ; virtual ;
                                        stdcall ; abstract ;

                                    function Arbitrary_Length : boolean ;
                                        virtual ;
                                        stdcall ; abstract ;

                                    property Context : longint
                                                 read Get_Context
                                                 write Set_Context ;

                                    property Title : PAnsiChar
                                                read Get_Title
                                                write Set_Title ;
                            end ;

type TDebug_Interface = TCOM_Debug_Interface ;
     PDebug_Interface = TDebug_Interface ;
{$ENDIF}

type PSDebug_Interface = ^TSDebug_Interface ;
     TSDebug_Interface = object( TObject )
                            public { API... }
                                function Count : longint ; virtual ;
                                { Returns the number of child components under
                                  this level.  If this is a terminal (leaf)
                                  node, the method returns 0. }

                                function Get_Title : PAnsiChar ; virtual ;
      	                        { Returns text representing this level (such as
                                  "Cached data"). }

                                function Has_Children : boolean ; virtual ;
                                { Returns whether or not there are child
                                  components under this level.  If this is a
                                  terminal (leaf) node, the method returns
                                  false. }

                                procedure Set_Title( Value : PAnsiChar ) ;
                                    virtual ;
      	                        { Sets text representing this level. }

                                procedure Activate ; virtual ;
      	                        { Indicates that the user wished to activate
                                  this level.  The behavior is undefined and up
                                  to the implementor.  Typically, this does
                                  nothing unless this level is a terminal
                                  representing some complex data which is not
                                  viewable in a hierarchical form. }

                                function Can_Change : boolean ; virtual ;
                                { Returns True if this item can be modified. }

                                function Child( Ordinal : longint ) : PSDebug_Interface ;
                                    virtual ;
      	                        { Returns a debug interface for the specified
                                  child level (first child level is 0). }

                                procedure Kill ; virtual ;
                                { Tells object to destroy itself }

                        	    function Modify( Data : PAnsiChar ; Size : integer ) : PAnsiChar ;
                                    virtual ;
                                { Modifies data.  If Size is -1, Data is assumed
                                  to be null-terminated and its length is
                                  automatically determined.  The result will be
                                  nil or a pointer to a null string to indicate
                                  success and error text otherwise. }

                                function Get_Context : longint ; virtual ;

                                procedure Set_Context( Value : longint ) ;
                                    virtual ;

                                function Recursive : boolean ; virtual ;

                                function Arbitrary_Length : boolean ;
                                    virtual ;

{$IFDEF STATIC_PROPERTIES}
                                property Title : PAnsiChar
                                    read Get_Title
                                    write Set_Title ;
{$ENDIF}
                        end ; { TSDebug_Interface }

implementation

{ TDebug_Interface methods... }

function TSDebug_Interface.Has_Children : boolean ;

begin
    Has_Children := ( Count <> 0 ) ;
    Abstract_Error ;
end ;


{ API... }

function TSDebug_Interface.Count : longint ;

begin
    Count := 0 ;
    Abstract_Error ;
end ;


function TSDebug_Interface.Get_Title : PAnsichar ;

begin
    Get_Title := nil ;
    Abstract_Error ;
end ;


procedure TSDebug_Interface.Set_Title( Value : PAnsichar ) ;

begin
    Abstract_Error ;
end ;


procedure TSDebug_Interface.Activate ;

begin
    Abstract_Error ;
end ;


function TSDebug_Interface.Can_Change : boolean ;

begin
    Can_Change := False ;
    Abstract_Error ;
end ;


function TSDebug_Interface.Child( Ordinal : longint ) : PSDebug_Interface ;

begin
    Child := nil ;
    Abstract_Error ;
end ;


procedure TSDebug_Interface.Kill ;

begin
    Abstract_Error ;
end ;


function TSDebug_Interface.Modify( Data : PAnsichar ; Size : integer ) : PAnsichar ;

begin
    Modify := nil ;
    Abstract_Error ;
end ;


function TSDebug_Interface.Get_Context : longint ;

begin
    Get_Context := 0 ;
    Abstract_Error ;
end ;


procedure TSDebug_Interface.Set_Context( Value : longint ) ;

begin
    Abstract_Error ;
end ;


function TSDebug_Interface.Recursive : boolean ;

begin
    Recursive := False ;
    Abstract_Error ;
end ;


function TSDebug_Interface.Arbitrary_Length : boolean ;

begin
    Arbitrary_Length := False ;
    Abstract_Error ;
end ;


end.
