/*
        Program Name : _DevMan
        Package Name : LB
        Purpose      : Device Manager class definitions
        Institution  : Conroy & Conroy Co.
        Date Written : 7-Jul-2006
        Written By   : Alan Conroy
        Version      : 1.1

        Copyright (C) 2006-2016 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TDevice_Manager is a class for managing access to devices for
        Operating system objects.
*/

#ifndef _DEVMANH
	#define _DEVMANH

	#include "eDefines.h"
	#include "cominter.h"

// Device Media IDs:
const int DMI_Unknown = -1 ; // Unknown media type
const int DMI_Remote = 4 ; // Remote (network) drive.
const int DMI_CDROM = 5 ; // CD or DVD.
const int DMI_RAM = 6 ; // RAM disk.
const int DMI_Removable = 0xF0 ; // 3.5" DS, 18 sectors (or other removable)
const int DMI_Fixed = 0xF8 ; // Fixed disk
const int DMI_Floppy1 = 0xF9 ; // 5.25", DS, 15 sectors or 3.5" DS, 9 sectors
const int DMI_Floppy2 = 0xFC ; // 5.25" SS, 9 sectors
const int DMI_Floppy3 = 0xFD ; // 5.25" DS, 9 sectors
const int DMI_Floppy4 = 0xFE ; // 5.25" SS, 8 sectors
const int DMI_Floppy5 = 0xFF ; // 5.25" DS, 8 sectors

// Device File System Flags:
const int DFSF_Case_Sensitive = 1 ; // Specifies that searches are case-sensitive.
const int DFSF_Preserve_Case = 2 ; // Preserves case in directory entries.
const int DFSF_UNICODE = 4 ; // Uses Unicode characters in file and directory names.
const int DFSF_Long_Filenames = 0x4000 ; // Supports new long filename functions.
const int DFSF_Volume_Compressed = 0x8000 ; // The volume is compressed.

// Device class:
const int DC_Unknown = 0 ;
const int DC_Disk = 1 ;
const int DC_Tape = 2 ;
const int DC_Serial = 3 ; // Includes NUL
const int DC_Network = 4 ;
const int DC_System = 5 ; // Includes clock
const int DC_Printer = 6 ;
const int DC_Optical = 7 ; // CD, DVD, etc.
const int DC_Other = 8 ; // Everything else

class TCOM_Device_Info : public TCommon_COM_Interface
{
    public: // API...
        virtual int __stdcall Media_ID() = 0;
        // Return type of media in/on device.  -1 if error or unknown.

        virtual char* __stdcall File_System() = 0;
        // Returns the name of the file system used on the device.  Returns a null string if the device is invalid or the file system is unknown.

        virtual int __stdcall File_System_Flags() = 0;
        // Returns the flags for the file system.  See DFSF_*.

        virtual bool __stdcall Fixed() = 0;
        // Returns true if the passed device is fixed (non-removable)

        virtual int __stdcall Max_Filename_Length() = 0;
        // Returns the maximum filename length for this device's file system.

        virtual int __stdcall Version() = 0;
        // Returns version number of this object.  11 = 1.1
} ;

class TCOM_Device_Manager : public TCommon_COM_Interface
{
    public: // API...
      virtual TCOM_Device_Info* __stdcall Device_Info( char* Device ) = 0;
      // Returns an information object for the requested device.

      virtual int64 __stdcall Disk_Clusters( char* Drive ) = 0;
      // Returns number of clusters on specified disk drive.  Returns -1 if an error or unknown.

      virtual int __stdcall Disk_Sector_Size( char* Drive ) = 0;
      // Returns size of disk sectors, in bytes. Returns -1 if an error or unknown.

      virtual char* __stdcall File_System( char* Device ) = 0;
      // Returns the name of the file system used on the device.  Returns a null string if the device is invalid or the file system is unknown.

      virtual int __stdcall File_System_Flags( char* Device ) = 0;
      // Returns the flags for the file system.  See DFSF_*.

      virtual bool __stdcall Fixed_Device( char* Device ) = 0;
      // Returns true if the passed device is fixed (non-removable)

      virtual int __stdcall Max_Filename_Length( char* Device ) = 0;
      // Returns the maximum filename length for this device's file system.

      virtual int __stdcall Media_ID( char* Device ) = 0;
      // Return type of media in/on device.  -1 if error or unknown.

      virtual bool __stdcall Valid_Device( char* Device ) = 0;
      // Returns true if the passed device is valid.

      virtual char* __stdcall Device_Name( int Clas, int Index ) = 0;
      // Return name of Indexth device of given class. if Class is unknown, all device classes are assumed.

      virtual char* __stdcall Current_Path( char* Device ) = 0;
      // Returns the current path (or default path if no current path) for the specified device. Returns null if device is not a hierarchical file system device.

      virtual int __stdcall Version() = 0;
	  // Returns version number of this object.  11 = 1.1

	  virtual bool __stdcall Media_Present( char* Device ) = 0;
	  // Returns true if media is present in the specified drive.
} ; // TCOM_Device_Manager
#endif
