{$IFNDEF FPC}
  {$E+}
  {$N+}
{$ENDIF}
{
        Program Name : _OS
        Package Name : Subroutine Library - Layer 1
        Purpose      : Operating system interface
        Institution  : Conroy & Conroy Co.
        Date Written : 6-Nov-2002
        Written By   : Alan Conroy
        Version      : 1.1

        Copyright (C) 2002-2018 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TOperating_System is an abstract Operating System interface.
}

unit _OS ;

interface

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

{$I EDEFINES.INC}

uses { Layer 0... }
     CommBase, { TBase_Object }
     ComInter, { TCommon_COM_Interface }
{$IFNDEF MOBILE}
     _DebugIn, { PDebug_Interface }
{$ENDIF}
     _Types, { int64 }
     _UE, { PSUnified_Exception }

     { Layer 1... }
     _TPM, { TSText_Presentation_Manager}
     _DevMan, { TSDevice_Manager }
     _FiP ; { PSFile_Processor }

const OS_Object_Version = 12; { Version 1.2}

      { Spawn flags... }
      Spawn_Quiet = 1 ;
      Spawn_Asynchronous = 2 ;

      { Application_Data_Path flags... }
      ADPF_Local = 0 ; { Return a data path specific to the current user }
      ADPF_Public = 1 ; { Return a public data path }
      ADPF_Create = 2 ; { Create path if it doesn't exist }

      { Message box option flags... }
      MBOF_OK = 0 ;
      MBOF_OKCancel = 1 ;
      MBOF_AbortRetryIgnore = 2 ;
      MBOF_YesNoCancel = 3 ;
      MBOF_YesNo = 4 ;
      MBOF_RetryCancel = 5 ;

      { Message box return values... }
      MBR_Abort = 1 ;
      MBR_Cancel = 2 ;
      MBR_No = 3 ;
      MBR_OK = 4 ;
      MBR_Retry = 5 ;
      MBR_Yes = 6 ;
      MBR_Ignore = 7 ;

{$IFDEF DYNAMIC_OBJECTS}
type TOperating_System = class( TBase_Delphi_Object )
                             public { API... }
                                 { Identification... }
                                 function Name : Ansistring ; virtual ; abstract ; { O/S name }
                                 function Version : Ansistring ;
                                     virtual ; abstract ; { O/S version }


                                 { Date/Time support... }
                                 procedure Get_Date( var Year, Month, Day : word ) ;
                                     virtual ; abstract ;
                                 procedure Get_Time( var Hour, Minute, Second, Hundreths, NS : word ) ;
                                     virtual ; abstract ;
                                 function Timestamp : int64 ;
                                     virtual ; abstract ;
                                 { Current date/time in Sirius format }


                                 { Process support... }
                                 function PID : longint ; virtual ; abstract ;
                                 procedure Reschedule ; virtual ; abstract ;
                                 { Reschedules process so other tasks can get
                                   some runtime. }

                                 function Spawn( const Path, Parameters, Default_Dir : Ansistring ;
                                     Flags : integer ) : boolean ;
                                     virtual ; abstract ; { Spawns a process.  See Spawn_* flags }
                                 function Thread_ID : longint ;
                                     virtual ; abstract ; { Current Thread ID }
                                 function Logical_CPUs : integer ;
                                     virtual ; abstract ;
                                 { Number of logical CPUs on this sytem. }

                                 function User_Name : Ansistring ;
                                     virtual ; abstract ;

                                 { Device support... }
                                 function Device_Manager : TDevice_Manager ;
                                     virtual ; abstract ;


                                 { I/O... }
                                 procedure Beep( Typ : integer ) ;
                                     virtual ; abstract ;
                                 { Beep the user. }

                                 procedure Cancel_Typeahead( Handle : integer ) ;
                                    virtual ; abstract ;
                                 { Cancel any pending keyboard input }

                                 function Get_Char( Wait : boolean ;
                                     var Valid : boolean ;
                                     Handle : integer ) : Ansichar ;
                                     virtual ; abstract ;

                                 function Pending_Input( Handle : integer ) : boolean ;
                                     virtual ; abstract ;
                                 { Returns true if any user input is pending }

                                 procedure Print( T : Ansistring ) ;
                                     virtual ; abstract ;
                                 { Print the text on the default printer }


                                 { Error processing... }
                                 function Error_Text( Code : longint ;
                                     const Language : Ansistring ) : Ansistring ;
                                     virtual ; abstract ;


                                 { Files and pathing... }
                                 function Application_Path : Ansistring ;
                                     virtual ; abstract ;
                                 { Returns application path with terminating
                                   delimiter. }

                                 function File_List_Delimiter : Ansistring ;
                                     virtual ; abstract ;
                                 { Delimiter used to separate files in a list. }

                                 function Get_Temp_Path : Ansistring ;
                                     virtual ; abstract ;
                                 { Returns path(s) for temporary files.  If
                                   there are multiple paths, they are delimited
                                   by the File_List_Delimiter. }

                                 function Swap_File_List : Ansistring ;
                                     virtual ; abstract ;
                                 { Returns list of swap files.  Returns Null if
                                   no swap files.  If more than one, they are
                                   delimited by the File_List_Delimiter. }

                                 function System_Path : Ansistring ;
                                     virtual ; abstract ;
                                 { Path(s) to the system directory.  Returns
                                   null string if not supported. }

                                 function Text_Presentation_Manager : TText_Presentation_Manager ;
                                     virtual ; abstract ;
                                 { Returns the Text_Presentation_Manager
                                   instance. }

                                 { Misc... }
                                 function Command_Line : Ansistring ;
                                     virtual ; abstract ;
                                 { Returns command line used to start program. }

{$IFNDEF MOBILE}
                                 function Debugger : TCOM_Debug_Interface ;
                                     override ; abstract ;
                                 { Returns a debugger for the operating system }
{$ENDIF}

                                 function Get_Symbol( const X : Ansistring ) : Ansistring ;
                                     virtual ; abstract ;
                                 { Return contents of OS/Shell symbol whose name
                                   is in X.  Returns null if not defined. }

                                 function Switch_Delimiter_Count : integer ;
                                     virtual ; abstract ;
                                 { Returns number of switch delimiters. }

                                 function Switch_Delimiters( Index : integer ) : Ansistring ;
                                     virtual ; abstract ;
                                 { Returns switch delimiter for Index. }

                                 function Get_Clipboard( Path : Ansistring ) : Ansistring ;
                                     virtual ; abstract ;
                                 { Get clipboard contents in path. }

                                 procedure Set_Clipboard( Path, Value : Ansistring ) ;
                                     virtual ; abstract ;
                                 { Get clipboard contents for path. }

                                 function Max_File_Size : int64 ;
                                     virtual ; abstract ;
                                 { Return the largest that a file could be on a
                                   device with infinite space, using the file
                                   system with the smallest max size files
                                   supported by the O/S. }

                                 function Object_Version : longint ;
                                     virtual ; abstract ;
                                 { Returns version number of this object.  11 = 1.1 }

                                 function File_Processor : TFile_Processor ;
                                     virtual ; abstract ;

                                 function Load_Library( Nam : Ansistring ) : longint ;
                                     virtual ; abstract ;
                                 { Load a library and return a handle to it. }

                                 function Get_Symbol_Address( Lib : longint ;
                                     Nam : Ansistring ) : int64 ;
                                     virtual ; abstract ;
                                 { Retrieve address of a symbol in a library }

                                 function Show_Message( Handle : longint ;
                                     Msg, Title : PAnsiChar ; Options : integer ) : integer ;
                                     virtual ; abstract ;
                                 { Prompt user and return response. }

                                 function Get_Country : Ansistring ;
                                     virtual ; abstract ;

                                 { Memory mapping... }
                                 function Max_Memory_Map : int64 ;
                                     virtual ; abstract ;
                                 { Maximum theoretical amount of memory
                                       mapping space, in bytes. }

                                 function Memory_Map_File( _Name : Ansistring ;
                                     Writeable : boolean ;
                                     Start, Size : int64 ;
                                     Flag : boolean ) : integer ;
                                     virtual ; abstract ;
                                 { Map a file into memory }

                                 procedure Unmap_File_From_Memory( Handle : integer ) ;
                                     virtual ; abstract ;

                                 function Memory_Mapped_File_Base( Handle : integer ) : pointer ;
                                     virtual ; abstract ;

                                 function Memory_Mapped_File_Size( Handle : integer ) : int64 ;
                                     virtual ; abstract ;

                                 { Memory... }
                                 function Total_Physical_Memory : int64 ;
                                     virtual ; abstract ;

                                 function Available_Physical_Memory : int64 ;
                                     virtual ; abstract ;

                                 function Total_Virtual_Memory : int64 ;
                                     virtual ; abstract ;

                                 function Available_Virtual_Memory : int64 ;
                                     virtual ; abstract ;


                                 { Additional... }
                                 function Application_Data_Path( Default_Root, Path : Ansistring ;
                                     Flags : integer ) : Ansistring ;
                                     virtual ; abstract ;

                                 function Create_Path( Path : Ansistring ) : integer ;
                                     virtual ; abstract ;

                                 function Shared_Library_Extension : Ansistring ;
                                     virtual ; abstract ;

                                 function Current_Directory : Ansistring ;
                                     virtual ; abstract ;
                                 { Returns current path }

                                 function Path_Separator : Ansistring ;
                                     virtual ; abstract ;

                                 function Hardware_Name : Ansistring ;
                                     virtual ; abstract ;
                         end ; { TOperating_System }

type TCOM_Operating_System = class( TCommon_COM_Interface )
                                 public { API... }
                                     { Identification... }
                                     function Name : PAnsiChar ;
                                         virtual ; stdcall ; abstract ; { O/S name }
                                     function Version : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { O/S version }


                                     { Date/Time support... }
                                     procedure Get_Date( var Year, Month, Day : word ) ;
                                         virtual ; stdcall ; abstract ;
                                     procedure Get_Time( var Hour, Minute, Second, Hundreths, NS : word ) ;
                                         virtual ; stdcall ; abstract ;
                                     function Timestamp : int64 ;
                                         virtual ; stdcall ; abstract ;
                                     { Current date/time in Sirius format }


                                     { Process support... }
                                     function PID : longint ;
                                         virtual ; stdcall ; abstract ;
                                     procedure Reschedule ;
                                         virtual ; stdcall ; abstract ;
                                     { Reschedules process so other tasks can
                                       get some runtime. }

                                     function Spawn( Path, Parameters, Default_Dir : PAnsiChar ;
                                         Flags : integer ) : boolean ;
                                         virtual ; stdcall ; abstract ; { Spawns a process. }
                                     function Thread_ID : longint ;
                                         virtual ; stdcall ; abstract ; { Get current thread ID }
                                     function Logical_CPUs : integer ;
                                         virtual ; stdcall ; abstract ; 
                                     { Number of logical CPUs on this sytem. }

                                     function User_Name : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;


                                     { Device support... }
                                     function Device_Manager : TCOM_Device_Manager ;
                                         virtual ; stdcall ; abstract ;


                                     { I/O... }
                                     procedure Beep( Typ : integer ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Beep the user. }

                                     procedure Cancel_Typeahead( Handle : integer ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Cancel any pending keyboard input }

                                     function Get_Char( Wait : boolean ;
                                         var Valid : boolean ;
                                         Handle : integer ) : Ansichar ;
                                         virtual ; stdcall ; abstract ;

                                     function Pending_Input( Handle : integer ) : boolean ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns true if any user input is pending }

                                     procedure Print( T : PAnsiChar ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Print the text on the default printer }


                                     { Error processing... }
                                     function Error_Text( Code : longint ;
                                         Language : PAnsiChar ) : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;


                                     { Misc... }
                                     function Application_Path : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns application path with terminating
                                       delimiter }

                                     function Command_Line : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns command line used to start program. }

{$IFNDEF MOBILE}
                                     function Debugger : TCOM_Debug_Interface ;
                                         override ; stdcall ; abstract ;
                                     { Returns a debugger for the operating system }
{$ENDIF}

                                     function Get_Symbol( X : PAnsiChar ) : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Return contents of OS/Shell symbol.  Returns
                                       null if not defined. }

                                     function Switch_Delimiter_Count : integer ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns number of switch delimiters. }

                                     function Switch_Delimiters( Index : integer ) : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns switch delimiter for Index. }

                                     function File_List_Delimiter : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Delimiter used to separate files in a list. }

                                     function Get_Temp_Path : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns path(s) for temporary files.  If
                                       there are multiple paths, they are delimited
                                       by the File_List_Delimiter. }

                                     function System_Path : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Path(s) to the system directory.  Returns
                                       null string if not supported. }

                                     function Text_Presentation_Manager : TCOM_Text_Presentation_Manager ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns the Text_Presentation_Manager
                                       instance. }

                                     function Swap_File_List : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns list of swap files.  Returns Null if
                                       no swap files. }

                                     function Get_Clipboard( Path : PAnsiChar ) : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;
                                     { Get clipboard contents in path. }

                                     procedure Set_Clipboard( Path, Value : PAnsiChar ) ;
                                         virtual ; stdcall ; abstract ;
                                     { Get clipboard contents for path. }

                                     function Max_File_Size : int64 ;
                                         virtual ; stdcall ; abstract ;
                                     { Return the largest that a file could be
                                       on a device with infinite space, using
                                       the file system with the smallest max
                                       size files supported by the O/S. }

                                     function Object_Version : longint ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns version number of this object.  11 = 1.1 }

                                     function File_Processor : TCOM_File_Processor ;
                                         virtual ; stdcall ; abstract ;
                                     { Returns file processor }

                                     function Load_Library( Nam : PAnsiChar ) : longint ;
                                         virtual ; stdcall ; abstract ;
                                     { Load a library and return a handle to it. }

                                     function Get_Symbol_Address( Lib : longint ;
                                         Nam : PAnsiChar ) : int64 ;
                                         virtual ; stdcall ; abstract ;
                                     { Retrieve address of a symbol in a library }

                                     function Show_Message( Handle : longint ;
                                         Msg, Title : PAnsiChar ;
                                         Options : integer ) : integer ;
                                         virtual ; stdcall ; abstract ;
                                     { Prompt user and return response. }

                                     function Get_Country : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;

                                     { Memory-mapping... }
                                     function Max_Memory_Map : int64 ;
                                         virtual ; stdcall ; abstract ;
                                     { Maximum theoretical amount of memory
                                       mapping space, in bytes. }

                                     function Memory_Map_File( _Name : PAnsiChar ;
                                         Writeable : boolean ;
                                         Start, Size : int64 ;
                                         Flag : boolean ) : integer ;
                                         virtual ; stdcall ; abstract ;
                                     { Map a file into memory }

                                     procedure Unmap_File_From_Memory( Handle : integer ) ;
                                         virtual ; stdcall ; abstract ;

                                     function Memory_Mapped_File_Base( Handle : integer ) : pointer ;
                                         virtual ; stdcall ; abstract ;

                                     function Memory_Mapped_File_Size( Handle : integer ) : int64 ;
                                         virtual ; stdcall ; abstract ;

                                     { Memory... }
                                     function Total_Physical_Memory : int64 ;
                                         virtual ; stdcall ; abstract ;

                                     function Available_Physical_Memory : int64 ;
                                         virtual ; stdcall ; abstract ;

                                     function Total_Virtual_Memory : int64 ;
                                         virtual ; stdcall ; abstract ;

                                     function Available_Virtual_Memory : int64 ;
                                         virtual ; stdcall ; abstract ;

                                     { Additional... }
                                     function Application_Data_Path( Default_Root, Path : PAnsiChar ;
                                         Flags : integer ) : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;

                                     function Create_Path( Path : PAnsiChar ) : integer ;
                                         virtual ; stdcall ; abstract ;

                                     function Shared_Library_Extension : PAnsiChar ;
                                         virtual ; stdcall ; abstract ;

                                     function Current_Directory : PAnsiChar ;
                                         virtual ; abstract ;
                                     { Returns current path }

                                     function Path_Separator : PAnsiChar ;
                                         virtual ; abstract ;

                                     function Hardware_Name : PAnsiChar ;
                                         virtual ; abstract ;
                            end ; { TCOM_Operating_System }
{$ENDIF}

type TSOperating_System = object( TBase_Object )
                             public { API... }
                                 { Identification... }
                                 function Name : Ansistring ; virtual ; { O/S name }
                                 function Version : Ansistring ; virtual ;
                                 { O/S version }


                                 { Date/Time support... }
                                 procedure Get_Date( var Year, Month, Day : word ) ;
                                     virtual ;
                                 procedure Get_Time( var Hour, Minute, Second, Hundreths, NS : word ) ;
                                     virtual ;
                                 function Timestamp : int64 ; virtual ;
                                 { Current date/time in Sirius format }


                                 { Process support... }
                                 function PID : longint ; virtual ;
                                 procedure Reschedule ; virtual ;
                                 { Reschedules process so other tasks can get
                                   some runtime. }

                                 function Spawn( const Path, Parameters, Default_Dir : Ansistring ;
                                     Flags : integer ) : boolean ;
                                     virtual ; { Spawns a process. }
                                 function Thread_ID : longint ;
                                     virtual ; { Get current thread ID. }
                                 function Logical_CPUs : integer ; virtual ;
                                 { Number of logical CPUs on this sytem. }

                                 function User_Name : Ansistring ;
                                     virtual ;

                                 { Device support... }
                                 function Device_Manager : PSDevice_Manager ;
                                     virtual ;


                                 { I/O... }
                                 procedure Beep( Typ : integer ) ; virtual ;
                                 { Beep the user. }

                                 procedure Cancel_Typeahead( Handle : integer ) ;
                                     virtual ;
                                 { Cancel any pending keyboard input }

                                 function Get_Char( Wait : boolean ;
                                     var Valid : boolean ;
                                     Handle : integer ) : Ansichar ;
                                     virtual ;

                                 function Show_Message( Handle : longint ;
                                     Msg, Title : Ansistring ;
                                     Options : integer ) : integer ; virtual ;
                                 { Prompt user and return response. }

                                 function Pending_Input( Handle : integer ) : boolean ;
                                     virtual ;
                                 { Returns true if any user input is pending }

                                 procedure Print( T : Ansistring ) ; virtual ;
                                 { Print the text on the default printer }


                                 { File and File system... }
                                 function Application_Data_Path( Default_Root, Path : Ansistring ;
                                     Flags : integer ) : Ansistring ;
                                     virtual ;

                                 function Application_Path : Ansistring ; virtual ;
                                 { Returns application path with terminating
                                   delimiter }

                                 function Create_Path( Path : Ansistring ) : integer ;
                                     virtual ;

                                 function Current_Directory : Ansistring ;
                                     virtual ;

                                 function File_List_Delimiter : Ansistring ;
                                     virtual ;
                                 { Delimiter used to separate files in a list. }

                                 function File_Processor : PSFile_Processor ;
                                     virtual ;

                                 function Get_Temp_Path : Ansistring ; virtual ;
                                 { Returns path(s) for temporary files.  If
                                   there are multiple paths, they are delimited
                                   by the File_List_Delimiter. }

                                 function Max_File_Size : int64 ; virtual ;
                                 { Return the largest that a file could be on a
                                   device with infinite space, using the file
                                   system with the smallest max size files
                                   supported by the O/S. }

                                 function Normalize_Filename( S : Ansistring ) : Ansistring ;
                                     virtual ;

                                 function Path_Separator : Ansistring ;
                                     virtual ;

                                 function Shared_Library_Extension : Ansistring ;
                                     virtual ;

                                 function System_Path : Ansistring ;
                                     virtual ;
                                 { Path(s) to the system directory.  Returns
                                   null string if not supported. }


                                 { Error processing... }
                                 function Last_Error : PSUnified_Exception ;
                                     virtual ;

                                 function Error_Text( Code : longint ;
                                     const Language : Ansistring ) : Ansistring ;
                                     virtual ;


                                 { Misc... }
                                 function Command_Line : Ansistring ; virtual ;
                                 { Returns command line used to start program. }

{$IFNDEF MOBILE}
                                 function Debugger : PSDebug_Interface ;
                                     virtual ;
                                 { Returns a debugger for the operating system }
{$ENDIF}

                                 function Get_Symbol( const X : Ansistring ) : Ansistring ;
                                     virtual ;
                                 { Return contents of OS/Shell symbol.  Returns
                                   null if not defined. }

                                 function Switch_Delimiter_Count : integer ;
                                     virtual ;
                                 { Returns number of switch delimiters. }

                                 function Switch_Delimiters( Index : integer ) : Ansistring ;
                                     virtual ;
                                 { Returns switch delimiter for Index. }

                                 function Text_Presentation_Manager : PSText_Presentation_Manager ;
                                     virtual ;
                                 { Returns the Text_Presentation_Manager
                                   instance. }

                                 function Swap_File_List : Ansistring ; virtual ;
                                 { Returns list of swap files.  Returns Null if
                                   no swap files. }

                                 function Get_Clipboard( Path : Ansistring ) : Ansistring ;
                                     virtual ;
                                 { Get clipboard contents in path. }

                                 procedure Set_Clipboard( Path, Value : Ansistring ) ;
                                     virtual ;
                                 { Get clipboard contents for path. }

                                 function Object_Version : longint ; virtual ;
                                 { Returns version number of this object.  11 = 1.1 }

                                 function Load_Library( Nam : Ansistring ) : longint ;
                                     virtual ;
                                 { Load a library and return a handle to it. }

                                 function Get_Symbol_Address( Lib : longint ;
                                     Nam : Ansistring ) : int64 ; virtual ;
                                 { Retrieve address of a symbol in a library }

                                 function Get_Country : Ansistring ; virtual ;


                                 { Memory mapping... }
                                 function Max_Memory_Map : int64 ; virtual ;
                                 { Maximum theoretical amount of memory
                                       mapping space, in bytes. }

                                 function Memory_Map_File( _Name : Ansistring ;
                                     Writeable : boolean ;
                                     Start, Size : int64 ;
                                     Flag : boolean ) : integer ;
                                     virtual ; { Map a file into memory }

                                 procedure Unmap_File_From_Memory( Handle : integer ) ;
                                     virtual ;

                                 function Memory_Mapped_File_Base( Handle : integer ) : pointer ;
                                     virtual ;

                                 function Memory_Mapped_File_Size( Handle : integer ) : int64 ;
                                     virtual ;


                                 { Memory... }
                                 function Total_Physical_Memory : int64 ;
                                     virtual ;

                                 function Available_Physical_Memory : int64 ;
                                     virtual ;

                                 function Total_Virtual_Memory : int64 ;
                                     virtual ;

                                 function Available_Virtual_Memory : int64 ;
                                     virtual ;

                                 function Hardware_Name : Ansistring ;
                                     virtual ;
                         end ; { TSOperating_System }
     PSOperating_System = ^TSOperating_System ;

var OS : PSOperating_System
{$IFDEF VARIABLE_DECLARATION_ASSIGN}
    = nil
{$ENDIF}
    ;

implementation

uses { Subroutine Library, Layer 1... }
     _ASCII, { NUL }
     Compatib, { MemAvail }
     CommonUt ; { Edit }


{ TOperating_System methods... }

{ API... }

function TSOperating_System.Name : Ansistring ; { O/S name }

begin
    Name := '' ;
end ;


function TSOperating_System.Version : Ansistring ; { O/S version }

begin
    Version := '' ;
end ;


function TSOperating_System.Application_Path : Ansistring ;

begin
    Application_Path := '' ;
end ;


procedure TSOperating_System.Beep( Typ : integer ) ;

begin
end ;


procedure TSOperating_System.Cancel_Typeahead( Handle : integer ) ;

var B : boolean ;

begin
    while( Pending_Input( Handle ) ) do
    begin
        Get_Char( False, B, 0 ) ;
    end ;
end ;


function TSOperating_System.Command_Line : Ansistring ;

begin
    Command_Line := '' ;
end ;


{$IFNDEF MOBILE}
function TSOperating_System.Debugger : PSDebug_Interface ;

begin
    Debugger := nil ;
end ;
{$ENDIF}


function TSOperating_System.User_Name : Ansistring ;

begin
    User_Name := '' ;
end ;


function TSOperating_System.Device_Manager : PSDevice_Manager ;

begin
    Device_Manager := new( PSDevice_Manager, Init ) ;
end ;


function TSOperating_System.Get_Char( Wait : boolean ;
    var Valid : boolean ; Handle : integer ) : Ansichar ;

begin
    Valid := False ;
    Get_Char := NUL ;
end ;


procedure TSOperating_System.Get_Date( var Year, Month, Day : word ) ;

begin
    Year := 0 ;
    Month := 0 ;
    Day := 0 ;
end ;


function TSOperating_System.Get_Symbol( const X : Ansistring ) : Ansistring ;

begin
    Get_Symbol := '' ;
end ;


procedure TSOperating_System.Get_Time( var Hour, Minute, Second, Hundreths, NS : word ) ;

begin
    Hour := 0 ;
    Minute := 0 ;
    Second := 0 ;
    Hundreths := 0 ;
    NS := 0 ;
end ;


function TSOperating_System.Last_Error : PSUnified_Exception ;

begin
    Last_Error := nil ;
end ;


function TSOperating_System.Error_Text( Code : longint ;
    const Language : Ansistring ) : Ansistring ;

begin
    Error_Text := '';
end ;


function TSOperating_System.Pending_Input( Handle : integer ) : boolean ;

begin
    Pending_Input := False ;
end ;



function TSOperating_System.Current_Directory : Ansistring ;

begin
    Result := '' ;
end ;


function TSOperating_System.Path_Separator : Ansistring ;

begin
    Result := '\' ;
end ;


function TSOperating_System.Hardware_Name : Ansistring ;

begin
    Result := '' ;
end ;


function TSOperating_System.Normalize_Filename( S : Ansistring ) : Ansistring ;

begin
    Result := Edit( S, 512 ) ;
end ;


procedure TSOperating_System.Print( T : Ansistring ) ;

begin
end ;


function TSOperating_System.PID : longint ;

begin
    PID := 0 ;
end ;


procedure TSOperating_System.Reschedule ;

begin
end ;


function TSOperating_System.Spawn( const Path, Parameters, Default_Dir : Ansistring ;
    Flags : integer ) : boolean ;

begin
    Result := False ;
end ;


function TSOperating_System.Thread_ID : longint ;

begin
    Thread_ID := 0 ;
end ;


function TSOperating_System.Logical_CPUs : integer ;

begin
    Logical_CPUs := 1 ;
end ;


function TSOperating_System.Switch_Delimiter_Count : integer ;

begin
    Switch_Delimiter_Count := 1 ;
end ;


function TSOperating_System.Switch_Delimiters( Index : integer ) : Ansistring ;

begin
    Switch_Delimiters := '/' ;
end ;


function TSOperating_System.File_List_Delimiter : Ansistring ;

begin
    File_List_Delimiter := ';' ;
end ;



function TSOperating_System.Get_Temp_Path : Ansistring ;

begin
    Get_Temp_Path := '' ;
end ;


function TSOperating_System.System_Path : Ansistring ;

begin
    System_Path := '' ;
end ;


function TSOperating_System.Text_Presentation_Manager : PSText_Presentation_Manager ;

begin
    Text_Presentation_Manager := new( PSText_Presentation_Manager, Init ) ;
end ;


function TSOperating_System.Swap_File_List : Ansistring ;

begin
    Swap_File_List := '' ;
end ;


function TSOperating_System.Timestamp : int64 ;

begin
    Timestamp := 0 ;
end ;


function TSOperating_System.Get_Clipboard( Path : Ansistring ) : Ansistring ;

begin
    Get_Clipboard := '' ;
end ;


procedure TSOperating_System.Set_Clipboard( Path, Value : Ansistring ) ;

begin
end ;


function TSOperating_System.Max_File_Size : int64 ;

begin
    Max_File_Size := 0 ;
end ;


function TSOperating_System.Object_Version : longint ;

begin
    Object_Version := OS_Object_Version ;
end ;


function TSOperating_System.File_Processor : PSFile_Processor ;

begin
    File_Processor := _FiP.File_Processor ;
end ;


function TSOperating_System.Load_Library( Nam : Ansistring ) : longint ;

begin
    Load_Library := 0 ;
end ;


function TSOperating_System.Get_Symbol_Address( Lib : longint ;
    Nam : Ansistring ) : int64 ;

begin
    Get_Symbol_Address := 0 ;
end ;


function TSOperating_System.Show_Message( Handle : longint ;
    Msg, Title : Ansistring ; Options : integer ) : integer ;

begin
    Show_Message := 0 ;
end ;


function TSOperating_System.Get_Country : Ansistring ;

begin
    Get_Country := '' ;
end ;


function TSOperating_System.Max_Memory_Map : int64 ;

begin
    Max_Memory_Map := 0 ; { Assume there is no memory-mapping support }
end ;


function TSOperating_System.Memory_Map_File( _Name : Ansistring ;
    Writeable : boolean ; Start, Size : int64 ; Flag : boolean ) : integer ;

begin
    Memory_Map_File := 0 ;
end ;


procedure TSOperating_System.Unmap_File_From_Memory( Handle : integer ) ;

begin
end ;


function TSOperating_System.Memory_Mapped_File_Base( Handle : integer ) : pointer ;

begin
    Memory_Mapped_File_Base := nil ;
end ;


function TSOperating_System.Memory_Mapped_File_Size( Handle : integer ) : int64 ;

begin
    Memory_Mapped_File_Size := 0 ;
end ;


function TSOperating_System.Total_Physical_Memory : int64 ;

begin
    Total_Physical_Memory := 0 ;
end ;


function TSOperating_System.Available_Physical_Memory : int64 ;

begin
    Available_Physical_Memory := 0 ;
end ;


function TSOperating_System.Total_Virtual_Memory : int64 ;

begin
    Total_Virtual_Memory := 0 ;
end ;


function TSOperating_System.Available_Virtual_Memory : int64 ;

begin
    Available_Virtual_Memory := MemAvail ;
end ;


function TSOperating_System.Application_Data_Path( Default_Root, Path : Ansistring ;
    Flags : integer ) : Ansistring ;

var S : Ansistring ;

begin
    S := '' ;
    if( Default_Root <> '' ) then
    begin
        if( copy( Default_Root, 1, 1 ) <> '\' ) then
        begin
            S := S + '\' ;
        end ;
        S := S + Default_Root ;
    end ;
    if( Path <> '' ) then
    begin
        if( copy( Path, 1, 1 ) <> '\' ) then
        begin
            S := S + '\' ;
        end ;
        S := S + Path ;
    end ;
    if( ( S <> '' ) and ( copy( S, length( S ), 1 ) <> '\' ) ) then
    begin
        S := S + '\' ;
    end ;
    Application_Data_Path := S ;
end ;


function TSOperating_System.Create_Path( Path : Ansistring ) : integer ;

begin
    Create_Path := -1 ;
end ;


function TSOperating_System.Shared_Library_Extension : Ansistring ;

begin
    Shared_Library_Extension := '.dll' ;
end ;


{$IFNDEF VARIABLE_DECLARATION_ASSIGN}
begin
    __OS = nil
{$ENDIF}
end.
