/*
        Program Name : _OS
        Package Name : Subroutine Library - Layer 1
        Purpose      : Operating system interface
        Institution  : Conroy & Conroy Co.
        Date Written : 6-Nov-2002
        Written By   : Alan Conroy
        Version      : 1.1

        Copyright (C) 2002-2009 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *    DATE        BY          REASON                         *
        *                                                           *
        *                                                           *
        *************************************************************

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          TOperating_System is an abstract Operating System interface.
*/

#ifndef _OSH
	#define _OSH

	#define int64 __int64

	#include "cominter.h"
	#include "_DevMan.h"
	#include "_fip.h"
	#include "_TPM.h"

// Spawn flags...
const int Spawn_Quiet = 1 ;
const int Spawn_Asynchronous = 2 ;

// Message box option flags...
const int MBOF_OK = 0 ;
const int MBOF_OKCancel = 1 ;
const int MBOF_AbortRetryIgnore = 2 ;
const int MBOF_YesNoCancel = 3 ;
const int MBOF_YesNo = 4 ;
const int MBOF_RetryCancel = 5 ;

// Message box return values...
const int MBR_Abort = 1 ;
const int MBR_Cancel = 2 ;
const int MBR_No = 3 ;
const int MBR_OK = 4 ;
const int MBR_Retry = 5 ;
const int MBR_Yes = 6 ;
const int MBR_Ignore = 7 ;

class TCOM_Operating_System : public TCommon_COM_Interface
{
	public: // API...
		// Identification...
		virtual char* __stdcall Name() = 0; // O/S name
		virtual char* __stdcall Version() = 0; // O/S version

		// Date/Time support...
		virtual void __stdcall Get_Date( unsigned short& Year, unsigned short& Month, unsigned short& Day ) = 0;
		virtual void __stdcall Get_Time( unsigned short& Hour, unsigned short& Minute, unsigned short& Second, unsigned short& Hundreths ) = 0;
		virtual int64 __stdcall Timestamp() = 0 ; // Current date/time in Sirius format

		// Process support...
		virtual int __stdcall PID() = 0;
		virtual void __stdcall Reschedule() = 0;
		// Reschedules process so other tasks can get some runtime.

		virtual bool __stdcall Spawn( char* Path, char* Parameters, char* Default_Dir, int Flags ) ;
		// Spawns a process.

		virtual int __stdcall Thread_ID() = 0; // Get current thread ID
		virtual int __stdcall Logical_CPUs() = 0; // Number of logical CPUs on this sytem.

		virtual char* __stdcall User_Name() = 0;

		// Device support...
		virtual TCOM_Device_Manager* __stdcall Device_Manager() = 0;


		// I/O...
		virtual void __stdcall Beep( int Typ ) = 0; // Beep the user.

		virtual void __stdcall Cancel_Typeahead( int Handle ) = 0;
		// Cancel any pending keyboard input

		virtual char* __stdcall Get_Char( bool Wait, bool& Valid, int Handle ) = 0;

		virtual bool __stdcall Pending_Input( int Handle ) = 0;
		// Returns true if any user input is pending

		virtual void __stdcall Print( char* T ) = 0;
		// Print the text on the default printer


		// Error processing...
		virtual void __stdcall Last_Error( TUEC& UEC ) = 0;
		virtual char* __stdcall Error_Text( TUEC UEC, char* Language ) = 0;


		// Files and pathing...
		virtual char* __stdcall Application_Path() = 0;
		// Returns application path with terminating delimiter

		virtual char* __stdcall File_List_Delimiter() = 0;
		// Delimiter used to separate files in a list.

		virtual char* __stdcall Get_Temp_Path() = 0;
		// Returns path(s) for temporary files.  If there are multiple paths, they are delimited by the File_List_Delimiter.

		virtual char* __stdcall Swap_File_List() = 0;
		// Returns list of swap files.  Returns Null if no swap files.

		virtual char* __stdcall System_Path() = 0;
		// Path(s) to the system directory.  Returns null string if not supported.

		virtual TCOM_Text_Presentation_Manager* __stdcall Text_Presentation_Manager() = 0;
		// Returns the Text_Presentation_Manager instance.


		// Misc...
		virtual char* __stdcall Command_Line() = 0;
		 // Returns command line used to start program.

		virtual char* __stdcall Get_Symbol( char* X ) = 0;
		// Return contents of OS/Shell symbol.  Returns null if not defined.

		virtual int __stdcall Switch_Delimiter_Count() = 0;
		// Returns number of switch delimiters.

		virtual char* __stdcall Switch_Delimiters( int Index ) = 0;
		// Returns switch delimiter for Index.

		virtual char* __stdcall Get_Clipboard( char* Path ) = 0;
		// Get clipboard contents in path.

		virtual void __stdcall Set_Clipboard( char* Path, char* Value ) = 0;
		// Get clipboard contents for path.

		virtual int64 __stdcall Max_File_Size() = 0;
		// Return the largest that a file could be on a device with infinite space, using the file system with the smallest max size files supported by the O/S.

		virtual int __stdcall Object_Version() = 0;
		// Returns version number of this object.  11 = 1.1

		virtual TCOM_File_Processor* __stdcall File_Processor() = 0;

		virtual int __stdcall Load_Library( char* Nam ) = 0;
		// Load a library and return a handle to it.

		virtual int64 __stdcall Get_Symbol_Address( int Lib, char* Nam ) = 0;
		// Retrieve address of a symbol in a library

		virtual int __stdcall Show_Message( int Handle, char* Msg, char* Title,
			 int Options ) = 0;
		// Prompt user and return response.

		virtual char* __stdcall Get_Country() = 0;


		// Memory mapping...
		virtual int64 __stdcall Max_Memory_Mapping() = 0;
		// Maximum memory mapping available.
} ; // TCOM_Operating_System
#endif
