{$IFNDEF FPC}
  {$N+}
{$ENDIF}
{
        Program Name : FormUtil
        Package Name : Subroutine Library
        Purpose      : Delphi form utilities
        Institution  : Conroy & Conroy Co.
        Date Written : 27-Sep-2008
        Written By   : Alan Conroy
        Version      : 1.0

        Copyright (C) 2008-2015 by Alan Conroy.  Released to the public domain.

        TO THE GLORY OF GOD THROUGH JESUS CHRIST

        *************************************************************
        *                                                           *
        *        M O D I F I C A T I O N   H I S T O R Y            *
        *                                                           *
        *************************************************************

            DATE        BY          REASON

        *************************************************************
        *                                                           *
        *        P R O G R A M   P U R P O S E                      *
        *                                                           *
        *************************************************************

          This unit provides utility routines for dealing with Delphi forms.
}

unit FormUtil ;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

{$I EDEFINES.INC}

uses { Borland... }
     Classes, { TStringList }
     Controls, { TControl }
     Graphics, { TBitmap }

     { Subroutine Library, Layer 0... }
     CommBase ; { TBase_Delphi_Object }

type TObject_Constructor = class
                               public // API...
                                   function Supported( Name : ansistring ) : boolean ;
                                       virtual ; abstract ;
                                   function Construct( O : TObject ;
                                       Name, Typ, Def : ansistring ;
                                       Parent : TControl ; Owner : TComponent ) : TObject ;
                                       virtual ; abstract ;
                           end ;

    TBitmap_Constructor = class
                              public // API...
                                  function Create_Image( Name : ansistring ) : TBitmap ;
                                      virtual ; abstract ;
                          end ;

type TVComponent_Definition = class( TBase_Delphi_Object )
                                  public // Constructors and destructors...
                                      destructor Destroy ; override ;

                                  private // Instance data...
                                      _Attribute_Names : TStringList ;
                                      _Attribute_Values : TStringList ;
                                      _Children : TList ;

                                  protected // Property handlers..
                                      function Get_Attribute_Names : TStringList ;
                                      function Get_Attribute_Values : TStringList ;
                                      function Get_Children : TList ;

                                      procedure Set_Attribute_Names( Value : TStringList ) ;
                                      procedure Set_Attribute_Values( Value : TStringList ) ;
                                      procedure Set_Children( Value : TList ) ;

                                  public // API...
                                      Name : ansistring ;
                                      Type_Name : ansistring ;

                                      procedure Clear ;

                                      property Attribute_Names : TStringList
                                          read Get_Attribute_Names
                                          write Set_Attribute_Names ;
                                      property Attribute_Values : TStringList
                                          read Get_Attribute_Values
                                          write Set_Attribute_Values ;
                                      property Children : TList
                                          read Get_Children
                                          write Set_Children ;
                              end ; // TVComponent_Definition

type TVComponent = class
                       public // Constructors and destructors...
                           destructor Destroy ; override ;

                       protected // Utility routines...
                           function Create_Control( Parent : TWinControl ;
                               Def : TVComponent_Definition ) : TControl ;
                               virtual ;
                           procedure Set_Control_Property( Name, Value : ansistring ;
                               Control : TControl ) ; virtual ;

                       public // API...
                           Definition : TVComponent_Definition ;

                           procedure Parse_Form( Def : ansistring ) ; virtual ;
                           // Parse a form definition and return the parsed definition

                           function Construct_From_Definition( Parent : TWinControl ;
                               Def : TVComponent_Definition ) : TControl ;
                               virtual ;
                           // Construct and return an object from a definition.
                           // Parent is the parent control for the returned control

                           function Construct( Parent : TWinControl ) : TControl ;
                               virtual ;
                   end ; // TVComponent

function Parse_Form( Def : ansistring ) : TVComponent_Definition ;
// Parse a form definition and return the parsed definition

function Construct_From_Definition( Parent : TWinControl ;
    Def : TVComponent_Definition ) : TControl ;
// Construct and return an object from a definition.  Parent is the parent control for the returned control


procedure Register_Bitmap_Constructor( O : TBitmap_Constructor ) ;

procedure Register_Object_Constructor( O : TObject_Constructor ) ;

function Construct_Form_From_Definition( Parent : TWinControl ;
    Def : ansistring ; Owner : TComponent ) : TObject ;

function Construct_Object_From_Definition( O : TObject ; Name, Typ, Def : ansistring ;
    Parent : TWinControl ; Owner : TComponent ) : TObject ;

function Next_Line( var Def : ansistring ) : ansistring ;

function Extract_Object_Definition( var S : ansistring ) : ansistring ;

function Get_String( S : ansistring ; var Def : ansistring ) : ansistring ;


implementation

uses // Borland...
     Buttons, // TBitBtn
     ComCtrls, // TPageControl
     Extctrls, // TPanel
     Forms, // TForm
     Menus, // TMainMenu
     StdCtrls, // TLabel
     SysUtils, // uppercase

     // C&C...
     _ASCII, // CR
     CommonUt, // Edit
     L10n, // Text_*
     Parse, // TString_Parser
     Spin, // TSpinEdit
     UStrings, // PString
     VCL_Std ; // Text_To_Color

// TVComponent_Definition methods...

// Constructors and destructors...

destructor TVComponent_Definition.Destroy ;

begin
    _Attribute_Names := nil ;
    _Attribute_Values := nil ;
    _Children := nil ;

    inherited Destroy ;
end ;


// Property handlers...

function TVComponent_Definition.Get_Attribute_Names : TStringList ;

begin
    if( _Attribute_Names = nil ) then
    begin
        _Attribute_Names := TStringList.Create ;
    end ;
    Result := _Attribute_Names ;
end ;


function TVComponent_Definition.Get_Attribute_Values : TStringList ;

begin
    if( _Attribute_Values = nil ) then
    begin
        _Attribute_Values := TStringList.Create ;
    end ;
    Result := _Attribute_Values ;
end ;


function TVComponent_Definition.Get_Children : TList ;

begin
    if( _Children = nil ) then
    begin
        _Children := TList.Create ;
    end ;
    Result := _Children ;
end ;


procedure TVComponent_Definition.Set_Attribute_Names( Value : TStringList ) ;

begin
    if( ( Value <> _Attribute_Names ) and ( _Attribute_Names <> nil ) ) then
    begin
        _Attribute_Names.Free ;
    end ;
    _Attribute_Names := Value ;
end ;


procedure TVComponent_Definition.Set_Attribute_Values( Value : TStringList ) ;

begin
    if( ( Value <> _Attribute_Values ) and ( _Attribute_Values <> nil ) ) then
    begin
        _Attribute_Values.Free ;
    end ;
    _Attribute_Values := Value ;
end ;


procedure TVComponent_Definition.Set_Children( Value : TList ) ;

begin
    if( ( Value <> _Children ) and ( _Children <> nil ) ) then
    begin
        Clear ;
        _Children.Free ;
    end ;
    _Children := Value ;
end ;


// API...

procedure TVComponent_Definition.Clear ;

var Loop : integer ;

begin
    if( _Children <> nil ) then
    begin
        for Loop := 0 to _Children.Count - 1 do
        begin
            TVComponent_Definition( _Children[ Loop ] ).Free ;
            _Children[ Loop ] := nil ;
        end ;
        _Children.Clear ;
    end ;
end ;



// TVComponent methods...

// Constructors and destructors...

destructor TVComponent.Destroy ;

begin
    Definition.Free ;
    Definition := nil ;

    inherited Destroy ;
end ;


type TFake_Form = class( TForm ) ; // TO get access to protected data

// Utility routines...

function TVComponent.Create_Control( Parent : TWinControl ;
    Def : TVComponent_Definition ) : TControl ;

var S : ansistring ;

begin
    S := uppercase( Def.Type_Name ) ;
    if( S = 'TLABEL' ) then
    begin
        Result := TLabel.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TPANEL' ) then
    begin
        Result := TPanel.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TEDIT' ) then
    begin
        Result := TEdit.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TBITBTN' ) then
    begin
        Result := TBitBtn.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TLISTBOX' ) then
    begin
        Result := TListBox.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TSPLITTER' ) then
    begin
        Result := TSplitter.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TSCROLLBAR' ) then
    begin
        Result := TScrollbar.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
(*
    if( S = 'TMAINMENU' ) then
    begin
        Result := TMainMenu.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
*)
    if( S = 'TSPEEDBUTTON' ) then
    begin
        Result := TSpeedButton.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TPAGECONTROL' ) then
    begin
        Result := TPageControl.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TTABSHEET' ) then
    begin
        Result := TTabSheet.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TCOMBOBOX' ) then
    begin
        Result := TComboBox.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TGROUPBOX' ) then
    begin
        Result := TGroupBox.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TCHECKBOX' ) then
    begin
        Result := TCheckBox.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TRADIOBUTTON' ) then
    begin
        Result := TRadioButton.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( S = 'TSPINEDIT' ) then
    begin
        Result := TSpinEdit.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
{$IFNDEF FPC}
    if( S = 'TRICHEDIT' ) then
    begin
        Result := TRichEdit.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
{$ENDIF}
    begin
        Result := TForm.Create( Parent ) ;
        Result.Parent := Parent ;
    end ;
    Result.Name := Def.Name ;
    TFake_Form( Result ).Caption := '' ;
    if( Result is TEdit ) then
    begin
        TEdit( Result ).Text := '' ;
    end ;
end ; // TVComponent.Create_Control


function String_To_FormBorderStyle( Value : ansistring ) : TFormBorderStyle ;

begin
    Value := uppercase( Value ) ;
    Result := bsNone ;
    if( Value = 'BSNONE' ) then
    begin
        Result := bsNone ;
    end else
    if( Value = 'BSSINGLE' ) then
    begin
        Result := bsSingle ;
    end else
    if( Value = 'BSSIZEABLE' ) then
    begin
        Result := bsSizeable ;
    end else
    if( Value = 'BSDIALOG' ) then
    begin
        Result := bsDialog ;
    end else
    if( Value = 'BSTOOLWINDOW' ) then
    begin
        Result := bsToolWindow ;
    end else
    if( Value = 'BSSIZETOOLWIN' ) then
    begin
        Result := bsSizeToolWin ;
    end ;
end ;


procedure TVComponent.Set_Control_Property( Name, Value : ansistring ;
    Control : TControl ) ;

var BB : TBitBtn ;
    CB : TComboBox ;
    Dummy : integer ;
    E : TEdit ;
    Form : TFake_Form ;
    S, Work : ansistring ;
    SB : TSpeedButton ;

begin
    S := uppercase( Name ) ;
    Value := Edit( Value, -1 ) ;
    if( S = 'ALIGN' ) then
    begin
        if( uppercase( Value ) = 'ALNONE' ) then
        begin
            Control.Align := alNone ;
        end else
        if( uppercase( Value ) = 'ALTOP' ) then
        begin
            Control.Align := alTop ;
        end else
        if( uppercase( Value ) = 'ALBOTTOM' ) then
        begin
            Control.Align := alBottom ;
        end else
        if( uppercase( Value ) = 'ALLEFT' ) then
        begin
            Control.Align := alLeft ;
        end else
        if( uppercase( Value ) = 'ALRIGHT' ) then
        begin
            Control.Align := alRight ;
        end ;
    end else
    if( S = 'BEVELINNER' ) then
    begin
        Form := TFake_Form( Control ) ;
        if( uppercase( Value ) = 'BVNONE' ) then
        begin
{$IFNDEF FPC}
            Form.BevelInner := bvNone ;
{$ENDIF}
        end else
        if( uppercase( Value ) = 'BVLOWERED' ) then
        begin
{$IFNDEF FPC}
            Form.BevelInner := bvLowered ;
{$ENDIF}
        end else
        if( uppercase( Value ) = 'BVRAISED' ) then
        begin
{$IFNDEF FPC}
            Form.BevelInner := bvRaised ;
{$ENDIF}
        end else
        if( uppercase( Value ) = 'BVSPACE' ) then
        begin
{$IFNDEF FPC}
            Form.BevelInner := bvSpace ;
{$ENDIF}
        end ;
    end else
    if( S = 'BEVELOUTER' ) then
    begin
        Form := TFake_Form( Control ) ;
        if( uppercase( Value ) = 'BVNONE' ) then
        begin
{$IFNDEF FPC}
            Form.BevelOuter := bvNone ;
{$ENDIF}
        end else
        if( uppercase( Value ) = 'BVLOWERED' ) then
        begin
{$IFNDEF FPC}
            Form.BevelOuter := bvLowered ;
{$ENDIF}
        end else
        if( uppercase( Value ) = 'BVRAISED' ) then
        begin
{$IFNDEF FPC}
            Form.BevelOuter := bvRaised ;
{$ENDIF}
        end else
        if( uppercase( Value ) = 'BVSPACE' ) then
        begin
{$IFNDEF FPC}
            Form.BevelOuter := bvSpace ;
{$ENDIF}
        end ;
    end else
    if( S = 'BORDERICONS' ) then
    begin
        Form := TFake_Form( Control ) ;
        Value := copy( Value, 2, length( Value ) ) ;
        Value := copy( Value, 1, length( Value ) - 1 ) ; // Strip []
        while( Value <> '' ) do
        begin
            Dummy := pos( ',', Value ) ;
            if( Dummy = 0 ) then
            begin
                Dummy := length( Value ) + 1 ;
            end ;
            Work := copy( Value, 1, Dummy - 1 ) ;
            Value := copy( Value, Dummy + 1, length( Value ) ) ;
            if( Work = 'BISYSTEMMENU' ) then
            begin
                Form.BorderIcons := Form.BorderIcons + [biSystemMenu] ;
            end else
            if( Work = 'BIMINIMIZE' ) then
            begin
                Form.BorderIcons := Form.BorderIcons + [biMinimize] ;
            end else
            if( Work = 'BIMAXIMIZE' ) then
            begin
                Form.BorderIcons := Form.BorderIcons + [biMaximize] ;
            end else
            if( Work = 'BIHELP' ) then
            begin
                Form.BorderIcons := Form.BorderIcons + [biHelp] ;
            end ;
        end ;
    end else
    if( S = 'BORDERSTYLE' ) then
    begin
        if( Control is TCustomEdit ) then
        begin
            TEdit( Control ).BorderStyle := String_To_FormBorderStyle( Value ) ;
        end else
        begin
            TFake_Form( Control ).BorderStyle := String_To_FormBorderStyle( Value ) ;
        end ;
    end else
    if( S = 'BORDERWIDTH' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.BorderWidth := strtoint( Value ) ;
    end else
    if( S = 'CAPTION' ) then
    begin
        Form := TFake_Form( Control ) ;
        Value := copy( Value, 2, length( Value ) ) ;
        Value := copy( Value, 1, length( Value ) - 1 ) ; // Trim quotes
        Form.Caption := Value ;
    end else
    if( ( S = 'CHECKED' ) and ( Control is TCheckBox ) ) then
    begin
        TCheckBox( Control ).Checked := uppercase( Value ) = 'TRUE' ;
    end else
    if( ( S = 'CHECKED' ) and ( Control is TRadioButton ) ) then
    begin
        TRadioButton( Control ).Checked := uppercase( Value ) = 'TRUE' ;
    end else
    if( S = 'CLIENTHEIGHT' ) then
    begin
        Control.ClientHeight := strtoint( Value ) ;
    end else
    if( S = 'CLIENTWIDTH' ) then
    begin
        Control.ClientWidth := strtoint( Value ) ;
    end else
    if( S = 'COLOR' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.Color := String_To_Color( Value ) ;
    end else
    if( S = 'CTL3D' ) then
    begin
{$IFNDEF FPC}
        Form := TFake_Form( Control ) ;
        if( Value = 'TRUE' ) then
        begin
            Form.Ctl3D := True ;
        end else
        begin
            Form.Ctl3D := False ;
        end ;
{$ENDIF}
    end else
    if( S = 'ENABLED' ) then
    begin
        Form := TFake_Form( Control ) ;
        if( Value = 'TRUE' ) then
        begin
            Form.Enabled := True ;
        end else
        begin
            Form.Enabled := False ;
        end ;
    end else
    if( S = 'FONT.COLOR' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.Font.Color := String_To_Color( Value ) ;
    end else
    if( S = 'FONT.HEIGHT' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.Font.Height := strtoint( Value ) ;
    end else
    if( S = 'FONT.NAME' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.Font.Name := Value ;
    end else
    if( S = 'FONT.SIZE' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.Font.Size := strtoint( Value ) ;
    end else
    if( S = 'FONT.STYLE' ) then
    begin
        Form := TFake_Form( Control ) ;
        Value := copy( Value, 2, length( Value ) ) ;
        Value := copy( Value, 1, length( Value ) - 1 ) ; // Strip []
        while( Value <> '' ) do
        begin
            Dummy := pos( ',', Value ) ;
            if( Dummy = 0 ) then
            begin
                Dummy := length( Value ) + 1 ;
            end ;
            Work := copy( Value, 1, Dummy - 1 ) ;
            Value := copy( Value, Dummy + 1, length( Value ) ) ;
            if( Work = 'FSBOLD' ) then
            begin
                Form.Font.Style := Form.Font.Style + [fsBold] ;
            end else
            if( Work = 'FSITALIC' ) then
            begin
                Form.Font.Style := Form.Font.Style + [fsItalic] ;
            end else
            if( Work = 'FSUNDERLINE' ) then
            begin
                Form.Font.Style := Form.Font.Style + [fsUnderline] ;
            end else
            if( Work = 'FSSTRIKEOUT' ) then
            begin
                Form.Font.Style := Form.Font.Style + [fsStrikeOut] ;
            end ;
        end ;
    end else
    if( S = 'HEIGHT' ) then
    begin
        Control.Height := strtoint( Value ) ;
    end else
    if( S = 'HELPCONTEXT' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.HelpContext := strtoint( Value ) ;
    end else
    if( S = 'ITEMHEIGHT' ) then
    begin
        CB := TComboBox( Control ) ;
        CB.ItemHeight := strtoint( Value ) ;
    end else
    if( S = 'ITEMINDEX' ) then
    begin
        CB := TComboBox( Control ) ;
        CB.ItemIndex := strtoint( Value ) ;
    end else
    if( S = 'ITEMS.STRINGS' ) then
    begin
        CB := TComboBox( Control ) ;
        Value := copy( Value, 2, length( Value ) ) ; // Trim parentheses
        Value := copy( Value, 1, length( Value ) - 1 ) ;
        while( Value <> '' ) do
        begin
            Value := copy( Value, 2, length( Value ) ) ; // Trim leading quote
            Dummy := pos( #39, Value ) ;
            if( Dummy = 0 ) then // Shouldn't happen in well-formed file
            begin
                Dummy := length( Value ) + 1 ;
            end ;
            CB.Items.Add( copy( Value, 1, Dummy - 1 ) ) ;
            Value := copy( Value, Dummy + 1, length( Value ) ) ;
        end ;
    end else
    if( S = 'LEFT' ) then
    begin
        Control.Left := strtoint( Value ) ;
    end else
    if( S = 'MARGIN' ) then
    begin
        SB := TSpeedButton( Control ) ;
        SB.Margin := strtoint( Value ) ;
    end else
    if( ( S = 'MAXVALUE' ) and ( Control is TSpinEdit ) ) then
    begin
        TSpinEdit( Control ).MaxValue := strtoint( Value ) ;
    end else
    if( ( S = 'MINVALUE' ) and ( Control is TSpinEdit ) ) then
    begin
        TSpinEdit( Control ).MinValue := strtoint( Value ) ;
    end else
    if( S = 'MODALRESULT' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.ModalResult := strtoint( Value ) ;
    end else
    if( S = 'PARENTBACKGROUND' ) then
    begin
{$IFNDEF FPC}
        Form := TFake_Form( Control ) ;
        Form.ParentBackground := Value = 'TRUE' ;
{$ENDIF}
    end else
    if( S = 'PARENTCOLOR' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.ParentColor := Value = 'TRUE' ;
    end else
    if( S = 'PARENTCTL3D' ) then
    begin
{$IFNDEF FPC}
        Form := TFake_Form( Control ) ;
        Form.ParentCtl3D := Value = 'TRUE' ;
{$ENDIF}
    end else
    if( S = 'PARENTFONT' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.ParentFont := Value = 'TRUE' ;
    end else
    if( S = 'TABORDER' ) then
    begin
        Form := TFake_Form( Control ) ;
        Form.TabOrder := strtoint( Value ) ;
    end else
    if( S = 'TEXT' ) then
    begin
        E := TEdit( Control ) ;
        Value := copy( Value, 2, length( Value ) ) ;
        Value := copy( Value, 1, length( Value ) - 1 ) ; // Trim quotes
        E.Text := Value ;
    end else
    if( S = 'TOP' ) then
    begin
        Control.Top := strtoint( Value ) ;
    end else
    if( ( S = 'VALUE' ) and ( Control is TSpinEdit ) ) then
    begin
        TSpinEdit( Control ).Value := strtoint( Value ) ;
    end else
    if( S = 'WIDTH' ) then
    begin
        Control.Width := strtoint( Value ) ;
    end else
    if( S = 'KIND' ) then
    begin
        BB := TBitBtn( Control ) ;
        if( uppercase( Value ) = 'BKOK' ) then
        begin
            BB.Kind := bkOK ;
        end else
        if( uppercase( Value ) = 'BKCANCEL' ) then
        begin
            BB.Kind := bkCancel ;
        end else
        if( uppercase( Value ) = 'BKHELP' ) then
        begin
            BB.Kind := bkHelp ;
        end else
        if( uppercase( Value ) = 'BKABORT' ) then
        begin
            BB.Kind := bkAbort ;
        end else
        if( uppercase( Value ) = 'BKALL' ) then
        begin
            BB.Kind := bkAll ;
        end else
        if( uppercase( Value ) = 'BKCLOSE' ) then
        begin
            BB.Kind := bkClose ;
        end else
        if( uppercase( Value ) = 'BKCUSTOM' ) then
        begin
            BB.Kind := bkCustom ;
        end else
        if( uppercase( Value ) = 'BKIGNORE' ) then
        begin
            BB.Kind := bkIgnore ;
        end else
        if( uppercase( Value ) = 'BKNO' ) then
        begin
            BB.Kind := bkNo ;
        end else
        if( uppercase( Value ) = 'BKRETRY' ) then
        begin
            BB.Kind := bkRetry ;
        end else
        if( uppercase( Value ) = 'BKYES' ) then
        begin
            BB.Kind := bkYes ;
        end ;
    end ; // if( S = 'KIND' )
    // Ignore all others
end ; // TVComponent.Set_Control_Property


// API...

procedure TVComponent.Parse_Form( Def : ansistring ) ;

var Parser : TSStrings_Parser ;

    procedure Process_Component( This : TVComponent_Definition ) ;

        procedure Error( X : ansistring ) ;

        begin
        end ;


        function Grab_Line : ansistring ;

        var P : UStrings.PString ;

        begin
            P := Parser.Grab_Line ;
            Result := Edit( P^.Text, -1 ) ;
            dispose( P, Done ) ;
        end ;


        function Token : ansistring ;

        var P : UStrings.PString ;

        begin
            P := Parser.Token ;
            Result := Edit( P^.Text, -1 ) ;
            dispose( P, Done ) ;
        end ;

    var O : TVComponent_Definition ;
        S, S1 : ansistring ;

    begin
        S := uppercase( Token ) ;
        if( S <> 'OBJECT' ) then // First token must be "object"
        begin
            Error( Text_Error_Invalid_Syntax ) ;
            exit ;
        end ;
        This.Name := Token ;
        if( This.Name = '' ) then
        begin
            Error( Text_Error_Object_Is_Missing_A_Name ) ;
            exit ;
        end ;
        S := Token ;
        if( S <> ':' ) then
        begin
            Error( Text_Error_Expected_Colon ) ;
            exit ;
        end ;
        This.Type_Name := Token ;
        if( This.Type_Name = '' ) then
        begin
            Error( Text_Error_Object_Is_Missing_Type ) ;
            exit ;
        end ;
        S := uppercase( Token ) ;
        while( S <> '' ) do
        begin
            if( S = 'OBJECT' ) then
            begin
                Parser.Put_Token( Temp_String( S ) ) ;
                O := TVComponent_Definition.Create ;
                Process_Component( O ) ;
                This.Children.Add( O ) ;
            end else
            if( S = 'END' ) then
            begin
                exit ; // All done with definition
            end else
            begin
                S1 := Token ;
                if( S1 = '.' ) then
                begin
                    S := S + '.' + Token ;
                    S1 := Token ;
                end ;
                This.Attribute_Names.Add( S ) ;
                S := S1 ;
                if( S <> '=' ) then
                begin
                    Error( Text_Error_Expected_Equal ) ;
                    S := '' ;
                end else
                begin
                    S := Grab_Line ;
                    if( ( pos( '(', S ) > 0 ) and ( copy( S, 1, 1 ) <> #39 ) ) then
                    begin
                        while( pos( ')', S ) = 0 ) do
                        begin
                            S := S + Token ;
                            if( pos( ')', S ) = 0 ) then
                            begin
                                S := S + Grab_Line ;
                            end ;
                        end ;
                    end ;
                end ;
                This.Attribute_Values.Add( S ) ;
            end ;
            S := uppercase( Token ) ;
        end ; // while( S <> '' )
    end ; // Process_Component

var Dummy : integer ;
    List : TStringList ;

begin
    Parser.Init ;
    List := TStringList.Create ;
    try
        while( Def <> '' ) do
        begin
            Dummy := pos( CR, Def ) ;
            if( Dummy = 0 ) then
            begin
                Dummy := length( Def ) + 1 ;
            end ;
            List.Add( copy( Def, 1, Dummy - 1 ) ) ;
            if( copy( Def, Dummy + 1, 1 ) = LF ) then
            begin
                inc( Dummy ) ;
            end ;
            Def := copy( Def, Dummy + 1, length( Def ) ) ;
        end ;
        Parser.Set_Source( List ) ;
        Definition := TVComponent_Definition.Create ;
        Process_Component( Definition ) ;
    finally
        Parser.Done ;
        List.Free ;
    end ;
end ; // TVComponent.Parse_Form


function TVComponent.Construct( Parent : TWinControl ) : TControl ;

begin
    Result := Construct_From_Definition( Parent, Definition ) ;
end ;


function TVComponent.Construct_From_Definition( Parent : TWinControl ;
    Def : TVComponent_Definition ) : TControl ;

var Loop : integer ;
    
begin
    // Setup...
    Result := nil ;
    if( Def = nil ) then
    begin
        exit ;
    end ;

    Result := Create_Control( Parent, Def ) ;
    if( Result = nil ) then
    begin
        exit ;
    end ;

    for Loop := 0 to Def.Attribute_Names.Count - 1 do
    begin
        Set_Control_Property( Def.Attribute_Names[ Loop ],
            Def.Attribute_Values[ Loop ], Result ) ;
    end ;

    for Loop := 0 to Def.Children.Count - 1 do
    begin
        Construct_From_Definition( TWinControl( Result ), TVComponent_Definition( Def.Children[ Loop ] ) ) ;
    end ;
end ; // TVComponent.Construct_From_Definition



// Unit API...

function Parse_Form( Def : ansistring ) : TVComponent_Definition ;

var Parser : TString_Parser ;

    procedure Process_Component( Parent : TVComponent_Definition ) ;

    var O : TVComponent_Definition ;
        Name, S, Type_Name : ansistring ;

    begin
        S := uppercase( Parser.Token ) ;
        if( S <> 'OBJECT' ) then // First token must be "object"
        begin
            exit ; // Invalid file
        end ;
        Name := Parser.Token ;
        if( Name = '' ) then
        begin
            exit ; // Missing name
        end ;
        S := Parser.Token ;
        if( S <> ':' ) then
        begin
            exit ; // Expected colon
        end ;
        Type_Name := Parser.Token ;
        if( Type_Name = '' ) then
        begin
            exit ; // Missing type
        end ;
        S := uppercase( Parser.Token ) ;
        while( S <> '' ) do
        begin
            if( S = 'OBJECT' ) then
            begin
                Parser.Put_Token( S ) ;
                O := TVComponent_Definition.Create ;
                Process_Component( O ) ;
                if( Result = nil ) then
                begin
                    Result := O ;
                end ;
            end else
            if( S = 'END' ) then
            begin
                exit ; // All done with definition
            end else
            if( Parent <> nil ) then
            begin
                Parent.Attribute_Names.Add( S ) ;
                S := Parser.Token ;
                if( S <> '=' ) then
                begin
                    exit ; // Expected equal
                end ;
                Parent.Attribute_Values.Add( Parser.Grab_Line ) ;
            end ;
            S := uppercase( Parser.Token ) ;
        end ;
    end ; // Process_Component

begin
    Result := nil ;
    Parser := TString_Parser.Create ;
    Parser.Set_Source( Def ) ;
    try
        Process_Component( nil ) ;
    finally
        Parser.Free ;
    end ;
end ; // Parse_Form


function Construct_From_Definition( Parent : TWinControl ;
    Def : TVComponent_Definition ) : TControl ;

//var Loop : integer ;
//    S, Value : ansistring ;

begin
    // Setup...
    Result := nil ;
    if( Def = nil ) then
    begin
        exit ;
    end ;

    if( uppercase( Def.Type_Name ) = 'TLABEL' ) then
    begin
        Result := TLabel.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TPANEL' ) then
    begin
        Result := TPanel.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TEDIT' ) then
    begin
        Result := TEdit.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TBITBTN' ) then
    begin
        Result := TBitBtn.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TLISTBOX' ) then
    begin
        Result := TListBox.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TSPLITTER' ) then
    begin
        Result := TSplitter.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TSCROLLBAR' ) then
    begin
        Result := TScrollbar.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
(*
    if( uppercase( Def.Type_Name ) = 'TMAINMENU' ) then
    begin
        Result := TMainMenu.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
*)
    if( uppercase( Def.Type_Name ) = 'TPAGECONTROL' ) then
    begin
        Result := TPageControl.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    if( uppercase( Def.Type_Name ) = 'TTABSHEET' ) then
    begin
        Result := TTabSheet.Create( Parent ) ;
        Result.Parent := Parent ;
    end else
    begin
        Result := TForm.Create( Parent ) ;
        Result.Parent := Parent ;
    end ;

(*
    for Loop := 0 to Attribute_Names.Count - 1 do
    begin
        S := uppercase( Attribute_Names[ Loop ] ) ;
        Value := Attribute_Values[ Loop ] ;
        if( S = 'LEFT' ) then
        begin
            Result.Left := strtoint( Value ) ;
        end else
        if( S = 'TOP' ) then
        begin
            Result.Top := strtoint( Value ) ;
        end else
        if( S = 'WIDTH' ) then
        begin
            Result.Width := strtoint( Value ) ;
        end else
        if( S = 'HEIGHT' ) then
        begin
            Result.Height := strtoint( Value ) ;
        end else
        if( S = 'CAPTION' ) then
        begin
            Result.Caption := Value ;
        end else
        if( S = 'COLOR' ) then
        begin
            Result.Color := String_To_Color( Value ) ;
        end else
        if( S = 'FONT.CHARSET' ) then
        begin
            if( uppercase( Value ) = 'DEFAULT_CHARSET' ) then
            begin
                Result.Font.Charset := DEFAULT_CHARSET
            end ;
        end else
        if( S = 'FONT.COLOR' ) then
        begin
            Result.Font.Color := String_To_Color( Value ) ;
        end else
        if( S = 'FONT.HEIGHT' ) then
        begin
            Result.Font.Height := strtoint( Value ) ;
        end else
        if( S = 'FONT.NAME' ) then
        begin
            Result.Font.Name := Value ;
        end else
        if( S = 'FONT.STYLE' ) then
        begin
            //Result.Font.Style := []
        end else
        if( S = 'TEXTHEIGHT' ) then
        begin
            Result.TextHeight := strtoint( Value ) ;
        end else
        if( S = 'ALIGN' ) then
        begin
            if( uppercase( Value ) = 'ALNONE' ) then
            begin
                Result.Align := alNone ;
            end else
            if( uppercase( Value ) = 'ALTOP' ) then
            begin
                Result.Align := alTop ;
            end else
            if( uppercase( Value ) = 'ALBOTTOM' ) then
            begin
                Result.Align := alBottom ;
            end else
            if( uppercase( Value ) = 'ALLEFT' ) then
            begin
                Result.Align := alLeft ;
            end else
            if( uppercase( Value ) = 'ALRIGHT' ) then
            begin
                Result.Align := alRight ;
            end ;
        end else
        if( S = 'KIND' ) then
        begin
            if( uppercase( Value ) = 'BKOK' ) then
            begin
                Result.Kind := bkOK ;
            end else
            if( uppercase( Value ) = 'BKCANCEL' ) then
            begin
                Result.Kind := bkCancel ;
            end else
            if( uppercase( Value ) = 'BKHELP' ) then
            begin
                Result.Kind := bkHelp ;
            end else
            if( uppercase( Value ) = 'BKABORT' ) then
            begin
                Result.Kind := bkAbort ;
            end else
            if( uppercase( Value ) = 'BKALL' ) then
            begin
                Result.Kind := bkAll ;
            end else
            if( uppercase( Value ) = 'BKCLOSE' ) then
            begin
                Result.Kind := bkClose ;
            end else
            if( uppercase( Value ) = 'BKCUSTOM' ) then
            begin
                Result.Kind := bkCustom ;
            end else
            if( uppercase( Value ) = 'BKIGNORE' ) then
            begin
                Result.Kind := bkIgnore ;
            end else
            if( uppercase( Value ) = 'BKNO' ) then
            begin
                Result.Kind := bkNo ;
            end else
            if( uppercase( Value ) = 'BKRETRY' ) then
            begin
                Result.Kind := bkRetry ;
            end else
            if( uppercase( Value ) = 'BKYES' ) then
            begin
                Result.Kind := bkYes ;
            end ;
        end ;
        // Ignore all others
    end ; // for Loop := 0 to Attribute_Names.Count - 1

    for Loop := 0 to Children.Count - 1
    begin
        Construct_From_Definition( Result, TVComponent_Definition( Children[ Loop ] ) ) ;
    end ;
*)
end ; // Construct_From_Definition


var Bitmap_Constructors : TList = nil ;

procedure Register_Bitmap_Constructor( O : TBitmap_Constructor ) ;

begin
    if( Bitmap_Constructors = nil ) then
    begin
        Bitmap_Constructors := TList.Create ;
    end ;
    Bitmap_Constructors.Add( O ) ;
end ;


function Get_Bitmap( Name : ansistring ) : TBitmap ;

var BC : TBitmap_Constructor ;
    Dummy : integer ;

begin
    Result := nil ;
    if( Bitmap_Constructors = nil ) then
    begin
        exit ;
    end ;
    for Dummy := 0 to Bitmap_Constructors.Count - 1 do
    begin
        BC := TBitmap_Constructor( Bitmap_Constructors[ Dummy ] ) ;
        Result := BC.Create_Image( Name ) ;
        if( Result <> nil ) then // Found the iamge
        begin
            exit ;
        end ;
    end ;
end ;


var Object_Constructors : TList = nil ;

procedure Register_Object_Constructor( O : TObject_Constructor ) ;

begin
    if( Object_Constructors = nil ) then
    begin
        Object_Constructors := TList.Create ;
    end ;
    Object_Constructors.Add( O ) ;
end ;


function Next_Line( var Def : ansistring ) : ansistring ;

var Attribute : ansistring ;
    Dummy, Dummy1 : integer ;

begin
    // Get next line...
    Dummy := pos( CR, Def + CR ) ;
    Dummy1 := pos( LF, Def + LF ) ;
    if( Dummy1 < Dummy ) then
    begin
        Dummy := Dummy1 ;
    end ;
    Result := trim( copy( Def, 1, Dummy - 1 ) ) ;
                                  
    // Strip line from definition...
    while( ( Dummy <= length( Def ) ) and ( pos( Def[ Dummy ], CRLF ) > 0 ) ) do
    begin
        inc( Dummy ) ; // Skip past all CRs and LFs
    end ;
    Def := copy( Def, Dummy, length( Def ) ) ;

    // Remove delegations...
    Dummy := pos( '=', Result ) ;
    Attribute := lowercase( trim( copy( Result, 1, Dummy - 1 ) ) ) ;
    if( Attribute = 'onchange' ) then
    begin
        Result := '' ;
    end else
    if( Attribute = 'onclick' ) then
    begin
        Result := '' ;
    end else
    if( Attribute = 'oncreate' ) then
    begin
        Result := '' ;
    end else
    if( Attribute = 'onresize' ) then
    begin
        Result := '' ;
    end ;
end ;


function Extract_Object_Definition( var S : ansistring ) : ansistring ;

var Attribute : ansistring ;
    Nest : integer ;
    Work : ansistring ;

begin
    Nest := 1 ;
    Result := '' ;
    while( ( length( S ) > 0 ) and ( Nest > 0 ) ) do
    begin
        Work := Next_Line( S ) ;
        Attribute := lowercase( trim( Work ) ) ;
        if( copy( Attribute, 1, 7 ) = 'object ' ) then
        begin
            inc( Nest ) ;
        end ;
        if( Attribute = 'end' ) then
        begin
            dec( Nest ) ;
            if( Nest = 0 ) then // End the outermost object
            begin
                exit ;
            end ;
        end ;
        Result := Result + Work + CR ;
    end ;
end ;


function Default_Construct( O : TObject ; Name, Typ : ansistring ; var S : ansistring ;
    Parent : TControl ; Owner : TComponent ) : TObject ; forward ;

function _Construct_Object_From_Definition( O : TObject ; Name, Typ, Def : ansistring ;
    Parent : TWinControl ; Owner : TComponent ) : TObject ;

var Attribute : ansistring ;
    Dummy : integer ;
    S, Work : ansistring ;
    Nest : integer ;
    OC : TObject_Constructor ;

begin
    // Setup...
    Result := O ;
    if( Def = '' ) then
    begin
        exit ;
    end ;
    if( Owner = nil ) then
    begin
        Owner := parent ;
    end ;

    // Handle case where we have a type...
    if( Typ <> '' ) then
    begin
        if( Object_Constructors <> nil ) then
        begin
            for Dummy := 0 to Object_Constructors.Count - 1 do
            begin
               OC := TObject_Constructor( Object_Constructors[ Dummy ] ) ;
               if( OC.Supported( Typ ) ) then
               begin
                   Result := OC.Construct( O, Name, Typ, Def, Parent, Owner ) ;
                   exit ;
               end ;
            end ; // for Dummy := 0 to Object_Constructors.Count - 1
        end ;
        Result := Default_Construct( O, Name, Typ, Def, Parent, Owner ) ;
        exit ;
    end ;

    // Process definition...
    while( Def <> '' ) do
    begin
        S := Next_Line( Def ) ;

        // Ignore anything that isn't an object definition
        if( uppercase( copy( S, 1, 7 ) ) <> 'OBJECT ' ) then
        begin
            continue ;
        end ;

        // Get component name and type...
        S := trim( copy( S, 8, length( S ) ) ) ;
        Dummy := pos( ':', S ) ;
        Name := trim( copy( S, 1, Dummy - 1 ) ) ;
        Typ := trim( copy( S, Dummy + 1, length( S ) ) ) ;

        // Get definition of the object...
        S := '' ;
        Nest := 1 ;
        while( ( Nest > 0 ) and ( Def <> '' ) ) do
        begin
            Work := Next_Line( Def ) ;
            Attribute := trim( lowercase( Work ) ) ;
            if( Attribute = 'end' ) then
            begin
                dec( Nest ) ;
                if( Nest > 0 ) then
                begin
                    S := S + 'end' + CR ;
                end ;
            end else
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                inc( Nest ) ;
                S := S + Work + CR ;
            end else
            begin
                S := S + Work + CR ;
            end ;
        end ;

        // Construct object...
        if( Object_Constructors <> nil ) then
        begin
            for Dummy := 0 to Object_Constructors.Count - 1 do
            begin
               OC := TObject_Constructor( Object_Constructors[ Dummy ] ) ;
               if( OC.Supported( Typ ) ) then
               begin
                   Result := OC.Construct( nil, Name, Typ, S, Parent, Owner ) ;
                   break ;
               end ;
            end ; // for Dummy := 0 to Object_Constructors.Count - 1
        end ;
        if( Result = nil ) then // Not constructed yet
        begin
            Result := Default_Construct( nil, Name, Typ, S, Parent, Owner ) ;
        end ;
    end ; // while( Def <> '' )
end ; // _Construct_Object_From_Definition


// Find a control by name that is a child of Parent
function FindObject( Parent : TWinControl ; Name : ansistring ) : TControl ;

var C : TControl ;
    Dummy : integer ;

begin
    Result := nil ;
    if( Parent = nil ) then
    begin
        exit ;
    end ;

    for Dummy := 0 to Parent.ControlCount - 1 do
    begin
        C := Parent.Controls[ Dummy ] ;
        if( lowercase( C.Name ) = Name ) then
        begin
            Result := C ;
            exit ;
        end ;
        if( C is TWinControl ) then
        begin
            C := FindObject( TWinControl( C ), Name ) ;
            if( C <> nil ) then
            begin
                Result := C ;
                exit ;
            end ;
        end ;
    end ; // for Dummy := 0 to Parent.ControlCount - 1
end ; // FindObject


var Focus_Controls : TStringList = nil ;

function Construct_Object_From_Definition( O : TObject ; Name, Typ, Def : ansistring ;
    Parent : TWinControl ; Owner : TComponent ) : TObject ;

var Dummy : integer ;
    C : TControl ;
    L : TLabel ;

begin
    Focus_Controls.Clear ;
    if( Owner = nil ) then
    begin
        Owner := Parent ;
    end ;
    Result := _Construct_Object_From_Definition( O, Name, Typ, Def, Parent, Owner ) ;
    if( Result <> nil ) then
    begin
        if( Result is TWinControl ) then
        begin
            for Dummy := 0 to Focus_Controls.Count - 1 do
            begin
                C := FindObject( TWinControl( Result ), Focus_Controls[ Dummy ] ) ;
                if( C <> nil ) then
                begin
                    L := TLabel( Focus_Controls.Objects[ Dummy ] ) ;
                    L.FocusControl := TWinControl( C ) ;
                end ;
            end ; // for Dummy := 0 to Focus_Controls.Count - 1
        end ; // if( Result is TWinControl )
    end ; // if( Result <> nil )
    Focus_Controls.Clear ;
end ; // Construct_Object_From_Definition


function Create_Child( Def, S : ansistring ; C : TControl ; Owner : TComponent ) : ansistring ;
// Construct a child object for C.  Def contains <name : type>, S contains the definition

var Dummy : integer ;
    O : TObject ;
    Temp : ansistring ;

begin
    Def := trim( Def ) ;
    Dummy := pos( ':', Def ) ;
    Temp := Extract_Object_Definition( S ) ;
    if( ( C <> nil ) and ( C is TWinControl ) ) then
    begin
        O := _Construct_Object_From_Definition( nil, trim( copy( Def, 1, Dummy - 1 ) ),
            trim( Copy( Def, Dummy + 1, length( Temp ) ) ), Temp,
            TWinControl( C ), Owner ) ;
        if( O is TControl ) then
        begin
            if( TControl( O ).Parent = nil ) then
            begin
                TControl( O ).Parent := TWinControl( C ) ;
            end ;
        end ;
    end else
    begin
        _Construct_Object_From_Definition( nil, trim( copy( Def, 1, Dummy - 1 ) ),
            trim( Copy( Def, Dummy + 1, length( Def ) ) ), Temp, nil,
            Owner ) ;
    end ;
    Result := S ;
end ;


function Construct_Form_From_Definition( Parent : TWinControl ;
    Def : ansistring ; Owner : TComponent ) : TObject ;

begin
    Result := Construct_Object_From_Definition( nil, '', '', Def, Parent, Owner ) ;
end ;


function Process_Component( O : TObject ; Def, Name : ansistring ;
    Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    C : TComponent ;
    Dummy : integer ;
    S, Temp : ansistring ;

begin
    C := TComponent( O ) ;
    C.Name := Name ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( C is TControl ) then
                begin
                    if( TControl( C ).Parent = nil ) then
                    begin
                        TControl( C ).Parent := TWinControl( Parent ) ;
                    end ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, TControl( C ), Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'tag' ) then
            begin
                C.Tag := strtoint( copy( S, Dummy + 1, length( S ) ) ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ; // if( S <> '' )
    end ; // while( Def <> '' )
end ; // Process_Component


function Color_From_Name( S : ansistring ) : TColor ;

begin
    S := lowercase( S ) ;
    if( S = 'clscrollbar' ) then Result := clscrollbar
    else if( S = 'clbackground' ) then Result := clbackground
    else if( S = 'clactivecaption' ) then Result := clactivecaption
    else if( S = 'clinactivecaption' ) then Result := clinactivecaption
    else if( S = 'clmenu' ) then Result := clmenu
    else if( S = 'clwindow' ) then Result := clwindow
    else if( S = 'clwindowframe' ) then Result := clwindowframe
    else if( S = 'clmenutext' ) then Result := clmenutext
    else if( S = 'clwindowtext' ) then Result := clwindowtext
    else if( S = 'clcaptiontext' ) then Result := clcaptiontext
    else if( S = 'clactiveborder' ) then Result := clactiveborder
    else if( S = 'clinactiveborder' ) then Result := clinactiveborder
    else if( S = 'clappworkspace' ) then Result := clappworkspace
    else if( S = 'clhighlight' ) then Result := clhighlight
    else if( S = 'clhighlighttext' ) then Result := clhighlighttext
    else if( S = 'clbtnface' ) then Result := clbtnface
    else if( S = 'clbtnshadow' ) then Result := clbtnshadow
    else if( S = 'clgraytext' ) then Result := clgraytext
    else if( S = 'clbtntext' ) then Result := clbtntext
    else if( S = 'clinactivecaptiontext' ) then Result := clinactivecaptiontext
    else if( S = 'clbtnhighlight' ) then Result := clbtnhighlight
    else if( S = 'cl3ddkshadow' ) then Result := cl3ddkshadow
    else if( S = 'cl3dlight' ) then Result := cl3dlight
    else if( S = 'clinfotext' ) then Result := clinfotext
    else if( S = 'clinfobk' ) then Result := clinfobk
    else if( S = 'clhotlight' ) then Result := clhotlight
    else if( S = 'clgradientactivecaption' ) then Result := clgradientactivecaption
    else if( S = 'clgradientinactivecaption' ) then Result := clgradientinactivecaption
    else if( S = 'clmenuhighlight' ) then Result := clmenuhighlight
    else if( S = 'clmenubar' ) then Result := clmenubar
    else if( S = 'clblack' ) then Result := clblack
    else if( S = 'clmaroon' ) then Result := clmaroon
    else if( S = 'clgreen' ) then Result := clgreen
    else if( S = 'clolive' ) then Result := clolive
    else if( S = 'clnavy' ) then Result := clnavy
    else if( S = 'clpurple' ) then Result := clpurple
    else if( S = 'clteal' ) then Result := clteal
    else if( S = 'clgray' ) then Result := clgray
    else if( S = 'clsilver' ) then Result := clsilver
    else if( S = 'clred' ) then Result := clred
    else if( S = 'cllime' ) then Result := cllime
    else if( S = 'clyellow' ) then Result := clyellow
    else if( S = 'clblue' ) then Result := clblue
    else if( S = 'clfuchsia' ) then Result := clfuchsia
    else if( S = 'claqua' ) then Result := claqua
    else if( S = 'clltgray' ) then Result := clltgray
    else if( S = 'cldkgray' ) then Result := cldkgray
    else if( S = 'clwhite' ) then Result := clwhite
    else if( S = 'clmoneygreen' ) then Result := clmoneygreen
    else if( S = 'clskyblue' ) then Result := clskyblue
    else if( S = 'clcream' ) then Result := clcream
    else if( S = 'clmedgray' ) then Result := clmedgray
    else raise Exception.Create( 'Invalid color mnuemonic' ) ;
end ;


function Get_String( S : ansistring ; var Def : ansistring ) : ansistring ;

var Dummy : integer ;
    Temp : ansistring ;

begin
    Dummy := pos( '=', S ) ;
    S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
    if( S = '' ) then // Must start on the next line
    begin
        S := trim( Next_Line( Def ) ) ;
    end ;
    while( copy( S, length( S ) - 2, 3 ) = #39' +' ) do // Concatenation
    begin
        setlength( S, length( S ) - 3 ) ; // Remove trailing quote and +
        Temp := trim( Next_Line( Def ) ) ;
        Temp := copy( Temp, 2, length( Temp ) ) ; // Trim initial quote of next line
        S := S + Temp ;
    end ;
    S := copy( S, 2, length( S ) ) ;
    setlength( S, length( S ) - 1 ) ; // Remove quotes
    Result := S ;
end ;


function Process_Control( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Align : TAlign ;
    BiDiMode : TBiDiMode ;
    C : TControl ;
    DockOrientation : TDockOrientation ;
    Dummy : integer ;
    S, Temp : ansistring ;

const Aligns : array[ TAlign ] of string = ( 'alnone',
                                             'altop',
                                             'albottom',
                                             'alleft',
                                             'alright',
                                             'alclient',
                                             'alcustom'
                                           ) ;

const BiDiModes : array[ TBiDiMode ] of string = ( 'bdlefttoright',
                                                   'bdrighttoleft',
                                                   'bdrighttoleftnoalign',
                                                   'bdrighttoleftreadingonly'
                                                 ) ;

const DockOrientations : array[ TDockOrientation ] of string = ( 'donoorient',
                                                                 'dohorizontal',
                                                                 'dovertical'
{$IFDEF FPC}
                                                                ,'doPages'
{$ENDIF}
                                                                 ) ;

begin // Process_Control
    Result := '' ;
    C := TControl( O ) ;
    if( C.Parent = nil ) then
    begin
        if( Parent is TWinControl ) then
        begin
            C.Parent := TWinControl( Parent ) ;
        end ;
    end ;
    Def := Process_Component( O, Def, Name, Parent ) ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( C.Parent = nil ) then
                begin
                    C.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, C, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'align' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for Align := low( Align ) to high( Align ) do
                begin
                    if( S = Aligns[ Align ] ) then
                    begin
                        C.Align := Align ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'autosize' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TImage( C ).Autosize := ( S = 'true' ) ;
            end else
            if( Attribute = 'bidimode' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for BiDiMode := low( BiDiMode ) to high( BiDiMode ) do
                begin
                    if( S = BiDiModes[ BiDiMode ] ) then
                    begin
                        C.BiDiMode := BiDiMode ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'caption' ) then
            begin
                S := Get_String( S, Def ) ;
                TLabel( C ).Caption := S ;
            end else
            if( Attribute = 'dockorientation' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for DockOrientation := low( DockOrientation ) to high( DockOrientation ) do
                begin
                    if( S = DockOrientations[ DockOrientation ] ) then
                    begin
                        C.DockOrientation := DockOrientation ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'dragkind' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                if( S = 'dkdrag' ) then
                begin
                    TGroupBox( C ).DragKind := dkDrag ;
                end else
                if( S = 'dkdock' ) then
                begin
                    TGroupBox( C ).DragKind := dkDock ;
                end ;
            end else
            if( Attribute = 'dragmode' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                if( S = 'dmmanual' ) then
                begin
                    TGroupBox( C ).Dragmode := dmManual ;
                end else
                if( S = 'dmautomatic' ) then
                begin
                    TGroupBox( C ).DragMode := dmAutomatic ;
                end ;
            end else
            if( Attribute = 'enabled' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Enabled := ( S = 'true' ) ;
            end else
            if( Attribute = 'font.charset' ) then
            begin
                //S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
               // C.Font.Charset := ~~~ ;
            end else
            if( Attribute = 'font.color' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                THintWindow( C ).Font.Color := Color_From_Name( S ) ;
            end else
            if( Attribute = 'font.height' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                THintWindow( C ).Font.height := strtoint( S ) ;
            end else
            if( Attribute = 'font.name' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                S := copy( S, 2, length( S ) ) ;
                setlength( S, length( S ) - 1 ) ; // Remove quotes
                THintWindow( C ).Font.Name := S ;
            end else
            if( Attribute = 'font.style' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                S := copy( S, 2, length( S ) ) ;
                setlength( S, length( S ) - 1 ) ; // Remove brackets
                THintWindow( C ).Font.Style := [] ;
            end else
            if( Attribute = 'height' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Height := strtoint( S ) ;
            end else
            if( Attribute = 'helpcontext' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.HelpContext := strtoint( S ) ;
            end else
            if( Attribute = 'helpkeyword' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                S := copy( S, 2, length( S ) ) ;
                setlength( S, length( S ) - 1 ) ; // Remove quotes
                C.HelpKeyword := S ;
            end else
            if( Attribute = 'helptype' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                if( S = 'htkeyword' ) then
                begin
                    C.HelpType := htKeyword ;
                end else
                if( S = 'htcontext' ) then
                begin
                    C.HelpType := htContext ;
                end ;
            end else
            if( Attribute = 'hint' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                S := copy( S, 2, length( S ) ) ;
                setlength( S, length( S ) - 1 ) ; // Remove quotes
                C.Hint := S ;
            end else
            if( Attribute = 'left' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Left := strtoint( S ) ;
            end else
            if( Attribute = 'lrdockwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.LRDockWidth := strtoint( S ) ;
            end else
            if( Attribute = 'parentcolor' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TGroupBox( C ).ParentColor := ( S = 'true' ) ;
            end else
            if( Attribute = 'parentbidimode' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TGroupBox( C ).ParentBiDiMode := ( S = 'true' ) ;
            end else
            if( Attribute = 'parentfont' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TGroupBox( C ).ParentFont := ( S = 'true' ) ;
            end else
            if( Attribute = 'parentshowhint' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TLabel( C ).ParentShowHint := ( S = 'true' ) ;
            end else
            if( Attribute = 'showhint' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.ShowHint := ( S = 'true' ) ;
            end else
            if( Attribute = 'tbdockheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.TBDockHeight := strtoint( S ) ;
            end else
            if( Attribute = 'text' ) then
            begin
                TEdit( C ).Text := Get_String( S, Def ) ;
            end else
            if( Attribute = 'top' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Top := strtoint( S ) ;
            end else
            if( Attribute = 'undockheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.UndockHeight := strtoint( S ) ;
            end else
            if( Attribute = 'undockwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.UndockWidth := strtoint( S ) ;
            end else
            if( Attribute = 'visible' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Visible := ( S = 'true' ) ;
            end else
            if( Attribute = 'width' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Width := strtoint( S ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
end ; // Process_Control


function String_To_Alignment( S : ansistring ) : TAlignment ;

var Alignment : TAlignment ;

const Alignments : array[ TAlignment ] of string =
    ('taLeftJustify', 'taRightJustify', 'taCenter') ;

begin
    Result := taLeftJustify ;
    S := lowercase( S ) ;
    for Alignment := low( Alignment ) to high( Alignment ) do
    begin
        if( S = Alignments[ Alignment ] ) then
        begin
            Result := Alignment ;
            break ;
        end ;
    end ;
end ;




function Process_Label( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
{$IFDEF DELPHI8UP}
    EllipsisPosition : TEllipsisPosition ;
{$ENDIF}
    L : TLabel ;
    S, Temp : ansistring ;
    TextLayout : TTextLayout ;

const TextLayouts : array[ TTextLayout ] of string =
    ('tlTop', 'tlCenter', 'tlBottom');
{$IFDEF DELPHI8UP}
const EllipsisPositions : array[ TEllipsisPosition ] of string =
    ('epNone', 'epPathEllipsis', 'epEndEllipsis', 'epWordEllipsis');
{$ENDIF}

begin
    L := TLabel( O ) ;
    Def := Process_Control( O, Def, Name, Parent ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( L.Parent = nil ) then
                begin
                    L.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, L, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'alignment' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Alignment := String_To_Alignment( S ) ;
            end else
            if( Attribute = 'autosize' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Autosize := ( S = 'true' ) ;
            end else
            if( Attribute = 'color' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Color := String_To_Color( S ) ;
            end else
            if( Attribute = 'ellipsisposition' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFDEF DELPHI8UP}
                for EllipsisPosition := low( EllipsisPosition ) to high( EllipsisPosition ) do
                begin
                    if( S = EllipsisPositions[ EllipsisPosition ] ) then
                    begin
                        L.EllipsisPosition := EllipsisPosition ;
                        break ;
                    end ;
                end ;
{$ENDIF}
            end else
            if( Attribute = 'focuscontrol' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Focus_Controls.AddObject( S, L ) ;
            end else
            if( Attribute = 'layout' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for TextLayout := low( TextLayout ) to high( TextLayout ) do
                begin
                    if( S = TextLayouts[ TextLayout ] ) then
                    begin
                        L.Layout := TextLayout ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'showaccelchar' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.ShowAccelChar := ( S = 'true' ) ;
            end else
            if( Attribute = 'transparent' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Transparent := ( S = 'true' ) ;
            end else
            if( Attribute = 'wordwrap' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Wordwrap := ( S = 'true' ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ; // if( S <> '' ) 
    end ; // while( Def <> '' )
end ; // Process_Label


function String_To_BorderStyle( S : ansistring ) : TBorderStyle ;

begin
    if( S = 'bssingle' ) then
    begin
        Result := bsSingle ;
    end else
    begin
        Result := bsnone ;
    end ;
end ;


function String_To_BevelCut( S : ansistring ) : TBevelCut ;

var BevelCut : TBevelCut ;

const BevelCuts : array [ TBevelCut ] of string =
    ('bvNone', 'bvLowered', 'bvRaised', 'bvSpace');

begin
    Result := bvNone ;
    for BevelCut := low( BevelCut ) to high( BevelCut ) do
    begin
        if( S = BevelCuts[ BevelCut ] ) then
        begin
            Result := BevelCut ;
            exit ;
        end ;
    end ;
end ;


{$IFNDEF FPC}
function String_To_BevelEdges( S : ansistring ) : TBevelEdges ;

var BevelEdge : TBevelEdge ;
    Dummy : integer ;
    Work : ansistring ;

const BevelEdges : array [ TBevelEdge ] of string =
    ('beLeft', 'beTop', 'beRight', 'beBottom') ;

begin
    Result := [] ;
    while( S <> '' ) do
    begin
        Dummy := pos( ',', S + ',' ) ;
        Work := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
        S := trim( copy( S, Dummy + 1, length( S ) ) ) ;
        for BevelEdge := low( BevelEdge ) to high( BevelEdge ) do
        begin
            if( Work = BevelEdges[ BevelEdge ] ) then
            begin
                Result := Result + [ BevelEdge ] ;
                break ;
            end ;
        end ;
    end ;
end ;
{$ENDIF}


function Process_WinControl( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
{$IFNDEF FPC}
    BevelKind : TBevelKind ;
    ImeMode : TImeMode ;
{$ENDIF}
    Dummy : integer ;
    S, Temp : ansistring ;
    W : TWinControl ;

{$IFNDEF FPC}
const BevelKinds : array[ TBevelKind ] of string =
    ('bkNone', 'bkTile', 'bkSoft', 'bkFlat') ;

const ImeModes : array [ TImeMode ] of string = ( 'imDisable',
                                                  'imClose',
                                                  'imOpen',
                                                  'imDontCare',
                                                  'imSAlpha',
                                                  'imAlpha',
                                                  'imHira',
                                                  'imSKata',
                                                  'imKata',
                                                  'imChinese',
                                                  'imSHanguel',
                                                  'imHanguel'
                                                );
{$ENDIF}

begin
    W := TWinControl( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( W.Parent = nil ) then
                begin
                    W.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, W, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'ctl3d' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                TGroupBox( W ).ctl3d := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'beveledges' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                S := copy( S, 2, length( S ) ) ;
                S := copy( S, 1, length( S ) - 1 ) ; // Trim brackets
                TEdit( W ).BevelEdges := String_To_BevelEdges( S ) ;
{$ENDIF}
            end else
            if( Attribute = 'bevelinner' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                TEdit( W ).BevelInner := String_To_BevelCut( S ) ;
{$ENDIF}
            end else
            if( Attribute = 'bevelkind' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                for BevelKind := low( BevelKind ) to high( BevelKind ) do
                begin
                    if( S = BevelKinds[ BevelKind ] ) then
                    begin
                        TEdit( W ).BevelKind := BevelKind ;
                        break ;
                    end ;
                end ;
{$ENDIF}
            end else
            if( Attribute = 'bevelouter' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                TEdit( W ).BevelOuter := String_To_BevelCut( S ) ;
{$ENDIF}
            end else
            if( Attribute = 'bevelwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFDEF DELPHI8UP}
                TEdit( W ).BevelWidth := strtoint( S ) ;
{$ENDIF}
            end else
            if( Attribute = 'borderwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TPanel( W ).BorderWidth := strtoint( S ) ;
            end else
            if( Attribute = 'docksite' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFDEF DELPHI8UP}
                W.Docksite := ( S = 'true' ) ;
{$ENDIF}
            end else
{$IFNDEF FPC}
            if( Attribute = 'imemode' ) then
            begin
                for ImeMode := low( ImeMode ) to high( ImeMode ) do
                begin
                    if( S = ImeModes[ ImeMode ] ) then
                    begin
                        TEdit( W ).ImeMode := ImeMode ;
                        break ;
                    end ;
                end ;
            end else
{$ENDIF}
            if( Attribute = 'imename' ) then
            begin
{$IFNDEF FPC}
                TEdit( W ).Imename := Get_String( S, Def ) ;
{$ENDIF}
            end else
            if( Attribute = 'parentctl3d' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                TGroupBox( W ).parentctl3d := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'taborder' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                W.Taborder := strtoint( S ) ;
            end else
            if( Attribute = 'tabstop' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                W.Tabstop := ( S = 'true' ) ;
            end else
            if( Attribute = 'usedockmanager' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFDEF DELPHI8UP}
                W.UseDockManager := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'end' ) then
            begin
                Result := Result + CR + 'end' + CR + Def ;
                break ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ; // if( S <> '' )
    end ; // while( Def <> '' )
    Result := Process_Control( O, Result, Name, Parent ) ;
end ; // Process_WinControl


function Process_Panel( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    P : TPanel ;
    S, Temp : ansistring ;
{$IFNDEF FPC}
  {$IFDEF DELPHI8UP}
    VerticalAlignment : TVerticalAlignment ;

const VerticalAlignments : array[ TVerticalAlignment ] of string =
    ('taAlignTop', 'taAlignBottom', 'taVerticalCenter');
  {$ENDIF}
{$ENDIF}

begin
    P := TPanel( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( P.Parent = nil ) then
                begin
                    P.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, P, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'alignment' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.Alignment := String_To_Alignment( S ) ;
            end else
            if( Attribute = 'bevelinner' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.BevelInner := String_To_BevelCut( S ) ;
            end else
            if( Attribute = 'bevelouter' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.BevelOuter := String_To_BevelCut( S ) ;
            end else
            if( Attribute = 'bevelwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.Bevelwidth := strtoint( S ) ;
            end else
            if( Attribute = 'borderstyle' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.BorderStyle := String_To_BorderStyle( S ) ;
            end else
            if( Attribute = 'borderwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.Borderwidth := strtoint( S ) ;
            end else
            if( Attribute = 'fullrepaint' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                P.FullRepaint := ( S = 'true' ) ;
            end else
            if( Attribute = 'locked' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                P.Locked := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'verticalalignment' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
  {$IFDEF DELPHI8UP}
                for VerticalAlignment := low( VerticalAlignment ) to high( VerticalAlignment ) do
                begin
                    if( S = VerticalAlignments[ VerticalAlignment ] ) then
                    begin
                        P.VerticalAlignment := VerticalAlignment ;
                        break ;
                    end ;
                end ;
  {$ENDIF}
{$ENDIF}
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_WinControl( O, Result, Name, Parent ) ;
end ; // Process_Panel


function Process_Edit( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    E : TEdit ;
    S, Temp : ansistring ;

begin
    E := TEdit( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( E.Parent = nil ) then
                begin
                    E.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, E, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'autoselect' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                E.AutoSelect := ( S = 'true' ) ;
            end else
            if( Attribute = 'autosize' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                E.AutoSize := ( S = 'true' ) ;
            end else
            if( Attribute = 'borderstyle' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                E.BorderStyle := String_To_BorderStyle( S ) ;
            end else
            if( Attribute = 'hideselection' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                E.HideSelection := ( S = 'true' ) ;
            end else
            if( Attribute = 'maxlength' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                E.MaxLength := strtoint( S ) ;
            end else
            if( Attribute = 'oemconvert' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                E.oemConvert := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'readonly' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                E.readonly := ( S = 'true' ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_WinControl( O, Result, Name, Parent ) ;
end ; // Process_Edit


function String_To_ModalResult( S : ansistring ) : TModalResult ;

begin
    S := lowercase( S ) ;
    if( S = 'mrOk' ) then Result := mrOk
    else if( S = 'mrCancel' ) then Result := mrCancel
    else if( S = 'mrAbort' ) then Result := mrAbort
    else if( S = 'mrRetry' ) then Result := mrRetry
    else if( S = 'mrIgnore' ) then Result := mrIgnore
    else if( S = 'mrYes' ) then Result := mrYes
    else if( S = 'mrNo' ) then Result := mrNo
    else if( S = 'mrAll' ) then Result := mrAll
    else if( S = 'mrNoToAll' ) then Result := mrNoToAll
    else if( S = 'mrYesToAll' ) then Result := mrYesToAll
    else Result := mrNone ;
end ;


function Process_Button( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    B : TButton ;
    Dummy : integer ;
    S, Temp : ansistring ;

begin
    B := TButton( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( B.Parent = nil ) then
                begin
                    B.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, B, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'cancel' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                B.Cancel := ( S = 'true' ) ;
            end else
            if( Attribute = 'default' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                B.Default := ( S = 'true' ) ;
            end else
            if( Attribute = 'glyph.data' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                // Ignored (for now)
            end else
            if( Attribute = 'modalresult' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                B.ModalResult := String_To_ModalResult( S ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_WinControl( O, Result, Name, Parent ) ;
end ; // Process_Button


function Process_Bitbtn( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    B : TBitbtn ;
    BitBtnKind : TBitBtnKind ;
    ButtonLayout : TButtonLayout ;
{$IFNDEF FPC}
    ButtonStyle : TButtonStyle ;
{$ENDIF}
    Dummy : integer ;
    S, Temp : ansistring ;

{$IFNDEF FPC}
const ButtonStyles : array [ TButtonStyle ] of string = ( 'bsAutoDetect', 'bsWin31', 'bsNew' ) ;
{$ENDIF}

const BitBtnKinds : array[ TBitBtnKind ] of string = ( 'bkCustom',
                                                       'bkOK',
                                                       'bkCancel',
                                                       'bkHelp',
                                                       'bkYes',
                                                       'bkNo',
                                                       'bkClose',
                                                       'bkAbort',
                                                       'bkRetry',
                                                       'bkIgnore',
                                                       'bkAll'
{$IFDEF FPC}
                                                      ,'bkNoToAll',
                                                       'bkYesToAll'
{$ENDIF}
                                                       );

const ButtonLayouts : array[ TButtonLayout ] of string =
    ( 'blGlyphLeft', 'blGlyphRight', 'blGlyphTop', 'blGlyphBottom' );

begin
    B := TBitbtn( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( B.Parent = nil ) then
                begin
                    B.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, B, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'kind' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for BitBtnKind := low( BitBtnKind ) to high( BitBtnKind ) do
                begin
                    if( S = BitBtnKinds[ BitBtnKind ] ) then
                    begin
                        B.Kind := BitBtnKind ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'layout' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for ButtonLayout := low( ButtonLayout ) to high( ButtonLayout ) do
                begin
                    if( S = ButtonLayouts[ ButtonLayout ] ) then
                    begin
                        B.Layout := ButtonLayout ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'margin' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                B.Margin := strtoint( S ) ;
            end else
            if( Attribute = 'spacing' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                B.Spacing := strtoint( S ) ;
            end else
            if( Attribute = 'style' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                for ButtonStyle := low( ButtonStyle ) to high( ButtonStyle ) do
                begin
                    if( S = ButtonStyles[ ButtonStyle ] ) then
                    begin
                        B.Style := ButtonStyle ;
                        break ;
                    end ;
                end ;
{$ENDIF}
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Button( O, Result, Name, Parent ) ;
end ; // Process_Bitbtn


function Process_MultiSelectListControl( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

{$IFNDEF FPC}
var Attribute : ansistring ;
    C : TCustomMultiSelectListControl ;
    Dummy : integer ;
    S, Temp : ansistring ;
{$ENDIF}

begin
{$IFNDEF FPC}
    C := TCustomMultiSelectListControl( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( lowercase( trim( copy( S, 1, 7 ) ) ) = 'object ' ) then
            begin
                if( C.Parent = nil ) then
                begin
                    C.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, C, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'multiselect' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.MultiSelect := ( S = 'true' ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
{$ELSE}
    Result := Def ;
{$ENDIF}
    Result := Process_Wincontrol( O, Result, Name, Parent ) ;
end ; // Process_MultiSelectListControl


function Process_Listbox( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    L : TListBox ;
    ListBoxStyle : TListBoxStyle ;
    S, Temp : ansistring ;

const ListBoxStyles : array[ TListBoxStyle ] of string = ( 'lbStandard',
                                                           'lbOwnerDrawFixed',
                                                           'lbOwnerDrawVariable',
                                                           'lbVirtual'
{$IFNDEF FPC}
                                                          ,'lbVirtualOwnerDraw'
{$ENDIF}
                                                         ) ;

begin
    L := TListBox( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( L.Parent = nil ) then
                begin
                    L.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, L, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'borderstyle' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.BorderStyle := String_To_BorderStyle( S ) ;
            end else
            if( Attribute = 'columns' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Columns := strtoint( S ) ;
            end else
            if( Attribute = 'extendedselect' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.ExtendedSelect := ( S = 'true' ) ;
            end else
            if( Attribute = 'integralheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.IntegralHeight := ( S = 'true' ) ;
            end else
            if( Attribute = 'itemheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.ItemHeight := strtoint( S ) ;
            end else
            if( Attribute = 'sorted' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                L.Sorted := ( S = 'true' ) ;
            end else
            if( Attribute = 'style' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for ListBoxStyle := low( ListBoxStyle ) to high( ListBoxStyle ) do
                begin
                    if( S = ListBoxStyles[ ListBoxStyle ] ) then
                    begin
                        L.Style := ListBoxStyle ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'tabwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                L.TabWidth := strtoint( S ) ;
{$ENDIF}
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_MultiSelectListControl( O, Result, Name, Parent ) ;
end ; // Process_Listbox


function Process_Splitter( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    ResizeStyle : TResizeStyle ;
    S, Temp : ansistring ;
    Sp : TSplitter ;

const ResizeStyles : array [ TResizeStyle ] of string = ('rsNone', 'rsLine', 'rsUpdate', 'rsPattern');

begin
    Sp := TSplitter( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( Sp.Parent = nil ) then
                begin
                    Sp.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, Sp, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'autosnap' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sp.Autosnap := ( S = 'true' ) ;
            end else
            if( Attribute = 'beveled' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sp.Beveled := ( S = 'true' ) ;
            end else
            if( Attribute = 'minsize' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sp.Minsize := strtoint( S ) ;
            end else
            if( Attribute = 'resizestyle' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for ResizeStyle := low( ResizeStyle ) to high( ResizeStyle ) do
                begin
                    if( S = ResizeStyles[ ResizeStyle ] ) then
                    begin
                        Sp.ResizeStyle := ResizeStyle ;
                        break ;
                    end ;
                end ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Control( O, Result, Name, Parent ) ;
end ; // Process_Splitter


function Process_Scrollbar( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    S, Temp : ansistring ;
    Sc : TScrollbar ;

begin
    Sc := TScrollbar( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( Sc.Parent = nil ) then
                begin
                    Sc.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, Sc, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'kind' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                if( S = 'sbHorizontal' ) then
                begin
                    Sc.Kind := sbHorizontal ;
                end else
                begin
                    Sc.Kind := sbVertical ;
                end ;
            end else
            if( Attribute = 'largechange' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sc.Largechange := strtoint( S ) ;
            end else
            if( Attribute = 'max' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sc.max := strtoint( S ) ;
            end else
            if( Attribute = 'min' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sc.min := strtoint( S ) ;
            end else
            if( Attribute = 'pagesize' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sc.Pagesize := strtoint( S ) ;
            end else
            if( Attribute = 'position' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sc.Position := strtoint( S ) ;
            end else
            if( Attribute = 'smallchange' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sc.Smallchange := strtoint( S ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Wincontrol( O, Result, Name, Parent ) ;
end ; // Process_Scrollbar


function Process_Tabsheet( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    S, Temp : ansistring ;
    T : TTabSheet ;

begin
    T := TTabSheet( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( T.Parent = nil ) then
                begin
                    T.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, T, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'highlighted' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                T.Highlighted := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'imageindex' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                T.Imageindex := strtoint( S ) ;
            end else
            if( Attribute = 'pageindex' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                T.Pageindex := strtoint( S ) ;
            end else
            if( Attribute = 'tabvisible' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                T.Tabvisible := ( S = 'true' ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Wincontrol( O, Result, Name, Parent ) ;
end ; // Process_Tabsheet


function Process_Buttoncontrol( O : TObject ; Def, Name : ansistring ;
    Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    B : TButtonControl ;
    Dummy : integer ;
    S, Temp : ansistring ;

begin
    B := TButtonControl( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( B.Parent = nil ) then
                begin
                    B.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, B, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'checked' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TCheckBox( B ).Checked := ( S = 'true' ) ;
            end else
            if( Attribute = 'wordwrap' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                TButton( B ).Wordwrap := ( S = 'true' ) ;
{$ENDIF}
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Wincontrol( O, Result, Name, Parent ) ;
end ; // Process_Buttoncontrol


function Process_RadioButton( O : TObject ; Def, Name : ansistring ;
    Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    R : TRadioButton ;
    S, Temp : ansistring ;

begin
    R := TRadioButton( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( R.Parent = nil ) then
                begin
                    R.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, R, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'alignment' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                R.Alignment := String_To_Alignment( S ) ;
{$ENDIF}
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Buttoncontrol( O, Result, Name, Parent ) ;
end ; // Process_RadioButton


function String_To_CheckBoxState( S : ansistring ) : TCheckBoxState ;

var CheckBoxState : TCheckBoxState ;

const CheckBoxStates : array[ TCheckBoxState ] of string = ('cbUnchecked', 'cbChecked', 'cbGrayed');

begin
    Result := cbUnchecked ;
    for CheckBoxState := low( CheckBoxState ) to high( CheckBoxState ) do
    begin
        if( S = CheckBoxStates[ CheckBoxState ] ) then
        begin
            Result := CheckBoxState ;
            break ;
        end ;
    end ;
end ;


function Process_CheckBox( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    C : TCheckBox ;
    Dummy : integer ;
    S, Temp : ansistring ;

begin
    C := TCheckBox( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( C.Parent = nil ) then
                begin
                    C.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, C, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'alignment' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                C.Alignment := String_To_Alignment( S ) ;
{$ENDIF}
            end else
            if( Attribute = 'allowgrayed' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.AllowGrayed := ( S = 'true' ) ;
            end else
            if( Attribute = 'state' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.State := String_To_CheckBoxState( S ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Buttoncontrol( O, Result, Name, Parent ) ;
end ; // Process_CheckBox


function Process_customcombo( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

{$IFNDEF FPC}
var Attribute : ansistring ;
    C : TCustomCombo ;
    Dummy : integer ;
    S, Temp : ansistring ;
{$ENDIF}

begin
{$IFNDEF FPC}
    C := TCustomCombo( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( C.Parent = nil ) then
                begin
                    C.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, C, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( copy( Attribute, 1, 7 ) = 'maxlength' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                TComboBox( C ).MaxLength := strtoint( S ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
{$ELSE}
    Result := Def ;
{$ENDIF}
    Result := Process_Wincontrol( O, Result, Name, Parent ) ;
end ;


function Process_List( S : ansistring ; var Def : ansistring ) : TStringList ;

var C : Ansichar ;
    Dummy : integer ;
    Work : ansistring ;

begin
    // Setup...
    Result := TStringList.Create ;

    // Find opening of list...
    while( pos( '(', S ) = 0 ) do
    begin
        S := S + Def ;
    end ;
    S := trim( copy( S, pos( '(', S ) + 1, length( S ) ) ) ;

    // Process list...
    Work := '' ;
    while( ( Def <> '' ) or ( S <> '' ) ) do
    begin
        if( S = '' ) then
        begin
            if( Work <> '' ) then
            begin
                Result.Add( Work ) ;
                Work := '' ;
            end ;
            S := Next_Line( Def ) ;
        end ;
        C := S[ 1 ] ;
        if( C = #39 ) then
        begin
            S := copy( S, 2, length( S ) ) ; // Trim quote
            Dummy := pos( #39, S ) ;
            Work := Work + copy( S, 1, Dummy - 1 ) ;
            S := trim( copy( S, Dummy + 1, length( S ) ) ) ;
            if( copy( S, 1, 1 ) = '+' ) then
            begin
                S := Next_Line( Def ) ;
                continue ;
            end ;
        end else
        if( C = ',' ) then // End of item
        begin
            Result.Add( Work ) ;
            Work := '' ;
            S := trim( copy( S, 2, length( S ) ) ) ;
        end else
        if( C = ')' ) then // End of list
        begin
            Result.Add( Work ) ;
            break ;
        end else
        begin
            raise Exception.Create( 'Unknown character in list: ' + C ) ;
        end ;
    end ; // while( Def <> '' )
end ; // Process_List


function Process_combobox( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    C : TComboBox ;
    Dummy : integer ;
    Have_ItemIndex : boolean ;
    ItemIndex : integer ;
    List : TStringList ;
    S, Temp : ansistring ;
    ComboBoxStyle : TComboBoxStyle ;
    EditCharCase : TEditCharCase ;

const ComboBoxStyles : array[ TComboBoxStyle ] of string = ( 'csDropDown',
                                                             'csSimple',
                                                             'csDropDownList',
                                                             'csOwnerDrawFixed',
                                                             'csOwnerDrawVariable' ) ;
const EditCharCases : array[ TEditCharCase ] of string = ( 'ecNormal', 'ecUpperCase', 'ecLowerCase' ) ;

begin
    Have_ItemIndex := False ;
    ItemIndex := -1 ;
    C := TComboBox( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute := lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( C.Parent = nil ) then
                begin
                    C.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, C, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'autocloseup' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                C.AutoCloseUp := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'autocomplete' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.AutoComplete := ( S = 'true' ) ;
            end else
            if( Attribute = 'autocompletedelay' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
  {$IFDEF DELPHI8UP}
                C.AutoCompleteDelay := strtoint( S ) ;
  {$ENDIF}
{$ENDIF}
            end else
            if( Attribute = 'autodropdown' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.AutoDropDown := ( S = 'true' ) ;
            end else
            if( Attribute = 'charcase' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for EditCharCase := low( EditCharCase ) to high( EditCharCase ) do
                begin
                    if( S = EditCharCases[ EditCharCase ] ) then
                    begin
                        C.CharCase := EditCharCase ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'itemheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.ItemHeight := strtoint( S ) ;
            end else
            if( Attribute = 'itemindex' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Have_ItemIndex := True ;
                ItemIndex := strtoint( S ) ;
            end else
            if( Attribute = 'items.strings' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                List := Process_List( S, Def ) ;
                C.Parent := TWinControl( Parent ) ;
                C.Items.Assign( List ) ;
                List.Free ;
            end else
            if( Attribute = 'sorted' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                C.Sorted := ( S = 'true' ) ;
            end else
            if( Attribute = 'style' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for ComboBoxStyle := low( ComboBoxStyle ) to high( ComboBoxStyle ) do
                begin
                    if( S = ComboBoxStyles[ ComboBoxStyle ] ) then
                    begin
                        C.Style := ComboBoxStyle ;
                        break ;
                    end ;
                end ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ; // if( S <> '' ) 
    end ; // while( Def <> '' )
    if( Have_ItemIndex ) then
    begin
        C.ItemIndex := ItemIndex ;
    end ;
    Result := Process_customcombo( O, Result, Name, Parent ) ;
end ; // Process_combobox


function String_To_BorderIcons( S : ansistring ) : TBorderIcons ;

const BorderIcons : array[ TBorderIcon ] of string =
    ( 'biSystemMenu', 'biMinimize', 'biMaximize', 'biHelp' ) ;

var Dummy : integer ;
    Work : ansistring ;

begin
    Result := [] ;
    S := uppercase( S ) ;
    while( S <> '' ) do
    begin
        Dummy := pos( ',', S ) ;
        if( Dummy = 0 ) then
        begin
            Dummy := length( S ) + 1 ;
        end ;
        Work := copy( S, 1, Dummy - 1 ) ;
        S := copy( S, Dummy + 1, length( S ) ) ;
        if( Work = 'BISYSTEMMENU' ) then
        begin
            Result := Result + [biSystemMenu] ;
        end else
        if( Work = 'BIMINIMIZE' ) then
        begin
            Result := Result + [biMinimize] ;
        end else
        if( Work = 'BIMAXIMIZE' ) then
        begin
            Result := Result + [biMaximize] ;
        end else
        if( Work = 'BIHELP' ) then
        begin
            Result := Result + [biHelp] ;
        end ;
    end ;
end ; // String_To_BorderIcons


function Process_Form( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    F : TForm ;
    FormStyle : TFormStyle ;
{$IFNDEF FPC}
    PrintScale : TPrintScale ;
{$ENDIF}
    S, Temp : ansistring ;

const FormStyles : array [ TFormStyle ] of string = ( 'fsNormal',
                                                      'fsMDIChild',
                                                      'fsMDIForm',
                                                      'fsStayOnTop'
{$IFDEF FPC}
                                                     ,'fsSplash',
                                                      'fsSystemStayOnTop'
{$ENDIF}
                                                      ) ;

{$IFNDEF FPC}
const PrintScales : array [ TPrintScale ] of string =
          ( 'poNone', 'poProportional', 'poPrintToFit' ) ;
{$ENDIF}

begin
    F := TForm( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute :=  lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( F.Parent = nil ) then
                begin
                    F.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, F, F ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'alphablend' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.Alphablend := ( S = 'true' ) ;
            end else
            if( Attribute = 'alphablendvalue' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.Alphablendvalue := strtoint( S ) ;
            end else
            if( Attribute = 'bordericons' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                S := copy( S, 2, length( S ) ) ;
                setlength( S, length( S ) - 1 ) ; // Remove brackets
                F.Bordericons := String_To_BorderIcons( S ) ;
            end else
            if( Attribute = 'borderstyle' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.BorderStyle := String_To_BorderStyle( S ) ;
            end else
            if( Attribute = 'clientheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.ClientHeight := strtoint( S ) ;
            end else
            if( Attribute = 'clientwidth' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.ClientWidth := strtoint( S ) ;
            end else
            if( Attribute = 'color' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.Color := String_To_Color( S ) ;
            end else
            if( Attribute = 'formstyle' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                for FormStyle := low( FormStyle ) to high( FormStyle ) do
                begin
                    if( S = FormStyles[ FormStyle ] ) then
                    begin
                        F.Formstyle := FormStyle ;
                        break ;
                    end ;
                end ;
            end else
            if( Attribute = 'transparentcolor' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                F.Transparentcolor := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'transparentcolorvalue' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                F.transparentcolorvalue := String_To_Color( S ) ;
{$ENDIF}
            end else
            if( Attribute = 'oldcreateorder' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                F.OldCreateOrder := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'pixelsperinch' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.Pixelsperinch := strtoint( S ) ;
            end else
            if( Attribute = 'printscale' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                for PrintScale := low( PrintScale ) to high( PrintScale ) do
                begin
                    if( S = PrintScales[ PrintScale ] ) then
                    begin
                        F.Printscale := PrintScale ;
                        break ;
                    end ;
                end ;
{$ENDIF}
            end else
            if( Attribute = 'scaled' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                F.OldCreateOrder := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'textheight' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                //F.TextHeight := strtoint( S ) ; // Ignore this (for now)
            end else
            if( Attribute = 'tilemode' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                if( S = 'tbhorizontal' ) then
                begin
                    F.TileMode := tbHorizontal ;
                end else
                begin
                    F.TileMode := tbVertical ;
                end ;
{$ENDIF}
            end else
            if( Attribute = 'visible' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                F.Visible := ( S = 'true' ) ;
            end else
            begin
                if( Result <> '' ) then
                begin
                    Result := Result + CR ;
                end ;
                Result := Result + S ;
            end ;
        end ;
    end ;
    Result := Process_Wincontrol( O, Result, Name, Parent ) ;
end ; // Process_Form


function Process_Spinedit( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    S, Temp : ansistring ;
    Sp : TSpinEdit ;

begin
    Sp := TSpinEdit( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute :=  lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( Sp.Parent = nil ) then
                begin
                    Sp.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, Sp, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'maxvalue' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sp.Maxvalue := strtoint( S ) ;
            end else
            if( Attribute = 'minvalue' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sp.Minvalue := strtoint( S ) ;
            end else
            if( Attribute = 'value' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                Sp.Value := strtoint( S ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Buttoncontrol( O, Result, Name, Parent ) ;
end ; // Process_Spinedit


function Process_Image( O : TObject ; Def, Name : ansistring ; Parent : TControl ) : ansistring ;

var Attribute : ansistring ;
    Dummy : integer ;
    S, Temp : ansistring ;
    I : TImage ;

begin
    I := TImage( O ) ;
    Result := '' ;
    while( Def <> '' ) do
    begin
        S := trim( Next_Line( Def ) ) ;
        if( S <> '' ) then
        begin
            Dummy := pos( '=', S + '=' ) ;
            Attribute :=  lowercase( trim( copy( S, 1, Dummy - 1 ) ) ) ;
            if( copy( Attribute, 1, 7 ) = 'object ' ) then
            begin
                if( I.Parent = nil ) then
                begin
                    I.Parent := TWinControl( Parent ) ;
                end ;
                Temp := Extract_Object_Definition( Def ) ;
                Temp := Create_Child( trim( copy( S, 8, length( S ) ) ), Temp, I, Parent ) ;
                if( length( Temp ) > 0 ) then
                begin
                    raise Exception.Create( 'Unknown item: ' + Temp ) ;
                end ;
            end else
            if( Attribute = 'center' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                I.Center := ( S = 'true' ) ;
            end else
            if( Attribute = 'incrementaldisplay' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
{$IFNDEF FPC}
                I.IncrementalDisplay := ( S = 'true' ) ;
{$ENDIF}
            end else
            if( Attribute = 'picture' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                I.Picture.Bitmap := Get_Bitmap( S ) ;
            end else
            if( Attribute = 'proportional' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                I.Proportional := ( S = 'true' ) ;
            end else
            if( Attribute = 'stretch' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                I.Stretch := ( S = 'true' ) ;
            end else
            if( Attribute = 'transparent' ) then
            begin
                S := trim( lowercase( copy( S, Dummy + 1, length( S ) ) ) ) ;
                I.Transparent := ( S = 'true' ) ;
            end else
            begin
                Result := Result + CR + S ;
            end ;
        end ;
    end ;
    Result := Process_Buttoncontrol( O, Result, Name, Parent ) ;
end ; // Process_Image


function Default_Construct( O : TObject ; Name, Typ : ansistring ; var S : ansistring ;
    Parent : TControl ; Owner : TComponent ) : TObject ;

var Temp : ansistring ;

begin
    // Setup...
    Result := O ;
    if( Owner = nil ) then
    begin
        Owner := Parent ;
    end ;
    Typ := uppercase( Typ ) ;

    if( Typ = 'TBITBTN' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TBitBtn.Create( Owner ) ;
        end ;
        S := Process_Bitbtn( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TCHECKBOX' ) or ( Typ = 'TCUSTOMCHECKBOX' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TCheckBox.Create( Owner ) ;
        end ;
        S := Process_checkbox( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TCOMBOBOX' ) or ( Typ = 'TCUSTOMCOMBOBOX' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TComboBox.Create( Owner ) ;
        end ;
        S := Process_combobox( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TCOMPONENT' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TComponent.Create( Owner ) ;
        end ;
        S := Process_component( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TEDIT' ) or ( Typ = 'TCUSTOMEDIT' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TEdit.Create( Owner ) ;
        end ;
        S := Process_Edit( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TFORM' ) or ( Typ = 'TCUSTOMFORM' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TForm.Create( Owner ) ;
        end ;
        S := Process_Form( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TGROUPBOX' ) or ( Typ = 'TCUSTOMGROUPBOX' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TGroupBox.Create( Owner ) ;
        end ;
        S := Process_WinControl( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TIMAGE' ) or ( Typ = 'TCUSTOMGROUPBOX' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TImage.Create( Owner ) ;
        end ;
        S := Process_Image( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TLABEL' ) or ( Typ = 'TCUSTOMLABEL' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TLabel.Create( Owner ) ;
        end ;
        S := Process_Label( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TLISTBOX' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TListBox.Create( Owner ) ;
        end ;
        S := Process_Listbox( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TPAGECONTROL' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TPageControl.Create( Owner ) ;
        end ;
        S := Process_Wincontrol( Result, S, Name, Parent ) ;
    end else
    if( ( Typ = 'TPANEL' ) or ( Typ = 'TCUSTOMPANEL' ) ) then
    begin
        if( Result = nil ) then
        begin
            Result := TPanel.Create( Owner ) ;
        end ;
        S := Process_Panel( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TRADIOBUTTON' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TRadioButton.Create( Owner ) ;
        end ;
        S := Process_RadioButton( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TSCROLLBAR' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TScrollbar.Create( Owner ) ;
        end ;
        S := Process_Scrollbar( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TSPINEDIT' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TSpinEdit.Create( Owner ) ;
        end ;
        S := Process_Spinedit( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TSPLITTER' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TSplitter.Create( Owner ) ;
        end ;
        S := Process_Splitter( Result, S, Name, Parent ) ;
    end else
    if( Typ = 'TTABSHEET' ) then
    begin
        if( Result = nil ) then
        begin
            Result := TTabSheet.Create( Owner ) ;
        end ;
        S := Process_Tabsheet( Result, S, Name, Parent ) ;
    end else
    begin
        raise Exception.Create( 'Unknown type: ' + Typ ) ;
    end ;
    if( S <> '' ) then
    begin
        Temp := lowercase( trim( Next_Line( S ) ) ) ;
        while( ( Temp = '' ) and ( S <> '' ) ) do
        begin
            Temp := lowercase( trim( Next_Line( S ) ) ) ;
        end ;
        if( Temp = 'end' ) then // End of this object definition
        begin
            exit ;
        end ;
        if( copy( Temp, 1, 7 ) = 'object ' ) then // Child object
        begin
            Temp := trim( Create_Child( copy( Temp, 8, length( Temp ) ), S, TControl( Result ), TComponent( Result ) ) ) ;
            if( Temp = '' ) then
            begin
                exit ;
            end ;
        end ;
        raise Exception.Create( 'Unknown attribute: ' + Temp ) ;
    end ; // if( S <> '' )
end ; // Default_Construct


initialization
    Focus_Controls := TStringList.Create ;

finalization
    Focus_Controls.Free ;
    Focus_Controls := nil ;
end.
