unit GUI ;

interface

type TRectangle = packed record
                      X, Y : integer ;
                  end ;

type TPixel = int64 ; // RGBA (word values)

type TImage = class // PNG image
                  public // API
              end ;

type TSurface = class // Something to draw upon
                    protected // Property handlers...
                        procedure Set_Width( Value : integer ) ; virtual ; abstract ;
                        procedure Set_Height( Value : integer ) ; virtual ; abstract ;
                        procedure Set_Depth( Value : integer ) ; virtual ; abstract ;
                        function Get_Width : integer ; virtual ; abstract ;
                        function Get_Height : integer ; virtual ; abstract ;
                        function Get_Depth : integer ; virtual ; abstract ;
                        function Get_Pixel( X, Y, Z : integer ) : TPixel ;
                            virtual ; abstract ;
                        procedure Set_Pixel( X, Y, Z : integer ; Value : TPixel ) ;
                            virtual ; abstract ;

                    public // API...
                        procedure Draw( const DestR, SrcR : TRectangle ; Source : Timage ) ;
                            virtual ; abstract ;
                        procedure Set_Dimensions( H, W, D : integer ) ;
                            virtual ; abstract ;

                    public // Properties...
                        property Pixels[ X, Y, Z : integer ] : TPixel
                            read Get_Pixel
                            write Set_Pixel ;
                        property Width : integer
                            read Get_Width
                            write Set_Width ;
                        property Height : integer
                            read Get_Height
                            write Set_Height ;
                        property Depth : integer
                            read Get_Depth
                            write Set_Depth ;
                end ; // TSurface

type TAnimation = class // Animated image
                      public // API
                          procedure Show( Surface : TSurface ) ; virtual ; abstract ;
                          procedure Stop ; virtual ; abstract ;
                          procedure Hide ; virtual ; abstract ;
                  end ;


type TPointer = class // Mouse (or other) pointer
                    public // API
                         procedure Show ; virtual ; abstract ;
                         procedure Hide ; virtual ; abstract ;
                         procedure Set_Position( X, Y, Z : integer ) ;
                              virtual ; abstract ;
                end ;

type TImage_Pointer = class( TPointer ) // pointer with an image
                          public // API...
                              procedure Set_Blink_Rate ; virtual ; abstract ;
                              function Get_Image : TImage ; virtual ; abstract ;
                              procedure Set_Image( Value : TImage ) ;
                                  virtual ; abstract ;
                      end ;

type TAnimated_Pointer = class( TPointer ) // pointer with an animation
                             public // API...
                                 function Get_Animation : TAnimation ;
                                     virtual ; abstract ;
                                 procedure Set_Animation( Value : TAnimation ) ;
                                     virtual ; abstract ;
                         end ;

implementation

end.

